# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from wxPython.wx import *
from Common import *

def SetTextColour(text, value):
    if value < 0:
        colour="#DD0000"
    elif value > 0:
        colour="#008800"
    else:
        colour="#000000"
    text.SetForegroundColour(colour)

def SetTextBold(text):
    fnt=text.GetFont()
    fnt.SetWeight(wxBOLD)
    text.SetFont(fnt)

FACE_NONE = 0x00
FACE_BOLD = 0x01
FACE_BIG = 0x02

def SetTextFace(text, face_flags):
    if face_flags != FACE_NONE:
        tfont = text.GetFont()
        if face_flags & FACE_BOLD:
            tfont.SetWeight(wxBOLD)
        if face_flags & FACE_BIG:
            tfont.SetPointSize(tfont.GetPointSize() + 2)
        text.SetFont(tfont)
    return text

def version_str_with_wx():
    return version_str() + (_("Using the gui toolkit wxWidgets %s") % wxVERSION_STRING,)

class SimpleColouredPanel(wxPanel):
    def __init__(self, parent, ID, colour = None, sizer_direction = wxHORIZONTAL):
        wxPanel.__init__(self, parent, ID)

        if colour is not None:
            self.SetBackgroundColour(colour)

        sizer = wxBoxSizer(sizer_direction)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

    def AddChild(self, child, size, flags, padding):
        sizer = self.GetSizer()

        sizer.Add(child, size, flags, padding)

        sizer.Fit(self)

class HeaderPanel(wxPanel):
    def __init__(self, parent, ID, text, colour = None):
        if wxPlatform == '__WXMSW__':
            pstyle = wxSTATIC_BORDER
        elif wxPlatform == '__WXGTK__':
            pstyle = wxRAISED_BORDER
        else:
            pstyle = wxSIMPLE_BORDER

        wxPanel.__init__(self, parent, ID, style = pstyle)

        if colour is not None:
            self.SetBackgroundColour(colour)

        psizer = wxBoxSizer(wxHORIZONTAL)

        self.header_text = wxStaticText(self, -1, text, style = wxALIGN_CENTRE)
        SetTextFace(self.header_text, FACE_BOLD | FACE_BIG)
        psizer.Add(self.header_text, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        self.SetSizer(psizer)
        self.SetAutoLayout(True)
        psizer.Fit(self)

class InvisibleSizer(wxPySizer):
    def __init__(self):
        wxPySizer.__init__(self)

    def CalcMin(self):
        items = self.GetChildren()
        if not items:
            return wxSize(1, 1)
        sizes = [item.CalcMin() for item in items]
        hw = ([s.width for s in sizes], [s.height for s in sizes])
        return wxSize(max(hw[0]), max(hw[1]))

    def RecalcSizes(self):
        mysize = self.GetSize()
        pos = self.GetPosition()
        for item in self.GetChildren():
            item.SetDimension(pos, mysize)
