# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import Players
import HTCalendar
from Common import *
import CommonData

import time

from wxPython.wx import *
from wxPython.calendar import *

class ComparisonPage(wxPanel):
    ID_NO_FILTER           = 225
    def __init__(self, parent, ID):
        wxPanel.__init__(self, parent, ID)

        sizer = wxBoxSizer(wxVERTICAL)
        hsizer = wxBoxSizer(wxHORIZONTAL)

        self.first_cal = HTCalendar.CalendarChoice(self, -1)
        self.last_cal = HTCalendar.CalendarChoice(self, -1)

        hsizer.Add(self.first_cal, 0, wxALIGN_LEFT | wxALL, 0)
        hsizer.Add((0,0),1)
        button = wxButton(self, -1, _("Today"))
        hsizer.Add(button, 0, wxALIGN_RIGHT | wxALL, 0)
        hsizer.Add((5,5), 0)
        hsizer.Add(self.last_cal, 0, wxALIGN_LEFT | wxALL, 0)
        HTCalendar.EVT_CALENDAR(self, self.first_cal.GetId(), self.OnFirstCalSelected)
        HTCalendar.EVT_CALENDAR(self, self.last_cal.GetId(), self.OnLastCalSelected)
        EVT_BUTTON(self, button.GetId(), self.OnToday)
        sizer.Add(hsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        hsizer = wxBoxSizer(wxHORIZONTAL)

        vsizer = wxBoxSizer(wxVERTICAL)
        self.no_filter = wxCheckBox(self, ComparisonPage.ID_NO_FILTER, _("No filter"))
        self.no_filter.SetValue(False)
        vsizer.Add(self.no_filter, 1, wxALIGN_CENTRE | wxALL | wxEXPAND, 2)
        EVT_CHECKBOX(self, ComparisonPage.ID_NO_FILTER, self.NoFilterChanged)

        self.choice = wxChoice(self, -1, choices = [_("Group by player"), _("Group by attribute")])
        self.choice.SetSelection(0)
        self.grouping = 0
        EVT_CHOICE(self, self.choice.GetId(), self.OnChoice)
        vsizer.Add(self.choice, 1, wxALIGN_CENTRE | wxALL | wxEXPAND, 2)

        hsizer.Add(vsizer, 0, wxALIGN_CENTRE | wxALL | wxEXPAND, 0)
        vsizer = wxBoxSizer(wxVERTICAL)

        default_filters = [Players.plattr_SKILL, Players.plattr_ECONOMY,
                           Players.plattr_STATUS, Players.plattr_PERSONALITY]

        self.filter_cb = {}
        c = (len(Players.plattr_strings.keys())) / 3
        i = 0
        j = 0
        isizer = None
        for f in Players.plattr_strings.keys():
            if i == 0 and isizer == None:
                isizer = wxBoxSizer(wxHORIZONTAL)

            self.filter_cb[f] = wxCheckBox(self, j + 230, Players.plattr_strings[f])
            self.filter_cb[f].SetValue(f in default_filters)
            isizer.Add(self.filter_cb[f], 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)
            EVT_CHECKBOX(self, j + 230, self.OnFilterChanged)

            if i == c-1 and isizer != None:
                vsizer.Add(isizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)
                isizer = None
            i = (i+1) % c
            j += 1

        if isizer != None:
            vsizer.Add(isizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        hsizer.Add(vsizer, 5, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)
        sizer.Add(hsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        self.textarea = wxTextCtrl(self, -1, style = wxHSCROLL | wxTE_MULTILINE | wxTE_READONLY)
        sizer.Add(self.textarea, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

    def OnFirstCalSelected(self, event):
        htdate = self.first_cal.GetDate()

        self.first_date = time.localtime(CommonData.HT2unix_date(htdate))[0:3] + (24,)
        self.Update()

    def OnToday(self, event):
        time2 = time.time()
        httime2 = CommonData.local2HT_date(time2)
        self.last_cal.SetDate(httime2)
        self.last_date = time.localtime(time2)[0:3] + (24,)

        self.Update()

    def OnLastCalSelected(self, event):
        htdate = self.last_cal.GetDate()

        self.last_date = time.localtime(CommonData.HT2unix_date(htdate))[0:3] + (24,)
        self.Update()

    def AfterInit(self):
        time2 = time.time()
        time1 = time2 - 86400*7

        httime1 = CommonData.local2HT_date(time1)
        httime2 = CommonData.local2HT_date(time2)

        self.first_cal.SetDate(httime1)
        self.last_cal.SetDate(httime2)

        self.first_date = time.localtime(time1)[0:4]
        self.last_date = time.localtime(time2)[0:4]

        self.Update()

    def ChangeXML(self):
        p1=Players.Players()
        self.last_date = time.localtime(p1.fetched_date)[0:4]

    def OnChoice(self, event):
        self.grouping = self.choice.GetSelection()
        self.Update()

    def NoFilterChanged(self, event):
        enabled = not self.no_filter.GetValue()
        for f, fcb in self.filter_cb.items():
            fcb.Enable(enabled)
        self.OnFilterChanged(event)

    def OnFilterChanged(self, event):
        self.Update()

    def Update(self):
        self.textarea.Clear()
        self.textarea.AppendText(self.__get_change_text())
        self.textarea.SetInsertionPoint(0)

    def __get_change_text(self):
        filter_list = []
        if not self.no_filter.GetValue():
            for f, fcb in self.filter_cb.items():
                if fcb.GetValue():
                    filter_list.append(f)
        p1 = Players.Players(self.first_date)
        p2 = Players.Players(self.last_date)
        cp = Players.Compare()

        if len(filter_list) == 0:
            filter_list = None
        return cp.get_text(p1, p2, self.grouping, filter_list)
