# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from Config import *
from Common import *
from Commonwx import *

import string

from wxPython.wx import *

apply_function_list = []

class ConfTypeIntegerPanel(wxPanel):
    def __init__(self, parent, ID, section, conf_info):
        wxPanel.__init__(self, parent, ID)

        self.conf_info = conf_info
        self.section = section

        if self.conf_info[4] is None:
            (min, max) = (-1000000, 1000000)
        else:
            (min, max) = self.conf_info[4]
            
        sizer = wxBoxSizer(wxHORIZONTAL)
        stext = wxStaticText(self, -1, _(conf_info[1]) + ":")
        sizer.Add(stext, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        self.sb = wxSpinCtrl(self, -1, min = min, max = max,
                             style = wxSP_ARROW_KEYS | wxTE_RIGHT)
        self.sb.SetValue(config.get(section, conf_info[0]))
        sizer.Add(self.sb, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

    def set(self, value):
        self.sb.SetValue(value)

    def get(self):
        return self.sb.GetValue()

class ConfTypeBooleanPanel(wxPanel):
    def __init__(self, parent, ID, section, conf_info):
        wxPanel.__init__(self, parent, ID)

        self.conf_info = conf_info
        self.section = section

        sizer = wxBoxSizer(wxHORIZONTAL)

        self.cb = wxCheckBox(self, -1, _(conf_info[1]))
        self.cb.SetValue(config.get(section, conf_info[0]))

        sizer.Add(self.cb, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

    def set(self, value):
        self.cb.SetValue(bool(value))

    def get(self):
        return self.cb.GetValue()

class ConfTypeFloatPanel(wxPanel):
    ID_TC = 446

    def __init__(self, parent, ID, section, conf_info):
        wxPanel.__init__(self, parent, ID)

        self.conf_info = conf_info
        self.section = section

        sizer = wxBoxSizer(wxVERTICAL)
        stext = wxStaticText(self, -1, _(conf_info[1]) + ":")
        sizer.Add(stext, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        self.tc = wxTextCtrl(self, ConfTypeFloatPanel.ID_TC, style = wxTE_PROCESS_ENTER)
        self.cval = str(config.get(section, conf_info[0]))
        self.tc.SetValue(self.cval)
        sizer.Add(self.tc, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        EVT_TEXT(self.tc, ConfTypeFloatPanel.ID_TC, self.OnTextUpdated)
        EVT_TEXT_ENTER(self.tc, ConfTypeFloatPanel.ID_TC, self.OnTextEnter)
        EVT_KILL_FOCUS(self.tc, self.OnCheckText)

    def set(self, value):
        self.cval = str(value)
        self.tc.SetValue()

    def get(self):
        self.__check_text(True)
        return float(self.cval)

    def OnTextUpdated(self, event):
        self.__check_text(False)

    def OnTextEnter(self, event):
        self.__check_text(True)

    def OnCheckText(self, event):
        self.__check_text(True)
        event.Skip()

    def __check_text(self, change_text = True):
        val = self.tc.GetValue()
        print self.cval, val

        try:
            fval = float(val)
            self.cval = val
        except:
            if change_text:
                self.tc.SetValue(self.cval)

class ConfTypeStringPanel(wxPanel):
    def __init__(self, parent, ID, section, conf_info):
        wxPanel.__init__(self, parent, ID)

        self.conf_info = conf_info
        if conf_info[4] == "password":
            style = wxPASSWORD
        else:
            style = 0
            
        self.section = section

        sizer = wxBoxSizer(wxVERTICAL)
        stext = wxStaticText(self, -1, _(conf_info[1]) + ":")
        sizer.Add(stext, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        self.tc = wxTextCtrl(self, -1, style = style)
        self.tc.SetValue(config.get(section, conf_info[0]))
        sizer.Add(self.tc, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

    def set(self, value):
        self.tc.SetValue(value)

    def get(self):
        return self.tc.GetValue()

class ConfTypeDirPanel(wxPanel):
    ID_DIR_DIALOG = 443

    def __init__(self, parent, ID, section, conf_info, with_dir):
        wxPanel.__init__(self, parent, ID)

        self.with_dir = with_dir
        self.conf_info = conf_info
        self.section = section

        sizer = wxBoxSizer(wxVERTICAL)
        stext = wxStaticText(self, -1, _(conf_info[1]) + ":")
        sizer.Add(stext, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        hsizer = wxBoxSizer(wxHORIZONTAL)

        self.tc = wxTextCtrl(self, -1)
        self.tc.SetValue(config.get(section, conf_info[0]))
        hsizer.Add(self.tc, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        button = wxButton(self, ConfTypeDirPanel.ID_DIR_DIALOG, _("Browse"), style = wxBU_EXACTFIT)
        hsizer.Add(button, 0, wxALIGN_CENTRE | wxALL, 2)

        sizer.Add(hsizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        EVT_BUTTON(self, ConfTypeDirPanel.ID_DIR_DIALOG, self.OnDirSelect)

    def OnDirSelect(self, event):
        if self.with_dir:
            defdir = os.path.expanduser(self.tc.GetValue())
        else:
            defdir = os.path.expanduser(os.path.dirname(self.tc.GetValue()))
        if not defdir:
            defdir = "."
        if self.with_dir:
            dlg = wxDirDialog(self, _("Select catalog to add to the list."),
                              defaultPath = defdir,
                              style = wxRESIZE_BORDER | wxCAPTION | wxSYSTEM_MENU)
        else:
            if self.conf_info[4] is None:
                wildcard = "*.*"
            else:
                wildcard = self.conf_info[4]
            dlg = wxFileDialog(self, _("Select file to add to the list."),
                               defaultDir = defdir, style = wxOPEN, wildcard = wildcard)
        res = dlg.ShowModal()
        if res == wxID_OK:
            self.tc.SetValue(dlg.GetPath())

    def set(self, value):
        self.tc.SetValue(value)

    def get(self):
        return self.tc.GetValue()

class ConfTypePathPanel(wxPanel):
    ID_CLIST = 440
    ID_ADD = 441
    ID_REMOVE = 442

    def __init__(self, parent, ID, section, conf_info, with_dir):
        wxPanel.__init__(self, parent, ID)

        self.with_dir = with_dir
        self.conf_info = conf_info
        self.section = section

        main_sizer = wxBoxSizer(wxVERTICAL)
        stext = wxStaticText(self, -1, _(conf_info[1]) + ":")
        main_sizer.Add(stext, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        sizer = wxBoxSizer(wxHORIZONTAL)

        self.clist = wxListBox(self, ConfTypePathPanel.ID_CLIST,
                               choices = config.get(section, conf_info[0]),
                               style = wxLB_NEEDED_SB | wxLB_MULTIPLE
                               )

        sizer.Add(self.clist, 1, wxEXPAND | wxALIGN_CENTRE | wxRIGHT | wxTOP | wxBOTTOM, 5)

        bsizer = wxBoxSizer(wxVERTICAL)

        button = wxButton(self, ConfTypePathPanel.ID_ADD, _("Add"))
        bsizer.Add(button, 1, wxALIGN_CENTRE | wxTOP | wxBOTTOM, 5)

        button = wxButton(self, ConfTypePathPanel.ID_REMOVE, _("Remove"))
        bsizer.Add(button, 1, wxALIGN_CENTRE | wxTOP | wxBOTTOM, 5)

        sizer.Add(bsizer, 0, wxEXPAND | wxALL, 0)
        main_sizer.Add(sizer, 1, wxEXPAND | wxALL, 5)

        self.SetSizer(main_sizer)
        self.SetAutoLayout(True)
        main_sizer.Fit(self)

        EVT_BUTTON(self, ConfTypePathPanel.ID_ADD, self.OnAdd)
        EVT_BUTTON(self, ConfTypePathPanel.ID_REMOVE, self.OnRemove)

    def OnAdd(self, event):
        if self.with_dir:
            dlg = wxDirDialog(self, _("Select catalog to add to the list."),
                              defaultPath = ".",
                              style = wxRESIZE_BORDER | wxCAPTION | wxSYSTEM_MENU)
        else:
            dlg = wxFileDialog(self, _("Select file to add to the list."),
                               defaultDir = ".", style = wxOPEN)
        res = dlg.ShowModal()
        if res == wxID_OK:
            self.clist.Append(dlg.GetPath())

    def OnRemove(self, event):
        sel = self.clist.GetSelections()
        for i in sel:
            minus = len(filter(lambda x: i > x, sel))
            self.clist.Delete(i - minus)

    def set(self, value):
        self.clist.SetSelection(value)

    def get(self):
        n = self.clist.GetCount()
        return [self.clist.GetString(i) for i in range(n)]

class ConfTypeChoicePanel(wxPanel):
    def __init__(self, parent, ID, section, conf_info):
        wxPanel.__init__(self, parent, ID)

        self.conf_info = conf_info
        self.section = section

        sizer = wxBoxSizer(wxHORIZONTAL)
        stext = wxStaticText(self, -1, _(conf_info[1]) + ":")
        sizer.Add(stext, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        n = 0
        for val, descr in self.conf_info[4]:
            if val == config.get(section, conf_info[0]):
                break
            n += 1
        if n == 4:
            n = 0

        self.t_info = []
        for v, d in self.conf_info[4]:
            self.t_info.append((v, _(d)))

        self.cw = wxChoice(self, -1, choices = list(zip(*self.t_info)[1]))
        sizer.Add(self.cw, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)
        self.cw.SetSelection(n)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

    def set(self, value):
        for v, d in self.t_info:
            if v == value:
                self.cw.SetStringSelection(d)

    def get(self):
        s = self.cw.GetStringSelection()
        for v, d in self.t_info:
            if s == d:
                return v
        return None

class ConfTypeMultipleChoicePanel(wxPanel):
    ADD_COLUMNS = 444
    REMOVE_COLUMNS = 445
    MOVE_UP = 446
    MOVE_DOWN = 447

    def __init__(self, parent, ID, section, conf_info):
        wxPanel.__init__(self, parent, ID)

        self.conf_info = conf_info
        self.section = section

        (option, description, type, value_list, choice_list) = conf_info
        description = _(description)

        (enabledstrings, disabledstrings) = self.__get_ed_strings(config.get(section, conf_info[0]))

        main_sizer = wxBoxSizer(wxVERTICAL)
        stext = wxStaticText(self, -1, description + ":")
        main_sizer.Add(stext, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        sizer = wxBoxSizer(wxHORIZONTAL)

        self.rcols = wxListBox(self, -1, size = (120, 120), style=wxLB_EXTENDED)
        self.rcols.InsertItems(disabledstrings, 0)
        sizer.Add(self.rcols, 1, wxEXPAND | wxALL, 5)

        tsizer = wxBoxSizer(wxVERTICAL)
        tsizer.Add(wxButton(self, self.REMOVE_COLUMNS, "<<"), 0, wxALIGN_CENTRE | wxALL, 5)
        tsizer.Add(wxButton(self, self.ADD_COLUMNS, ">>"), 0, wxALIGN_CENTRE | wxALL, 5)
        sizer.Add(tsizer, 0, wxALIGN_CENTRE | wxALL, 5)

        asizer = wxBoxSizer(wxVERTICAL)
        self.acols = wxListBox(self, -1, size = (120, 120), style=wxLB_EXTENDED)
        self.acols.InsertItems(enabledstrings, 0)
        asizer.Add(self.acols, 1, wxEXPAND | wxALL, 5)
        asizer.Add(wxButton(self, self.MOVE_UP, _("Up")), 0, wxALIGN_CENTRE | wxALL, 0)
        asizer.Add(wxButton(self, self.MOVE_DOWN, _("Down")), 0, wxALIGN_CENTRE | wxALL, 0)
        sizer.Add(asizer, 1, wxEXPAND | wxALL, 5)

        main_sizer.Add(sizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        self.SetSizer(main_sizer)
        self.SetAutoLayout(True)
        main_sizer.Fit(self)

        EVT_BUTTON(self, self.REMOVE_COLUMNS, self.OnRemoveColumns)
        EVT_BUTTON(self, self.ADD_COLUMNS, self.OnAddColumns)
        EVT_BUTTON(self, self.MOVE_UP, self.OnMoveUp)
        EVT_BUTTON(self, self.MOVE_DOWN, self.OnMoveDown)

    def OnRemoveColumns(self, event):
        self.MoveColumns(self.acols, self.rcols)

    def OnAddColumns(self, event):
        self.MoveColumns(self.rcols, self.acols)

    def OnMoveUp(self, event):
        tuple = self.acols.GetSelections()
        for x in tuple:
            if (x==0):
                self.acols.Deselect(0)
            else:
                temp = self.acols.GetString(x)
                self.acols.SetString(x, self.acols.GetString(x-1))
                self.acols.SetString(x-1, temp)
                self.acols.Deselect(x)
                self.acols.SetSelection(x-1)

    def OnMoveDown(self, event):
        tuple = self.acols.GetSelections()
        count = self.acols.GetCount()
        list = range(count)
        list.reverse()
        for x in list:
            if (self.acols.Selected(x)):
                if(x!=(count-1)):
                    temp = self.acols.GetString(x)
                    self.acols.SetString(x, self.acols.GetString(x+1))
                    self.acols.SetString(x+1, temp)
                    self.acols.Deselect(x)
                    self.acols.SetSelection(x+1)
                else:
                    self.acols.Deselect(count-1)

    def MoveColumns(self, srccolumns, dstcolumns):
        self.mytuple = srccolumns.GetSelections()
        y = 0
        for x in range(len(self.mytuple)):
            dstcolumns.Append(srccolumns.GetString(self.mytuple[x] - y))
            srccolumns.Delete(self.mytuple[x] - y)
            y += 1

    def set(self, value):
        (enabledstrings, disabledstrings) = self.__get_ed_strings(value)

        self.rcols.Clear()
        self.acols.Clear()
        self.rcols.InsertItems(disabledstrings, 0)
        self.acols.InsertItems(enabledstrings, 0)

    def get(self):
        res = []
        for x in range(self.acols.GetCount()):
            res.append(self.acols.GetString(x))
        return res

    def __get_ed_strings(self, value_list):
        choice_list = self.conf_info[4]

        enabledstrings = []
        disabledstrings = []
        for c in value_list:
            enabledstrings.append(c)
        for c in choice_list:
            if not c in value_list:
                disabledstrings.append(c)

        return (enabledstrings, disabledstrings)

type_panels = {
    CONF_TYPE_INTEGER: (ConfTypeIntegerPanel, ()),
    CONF_TYPE_BOOLEAN: (ConfTypeBooleanPanel, ()),
    CONF_TYPE_FLOAT: (ConfTypeFloatPanel, ()),
    CONF_TYPE_STRING: (ConfTypeStringPanel, ()),
    CONF_TYPE_DIR: (ConfTypeDirPanel, (True,)),
    CONF_TYPE_PATH: (ConfTypePathPanel, (True,)),
    CONF_TYPE_FILE: (ConfTypeDirPanel, (False,)),
    CONF_TYPE_MULTIPLE_FILES: (ConfTypePathPanel, (False,)),
    CONF_TYPE_CHOICE: (ConfTypeChoicePanel, ()),
    CONF_TYPE_MULTIPLE_CHOICE: (ConfTypeMultipleChoicePanel, ()),
    }

class ConfSectionPanel(wxPanel):
    def __init__(self, parent, ID, section):
        wxPanel.__init__(self, parent, ID)

        self.section = section

        main_sizer = wxBoxSizer(wxVERTICAL)

        conf_info = config.config_info[section]
        self.config_items = {}

        for ci in conf_info:
            tp = type_panels[ci[2]]
            if tp != None:
                panel = tp[0](*((self, -1, section, ci) + tp[1]))
                main_sizer.Add(panel, 0, wxEXPAND | wxALIGN_LEFT | wxTOP | wxBOTTOM, 5)
                self.config_items[ci[0]] = panel

        self.SetSizer(main_sizer)
        self.SetAutoLayout(True)
        main_sizer.Fit(self)

    def set_option(self, option, value):
        self.config_items[option].set(value)

    def get_option(self, option):
        return self.config_items[option].get()

    def get(self):
        result = {}
        for option, panel in self.config_items.items():
            result[option] = panel.get()
        return result

class ConfigPanel(wxPanel):
    def __init__(self, parent, ID):
        wxPanel.__init__(self, parent, ID)

        self.type = "General"

        self.all_panels = {}
        main_sizer = wxBoxSizer(wxVERTICAL)

        if wxPlatform == '__WXMSW__':
            pstyle = wxSTATIC_BORDER
        elif wxPlatform == '__WXGTK__':
            pstyle = wxRAISED_BORDER
        else:
            pstyle = wxSIMPLE_BORDER

        panel = wxPanel(self, -1, style=pstyle)
        psizer = wxBoxSizer(wxVERTICAL)
        self.header_text = wxStaticText(panel, -1, config_sections[self.type],
                                        style=wxALIGN_CENTRE)
        SetTextFace(self.header_text, FACE_BOLD | FACE_BIG)
        psizer.Add(self.header_text, 1, wxALIGN_CENTRE | wxALL, 5)
        panel.SetSizer(psizer)
        panel.SetAutoLayout(True)
        psizer.Fit(panel)
        main_sizer.Add(panel, 0, wxEXPAND | wxALL, 0)

        isizer = InvisibleSizer()
        for type, descr in config_sections.items():
            panel = ConfSectionPanel(self, -1, type)
            self.all_panels[type] = panel
            isizer.Add(panel, 1, wxEXPAND | wxALL, 0)
            if self.type == type:
                panel.Show()
            else:
                panel.Hide()
        main_sizer.Add(isizer, 1, wxEXPAND | wxALL, 0)

        self.SetSizer(main_sizer)
        self.SetAutoLayout(True)
        main_sizer.Fit(self)

    def change_type(self, new_type):
        if new_type == self.type:
            return

        old_type = self.type
        old_panel = self.all_panels[old_type]
        new_panel = self.all_panels[new_type]
        old_panel.Hide()
        new_panel.Show()
        self.type = new_type
        self.header_text.SetLabel(config_sections[self.type])
        self.Refresh()

    def get_changed_vars(self):
        res = {}
        for s, p in self.all_panels.items():
            vars = p.get()
            cvars = {}
            for v, val in vars.items():
                if val != config.get(s,v):
                    cvars[v] = val
            if len(cvars) > 0:
                res[s] = cvars
        return res

class ConfigDialog(wxDialog):
    ID_APPLY = 401
    ID_SAVE = 402
    ID_CLOSE = 403

    conf_dialog_running = False

    def __init__(self, parent, ID):
        wxDialog.__init__(self, parent, ID, _("Preferences"),
                          style = wxRESIZE_BORDER | wxCAPTION | wxSYSTEM_MENU)

        ConfigDialog.conf_dialog_running = True

        main_sizer = wxBoxSizer(wxVERTICAL)

        sizer = wxBoxSizer(wxHORIZONTAL)

        panel = wxPanel(self, -1, style=wxSUNKEN_BORDER)
        psizer = wxBoxSizer(wxVERTICAL)

        types = config_sections.keys()
        types.sort()
        types = ["General"] + filter(lambda x: x != "General", types)
        self.type_ids = {}
        id = 410
        for type in types:
            descr = config_sections[type]
            id += 1
            button = wxButton(panel, id, descr)
            psizer.Add(button, 0, wxEXPAND | wxALIGN_TOP | wxALL, 0)
            self.type_ids[id] = type
            EVT_BUTTON(self, id, self.OnChangeType)

        panel.SetSizer(psizer)
        panel.SetAutoLayout(True)
        psizer.Fit(panel)

        sizer.Add(panel, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        self.conf_panel = ConfigPanel(self, -1)
        sizer.Add(self.conf_panel, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        main_sizer.Add(sizer, 1, wxEXPAND | wxALL, 5)

        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        main_sizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        sizer = wxBoxSizer(wxHORIZONTAL)

        buttons = [(ConfigDialog.ID_APPLY, _("Apply")),
                   (ConfigDialog.ID_SAVE, _("Save configuration")),
                   (ConfigDialog.ID_CLOSE, _("Close"))]
        for id, info in buttons:
            button = wxButton(self, id, info)
            sizer.Add(button, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        main_sizer.Add(sizer, 0, wxEXPAND | wxALL, 5)

        self.SetSizer(main_sizer)
        self.SetAutoLayout(True)
        main_sizer.Fit(self)

        EVT_BUTTON(self, ConfigDialog.ID_APPLY, self.OnApply)
        EVT_BUTTON(self, ConfigDialog.ID_SAVE, self.OnSave)
        EVT_BUTTON(self, ConfigDialog.ID_CLOSE, self.OnClose)
        EVT_CLOSE(self, self.OnClose)

    def OnChangeType(self, event):
        type = self.type_ids[event.GetId()]
        self.conf_panel.change_type(type)

    def OnApply(self, event):
        self.__apply()

    def OnSave(self, event):
        self.__apply()
        config.save()

    def OnClose(self, event):
        ConfigDialog.conf_dialog_running = False
        self.Destroy()

    def __apply(self):
        changed_vars = self.conf_panel.get_changed_vars()
        if len(changed_vars) == 0:
            return

        cv = {}
        for s, vars in changed_vars.items():
            l = []
            for v, val in vars.items():
                config.set(s, v, val)
                l.append(v)
            cv[s] = l

        for func in apply_function_list:
            func(cv)

config_sections = {"General": _("General"),
                   "Fetch": _("Fetching"),
                   "Team info": _("Team information"),
                   "Match viewer": _("Match viewer"),
                   "Player evaluation": _("Player evaluation"),
                   "Format": _("Value format"),
                   "Statistics": _("Statistics"),
                   "Comparison page": _("Comparisons"),
                   }
