# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""All basic database handling is done in these functions and classes.

There is automatically one instance of Database declared in the
variable databases.

Example:

>>> import Database
>>> Database.databases.get(None, False, \"My.key\")
"""

from Common import *

import Config

import bsddb
import os.path

class Database:
    """Class responsible for handling with the databases."""

    def __init__(self):
        self.dbs = {}
        self.memdbs = {}

    def get(self, userid, memory_only, key):
        """Get single data in specified database.

        If userid is None, then it is assumed to be from the common database."""

        db = self.__get_db(userid, memory_only)
        if db.has_key(key):
            return db[key]
        return None

    def gets(self, userid, memory_only, keys):
        """Get some data in specified database.

        If userid is None, then it is assumed to be from the common database.

        Keys is a list of keys to get."""

        data = {}
        db = self.__get_db(userid, memory_only)
        for key in keys:
            if db.has_key(key):
                data[key] = db[key]
            else:
                data[key] = None
        return data

    def get_all(self, userid, memory_only, key_start):
        """Get all data in specified database with keys beginning with key_start.

        If userid is None, then it is assumed to be from the common database."""

        db = self.__get_db(userid, memory_only)
        data = {}
        try:
            key, value = db.set_location(key_start)

            while key.startswith(key_start):
                data[key] = value
                key, value = db.next()

        except KeyError, err:
            pass

        return data


    def sets(self, userid, memory_only, data, key_start = ""):
        """Set some data in specified database.

        If userid is None, then it is assumed to be from the common database.

         - data -- should be a dictionary of key, value pairs.
         - key_start -- all keys beginning with key will before storage
                        be removed. If key_start has zero length, nothing will
                        be removed."""

        db = self.__get_db(userid, memory_only)

        if len(key_start) > 0:
            try:
                key, value = db.set_location(key_start)

                while key.startswith(key_start):
                    del db[key]
                    key, value = db.next()

            except KeyError, err:
                pass

        for key, value in data.items():
            if value is None:
                del db[key]
            else:
                db[key] = value
        db.sync()

    def __get_db(self, userid, memory_only):
        """Returns the actual database.

        If userid is None, then it is assumed to be from the common database."""

        if userid == None:
            dbname = "data-common.db"
        else:
            dbname = "data-u%d.db" % userid

        dbs = self.dbs

        if memory_only:
            dbs = self.memdbs
            if not dbs.has_key(dbname):
                dbs[dbname] = bsddb.btopen(None, 'c')
        else:
            if not dbs.has_key(dbname):
                fnpath = Config.get_database_filename(dbname)
                dbs[dbname] = bsddb.btopen(fnpath, 'c')

        return dbs[dbname]

databases = Database()
