# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import Countries

from Common import *
from Commonwx import *

from wxPython.wx import *
from math import *

heading_list = [_("Economist"), _("Revenue"), _("Expenses"), _("Profit"), _("Acc. profit"), _("Weeks")]

class EconomyDialog(wxDialog):
    """Dialog showing revenue for a given number of economists"""

    def __init__(self, parent, money, economists_now, leagueid):
        """Dialog initialization"""
        pos = wxDefaultPosition
        size = wxDefaultSize,
        style = wxDEFAULT_DIALOG_STYLE

        wxDialog.__init__(self, parent, -1, _("Economists"), pos, wxSize(500, 400), style, "EconomyDialog")

        sizer = wxBoxSizer(wxVERTICAL)

        (dummy, dummy, currency,
         currency_is_prefix, exchangerate, dummy) = Countries.country_info[leagueid]

        weekly_cost = (15000/exchangerate)

        # The table with increasing number of economists
        fgs=wxFlexGridSizer(cols=6, vgap=5, hgap=10)
        for text in heading_list:
            fgs.Add(wxStaticText(self,-1,text))

        best = -1
        last = -1
        accprofit=0
        for eco in range(10):
            fgs.Add(wxStaticText(self,-1,str(eco+1)))
            income=int(self.GetEconomyIncome(money, eco+1, exchangerate)-self.GetEconomyIncome(money, eco, exchangerate))
            fgs.Add(wxStaticText(self, -1,
                                 add_currency_to_money(income, currency, currency_is_prefix)),
                    0, wxALIGN_RIGHT)
            fgs.Add(wxStaticText(self,-1,add_currency_to_money(weekly_cost, currency,
                                                               currency_is_prefix)),
                    0, wxALIGN_RIGHT)
            profit = int(income-weekly_cost)
            profittext=wxStaticText(self,-1,add_currency_to_money(profit, currency,
                                                                  currency_is_prefix, True))
            fgs.Add(profittext, 0, wxALIGN_RIGHT)
            SetTextColour(profittext,profit)
            if profit>0:
                accprofit=accprofit+profit
                accprofittext=wxStaticText(self,-1,add_currency_to_money(accprofit, currency,
                                                                  currency_is_prefix, True))
                weeks=str(int(weekly_cost/profit))
            else:
                weeks=""
                accprofittext=wxStaticText(self,-1,"")
                
            SetTextColour(accprofittext,accprofit)
            fgs.Add(accprofittext, 0, wxALIGN_RIGHT)
            fgs.Add(wxStaticText(self,-1,weeks), 0, wxALIGN_RIGHT)
            if profit > weekly_cost:
                best = eco

            # Check if the economists won't invoke any revenue
            if profit <= 0:
                break
            else:
                last = eco

        # Text with the best number of economists   
        text=wxStaticText(self,-1,ngettext("It seems to be best with %d economist for you.",
                                           "It seems to be best with %d economists for you.",
                                           best+1) % (best+1))
        sizer.Add(text, 0,wxALL, 5)
        if last != best:
            text = wxStaticText(self, -1,
                                ngettext("If you are long term saving it might be best with up to %d economist for you.",
                                         "If you are long term saving it might be best with up to %d economists for you.",
                                         (last+1)) % (last+1))
            sizer.Add(text, 0,wxALL, 5)

        # Information about the current number of economists
        text=wxStaticText(self,-1,ngettext("You have now %d economist employed.",
                                           "You have now %d economists employed.",
                                           economists_now) % economists_now)
        sizer.Add(text, 0,wxALL, 5)

        # Add the table
        sizer.Add(fgs, 0, wxALIGN_CENTER | wxALL, 10)

        # Small warning text
        text=wxStaticText(self, -1,_("This function is not verified by HT."))
        SetTextBold(text)
        sizer.Add(text, 0,wxALL, 5)

        # Ok button
        button = wxButton(self, wxID_OK, _("Ok"))
        button.SetDefault()

        sizer.Add(button, 0, wxALIGN_RIGHT | wxALL, 10)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        EVT_BUTTON(self, wxID_OK, self.OnOK)


    def OnOK(self, event):
            self.EndModal(wxID_OK)

    def GetEconomyIncome(self, money, economists, exchangerate):
        """Calculate the revenue of a certain number of economists"""
        if money > 0:
            # Positive amount of money
            ret=1000*floor(money*(0.003*sqrt(economists)/1000))
        else:
            # Negative amount of money
            ret=1000*floor(-money*(0.005*sqrt(economists)/1000))
        if ret > 1000000:
            # Max income of 1000000 of the base currency
            ret=1000000
        return int(ret/exchangerate)
