# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, os.path
import time

import Countries
import Terms
import SelectDateMenu
import EconomyDialog
import CommonData
import XMLParse
import ArenaPage

from Common import *
from Commonwx import *
from wxPython.wx import *

class EconomyPage(wxPanel):
    ID_POPUP_MENU = 211
    ID_DATE_SELECTED = 212
    ID_EKONOM_BUTTON = 213
    ID_ARENA_BUTTON = 214

    Columns = [(_("This week"), "this"),
               (_("Last week"), "last"),
               (_("Difference"), "diff"),
               (_("Two weeks"), "two")]
    Rows = [(_("Revenue"), FACE_BOLD | FACE_BIG, "#C8F8C8", None, None),
            (_("Gate receipts"), FACE_NONE, "#F0F0F0", "Publik", "IncomeSpectators"),
            (_("Sponsors"), FACE_NONE, "#E8E8E8", "Sponsorer", "IncomeSponsors"),
            (_("Financial"), FACE_NONE, "#F0F0F0", "Finansiella", "IncomeFinancial"),
            (_("Temporary"), FACE_NONE, "#E8E8E8", "Tillfalliga", "IncomeTemporary"),
            (_("Total"), FACE_BOLD, "#F0F0F0", "Summa", "IncomeSum"),
            None,
            (_("Expenses"), FACE_BOLD | FACE_BIG, "#F8C8C8", None, None),
            (_("Arena"), FACE_NONE, "#F0F0F0", "cArena", "CostsArena"),
            (_("Player salaries"), FACE_NONE, "#E8E8E8", "cSpelare", "CostsPlayers"),
            (_("Interests"), FACE_NONE, "#F0F0F0", "cRantor", "CostsFinancial"),
            (_("Temporary"), FACE_NONE, "#E8E8E8", "cTillfalliga", "CostsTemporary"),
            (_("Staff"), FACE_NONE, "#F0F0F0", "cPersonal", "CostsStaff"),
            (_("Youth squad"), FACE_NONE, "#E8E8E8", "CJuniorverksamhet", "CostsYouth"),
            (_("Total"), FACE_BOLD, "#F0F0F0", "cSumma", "CostsSum"),
            None,
            (_("Profit"), FACE_BOLD | FACE_BIG, "#C8C8F8", "Total", "ExpectedWeeksTotal")]

    def __init__(self, parent, ID):

        wxPanel.__init__(self, parent, ID)

        sizer = wxBoxSizer(wxVERTICAL)
        
        self.fgs = wxFlexGridSizer(cols=5, vgap=0, hgap=0)
        self.fgs.AddGrowableCol(1)
        self.fgs.AddGrowableCol(2)
        self.fgs.AddGrowableCol(3)
        self.fgs.AddGrowableCol(4)

        self.fgs.Add(wxStaticText(self, -1, ""))
        for col_name, col_attr in EconomyPage.Columns:
            stext = wxStaticText(self, -1, col_name, style = wxALIGN_RIGHT)
            SetTextBold(stext)
            self.fgs.Add(stext, 1, wxEXPAND | wxADJUST_MINSIZE | wxALL, 5)

        self.economy_texts = {}
        i = 0
        col_count = 0
        for row in EconomyPage.Rows:
            if row != None:
                row_name, row_face, row_colour, row_attr, row_data = row
            else:
                row_colour = None
            gpanel = SimpleColouredPanel(self, -1, row_colour)
            if row == None:
                stext = wxStaticText(gpanel, -1, "")
            else:
                stext = SetTextFace(wxStaticText(gpanel, -1, row_name,
                                                 style = wxALIGN_LEFT), row_face)
            gpanel.AddChild(stext, 1, wxEXPAND | wxADJUST_MINSIZE | wxALL, 5)
            self.fgs.Add(gpanel, 0, wxEXPAND | wxADJUST_MINSIZE | wxALL, 0)

            for col_name, col_attr in EconomyPage.Columns:
                gpanel = SimpleColouredPanel(self, -1, row_colour)
                stext = SetTextFace(wxStaticText(gpanel, -1, "",
                                                 style = wxALIGN_RIGHT | wxST_NO_AUTORESIZE),
                                    (row_face | FACE_BIG)^FACE_BIG)
                gpanel.AddChild(stext, 1, wxADJUST_MINSIZE | wxALIGN_RIGHT | wxALL, 5)
                self.fgs.Add(gpanel, 0, wxEXPAND | wxADJUST_MINSIZE | wxALL, 0)
                if row != None and row_attr != None and col_attr != None:
                    self.economy_texts[col_attr + row_attr] = stext
            self.fgs.AddGrowableRow(i)
            i += 1

        hsizer2 = wxBoxSizer(wxHORIZONTAL)
        hsizer2.Add(self.fgs, 1, wxEXPAND |wxALL, 5)

        self.fgs2 = wxFlexGridSizer(cols=2, vgap=5, hgap=10)
        self.fgs2.AddGrowableCol(1)

        kapital=wxStaticText(self, -1, _("Capital"))
        SetTextBold(kapital)
        self.fgs2.Add(kapital)
        self.kapital=wxStaticText(self, -1, "")
        self.fgs2.Add(self.kapital, 1, wxADJUST_MINSIZE | wxALIGN_RIGHT | wxALIGN_BOTTOM | wxALIGN_TOP)
        resultat=wxStaticText(self, -1, _("Profit"))
        SetTextBold(resultat)
        self.fgs2.Add(resultat)
        self.resultat=wxStaticText(self, -1, "")
        self.fgs2.Add(self.resultat, 1, wxADJUST_MINSIZE | wxALIGN_RIGHT | wxALIGN_BOTTOM | wxALIGN_TOP)
        tillgang=wxStaticText(self, -1, _("Assets"))
        SetTextBold(tillgang)
        self.fgs2.Add(tillgang)
        self.tillgang=wxStaticText(self, -1, "")
        self.fgs2.Add(self.tillgang, 1, wxADJUST_MINSIZE | wxALIGN_RIGHT | wxALIGN_BOTTOM | wxALIGN_TOP)

        vsizer2 = wxBoxSizer(wxVERTICAL)
        vsizer2.Add(self.fgs2, 1, wxEXPAND |wxALL, 20)

        self.economybutton = wxButton(self, EconomyPage.ID_EKONOM_BUTTON, \
                                      _("Economists"), style = wxBU_EXACTFIT)

        vsizer2.Add(self.economybutton, 0, wxEXPAND |wxALL | wxALIGN_CENTER, 20)

        self.arenabutton = wxButton(self, EconomyPage.ID_ARENA_BUTTON, \
                                       _("Arena sizer"), style = wxBU_EXACTFIT)

        vsizer2.Add(self.arenabutton, 0, wxEXPAND |wxALL | wxALIGN_CENTER, 20)
        hsizer2.Add(vsizer2, 0, wxEXPAND |wxALL, 20)
        sizer.Add(hsizer2, 0, wxEXPAND |wxALL, 5)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        EVT_BUTTON(self, EconomyPage.ID_EKONOM_BUTTON, self.EconomyButton)
        EVT_BUTTON(self, EconomyPage.ID_ARENA_BUTTON, self.ArenaButton)

    def AfterInit(self):
        teamid = CommonData.get_current_teamid()

        if teamid is not None:
            self.league_id = XMLParse.get(["team details", teamid],
                                          ["Team.League.LeagueID"])

        self.UpdateUI()

    def ChangeXML(self):
        self.UpdateUI()

    def EconomyButton(self, event):
        teamid = CommonData.get_current_teamid()

        if teamid is None:
            return

        if not hasattr(self, "league_id"):
            self.league_id = XMLParse.get(["team details", teamid],
                                          ["Team.League.LeagueID"])

        cdata = XMLParse.get(["club"], nofetch = True, get_latest_available = True)
        tillgangar= self.edata.get("Team.ExpectedCash")
        ekonom = cdata.get("Team.Specialists.Economists",0)
        dlg = EconomyDialog.EconomyDialog(self, tillgangar, ekonom, self.league_id)
        ret = dlg.ShowModal()

    def ArenaButton(self, event):
        dlg = ArenaPage.ArenaDialog(self)
        ret = dlg.ShowModal()

    def UpdateUI(self):
        teamid = CommonData.get_current_teamid()
        if teamid is None:
            return

        if not hasattr(self, "league_id"):
            self.league_id = XMLParse.get(["team details", teamid],
                                          ["Team.League.LeagueID"])
   

        (dummy, dummy, currency,
         currency_is_prefix, exchangerate, dummy) = Countries.country_info[self.league_id]
        self.edata = XMLParse.get(["economy"], nofetch = True, get_latest_available = True)

        if not len(self.edata):
            return

        for row in EconomyPage.Rows:
            if row == None:
                continue

            row_name, row_face, row_colour, row_attr, row_data = row
            if row_attr == None:
                continue

            if row_data == "ExpectedWeeksTotal":
                lastrow_data = "LastWeeksTotal"
            else:
                lastrow_data = "Last" + row_data

            for col_name, col_attr in EconomyPage.Columns:
                if col_name == None or col_attr == None:
                    continue
                value = None

                text = self.economy_texts[col_attr + row_attr]
                thisvalue = int(self.edata.get("Team.%s" % row_data)/exchangerate)
                lastvalue = int(self.edata.get("Team.%s" % lastrow_data)/exchangerate)
                diffvalue = thisvalue - lastvalue
                twovalue = thisvalue + lastvalue
                if col_attr == "this":
                    value = thisvalue
                elif col_attr == "last":
                    value = lastvalue
                elif col_attr == "diff":
                    value = diffvalue
                elif col_attr == "two":
                    value = twovalue
                if col_attr == "diff" or row_attr == "Total":
                    strvalue = add_currency_to_money(value, currency, currency_is_prefix, True)
                    if row_attr[0] == "c":
                        colval = -value
                    else:
                        colval = value
                    SetTextColour(text, colval)
                else:
                    strvalue = add_currency_to_money(value, currency, currency_is_prefix)
                text.SetLabel(strvalue)


        kapvalue=int(self.edata.get("Team.Cash")/exchangerate)
        self.kapital.SetLabel(add_currency_to_money(kapvalue, currency, currency_is_prefix))
        self.resultat.SetLabel(add_currency_to_money(thisvalue, currency,
                                                     currency_is_prefix, True))
        SetTextColour(self.resultat,thisvalue)
        tillvalue=kapvalue+thisvalue
        self.tillgang.SetLabel(add_currency_to_money(tillvalue, currency, currency_is_prefix))

        self.Layout()
