# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from Common import *
from Commonwx import *

import cStringIO
import sys
import time
import traceback

import Version

if Version.version_type == "release":
    _debug = False
else:
    _debug = True

class ErrorDialog(wxDialog):
    ID_DISMISS = 473
    ID_EXIT = 474

    def __init__(self, parent, tback, use_exit_button = True):
        wxDialog.__init__(self, parent, -1, "Error",
                          style = wxRESIZE_BORDER | wxCAPTION | wxSYSTEM_MENU)

        main_sizer = wxBoxSizer(wxVERTICAL)

        stext = wxStaticText(self, -1,
                             _("""An error has been encountered in Mattricks. This shouldn't happen.
Please report this bug to mattricks@lists.lysator.liu.se or in the
Mattricks conference at Hattrick and include the traceback below in
your report. Also include the version of Mattricks and what OS you
ran it on and finally what you did to achieve this bug."""))
        main_sizer.Add(stext, 0, wxEXPAND | wxALL, 5)

        traceback_ctrl = wxTextCtrl(self, -1,
                                    size = (600, 200),
                                    style = wxTE_MULTILINE | wxTE_READONLY | wxHSCROLL)
        traceback_ctrl.SetValue(tback)

        main_sizer.Add(traceback_ctrl, 1, wxEXPAND | wxALL, 5)

        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        main_sizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        bsizer = wxBoxSizer(wxHORIZONTAL)

        button = wxButton(self, ErrorDialog.ID_DISMISS, _("Close"))
        button.SetDefault()
        bsizer.Add(button, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        if use_exit_button:
            button = wxButton(self, ErrorDialog.ID_EXIT, _("Exit program"))
            bsizer.Add(button, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        main_sizer.Add(bsizer, 0, wxALIGN_RIGHT | wxALL, 5)

        self.SetSizer(main_sizer)
        self.SetAutoLayout(True)
        main_sizer.Fit(self)

        EVT_BUTTON(self, ErrorDialog.ID_DISMISS, self.OnDismiss)
        EVT_BUTTON(self, ErrorDialog.ID_EXIT, self.OnExit)

    def OnDismiss(self, event):
        self.Destroy()

    def OnExit(self, event):
        self.Destroy()
        sys.exit(0)

class Errorwx:
    def __init__(self, parent_frame, old_sys_err):
        self.old_sys_err = old_sys_err
        self.parent_frame = parent_frame
        self.this_exception = None

    def write(self, s):
        if hasattr(sys, "last_traceback"):
            if self.this_exception is not sys.last_traceback:
                self.this_exception = sys.last_traceback

                strfile = cStringIO.StringIO()
                traceback.print_last(None, strfile)

                tback = strfile.getvalue()

                if _debug:
                    self.old_sys_err.write(tback)

                dlg = ErrorDialog(self.parent_frame, tback)
                dlg.Show()

class Error:
    def __init__(self):
        self.old_sys_err = sys.stderr

    def init_errorwx(self, parent_frame):
        sys.stderr = Errorwx(parent_frame, self.old_sys_err)

error = Error()
