# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from Common import *

import urllib
import os.path
import re
import string
import sys
import time
from xml.sax import SAXException

import Config
import Version
import HRFData
import XMLParse
import XMLDataInfo
import DialogSet

FETCH_ERROR_STRANGE_RESPONSE = 1
FETCH_ERROR_LOGIN_ERROR = 2
FETCH_ERROR_PLAYING_MATCH = 3
FETCH_ERROR_ERRONEOUS_DATA = 4
FETCH_ERROR_COULDNT_GET_COOKIE = 5
FETCH_ERROR_COULDNT_CONNECT = 6
FETCH_ERROR_UNABLE_TO_PARSE_DATA = 7

class FetchError(Exception):
    def __init__(self, s, type = -1, extra_info = ""):
        self.type = type
        self.extra_info = extra_info
        self.s = s

    def __str__(self):
        return str(self.s)

class MattricksURLopener(urllib.FancyURLopener):
    def __init__(self, *args):
        self.version = "Mattricks/" + Version.version
        urllib.FancyURLopener.__init__(self, *args)

urllib._urlopener = MattricksURLopener()

def make_hrf_filename(path, always_download, overwrite_old):
    def make_real_path(path, amount_numbers):
        filename = string.join(["%04d", "-%02d", "-%02d", \
                                "_%02d", "-%02d", "-%02d"][:amount_numbers], "") % \
                                time.localtime(time.time())[:amount_numbers] + ".hrf"
        return os.path.realpath(os.path.join(os.path.expanduser(path), filename))

    filename = make_real_path(path, 3)
    if os.path.exists(filename):
        if not always_download:
            return
        elif overwrite_old:
            return filename
        else:
            filename = make_real_path(path, 6)
    return filename

class Connection:
    re_timeout = re.compile("<title>TimeOut</title>", re.IGNORECASE | re.MULTILINE)
    re_msoledb_error = re.compile("Microsoft OLE DB Provider for SQL Server", re.IGNORECASE | re.MULTILINE)
    re_playing_match = re.compile("playingMatch=true", re.IGNORECASE | re.MULTILINE)
    re_script_timeout = re.compile("Active Server Pages.*Script timed out", re.IGNORECASE | re.MULTILINE | re.DOTALL)

    def __init__(self):
        self.current_user = None
        self.recommended_url = None
        self.cookie = None

    def get_recommended_url(self):
        o = MattricksURLopener()
        try:
            u = o.open("http://www.hattrick.org/Common/menu.asp",
                       urllib.urlencode([("outputType", "XML")]))
            content = u.read()
        except IOError, e:
            raise FetchError(_("Unable to connect to the server."),
                             FETCH_ERROR_COULDNT_CONNECT)

        m = self.re_msoledb_error.search(content)
        if m is not None:
            raise FetchError(_("Hattrick database error."),
                             FETCH_ERROR_ERRONEOUS_DATA)

        m = self.re_script_timeout.search(content)
        if m is not None:
            raise FetchError(_("Hattrick server error."),
                             FETCH_ERROR_ERRONEOUS_DATA)

        if not content[:5] == "<?xml":
            clen = len(content)
            if clen < 150:
                err_txt = "%s\n\n%s" % (_("Got this message from the Hattrick server:"),
                                        content)
            else:
                err_txt = "%s\n\n%s\n(%s)" % (_("Got this message from the Hattrick server:"),
                                              content[:150],
                                              _("message truncated"))
            raise FetchError(err_txt, FETCH_ERROR_ERRONEOUS_DATA)

        try:
            p = XMLParse.parse_xml_string(content)
        except SAXException, err:
            sys.stderr.write(str(err))
            print content
            xml_err = True

        self.recommended_url = p.get("RecommendedURL")
        if self.recommended_url is None:
            raise FetchError(_("Couldn't get recommended url"),
                             FETCH_ERROR_ERRONEOUS_DATA)

    def login(self, loginname, password):
        if self.cookie == None or \
           self.current_user != loginname:
            if self.recommended_url is None:
                self.get_recommended_url()

            try:
                u = urllib.urlopen(self.recommended_url + "/Common/default.asp",
                                   urllib.urlencode([("loginname", loginname),
                                                     ("password", password),
                                                     ("loginType","CHPP"),
                                                     ("actionType", "login"),
                                                     ("submit", "Log on")]))
            except IOError, e:
                raise FetchError(_("Unable to connect to the server."),
                                 FETCH_ERROR_COULDNT_CONNECT)

            try:
                cookie = u.info()['set-cookie']
                cookie = cookie[:string.find(cookie, ";")]
            except:
                raise FetchError(_("Unable to get cookie."),
                                 FETCH_ERROR_COULDNT_GET_COOKIE)

            regexp = re.compile("loginError=(Yes|)&", re.IGNORECASE)
            pwdata = u.read()
            result = regexp.search(pwdata)
            if not result:
                raise FetchError(_("Strange response in fetch."),
                                 FETCH_ERROR_STRANGE_RESPONSE,
                                 pwdata)
            if len(result.group(1)) > 0:
                raise FetchError(_("Failed to login, please try again!"),
                                 FETCH_ERROR_LOGIN_ERROR)

            self.current_user = loginname
            self.cookie = cookie

    def fetch_hrf(self, loginname, password, path = "",
                  dlg = None, filename = None,
                  always_download = True, overwrite_old = False):

        if filename == None:
            filename = make_hrf_filename(path, always_download, overwrite_old)

        if filename == None:
            return None

        self.login(loginname, password)

        if dlg != None:
            dlg.Update(2)

        o = MattricksURLopener()

        if dlg != None:
            dlg.Update(5, _("Downloading hrf-file ..."))

        o.addheader("Cookie", self.cookie)
        try:
            u = o.open(self.recommended_url +"/Common/hrf.asp")
            content = u.read()
        except IOError, e:
            raise FetchError(_("Unable to connect to the server."),
                             FETCH_ERROR_COULDNT_CONNECT)

        m = self.re_timeout.search(content)
        if m is not None:
            self.current_user = None
            self.recommended_url = None
            self.cookie = None
            return self.fetch_hrf(loginname, password, path, dlg, filename,
                                  always_download, overwrite_old)

        m = self.re_playing_match.search(content)
        if m is not None:
            raise FetchError(_("Team is currently playing match."),
                             FETCH_ERROR_PLAYING_MATCH)

        m = self.re_msoledb_error.search(content)
        if m is not None:
            raise FetchError(_("Hattrick database error."),
                             FETCH_ERROR_ERRONEOUS_DATA)

        m = self.re_script_timeout.search(content)
        if m is not None:
            raise FetchError(_("Hattrick server error."),
                             FETCH_ERROR_ERRONEOUS_DATA)

        dm = HRFData.HTDir.re_date.search(content)
        if dm is not None:
            open(filename, 'w').write(content)
        else:
            raise FetchError(_("The retrieved data is erroneous."),
                             FETCH_ERROR_ERRONEOUS_DATA)

        if dlg != None:
            dlg.Update(9, _("Updating tables ..."))

        return filename

    def fetch_xml(self, loginname, password, xmlname, args,
                  filename = None, dlg = None):
        self.__update_dlg(dlg, 1, _("Logging in ..."))

        self.login(loginname, password)

        self.__update_dlg(dlg, 2, _("Logged in ..."))

        o = MattricksURLopener()

        self.__update_dlg(dlg, 5, _("Downloading xml file ..."))

        o.addheader("Cookie", self.cookie)
        post = [("outputType", "XML")] + args

        try:
            u = o.open(self.recommended_url +"/Common/" + xmlname,
                       urllib.urlencode(post))
            content = u.read()
        except IOError, e:
            raise FetchError(_("Unable to connect to the server."),
                             FETCH_ERROR_COULDNT_CONNECT)

        self.__update_dlg(dlg, 6, _("Checking correctness ..."))

        m = self.re_timeout.search(content)
        if m is not None:
            self.current_user = None
            self.recommended_url = None
            self.cookie = None
            return self.fetch_xml(loginname, password, xmlname, args,
                                  filename, dlg)

        m = self.re_msoledb_error.search(content)
        if m is not None:
            raise FetchError(_("Hattrick database error."),
                             FETCH_ERROR_ERRONEOUS_DATA)

        m = self.re_script_timeout.search(content)
        if m is not None:
            raise FetchError(_("Hattrick server error."),
                             FETCH_ERROR_ERRONEOUS_DATA)

        self.__update_dlg(dlg, 7, _("Saving xml file ..."))

        if not content[:5] == "<?xml":
            clen = len(content)
            if clen < 150:
                err_txt = "%s\n\n%s" % (_("Got this message from the Hattrick server:"),
                                        content)
            else:
                err_txt = "%s\n\n%s\n(%s)" % (_("Got this message from the Hattrick server:"),
                                              content[:150],
                                              _("message truncated"))
            raise FetchError(err_txt, FETCH_ERROR_ERRONEOUS_DATA)

        xml_err = False
        try:
            p = XMLParse.parse_xml_string(content)

            identifier = XMLParse.save_data(p)
            if filename == None:
                xml_info = XMLDataInfo.xml_known_xmls[XMLDataInfo.xml_name_table[identifier[0]]]
                filename = xml_info[5](identifier)
                filename = filename.replace(" ", "_") + ".xml"
        except SAXException, err:
            sys.stderr.write(str(err))
            print content
            xml_err = True

        if filename == None:
            filename = "temporary_data.xml"

        if Config.config.get("Fetch", "default save directory"):
            Config.save_cfg_file(filename, content, "fetchedXMLs")
        else:
            path = Config.config.get("Fetch", "save directory")

            filename = os.path.realpath(os.path.join(os.path.expanduser(path), filename))
            open(filename, 'w').write(content)

        if xml_err:
            raise FetchError(_("Unable to parse retrieved data."),
                             FETCH_ERROR_UNABLE_TO_PARSE_DATA)

        self.__update_dlg(dlg, 10, _("Download finished ..."))

        return identifier

    def __update_dlg(self, dlg, num, text):
        if dlg != None:
            dlg.Update(num, text)

def xml_import(xmlfile):
    """Imports an XML file into the database."""

    try:
        p = XMLParse.parse_xml_file(xmlfile)

        identifier = XMLParse.save_data(p)
    except SAXException, err:
        sys.stderr.write(str(err))
        raise FetchError(_("Unable to parse xml file %s.") % xmlfile,
                         FETCH_ERROR_UNABLE_TO_PARSE_DATA)

conn = Connection()
