# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from Common import *

from Database import databases

import os.path
from wxPython.wx import *

import Config

class FetchDialog(wxDialog):

    DIR_DIALOG = 101

    def __init__(self, parent, title, pos = wxDefaultPosition,
                 size = wxDefaultSize,
                 style = wxDEFAULT_DIALOG_STYLE,
                 defaultDir = None, defaultUser = None, defaultPass = None,
                 defaultJSpy = False, defaultADownload = True, defaultOverride = False):

        wxDialog.__init__(self, parent, -1, title, pos, size, style, "FetchDialog")

        sizer = wxBoxSizer(wxVERTICAL)

        # Set directory.
        hsizer = wxBoxSizer(wxHORIZONTAL)

        label = wxStaticText(self, -1, _("Save in catalog") + ":")
        hsizer.Add(label, 0, wxALIGN_CENTRE | wxALL, 5)

        self.diredit = wxTextCtrl(self, -1, size = (200,-1))
        if defaultDir != None:
            self.diredit.SetValue(defaultDir)
        hsizer.Add(self.diredit, 1, wxALIGN_CENTRE | wxALL, 5)

        button = wxButton(self, FetchDialog.DIR_DIALOG, _("Browse"), style = wxBU_EXACTFIT)
        hsizer.Add(button, 0, wxALIGN_CENTRE | wxALL, 5)

        sizer.Add(hsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        # Username / Password.

        hsizer = wxBoxSizer(wxHORIZONTAL)
        label = wxStaticText(self, -1, _("Username") + ":")
        hsizer.Add(label, 0, wxALIGN_CENTRE | wxALL, 5)

        self.useredit = wxTextCtrl(self, -1)
        self.useredit.SetFocus()
        if defaultUser != None:
            self.useredit.SetValue(defaultUser)
        else:
            self.useredit.SetValue(Config.config.get("Fetch","user name"))
            
        hsizer.Add(self.useredit, 1,  wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        label = wxStaticText(self, -1, _("Password") + ":")
        hsizer.Add(label, 0, wxALIGN_CENTRE | wxALL, 5)

        self.passedit = wxTextCtrl(self, -1, style = wxTE_PASSWORD)
        if defaultPass != None:
            self.passedit.SetValue(defaultPass)
        else:
            self.useredit.SetValue(Config.config.get("Fetch","password"))
            
        hsizer.Add(self.passedit, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        sizer.Add(hsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        # Line

        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        sizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        # Questions

        self.always_download = wxCheckBox(self, -1, _("Always download a new hrf file."))
        self.always_download.SetValue(defaultADownload)
        sizer.Add(self.always_download, 0, wxEXPAND | wxALIGN_LEFT | wxALL, 5)

        self.overwrite_old = wxCheckBox(self, -1, _("Overwrite day old hrf file if one exists."))
        self.overwrite_old.SetValue(defaultOverride)
        sizer.Add(self.overwrite_old, 0, wxEXPAND | wxALIGN_LEFT | wxALL, 5)

        # Line

        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        sizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        # Ok / Cancel buttons

        hsizer = wxBoxSizer(wxHORIZONTAL)

        self.jsbox = wxCheckBox(self, -1, _("Start Junior Spy"))
        self.jsbox.SetValue(defaultJSpy)
        hsizer.Add(self.jsbox, 1, wxEXPAND | wxALIGN_LEFT | wxALL, 5)

        button = wxButton(self, wxID_OK, _("Ok"))
        button.SetDefault()
        hsizer.Add(button, 0, wxALIGN_CENTRE | wxALL, 5)

        button = wxButton(self, wxID_CANCEL, _("Cancel"))
        hsizer.Add(button, 0, wxALIGN_CENTRE | wxALL, 5)

        sizer.Add(hsizer, 0, wxEXPAND | wxALIGN_RIGHT | wxALL, 5)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        EVT_BUTTON(self, FetchDialog.DIR_DIALOG, self.OnDirSelect)
        EVT_BUTTON(self, wxID_OK, self.OnOK)

    def GetResult(self):
        dir = self.diredit.GetValue()
        loginname = self.useredit.GetValue()
        password = self.passedit.GetValue()
        juniorspy = self.jsbox.GetValue()
        always_download = self.always_download.GetValue()
        overwrite_old = self.overwrite_old.GetValue()

        return (dir, loginname, password, juniorspy, always_download, overwrite_old)

    def OnOK(self, event):
        loginname = self.useredit.GetValue()
        password = self.passedit.GetValue()
        if len(loginname) == 0 or len(password) == 0:
            mdlg = wxMessageDialog(self, _("You have to supply both username and password"),
                                  _("Error"), wxOK | wxICON_EXCLAMATION)
            mdlg.ShowModal()
            mdlg.Destroy()
        else:
            self.EndModal(wxID_OK)

    def OnDirSelect(self, event):
        defdir = os.path.expanduser(self.diredit.GetValue())
        if len(defdir) == 0:
            defdir = "."
        dlg = wxDirDialog(self, _("Choose save catalog for the hrf file."),
                          defaultPath = defdir,
                          style = wxDD_NEW_DIR_BUTTON |
                          wxRESIZE_BORDER | wxCAPTION | wxSYSTEM_MENU)
        res = dlg.ShowModal()
        if res == wxID_OK:
            self.diredit.SetValue(dlg.GetPath())
        dlg.Destroy()

class SimpleFetchDialog(wxDialog):
    def __init__(self, parent):
        wxDialog.__init__(self, parent, -1, _("Enter your Hattrick username and password."))

        sizer = wxBoxSizer(wxVERTICAL)

        # Username / Password.

        hsizer = wxBoxSizer(wxHORIZONTAL)
        label = wxStaticText(self, -1, _("Username") + ":")
        hsizer.Add(label, 0, wxALIGN_CENTRE | wxALL, 5)

        username = Config.config.get("Fetch","user name")
        self.useredit = wxTextCtrl(self, -1)
        self.useredit.SetValue(username)
        hsizer.Add(self.useredit, 1,  wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        label = wxStaticText(self, -1, _("Password") + ":")
        hsizer.Add(label, 0, wxALIGN_CENTRE | wxALL, 5)

        self.passedit = wxTextCtrl(self, -1, style = wxTE_PASSWORD)
        self.passedit.SetValue(Config.config.get("Fetch","password"))
                    
        hsizer.Add(self.passedit, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        sizer.Add(hsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        if username == "":
            self.useredit.SetFocus()
        else:
            self.passedit.SetFocus()
            
        # Line

        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        sizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        # Ok / Cancel buttons

        hsizer = wxBoxSizer(wxHORIZONTAL)

        button = wxButton(self, wxID_OK, _("Ok"))
        button.SetDefault()
        hsizer.Add(button, 0, wxALIGN_RIGHT | wxALL, 5)

        button = wxButton(self, wxID_CANCEL, _("Cancel"))
        hsizer.Add(button, 0, wxALIGN_RIGHT | wxALL, 5)

        sizer.Add(hsizer, 0, wxALIGN_RIGHT | wxALL, 5)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        EVT_BUTTON(self, wxID_OK, self.OnOK)

    def GetResult(self):
        loginname = self.useredit.GetValue()
        password = self.passedit.GetValue()

        return (loginname, password)

    def OnOK(self, event):
        loginname = self.useredit.GetValue()
        password = self.passedit.GetValue()
        if len(loginname) == 0 or len(password) == 0:
            mdlg = wxMessageDialog(self, _("You have to supply both username and password"),
                                  _("Error"), wxOK | wxICON_EXCLAMATION)
            mdlg.ShowModal()
            mdlg.Destroy()
        else:
            self.EndModal(wxID_OK)

def graphical_password_dialog(parent):
    """Graphical dialog for getting loginname/passwords."""
    dlg = SimpleFetchDialog(parent)
    res = dlg.ShowModal()
    if res == wxID_OK:
        (loginname, password) = dlg.GetResult()
        databases.sets(None, True, {"Mattricks.loginname": loginname,
                                    "Mattricks.password": password})
        return (loginname, password)
    return None

def graphical_error_msg_dialog(parent, err_message):
    """Graphical dialog for displaying error messages."""

    mdlg = wxMessageDialog(parent, err_message, _("Error"), wxOK | wxICON_ERROR)
    mdlg.ShowModal()
    mdlg.Destroy()
