# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import ConfigParser
import time
import string
import os
import os.path
import re
import cStringIO

from Common import *
import DataInfo
import I18n
import TeamSelector
import Terms

player_sort_attribute = "name"

# Takes two Data objects and compares them, the result being another Data object.
# Each attribute of the data objects will be in the result.
# If only one of the objects has an attribute, the resulting value will be None.
# If the values of both the objects for an attribute is int or float, the difference
# will be calculated. If not, the resulting value will be "differ" or "same" depending
# on whether the values of the objects is differ or not.
#
# If filter_same is True, then values that become 0 or 'same' is filtered out.
def compare(data1, data2, filter_same = False):
    res = {}
    for key in data1.__dict__.keys() + data2.__dict__.keys():
        if not res.has_key(key):
            value1 = getattr(data1, key, None)
            value2 = getattr(data2, key, None)
            v = None
            try:
                v = value2 - value1
            except:
                if value1 == value2:
                    v = "same"
                else:
                    v = "differ"
            res[key] = v
    if filter_same:
        for key, value in res.items():
            if value == "same" or value == 0:
                del res[key]
    return Data(res)

class Data:
    def __init__(self, attributes):
        self.__dict__ = attributes

    def __ne__(self, y):
        global player_sort_attribute
        return self.__getattribute__(player_sort_attribute) != \
               y.__getattribute__(player_sort_attribute)

    def __eq__(self, y):
        global player_sort_attribute
        return self.__getattribute__(player_sort_attribute) == \
               y.__getattribute__(player_sort_attribute)

    def __le__(self, y):
        global player_sort_attribute
        return self.__getattribute__(player_sort_attribute) <= \
               y.__getattribute__(player_sort_attribute)

    def __lt__(self, y):
        global player_sort_attribute
        return self.__getattribute__(player_sort_attribute) < \
               y.__getattribute__(player_sort_attribute)

    def __ge__(self, y):
        global player_sort_attribute
        return self.__getattribute__(player_sort_attribute) >= \
               y.__getattribute__(player_sort_attribute)

    def __gt__(self, y):
        global player_sort_attribute
        return self.__getattribute__(player_sort_attribute) > \
               y.__getattribute__(player_sort_attribute)

    def __len__(self):
        return len(self.__dict__)

    def __str__(self):
        return str(self.__dict__)

    def __repr__(self):
        return repr(self.__dict__)

    def __getattribute__(self, name):
        return self.__dict__[name]

    def __setattr__(self, name, value):
        self.__dict__[name] = value

class HTStore:
    def __init__(self):
        # In the form filename: (hrfdata, timestamp)
        self.hrfdata = {}

    def __getitem__(self, f):
        t = os.path.getmtime(f)
        if self.hrfdata.has_key(f):
            h, ot = self.hrfdata[f]
            if t == ot:
                return h
        self.hrfdata[f] = (HRFData(f), t)
        return self.hrfdata[f][0]

class HTDir:
    re_date = re.compile("[basics][^[]*^\s*date\s*=\s*([ :0-9-]+)\s*$", re.MULTILINE)

    def __init__(self, *dirs):
        self.dirs = []
        self.hrflist = []
        self.add_dirs(*dirs)

    def add_dirs(self, *dirs):
        hrflist = []
        for cdir in dirs:
            cdir = os.path.realpath(os.path.expanduser(cdir))
            filelist = os.listdir(cdir)
            filelist = filter(lambda x: x[-4:] == ".hrf", os.listdir(cdir))
            filelist = [os.path.join(cdir, x) for x in filelist]
            for f in filelist:
                if os.path.isfile(f):
                    c = open(f, 'r').read()
                    m = HTDir.re_date.search(c)
                    if m:
                        t = string2time(m.group(1))
                        fi = None
                        for i in range(len(self.hrflist)):
                            if self.hrflist[i][1] == f:
                                fi = i
                        if fi == None:
                            hrflist.append((t, f))
                        else:
                            self.hrflist[fi] == (t,f)
            if not cdir in self.dirs:
                self.dirs.append(cdir)

        self.hrflist += hrflist
        self.hrflist.sort()
        self.hrflist.reverse()

    def add_from_filename(self, filename):
        filename = os.path.realpath(os.path.expanduser(filename))
        added_dir = False
        while 1:
            for i in range(len(self.hrflist)):
                t, f = self.hrflist[i]
                if f == filename:
                    return True
            # If we get here, the filename in question isn't in our list.
            if not added_dir:
                self.add_dirs(os.path.dirname(filename))
                added_dir = True
            else:
                raise IOError("Couldn't find file: '%s'" % filename)

    # Always gives the hrffile nearest _earlier_ in time if that exists.
    # Otherwise, gives the oldest existing file.
    # If the list is empty, return None.
    def get_from_time(self, ctime):
        f = None
        for t, f in self.hrflist:
            if t < ctime:
                return f
        return f

    def get_nearest(self, hrffile, eariler = True):
        nearest = None
        for t, f in self.hrflist:
            if earlier:
                nearest = f
            if f == hrffile:
                if eariler:
                    return nearest

# The storage
hrfstorage = HTStore()
# The files
htdirs = HTDir()

class HRFData:
    def __init__(self, hrf_file):
        self.hrf_file = hrf_file
        self.config = ConfigParser.ConfigParser()
        self.config.optionxform = str

        f = open(hrf_file, 'r').read()
        try:
            f = f.decode("utf8")
        except UnicodeError, e:
            try:
                f = f.decode(I18n.def_charset)
            except:
                print "Strange file"

        try:
            f = f.encode(I18n.def_charset)
        except UnicodeError, e:
            f = f.encode("utf8")
        file_obj = cStringIO.StringIO(f)
        self.config.readfp(file_obj)

        self.__extract_data()

    def get_section(self, section, display = False):
        if display:
            return self.__sections_display[section]
        return self.__sections_convert[section]

    def get_player(self, pid, display = False):
        if display:
            return self.__players_display[pid]
        return self.__players_convert[pid]

    def get_section_names(self):
        return self.__sections_convert.keys()

    def get_pids(self):
        return self.__players_convert.keys()

    def get_all_sections(self, display = False):
        if display:
            return self.__sections_display
        return self.__sections_convert

    def get_all_players(self, display = False):
        if display:
            return self.__players_display
        return self.__players_convert

    def has_section(self, section):
        return self.__sections_convert.has_key(section)

    def has_player(self, pid):
        return self.__players_convert.has_key(pid)

    # Private functions
    def __extract_data(self):
        DataInfo.current_htdata.htdata = self

        self.__sections_convert = {}
        self.__players_convert = {}

        self.__sections_display = {}
        self.__players_display = {}

        # pass 1
        for section, attr_list in DataInfo.known_general_data_info.items():
            if self.config.has_section(section):
                section_attributes_convert = self.__extract_data_section(section, attr_list)
                self.__sections_convert[section] = Data(section_attributes_convert)
            else:
                self.__sections_convert[section] = Data({})

        attr_list = DataInfo.known_player_data_info
        for section in self.config.sections():
            if section[:6].lower() == "player":
                player_id = int(section[6:])
                section_attributes_convert = self.__extract_data_section(section, attr_list)
                section_attributes_convert["playerID"] = player_id
                section_attributes_convert["gttot"] = section_attributes_convert["gtc"] + section_attributes_convert["gtl"]
                section_attributes_convert["gtall"] = section_attributes_convert["gttot"] + section_attributes_convert["gtt"]
                self.__players_convert[player_id] = Data(section_attributes_convert)
            elif not self.__sections_convert.has_key(section):
                section_attributes_convert = self.__extract_data_section(section, [])
                self.__sections_convert[section] = Data(section_attributes_convert)

        # Add the teamselecting stuff.
        ts = TeamSelector.TeamSelector(self)
        pt = ts.get_player_table()

        pl_best_list = {}
        for f, plist in pt.items():
            i = 0
            for val, pid in plist:
                i += 1
                self.__players_convert[pid].__dict__["ts_" + f] = (val, i)
                if not pl_best_list.has_key(pid) or \
                       val > pl_best_list[pid][1]:
                    pl_best_list[pid] = (f, val)

        for pid, (f, val) in pl_best_list.items():
            for i, (ff, d) in Terms.team_positions.items():
                if f == ff:
                    self.__players_convert[pid].__dict__["ts_best_position"] = (i, val)
                    break

        elements = ["keeper",
                    "rightBack", "insideBack1", "insideBack2", "leftBack",
                    "rightWinger", "insideMid1", "insideMid2", "leftWinger",
                    "forward1", "forward2"]

        lineup = {}
        for position in elements:
            pid = getattr(self.__sections_convert["lineup"], position)
            if not pid:
                lineup = None
                break

            if position != "keeper":
                behposition = "beh" + position[0].upper() + position[1:]
                order_val = getattr(self.__sections_convert["lineup"], behposition)
            else:
                order_val = 0

            pos = None
            for d, p, rp in TeamSelector.TeamSelector.lineup_info:
                if p == position:
                    pos = rp
                    break

            pos = TeamSelector.TeamSelector.set_real_pos(pos, order_val)
            if pid not in self.__players_convert.keys():
                val = (0.0, -1)
            else:
                val = getattr(self.__players_convert[pid], "ts_" + pos)
            lineup[position] = (pid, val[0], val[1], rp, order_val)
        if lineup == None:
            self.__sections_convert["team"].__dict__["teamval"] = 0.0
        else:
            self.__sections_convert["team"].__dict__["teamval"] = ts.de.teamval(lineup)

        self.teamselector = ts

        # pass 2
        for section, secdata in self.__sections_convert.items():
            if DataInfo.known_general_data_info.has_key(section):
                attr_list = DataInfo.known_general_data_info[section]
            else:
                attr_list = None
            section_attributes_display = self.__make_displayval_in_section(secdata, attr_list)
            self.__sections_display[section] = Data(section_attributes_display)

        for pid, secdata in self.__players_convert.items():
            attr_list = DataInfo.known_player_data_info
            player_attributes_display = self.__make_displayval_in_section(secdata, attr_list)
            self.__players_display[pid] = Data(player_attributes_display)

    def __extract_data_section(self, section, attr_list):
        section_attributes_convert = {}

        for data_name, (hrf_name, description, convert_func, display_func) in attr_list.items():
            if self.config.has_option(section, hrf_name):
                val = self.config.get(section, hrf_name)
                if convert_func != None:
                    try:
                        convertval = convert_func(val)
                    except Exception, e:
                        #print "Error:", data_name, e
                        convertval = None
                else:
                    convertval = val

                section_attributes_convert[data_name] = convertval
            else:
                section_attributes_convert[data_name] = None

        for option in self.config.options(section):
            if not section_attributes_convert.has_key(option):
                section_attributes_convert[option] = self.config.get(section, option)

        return section_attributes_convert

    def __make_displayval_in_section(self, secdata, attr_list):
        section_attributes_display = {}
        for attribute, convertval in secdata.__dict__.items():
            if attr_list != None and attr_list.has_key(attribute):
                display_func = attr_list[attribute][3]
            else:
                display_func = None

            if display_func != None:
                try:
                    displayval = display_func(convertval)
                except Exception, e:
                    #print "Error:", attribute, e
                    displayval = convertval
            else:
                displayval = convertval

            section_attributes_display[attribute] = str(displayval)
        return section_attributes_display
