# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from Common import *
from Images import Images

import cStringIO

from wxPython.wx import *

DayNames = [(_("Sunday"), _("Sun")),
            (_("Monday"), _("Mon")),
            (_("Tuesday"), _("Tue")),
            (_("Wednesday"), _("Wed")),
            (_("Thursday"), _("Thu")),
            (_("Friday"), _("Fri")),
            (_("Saturday"), _("Sat"))]

mEVT_CALENDAR = wxNewEventType()

def EVT_CALENDAR(win, id, func):
    win.Connect(id, -1, mEVT_CALENDAR, func)

class CalendarEvent(wxPyCommandEvent):
    def __init__(self, evtType, id):
        wxPyCommandEvent.__init__(self, evtType, id)
        self.__date = None

    def SetDate(self, date):
        self.__date = date

    def GetDate(self):
        return self.__date

class CalendarChoice(wxPanel):
    ID_DATE = 429
    ID_EARLIER = 430
    ID_LATRE = 431
    ID_CALENDAR = 432

    def __init__(self, parent, ID = -1, current_date = (1, 1)):
        wxPanel.__init__(self, parent, ID)

        self.__pop_dialog = None

        sizer = wxBoxSizer(wxVERTICAL)

        self.__current_date = current_date

        self.date_button = wxButton(self, self.ID_DATE, self.__format_date(self.__current_date))
        sizer.Add(self.date_button, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        EVT_BUTTON(self, self.ID_DATE, self.OnDatePressed)

    def SetDate(self, date):
        if date is not None:
            self.__current_date = date
            self.date_button.SetLabel(self.__format_date(self.__current_date))

    def GetDate(self):
        return self.__current_date

    def OnDatePressed(self, event):
        if self.__pop_dialog is None:
            self.__pop_dialog = CalendarPopup(self, self.ID_CALENDAR,
                                              marked_date = self.__current_date)

            wsize = self.__pop_dialog.GetSize()
            bsize = self.date_button.GetSize()

            pos = self.date_button.ClientToScreen(((bsize[0] - wsize[0]) // 2,0))

            self.__pop_dialog.Position(pos, (0, bsize[1]))

            self.__pop_dialog.Show(True)

            EVT_CALENDAR(self.__pop_dialog, self.ID_CALENDAR, self.OnCalendar)
        else:
            self.__pop_dialog.Destroy()
            self.__pop_dialog = None

    def OnCalendar(self, event):
        date = event.GetDate()
        if date is not None:
            self.__current_date = date
            self.date_button.SetLabel(self.__format_date(self.__current_date))

        self.__pop_dialog.Destroy()
        self.__pop_dialog = None

        self.__generate_event(self.__current_date)

    def __format_date(self, date):
        r, d = divmod((date[1] - 1), 7)
        r += 1
        return (_("Season %d") + ", " + _("Round %d") + ", %s") % (date[0], r, DayNames[d][0])

    def __generate_event(self, date):
        event = CalendarEvent(mEVT_CALENDAR, self.GetId())
        event.SetDate(date)
        self.GetEventHandler().ProcessEvent(event)

class CalendarPopup(wxDialog):
    def __init__(self, parent, ID = -1, marked_date = None, available = None):
        wxDialog.__init__(self, parent, ID, '', style = wxSIMPLE_BORDER | wxSTAY_ON_TOP)

        sizer = wxBoxSizer(wxVERTICAL)

        panel = wxPanel(self, -1, style = wxRAISED_BORDER)
        ssizer = wxBoxSizer(wxVERTICAL)

        self.calctrl = CalendarCtrl(panel, ID, marked_date = marked_date, available = available)

        ssizer.Add(self.calctrl, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)
        panel.SetSizer(ssizer)
        panel.SetAutoLayout(True)
        ssizer.Fit(panel)

        sizer.Add(panel, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        self.SetClientSize((250, 110))

    def Position(self, pos, bsize):
        dsize = wxGetDisplaySize()
        msize = self.GetSize()

        pref_posx = pos[0] + bsize[0]
        pref_posy = pos[1] + bsize[1]

        if pref_posx < 0:
            pref_posx = 0

        if pref_posx + msize[0] > dsize[0]:
            pref_posx = dsize[0] - msize[0]

        if pref_posy + msize[1] > dsize[1]:
            pref_posy = pos[1] - msize[1]

        pos = (pref_posx, pref_posy)

        self.Move(pos)

class CalendarCtrl(wxWindow):
    BP_TIMER_ID = 675

    def __init__(self, parent, ID = -1,
                 pos = wxDefaultPosition, size = wxDefaultSize,
                 style = 0, name = 'CalendarCtrl',
                 marked_date = None, available = None):

        wxWindow.__init__(self, parent, ID, pos, size, style, name)

        self.__mouse_areas = []
        self.__mousewheel_areas = []

        self.__hovered_area = None
        self.__current_pressed_area = None

        self.__font_size = 8
        self.__font_cache = {}

        if marked_date is not None:
            self.__start_round = marked_date[1] // 7 - 1
            self.__start_round = max(self.__start_round, 0)
            self.__start_round = min(self.__start_round, 12)
            self.__season = marked_date[0]
        else:
            self.__start_round = 0
            self.__season = 22
        self.__marked_date = marked_date
        self.__available = available

        self.__header_bg_colour = (200, 200, 200)
        self.__hover_bg_colour = (200, 200, 240)

        self.__marked_fg_colour = (255, 0, 0)
        self.__unavailable_fg_colour = (175, 175, 175)

        self.__button_timer_long = 600
        self.__button_timer_short = 100
        self.__button_pressed = None
        self.__button_pressed_timer = wxTimer(self, self.BP_TIMER_ID)

        self.__text_border = 2

        self.__buffer = None

        EVT_PAINT(self, self.OnPaint)
        EVT_SIZE(self, self.OnSize)

        EVT_MOTION(self, self.OnMotion)
        EVT_ENTER_WINDOW(self, self.OnEnter)
        EVT_LEAVE_WINDOW(self, self.OnLeave)
        EVT_LEFT_DOWN(self, self.OnLeftDown)
        EVT_LEFT_UP(self, self.OnLeftUp)
        EVT_RIGHT_DOWN(self, self.OnRightDown)
        EVT_MOUSEWHEEL(self, self.OnMousewheel)

        EVT_TIMER(self, self.BP_TIMER_ID, self.OnRoundTimer)

    def OnSize(self, event):
        self.__size = self.GetClientSizeTuple()
        self.__buffer = wxEmptyBitmap(*self.__size)

        self.__draw()

    def OnPaint(self, event):
        if self.__buffer is None:
            dc = wxBufferedPaintDC(self)
            self.__draw()
        else:
            dc = wxBufferedPaintDC(self, self.__buffer)

    def OnMotion(self, event):
        evx = event.GetX()
        evy = event.GetY()

        old_hov = self.__hovered_area
        for etype, data, x1, y1, x2, y2 in self.__mouse_areas:
            if x1 <= evx < x2 and y1 <= evy < y2:
                if self.__available is not None and (self.__season, data) not in self.__available:
                    self.__hovered_area = None
                else:
                    self.__hovered_area = (etype, data)
                break
        else:
            self.__hovered_area = None

        if self.__hovered_area != old_hov:
            self.Redraw()

    def OnEnter(self, event):
        pass

    def OnLeave(self, event):
        old_hov = self.__hovered_area
        self.__hovered_area = None

        if self.__hovered_area != old_hov:
            self.Redraw()

    def OnLeftDown(self, event):
        evx = event.GetX()
        evy = event.GetY()
        for etype, data, x1, y1, x2, y2 in self.__mouse_areas:
            if x1 <= evx < x2 and y1 <= evy < y2:
                self.__current_pressed_area = (etype, data)
                break
        else:
            self.__current_pressed_area = None

        if self.__current_pressed_area is not None and self.__current_pressed_area[0] == "round":
            self.__button_pressed = (True, data)
            self.OnRoundTimer()

    def OnLeftUp(self, event):
        if self.__current_pressed_area is not None:
            if self.__current_pressed_area[0] == "round":
                self.__button_pressed = None
                self.__button_pressed_timer.Stop()
            elif self.__current_pressed_area[0] == "date":
                if self.__available is None or \
                       (self.__season, self.__current_pressed_area[1]) in self.__available:
                    self.__generate_event((self.__season, self.__current_pressed_area[1]))

    def OnRightDown(self, event):
        self.__generate_event(None)

    def OnRoundTimer(self, event = None):
        if self.__button_pressed is None:
            return

        long_time_wait, (add, rs_type) = self.__button_pressed
        self.__round_change(rs_type, add)
        if long_time_wait:
            self.__button_pressed = (False, (add, rs_type))
            self.__button_pressed_timer.Start(self.__button_timer_long, True)
        else:
            self.__button_pressed_timer.Start(self.__button_timer_short, True)

    def OnMousewheel(self, event):
        evx = event.GetX()
        evy = event.GetY()
        for data, x1, y1, x2, y2 in self.__mousewheel_areas:
            if x1 <= evx < x2 and y1 <= evy < y2:
                self.__round_change(data, sgn(event.GetWheelRotation()))

    def Redraw(self):
        self.__draw()

    def __generate_event(self, date):
        event = CalendarEvent(mEVT_CALENDAR, self.GetId())
        event.SetDate(date)
        self.GetEventHandler().ProcessEvent(event)

    def __round_change(self, rs_type, add):
        if rs_type == "round":
            self.__start_round += add
            if self.__start_round < 0:
                self.__start_round = 12
                self.__season -= 1
            elif self.__start_round > 12:
                self.__start_round = 0
                self.__season += 1
        elif rs_type == "season":
            self.__season += add
        self.__season = max(self.__season, 1)
        self.__season = min(self.__season, 100)
        self.Redraw()

    def __draw(self):
        dc = wxBufferedDC(wxClientDC(self), self.__buffer)
        bg_col = self.GetBackgroundColour()
        dc.SetBackground(wxBrush(bg_col))
        dc.SetTextBackground(bg_col)
        dc.Clear()

        self.__mouse_areas = []
        self.__mousewheel_areas = []

        dc.SetFont(self.__get_font(self.__font_size))

        short_days_ext = (0, 0)
        for sd_str in zip(*DayNames)[1]:
            mx, my = dc.GetTextExtent(sd_str)
            short_days_ext = (max(short_days_ext[0], mx), max(short_days_ext[1], my))

        round_ext = (0, 0)
        top_round_ext = (0, 0)
        for r in range(1, 16):
            mx, my = dc.GetTextExtent(_("Rnd %d") % r)
            round_ext = (max(round_ext[0], mx), max(round_ext[1], my))

            if r <= 13:
                tmx, tmy = dc.GetTextExtent(_("Round %d - %d") % (r, r + 3))
                top_round_ext = (max(top_round_ext[0], tmx), max(top_round_ext[1], tmy))

        top_season_ext = dc.GetTextExtent(_("Season %d") % 100)
        t_height = max(top_round_ext[1], top_season_ext[1]) + self.__text_border * 2

        rw_extra = round_ext[0] + self.__text_border * 2
        rh_extra = t_height

        w, h = self.__size
        h -= rh_extra
        nw = len(DayNames)
        nh = 5
        sh = int(float(h) / float(nh))

        dc.SetPen(wxPen((255, 255, 255)))
        dc.SetBrush(wxBrush((255, 255, 255)))
        dc.DrawRectangle(0, rh_extra, w, h)

        self.__draw_top_selection(dc, w, h, top_round_ext, top_season_ext, t_height)

        # Actual calendar
        dc.SetPen(wxPen(self.__header_bg_colour))
        dc.SetBrush(wxBrush(self.__header_bg_colour))
        dc.DrawRectangle(0, rh_extra, w, sh)
        dc.DrawRectangle(0, rh_extra, rw_extra, h)

        w -= rw_extra
        sw = int(float(w) / float(nw))

        dc.SetTextForeground((0, 0, 0))
        dc.SetTextBackground((255, 255, 255))
        for i in range(nw):
            sd_str = DayNames[i][1]
            mx, my = dc.GetTextExtent(sd_str)
            x = rw_extra + int((float(i) / float(nw)) * w) + (sw - mx) // 2
            y = rh_extra + (sh - my) // 2
            dc.DrawText(sd_str, x, y)

        for i in range(4):
            yh = rh_extra + int((float(i + 1) / float(nh)) * h)
            y = yh + (sh - my) // 2
            dc.DrawText(_("Rnd %d") % (i + self.__start_round + 1), self.__text_border, y)
            for j in range(7):
                n = ((i + self.__start_round) * 7) + j + 1

                mx, my = dc.GetTextExtent(str(n))
                xw = rw_extra + int((float(j) / float(nw)) * w)
                x = xw + (sw - mx) // 2

                self.__paint_hover(dc, "date", n, xw, yh, sw, sh)

                if (self.__season, n) == self.__marked_date:
                    dc.SetTextForeground(self.__marked_fg_colour)
                elif self.__available is not None and (self.__season, n) not in self.__available:
                    dc.SetTextForeground(self.__unavailable_fg_colour)

                dc.DrawText(str(n), x, y)

                dc.SetTextForeground((0, 0, 0))

                self.__mouse_areas.append(("date", n, xw, yh, xw + sw, yh + sh))

    def __draw_top_selection(self, dc, w, h, top_round_ext, top_season_ext, t_height):
        # Top selection
        left_bmp = Images.get_bitmap_from_file("left.png")
        right_bmp = Images.get_bitmap_from_file("right.png")

        left_ext = (left_bmp.GetWidth(), left_bmp.GetHeight())
        right_ext = (right_bmp.GetWidth(), right_bmp.GetHeight())

        x = 0
        xw = left_ext[0] + self.__text_border * 2
        self.__paint_hover(dc, "round", (-1, "round"), x, 0, xw, t_height)
        dc.DrawBitmap(left_bmp, x + self.__text_border, (t_height - left_ext[1]) // 2, True)
        self.__mouse_areas.append(("round", (-1, "round"), x, 0, x + xw, t_height))

        x += xw
        xw = top_round_ext[0] + self.__text_border * 2
        t_str = _("Round %d - %d") % (self.__start_round + 1, self.__start_round + 4)
        t_ext = dc.GetTextExtent(t_str)
        dc.DrawText(t_str, x + self.__text_border + (top_round_ext[0] - t_ext[0]) // 2,
                    self.__text_border)

        x += xw
        xw = right_ext[0] + self.__text_border * 2
        self.__paint_hover(dc, "round", (1, "round"), x, 0, xw, t_height)
        dc.DrawBitmap(right_bmp, x + self.__text_border, (t_height - right_ext[1]) // 2, True)
        self.__mouse_areas.append(("round", (1, "round"), x, 0, x + xw, t_height))

        self.__mousewheel_areas.append(("round", 0, 0, x + xw, t_height))

        xw = right_ext[0] + self.__text_border * 2
        x = w - xw
        self.__paint_hover(dc, "round", (1, "season"), x, 0, xw, t_height)
        dc.DrawBitmap(right_bmp, x + self.__text_border, (t_height - right_ext[1]) // 2, True)
        self.__mouse_areas.append(("round", (1, "season"), x, 0, x + xw, t_height))

        xw = top_season_ext[0] + self.__text_border * 2
        x -= xw
        t_str = _("Season %d") % self.__season
        t_ext = dc.GetTextExtent(t_str)
        dc.DrawText(t_str, x + self.__text_border + (top_season_ext[0] - t_ext[0]) // 2,
                    self.__text_border)

        xw = left_ext[0] + self.__text_border * 2
        x -= xw
        self.__paint_hover(dc, "round", (-1, "season"), x, 0, xw, t_height)
        dc.DrawBitmap(left_bmp, x + self.__text_border, (t_height - left_ext[1]) // 2, True)
        self.__mouse_areas.append(("round", (-1, "season"), x, 0, x + xw, t_height))

        self.__mousewheel_areas.append(("season", x, 0, w, t_height))

    def __paint_hover(self, dc, etype, data, x, y, w, h):
        if self.__hovered_area == (etype, data):
            dc.SetPen(wxPen(self.__hover_bg_colour))
            dc.SetBrush(wxBrush(self.__hover_bg_colour))
            dc.DrawRectangle(x, y, w, h)

    def __get_font(self, size):
        of = self.GetFont()

        key = (size, of.GetFamily(), of.GetStyle(), of.GetWeight())
        font = self.__font_cache.get(key, None)

        if font is None:
            font = wxFont(*key)
            self.__font_cache[key] = font
        return font

def __test():

    class AppFrame(wxFrame):
        def __init__(self, parent, id, title):
            wxFrame.__init__(self, parent, id, title)

            sizer = wxBoxSizer(wxVERTICAL)

            cal = CalendarChoice(self)
            sizer.Add(cal, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

#             panel = wxPanel(self, -1, style = wxRAISED_BORDER)
#             ssizer = wxBoxSizer(wxVERTICAL)

#             available = []
#             for i in range(26, 73):
#                 available.append((i, 22))

#             self.cc = CalendarCtrl(panel, marked_date = (66, 22), available = available)

#             ssizer.Add(self.cc, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)
#             panel.SetSizer(ssizer)
#             panel.SetAutoLayout(True)
#             ssizer.Fit(panel)

#            sizer.Add(panel, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

            self.SetSizer(sizer)
            self.SetAutoLayout(True)
            sizer.Fit(self)

            #self.SetClientSize((250, 110))

    class MyApp(wxApp):
        def OnInit(self):
            frame = AppFrame(None, -1, "PlotCtrl")
            frame.Centre()
            frame.Show(True)
            self.SetTopWindow(frame)
            return True

    app = MyApp(0)
    app.MainLoop()

if __name__ == '__main__':
    __test()
