# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys
import os.path

def_charset = 'ISO-8859-1'

def no_translation(s):
    return s

def no_translation_plural(s, p, n):
    if n == 1:
        return s
    return p

def install_translation():
    global def_charset

    import __builtin__

    __builtin__.__dict__['N_'] = no_translation

    try:
        import gettext
        from Config import config

        path = os.path.join(os.path.realpath(os.path.dirname(sys.argv[0])), "locale")
        if not os.path.isdir(path):
            path = os.path.normpath("/usr/share/locale")
            if not os.path.isdir(path):
                path = None
        def_lang = config.get("General", "language")
        if def_lang == "":
            def_lang = None
            if sys.platform.startswith("win"):
                if not os.getenv("LANGUAGE"):
                    try:
                        import win32api

                        # Get from
                        # http://msdn.microsoft.com/library/default.asp?url=/library/en-us/intl/nls_61df.asp
                        # http://msdn.microsoft.com/library/default.asp?url=/library/en-us/intl/nls_238z.asp

                        windows_languages = [(0x09,
                                              [0x0409, 0x0809, 0x0c09,
                                               0x1009, 0x1409, 0x1809,
                                               0x1c09, 0x2009, 0x2409,
                                               0x2809, 0x2c09, 0x3009,
                                               0x3409],
                                              "C"),

                                             (0x07,
                                              [0x0407, 0x0807, 0x0c07,
                                              0x1007, 0x1407],
                                              "de"),

                                             (0x0a,
                                              [0x040a, 0x080a, 0x0c0a,
                                               0x100a, 0x140a, 0x180a,
                                               0x1c0a, 0x200a, 0x240a,
                                               0x280a, 0x2c0a, 0x300a,
                                               0x340a, 0x380a, 0x3c0a,
                                               0x400a, 0x440a, 0x480a,
                                               0x4c0a, 0x500a],
                                              "es"),

                                             (0x10,
                                              [0x0410, 0x0810],
                                              "it"),

                                             (0x13,
                                              [0x0413, 0x0813],
                                              "nl"),

                                             (0x15,
                                              [0x0415],
                                              "pl"),

                                             (0x16,
                                              [0x0416, 0x0816],
                                              "pt"),

                                             (0x1d,
                                              [0x041d, 0x081d],
                                              "sv"),
                                             ]

                        lcid = win32api.GetUserDefaultLCID()

                        for pl, subls, code in windows_languages:
                            if lcid in subls:
                                def_lang = [code, 'C']
                                break

                    except ImportError, e:
                        pass
        else:
            def_lang = [def_lang, 'C']
        t = gettext.translation("mattricks",
                                localedir = path,
                                languages = def_lang,
                                fallback = True)
        t.install(False) #not not os.getenv("UNICODE"))

        if t._charset is not None:
            def_charset = t._charset

        try:
            __builtin__.__dict__['ngettext'] = t.ngettext
        except AttributeError, e:
            __builtin__.__dict__['ngettext'] = no_translation_plural

    except ImportError, e:
        __builtin__.__dict__['_'] = no_translation
        __builtin__.__dict__['ngettext'] = no_translation_plural
