# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os.path
import sys

from wxPython.wx import *

wxInitAllImageHandlers()

class ImageStorage:
    def __init__(self, search_path):
        self.search_path = [os.path.abspath(p) for p in search_path.split(':')]
        self.images = {}

    def get_bitmap_from_file(self, img_path):
        if img_path == "":
            return wxNullBitmap

        img_path = os.path.normpath(img_path)
        if self.images.has_key(img_path):
            return self.images[img_path]

        bmp = wxEmptyBitmap(1, 1)
        if os.path.isabs(img_path):
            sp = [""]
        else:
            sp = self.search_path

        for p in sp:
            file = os.path.join(p, img_path)
            #print file
            if os.path.exists(file):
                img = wxImage(file, wxBITMAP_TYPE_ANY)
                if img.Ok():
                    bmp = wxBitmapFromImage(img)
                    break
        self.images[img_path] = bmp
        return bmp

datapath = "Data:/usr/share/Mattricks:/usr/local/share/Mattricks:" + \
               os.path.abspath(os.path.dirname(sys.argv[0])) + \
               "/Data"
Images = ImageStorage(datapath)

def create_star_image(rating):
    star = Images.get_bitmap_from_file("star.png")
    star_half = Images.get_bitmap_from_file("star_half.png")
    (nstars, nhalfstars) = divmod(rating, 2)

    ws = star.GetWidth()
    wh = star_half.GetWidth()
    wr = ws*nstars + wh*nhalfstars
    hs = star.GetHeight()
    hh = star_half.GetHeight()
    hr = max(hs,hh)

    bmp = wxEmptyBitmap(wr,hh)

    image = wxMemoryDC()
    image.SelectObject(bmp)

    image.SetBrush(wxBrush("#FF00FF"))
    image.Clear()

    cw = 0
    for i in range(nstars):
        image.DrawBitmap(star, cw, 0, True)
        cw += ws
    for i in range(nhalfstars):
        image.DrawBitmap(star_half, cw, 0, True)
        cw += wh

    del image

    bmp.SetMaskColour("#FF00FF")

    return bmp

def create_dress_images(type, c_shirt, c_shorts, c_socks):
    shirt_body_mask = Images.get_bitmap_from_file("dress_shirt_body_mask.png")
    shirt_fringe_mask = Images.get_bitmap_from_file("dress_shirt_fringe_mask.png")
    shorts_body_mask = Images.get_bitmap_from_file("dress_shorts_body_mask.png")
    shorts_fringe_mask = Images.get_bitmap_from_file("dress_shorts_fringe_mask.png")
    socks_body_mask = Images.get_bitmap_from_file("dress_socks_body_mask.png")
    socks_fringe_mask = Images.get_bitmap_from_file("dress_socks_fringe_mask.png")

    res_imgs = []
    sfc = (c_shirt == c_socks) and c_shorts or c_shirt
    for body_mask, fringe_mask, c1, c2 in [(shirt_body_mask, shirt_fringe_mask, c_shirt, c_shorts),
                                           (shorts_body_mask, shorts_fringe_mask, c_shorts, c_shirt),
                                           (socks_body_mask, socks_fringe_mask, c_socks, sfc)]:
        w = body_mask.GetWidth()
        h = body_mask.GetHeight()

        resbmp = wxEmptyBitmap(w,h)
        resimage = wxMemoryDC()
        resimage.SelectObject(resbmp)
        resimage.SetBackground(wxBrush("#FF00FF"))
        resimage.Clear()

        for mask, c in [(body_mask, c1), (fringe_mask, c2)]:
            m = wxMaskColour(mask, wxBLACK)

            bmp = wxEmptyBitmap(w,h)
            bmp.SetMask(m)

            image = wxMemoryDC()
            image.SelectObject(bmp)

            image.SetBackground(wxBrush(c))
            image.Clear()

            resimage.Blit(0, 0, w, h, image, 0, 0, wxCOPY, True)

            del image

        del resimage

        resbmp.SetMaskColour("#FF00FF")

        res_imgs.append(resbmp)

    return res_imgs

def create_small_dress(c):
    small_dress = Images.get_bitmap_from_file("dress_small.png")
    
    w = small_dress.GetWidth()
    h = small_dress.GetHeight()

    resbmp = wxEmptyBitmap(w,h)
    resimage = wxMemoryDC()
    resimage.SelectObject(resbmp)
    resimage.SetBackground(wxBrush("#FF00FF"))
    resimage.Clear()
    m = wxMaskColour(small_dress, wxWHITE)

    bmp = wxEmptyBitmap(w,h)
    bmp.SetMask(m)

    image = wxMemoryDC()
    image.SelectObject(bmp)

    image.SetBackground(wxBrush(c))
    image.Clear()

    resimage.Blit(0, 0, w, h, image, 0, 0, wxCOPY, True)

    del image

    del resimage

    resbmp.SetMaskColour("#FF00FF")
    return resbmp
