# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""A panel suited for placement in the main notebook."""

from Common import *
from Commonwx import *

import CommonData
import Fixtures
import Images
import Matchdialogs
import XMLParse
import TeamList
import TablePlotsDialog

from wxPython.wx import *

class LeaguePage(wxPanel):
    ID_LEAGUE = 523
    ID_SEASON = 524
    ID_TYPE = 525
    ID_CHANGE = 526
    ID_SLIDER = 527
    ID_ALL = 528
    ID_UPDATE = 529
    ID_TEAMS = 530
    ID_PLOTS = 531
    ID_BUTTONS = 533

    def __init__(self, parent, ID):
        wxPanel.__init__(self, parent, ID)

        sizer = wxBoxSizer(wxVERTICAL)
        hsizer = wxBoxSizer(wxHORIZONTAL)

        stext = wxStaticText(self, -1, _("Matches") + ":")
        hsizer.Add(stext, 0, wxALIGN_CENTRE | wxALL, 5)

        type_choices=[_("All"), _("Home"), _("Away")]

        self.type_choice = wxChoice(self, self.ID_TYPE,
                                    choices = type_choices)
        self.type_choice.SetSelection(0)
        hsizer.Add(self.type_choice, 0, wxALIGN_CENTRE | wxALL, 5)

        hsizer.Add((1, 1), 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        stext = wxStaticText(self, -1, _("League") + ":")
        hsizer.Add(stext, 0, wxALIGN_CENTRE | wxALL, 5)

        self.league_select = wxTextCtrl(self, -1, style = wxTE_RIGHT)
        self.league_select.SetValue("-1")
        hsizer.Add(self.league_select, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        stext = wxStaticText(self, -1, _("Season") + ":")
        hsizer.Add(stext, 0,  wxALIGN_CENTRE | wxALL, 5)

        self.season_select = wxTextCtrl(self, -1, style = wxTE_RIGHT)
        self.season_select.SetValue("-1")
        hsizer.Add(self.season_select, 0, wxALIGN_CENTRE | wxALL, 5)

        button = wxButton(self, self.ID_CHANGE, _("Change"), style = wxBU_EXACTFIT)
        hsizer.Add(button, 0, wxALIGN_CENTRE | wxALL, 5)

        button = wxButton(self, self.ID_UPDATE, _("Update data"), style = wxBU_EXACTFIT)
        hsizer.Add(button, 0, wxALIGN_CENTRE | wxALL, 5)

        sizer.Add(hsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        hsizer = wxBoxSizer(wxHORIZONTAL)
        seriestext = wxStaticText(self,-1, _("League") + ": ", style=wxALIGN_LEFT)
        SetTextFace(seriestext, FACE_BOLD | FACE_BIG)
        hsizer.Add(seriestext, 0, wxEXPAND | wxALL|wxALIGN_BOTTOM, 5)
        self.league_name = wxStaticText(self,-1, "                      ", style=wxALIGN_LEFT)
        SetTextFace(self.league_name, FACE_BOLD | FACE_BIG)
        hsizer.Add(self.league_name, 0, wxEXPAND | wxALL|wxALIGN_BOTTOM, 5)
        countrytext = wxStaticText(self,-1, _("Country") + ": ", style=wxALIGN_LEFT)
        SetTextFace(countrytext, FACE_BOLD)
        hsizer.Add(countrytext, 0, wxEXPAND | wxALL|wxALIGN_BOTTOM, 5)
        self.country_name = wxStaticText(self,-1, "                      ", style=wxALIGN_LEFT)
        hsizer.Add(self.country_name, 0, wxEXPAND | wxALL|wxALIGN_BOTTOM, 5)
        leveltext = wxStaticText(self,-1, _("Level") + ": ", style=wxALIGN_LEFT)
        SetTextFace(leveltext, FACE_BOLD)
        hsizer.Add(leveltext, 0, wxEXPAND | wxALL|wxALIGN_BOTTOM, 5)
        self.level_name = wxStaticText(self,-1, "                      ", style=wxALIGN_LEFT)
        hsizer.Add(self.level_name, 0, wxEXPAND | wxALL|wxALIGN_BOTTOM, 5)
        hsizer.Add((1, 1), 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)
        button_myleague = wxButton(self, -1, _("My league"), style = wxBU_EXACTFIT)
        hsizer.Add(button_myleague, 0, wxALIGN_RIGHT | wxALL, 0)
        hsizer.Add((1, 1), 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)
        button_plots = wxButton(self, self.ID_PLOTS, _("Plots"), style = wxBU_EXACTFIT)
        hsizer.Add(button_plots, 0, wxALIGN_RIGHT | wxALL, 0)
        hsizer.Add((1, 1), 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)
        button_teams = wxButton(self, self.ID_TEAMS, _("Teams"), style = wxBU_EXACTFIT)
        hsizer.Add(button_teams, 0, wxALIGN_RIGHT | wxALL, 0)
        hsizer.Add((1, 1), 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)
        button_all = wxButton(self, self.ID_ALL, _("Fetch all games"), style = wxBU_EXACTFIT)
        hsizer.Add(button_all, 0, wxALIGN_RIGHT | wxALL, 0)

        sizer.Add(hsizer, 0, wxEXPAND | wxALL, 5)

        fgsizer = wxFlexGridSizer(cols=9, vgap=0, hgap=0)
        fgsizer.AddGrowableCol(1)

        aligns = [wxALIGN_RIGHT, wxALIGN_LEFT, wxALIGN_CENTRE] + \
                 [wxALIGN_RIGHT]*4 + [wxALIGN_CENTRE, wxALIGN_RIGHT]
        sizes = [0, 1] + [0]*7
        i = 0
        for h in [_("Pos"), _("Name"), _("Id"), _("Pl"), _("W"), _("D"),
                  _("L"), _("For - Against"), _("Po")]:
            stext = wxStaticText(self, -1, h, style = aligns[i])
            SetTextBold(stext)
            fgsizer.Add(stext, sizes[i], wxEXPAND | wxADJUST_MINSIZE | wxALL, 5)
            i += 1

        self.table_texts = {}
        self.gpanels = {}
        row = 0
        for rc, minxsize in zip(["#F0F0F0", "#E8E8E8"]*4,
                                [30, 100, 80, 40, 40, 40, 40, 100, 40]):
            col = 0
            for align, rsize in zip(aligns, sizes):
                gpanel= SimpleColouredPanel(self, -1, rc)
                stext = wxStaticText(gpanel, -1, "", size = (minxsize, -1), style = align)
                gpanel.AddChild(stext, 1, wxEXPAND | wxADJUST_MINSIZE | align | wxALL, 3)
                fgsizer.Add(gpanel, rsize, wxEXPAND | wxADJUST_MINSIZE | wxALL, 0)
                self.table_texts[(row, col)] = stext
                self.gpanels[(row,col)]=gpanel
                col += 1
            row += 1

        sizer.Add(fgsizer, 0, wxEXPAND |wxALL, 5)

        hsizer = wxBoxSizer(wxHORIZONTAL)
        vsizer = wxBoxSizer(wxVERTICAL)

        self.fixtures_texts = []
        self.fixture_buttons = {}
        star_img = Images.Images.get_bitmap_from_file("star.png")
        for r in range(2):
            rstext = wxStaticText(self, -1, _("Round") + ":")
            SetTextBold(rstext)
            vsizer.Add(rstext, 0, wxEXPAND | wxALL, 5)

            fgsizer = wxFlexGridSizer(cols=9, vgap=0, hgap=0)
            for i in [2,4]:
                fgsizer.AddGrowableCol(i)

            all_mtexts = []
            for m in range(4):
                texts = []
                ttexts = [0, 1, "", "-", "", 2, "", "-", ""]
                for t in range(9):
                    if ttexts[t] in [0,1,2]:
                        id = self.ID_BUTTONS + r*16 + m*4 + ttexts[t]
                        if ttexts[t] == 0:
                            widget = wxButton(self, id, _("Match details"), style = wxBU_EXACTFIT)
                        else:
                            widget = wxBitmapButton(self, id, star_img, style = wxBU_EXACTFIT)
                        widget.Disable()
                        EVT_BUTTON(self, id, self.OnMatchReport)
                    else:
                        widget = wxStaticText(self, -1, ttexts[t])
                    if t in [2,4]:
                        size = 1
                    else:
                        size = 0
                    fgsizer.Add(widget, size, wxEXPAND | wxADJUST_MINSIZE | wxLEFT | wxRIGHT, 5)

                    if ttexts[t] == "":
                        texts.append(widget)

                    if ttexts[t] in [0,1,2]:
                        self.fixture_buttons[(r, m, ttexts[t])] = widget
                all_mtexts.append(texts)

            vsizer.Add(fgsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)
            self.fixtures_texts.append((rstext, all_mtexts))

        hsizer.Add(vsizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        self.round_slider = wxSlider(self, self.ID_SLIDER, 1, 1, 13, style = wxSL_VERTICAL)
        hsizer.Add(self.round_slider, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        sizer.Add(hsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        self.fixtures = None

        EVT_BUTTON(self, self.ID_CHANGE, self.OnLeagueChange)
        EVT_BUTTON(self, self.ID_UPDATE, self.OnDataUpdate)
        EVT_BUTTON(self, self.ID_TEAMS, self.OnTeams)
        EVT_BUTTON(self, button_myleague.GetId(), self.OnMyLeague)
        EVT_BUTTON(self, self.ID_PLOTS, self.OnPlots)
        EVT_BUTTON(self, self.ID_ALL, self.OnFetchAllGames)
        EVT_CHOICE(self, self.ID_TYPE, self.OnTypeChange)
        EVT_COMMAND_SCROLL(self, self.ID_SLIDER, self.OnSliderChange)

    def AfterInit(self):
        teamid = CommonData.get_current_teamid()
        
        if teamid is not None:
            data = XMLParse.get(["team details", teamid],
                                ["Team.League.LeagueID",
                                 "Team.LeagueLevelUnit.LeagueLevelUnitID"])
            lid = data.get("Team.League.LeagueID")
            lluid = data.get("Team.LeagueLevelUnit.LeagueLevelUnitID")
            if lid is not None:
                season = CommonData.get_league_season(lid)
            else:
                season = None
            
            if lluid is not None:
                self.league_select.SetValue(str(lluid))
            if season is not None:
                self.season_select.SetValue(str(season))

            if lluid is not None and season is not None:
                self.Update()

    def OnMyLeague(self, event):
        self.AfterInit()
        
    def OnLeagueChange(self, event):
        self.Update()

    def OnDataUpdate(self, event):
        self.Update(True)

    def OnTypeChange(self, event):
        self.__change_table()

    def OnSliderChange(self, event):
        self.__update_fixtures()

    def OnTeams(self, event):
        tl=TeamList.TeamList(self)
        tl.Show()

    def OnPlots(self, event):
        ltp=TablePlotsDialog.TablePlotsDialog(self, self.fixtures)
        ltp.Show()

    def SetLeague(self, league_level_id, league_id):
        self.league_select.SetValue(str(league_level_id))
        season = CommonData.get_league_season(int(league_id))
        self.season_select.SetValue(str(season))

        self.Update()

    def OnMatchReport(self, event):
        id = event.GetId()
        r, m, t = divmod(id - self.ID_BUTTONS, 4, 4)
        round = self.__get_current_round() + r
        l_round = self.fixtures.GetLatestRound()
        matches = self.fixtures.matches

        match = matches[round].items()[m]
        match_id = match[0]

        if round <= l_round:
            if t == 0:
                dlg = Matchdialogs.MatchDetailsDialog(self, match_id)
                if not dlg.no_data:
                    ret = dlg.Show()
            elif t in [1,2]:
                if t == 1:
                    team_id = match[1][1][0]
                else:
                    team_id = match[1][2][0]

                dlg = Matchdialogs.MatchLineupDialog(self, match_id, team_id)
                if not dlg.no_data:
                    ret = dlg.Show()
        else:
            gp = self.GetGrandParent()
            matchviewpage = gp.GetTabHandle("matchview")
            matchviewpage.TryAddMatch(match_id)

    def OnFetchAllGames(self, event):
        if self.fixtures == None:
            mdlg = wxMessageDialog(self, _("No current fixture!"), _("Error"), wxOK | wxICON_ERROR)
            mdlg.ShowModal()
            mdlg.Destroy()
            return

        matches = self.fixtures.matches
        round = 14
        topframe = self.GetGrandParent()
        sbar = topframe.GetStatusBar()
        sbar.SetRange(round*4*3*10)
        pbw = topframe.GetPBW()
        pbw.progress_start(_("Fetching all games."))
        for r in range(round):
            for m in range(4):
                match_number=4*r+m+1
                match = matches[r+1].items()[m]
                played=match[1][3]
                sbar.SetBaseValue((match_number - 1)*30)
                if played:
                    sbar.PushExtraMessage(_("Checking game %d of %d.") % (match_number, round*4))
                    sbar.Update(0, "")

                    match_id = match[0]
                    dummy = XMLParse.get(["match details", match_id], ["FileName"],
                                         progress_dialog = False)

                    sbar.SetBaseValue((match_number - 1)*30 + 10)
                    team_id = match[1][1][0]
                    dummy = XMLParse.get(["match lineup", match_id, team_id], ["FileName"],
                                         progress_dialog = False)

                    sbar.SetBaseValue((match_number - 1)*30 + 20)
                    team_id = match[1][2][0]
                    dummy = XMLParse.get(["match lineup", match_id, team_id], ["FileName"],
                                         progress_dialog = False)
                else:
                    sbar.PushExtraMessage(_("Skipping unplayed game %d of %d.") %
                                          (match_number, round*4))
                    sbar.Update(0, "")
                sbar.PopExtraMessage()

        pbw.progress_end()

    def Update(self, force_update = False):
        current_league = self.__get_current_league()
        current_season = self.__get_current_season()

        err_message = None

        if current_league == None or current_league == 0 or current_league < -1:
            err_message = _("Entered league id is not valid.")

        elif current_season == None or current_season == 0 or current_season < -1:
            err_message = _("Entered season is not valid.")

        if err_message != None:
            mdlg = wxMessageDialog(self, err_message,
                                   _("Error"), wxOK | wxICON_ERROR)
            mdlg.ShowModal()
            mdlg.Destroy()
            return

        if self.fixtures != None:
            old_league = self.fixtures.league_id
            old_season = self.fixtures.season
        else:
            old_league = None
            old_season = None

        new_fixtures = Fixtures.Fixtures(current_league, current_season, force_update)

        if new_fixtures is None or new_fixtures.league_id is None:
            return

        self.fixtures = new_fixtures

        self.league_name.SetLabel(self.fixtures.league_name)
        self.level_name.SetLabel(_("%d of %d") % (self.fixtures.league_level,
                                                  self.fixtures.max_level))
        self.country_name.SetLabel(self.fixtures.country_name)
        self.league_select.SetValue(str(self.fixtures.league_id))
        self.season_select.SetValue(str(self.fixtures.season))

        self.__change_table()

        for ir in range(2):
            self.fixture_buttons[(ir, 0, 0)].SetLabel(_("Match details"))

        self.GetSizer().Layout()
        self.Refresh()

        if old_league != self.fixtures.league_id or \
           old_season != self.fixtures.season:

            round = self.fixtures.GetLatestRound()
            self.round_slider.SetValue(round)

        self.__update_fixtures()

        self.Refresh()

    def __change_table(self):
        if not self.fixtures or self.fixtures.league_id is None:
            for stext in self.table_texts.values():
                stext.SetLabel("")
            return

        current_type = self.__get_current_type()

        (allteams, hteams, ateams) = self.fixtures.calc_table()

        if current_type == 0:
            teams = allteams
        elif current_type == 1:
            teams = hteams
        elif current_type == 2:
            teams = ateams
        else:
            return

        for row in range(8):
            if row % 2 == 0:
                colour = 0xF0F0F0
            else:
                colour = 0xE8E8E8

            if (row == 0 or (row == 1 and self.fixtures.league_level >= 7)):

                colour = (colour & 0xDDFFDD)

            elif (row in [4,5] and self.fixtures.league_level <= 5 and \
                  self.fixtures.league_level != self.fixtures.max_level):

                colour = (colour & 0xFFFFDD)

            elif (row in [6,7] and self.fixtures.league_level != self.fixtures.max_level):

                colour = (colour & 0xFFDDDD)

            colour = "#%06X" % colour

            for col in range(9):
                self.gpanels[(row,col)].SetBackgroundColour(colour)
                self.table_texts[(row,col)].SetBackgroundColour(colour)
                self.gpanels[(row,col)].Refresh()
        row = 0
        for pl, w, d, l, f, a, po, n, tid in teams:
            self.table_texts[(row, 0)].SetLabel(str(row + 1))
            self.table_texts[(row, 1)].SetLabel(n)
            self.table_texts[(row, 2)].SetLabel(str(tid))
            self.table_texts[(row, 3)].SetLabel(str(pl))
            self.table_texts[(row, 4)].SetLabel(str(w))
            self.table_texts[(row, 5)].SetLabel(str(d))
            self.table_texts[(row, 6)].SetLabel(str(l))
            self.table_texts[(row, 7)].SetLabel("%d - %d" % (f, a))
            self.table_texts[(row, 8)].SetLabel(str(po))
            row += 1

    def __update_fixtures(self):
        if not self.fixtures or self.fixtures.league_id is None:
            for rstext, all_mtexts in self.fixtures_texts:
                rstext.SetLabel(_("Round") + ":")
                self.__reset_texts(all_mtexts)
            for r in range(2):
                for m in range(4):
                    for t in range(3):
                        self.fixture_buttons[(r,m,t)].Disable()
            return

        def goals2str(g):
            if g < 0:
                return "x"
            return str(g)

        s_round = self.__get_current_round()
        l_round = self.fixtures.GetLatestRound()
        matches = self.fixtures.matches

        for ir in range(2):
            r = s_round + ir
            self.fixtures_texts[ir][0].SetLabel(_("Round %d") % r + ":")

            rmatches = matches.get(r)
            if rmatches == None:
                self.__reset_texts(self.fixtures_texts[ir][1])
                break

            i = 0
            for m_id, vals in rmatches.items():
                if i > 3:
                    break

                (date, (hid, hname), (aid, aname), played, (hgoals, agoals)) = vals

                self.fixtures_texts[ir][1][i][0].SetLabel(hname)
                self.fixtures_texts[ir][1][i][1].SetLabel(aname)
                if played:
                    self.fixtures_texts[ir][1][i][2].SetLabel(str(hgoals))
                    self.fixtures_texts[ir][1][i][3].SetLabel(str(agoals))
                    self.fixture_buttons[(ir, i, 0)].SetLabel(_("Match details"))
                    for x in range(3):
                        self.fixture_buttons[(ir, i, x)].Enable(True)
                else:
                    self.fixtures_texts[ir][1][i][2].SetLabel("x")
                    self.fixtures_texts[ir][1][i][3].SetLabel("x")
                    if l_round+1 == r:
                        self.fixture_buttons[(ir, i, 0)].Enable(True)
                        self.fixture_buttons[(ir, i, 0)].SetLabel(_("View live"))
                    else:
                        self.fixture_buttons[(ir, i, 0)].SetLabel(_("Match details"))
                        for x in range(3):
                            self.fixture_buttons[(ir, i, x)].Disable()

                i += 1



    def __reset_texts(self, all_mtexts):
        for ttexts in all_mtexts:
            for stext in ttexts:
                stext.SetLabel("")

    def __get_current_league(self):
        current_league = self.league_select.GetValue()
        try:
            current_league = int(current_league)
        except ValueError, e:
            return None
        return current_league

    def __get_current_season(self):
        current_season = self.season_select.GetValue()
        try:
            current_season = int(current_season)
        except ValueError, e:
            return None
        return current_season

    def __get_current_type(self):
        current_type = self.type_choice.GetSelection()
        return current_type

    def __get_current_round(self):
        return self.round_slider.GetValue()
