# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from Config import config
from Configdialog import apply_function_list
import DataInfo
import TeamSelector
import Terms
from Images import Images
import Players

import operator

from Common import *
from Commonwx import *

from wxPython.wx import *

class LineupCtrl(wxPanel):
    basic_elements = [["keeper"],
                      ["rightBack", "insideBack1", "insideBack2", "leftBack"],
                      ["rightWinger", "insideMid1", "insideMid2", "leftWinger"],
                      ["forward1", "forward2"]]

    subst_elements = [["kicker1", "substKeeper", "captain"],
                      ["substBack", "substInsideMid", "substForward", "substWinger"]]

    def __init__(self, parent, ID, use_substs, data_type, data, *extra_data):
        wxPanel.__init__(self, parent, ID, style=wxSUNKEN_BORDER)

        self.SetBackgroundColour("#42A042")
        self.SetForegroundColour("#F0F0F0")

        main_sizer = wxBoxSizer(wxVERTICAL)

        sizer = None
        self.dir_sizer = sizer

        self.current_direction = config.get("General", "lineup direction").lower()


        n_elements = self.basic_elements[:]

        if use_substs:
            n_elements += [[]] + self.subst_elements[:]

        use_beh = True
        self.static_texts = {}
        for elmnts in n_elements:
            if len(elmnts) == 0:
                line = wxStaticLine(self, -1, style = wxHORIZONTAL)
                main_sizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)
                use_beh = False
                continue

            elements = elmnts[:]

            for position in elements:
                description = DataInfo.known_general_data_info["lineup"][position][1] + ":"
                pos_text = ""
                postexts = [(description, False), (pos_text, True)]
                if use_beh:
                    postexts.append(("", False))
                else:
                    postexts.append((None, False))

                savetexts = []
                for text, bold in postexts:
                    if text != None:
                        stext = wxStaticText(self, -1, text,
                                             style = wxALIGN_CENTRE)

                        if bold:
                            SetTextBold(stext)
                    else:
                        stext = None
                    savetexts.append(stext)
                self.static_texts[position] = savetexts

        sizer = self.__create_dir_sizers(self.basic_elements[:], False)
        self.dir_sizer = sizer
        main_sizer.Prepend(sizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 1)

        if use_substs:
            sizer = self.__create_dir_sizers(self.subst_elements[:], True)
            self.subst_sizer = sizer
            main_sizer.Add(sizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 1)

        self.main_sizer = main_sizer

        self.SetSizer(main_sizer)
        self.SetAutoLayout(True)
        main_sizer.Fit(self)

        self.data = self.__convert_data(data_type, data, *extra_data)
        self.Update()

        apply_function_list.append(self.ConfigChanged)

        self.bg_backup = None
        EVT_ERASE_BACKGROUND(self, self.OnBackgroundPaint)

    def __del__(self):
        apply_function_list.remove(self.ConfigChanged)

    def OnBackgroundPaint(self, event):
        t = time.time()

        dc = event.GetDC()
        if dc == None:
            dc = wxClientDC(self)

        dc.BeginDrawing()

        (dw,dh) = dc.GetSizeTuple()

        draw = False
        if self.bg_backup == None or \
               dw != self.bg_backup.GetWidth() or \
               dh != self.bg_backup.GetHeight():
            if self.bg_backup != None:
                self.bg_backup = None
            self.bg_backup = wxEmptyBitmap(dw, dh)
            draw = True

        mdc = wxMemoryDC()
        mdc.SelectObject(self.bg_backup)

        if draw:
            mdc.BeginDrawing()

            bg = Images.get_bitmap_from_file("football_bg.png")
            pw = bg.GetWidth()
            ph = bg.GetHeight()
            bgimage = wxMemoryDC()
            bgimage.SelectObject(bg)

            (dw,dh) = dc.GetSizeTuple()

            for x in range(0,dw,pw):
                for y in range(0,dh,ph):
                    w = min(dw-x, pw)
                    h = min(dh-y, ph)
                    mdc.Blit(x, y, w, h, bgimage, 0, 0)

            mdc.EndDrawing()

        dc.Blit(0, 0, dw, dh, mdc, 0, 0)

        dc.EndDrawing()

    def Update(self):
        self.Freeze()

        if self.data == None:
            for position, (pos_descr, pos_name, beh) in self.static_texts.items():
                pos_name.SetLabel("")
                if beh != None:
                    beh.SetLabel("")
        else:
            for position, (pos_descr, pos_name, beh) in self.static_texts.items():
                pname, order, posval = self.data[position]
                if pname == "0":
                    pname = ""
                pos_descr.SetBackgroundColour("#42A042")
                pos_name.SetLabel(pname)
                pos_name.SetBackgroundColour("#42A042")
                if beh != None:
                    if pname == "":
                        beh.SetLabel("")
                    else:
                        if posval != None:
                            order += "   " + posval
                        beh.SetLabel(order)
                        beh.SetBackgroundColour("#42A042")

        self.Thaw()

        self.GetSizer().Layout()

    def ChangeData(self, data_type, data, *extra_data):
        self.data = self.__convert_data(data_type, data, *extra_data)
        self.Update()

    def ConfigChanged(self, changed_vars):
        if not (changed_vars.has_key("General") and
                "lineup direction" in changed_vars["General"]):
            return

        old_direction = self.current_direction
        self.current_direction = config.get("General", "lineup direction").lower()

        self.main_sizer.RemoveSizer(self.dir_sizer)
        self.hsizers = []
        self.dir_sizer = None

        sizer = self.__create_dir_sizers(self.basic_elements[:], False)
        self.dir_sizer = sizer

        self.main_sizer.PrependSizer(sizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 1)
        self.main_sizer.Layout()

    def __convert_data(self, type, data, *extra_data):
        if data == None:
            return None

        if type == "htdata":
            dt = {}
            for position, (pos_descr, pos_name, beh) in self.static_texts.items():
                pid = getattr(data.get_section("lineup", False), position)
                pname = getattr(data.get_section("lineup", True), position)
                order = None
                if pid:
                    if beh != None:
                        behposition = "beh" + position[0].upper() + position[1:]
                        if DataInfo.known_general_data_info["lineup"].has_key(behposition):
                            order = getattr(data.get_section("lineup", True), behposition)
                        else:
                            order = ""

                        pos = None
                        for d, p, rp in TeamSelector.TeamSelector.lineup_info:
                            if p == position:
                                pos = rp
                                break
                        if pos == "goalie":
                            ep = 0
                        else:
                            ep = getattr(data.get_section("lineup", False), behposition)
                        pos = TeamSelector.TeamSelector.set_real_pos(pos, ep)
                        try:
                            posval = getattr(data.get_player(pid, True), "ts_" + pos)
                        except:
                            pname = _("Sold or fired player!")
                            order = ""
                            posval = ""
                else:
                    pname = ""
                    order = ""
                    posval = ""
                dt[position] = (pname, order, posval)
            return dt

        if type == "teamselect":
            players = Players.Players()
            dt = {}
            for position, (pos_descr, pos_name, beh) in self.static_texts.items():
                pid, val, n, llpos, order_val = data[position]
                if pid is not None:
                    pname = players.all_players[pid].name
                else:
                    pname = _("No player")
                order = None
                if beh != None:
                    if order_val is not None:
                        order = Terms.individual_order[order_val]
                    else:
                        order = ""
                if val is not None:
                    extra_value = "%.2f (%d)" % (val, n)
                else:
                    extra_value = ""
                dt[position] = (pname, order, extra_value)
            return dt

    def __create_dir_sizers(self, n_elements, force_up):
        if not force_up and self.current_direction in ["down", "right"]:
            n_elements.reverse()

        if not force_up and self.current_direction in ["left", "down"]:
            reverse_elements = True
        else:
            reverse_elements = False

        if not force_up and self.current_direction in ["left", "right"]:
            sizer_direction = wxHORIZONTAL
            inner_sizer_direction = wxVERTICAL
        else:
            sizer_direction = wxVERTICAL
            inner_sizer_direction = wxHORIZONTAL

        sizer = wxBoxSizer(sizer_direction)

        for elmnts in n_elements:
            elements = elmnts[:]
            if reverse_elements:
                elements.reverse()

            hsizer = wxBoxSizer(inner_sizer_direction)
            for position in elements:
                isizer = wxBoxSizer(wxHORIZONTAL)
                vsizer = wxBoxSizer(wxVERTICAL)
                for stext in self.static_texts[position]:
                    if stext != None:
                        vsizer.Add(stext, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 1)
                isizer.Add(vsizer, 1, wxALIGN_CENTRE | wxALL, 5)
                hsizer.Add(isizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)
            sizer.Add(hsizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        return sizer
