# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import HRFData
import SelectDateMenu
import DataInfo
import Terms
import FormationDialog
from LineupCtrl import LineupCtrl

from Common import *
from Commonwx import *

from wxPython.wx import *

class Formation(wxPanel):
    ID_FORMATION_BUTTON = 233
    def __init__(self, parent, ID, htdata):
        self.htdata = htdata

        wxPanel.__init__(self, parent, ID)

        sizer=wxBoxSizer(wxHORIZONTAL)

        formationtext=wxStaticText(self,-1,_("Formation") + ":")
        SetTextBold(formationtext)
        sizer.Add(formationtext, 0, wxEXPAND | wxALL, 5)
        self.formation=wxStaticText(self,-1,"")
        sizer.Add(self.formation, 1, wxEXPAND | wxALL, 5)

        teamvaltext=wxStaticText(self,-1,_("Team value") + ":")
        SetTextBold(teamvaltext)
        sizer.Add(teamvaltext, 0, wxEXPAND | wxALL, 5)
        self.teamval=wxStaticText(self,-1,"")
        sizer.Add(self.teamval, 1, wxEXPAND | wxALL, 5)

        experiencetext=wxStaticText(self,-1, _("Experience") + ":")
        SetTextBold(experiencetext)
        sizer.Add(experiencetext, 0, wxEXPAND | wxALL, 5)
        self.experience=wxStaticText(self,-1,"")
        sizer.Add(self.experience, 1, wxEXPAND | wxALL, 5)

        self.formationbutton = wxButton(self, Formation.ID_FORMATION_BUTTON, \
                                        _("Formations"), style = wxBU_EXACTFIT)

        sizer.Add(self.formationbutton, 0, wxEXPAND |wxALL | wxALIGN_CENTRE, 5)


        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        self.Update()

        EVT_BUTTON(self, Formation.ID_FORMATION_BUTTON, self.FormationButton)

    def Update(self):
        if self.htdata == None:
            return

        self.teamval.SetLabel("%.2f" % self.htdata.get_section("team").teamval)

        self.no_centraldefenders=0
        self.no_wingbacks=0
        self.no_innermidfielders=0
        self.no_wingers=0
        self.no_forwards=0

        order_elements=[["no_centraldefenders", ["behInsideBack1", "behInsideBack2"]],
                        ["no_wingbacks", ["behLeftBack", "behRightBack"]],
                        ["no_innermidfielders", ["behInsideMid1", "behInsideMid2"]],
                        ["no_wingers", ["behLeftWinger", "behRightWinger"]],
                        ["no_forwards", ["behForward1", "behForward2"]]]


        for pos in order_elements:
            for pos2 in pos[1]:
                order=getattr(self.htdata.get_section("lineup"),pos2)
                if order <= 4:
                    setattr(self, pos[0], getattr(self,pos[0])+1)
                elif order == 5:
                    self.no_forwards=self.no_forwards+1
                elif order == 6:
                    self.no_innermidfielders=self.no_innermidfielders+1
                elif order == 7:
                    self.no_centraldefenders=self.no_centraldefenders+1

        total_players=0
        if self.htdata.get_section("lineup").keeper > 0:
            total_players=1
        for pos in order_elements:
            total_players += getattr(self,pos[0])

        formation = str(self.no_centraldefenders+self.no_wingbacks) + "-" + str(self.no_innermidfielders+self.no_wingers) + "-" + str(self.no_forwards)
        experform = "exper" + str(self.no_centraldefenders+self.no_wingbacks) + str(self.no_innermidfielders+self.no_wingers) + str(self.no_forwards)

        try:
            experience = getattr(self.htdata.get_section("team",True),experform)
        except:
            experience = Terms.formation_experience[1]

        if formation == "4-4-2":
            experience = Terms.formation_experience[8]

        self.formation.SetLabel(formation)
        if total_players < 9 or self.no_innermidfielders > 3 or self.no_forwards > 3 or self.no_centraldefenders > 3:
            SetTextColour(self.formation,-1)
        else:
            SetTextColour(self.formation,0)

        self.experience.SetLabel(experience)
        self.GetSizer().RecalcSizes()
        self.GetSizer().Layout()

    def ChangeHRF(self, new_htdata):
        self.htdata = new_htdata
        self.Update()

    def FormationButton(self, event):
        dlg = FormationDialog.FormationDialog(self)
        ret = dlg.ShowModal()

class Spirit(wxPanel):
    def __init__(self, parent, ID, htdata):
        self.htdata = htdata

        wxPanel.__init__(self, parent, ID)

        sizer=wxBoxSizer(wxHORIZONTAL)

        texts = [(_("Team spirit"), "team", "stamningValue"),
                 (_("Attitude"), "lineup", "installning"),
                 (_("Confidence"), "team", "sjalvfortroendeValue")]
        self.texts = []
        for text, section, attribute in texts:
            hsizer = wxBoxSizer(wxHORIZONTAL)

            stext = wxStaticText(self, -1, text + ":")
            SetTextBold(stext)
            hsizer.Add(stext, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

            vtext = wxStaticText(self, -1, "")
            hsizer.Add(vtext, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

            self.texts.append((vtext, section, attribute))

            sizer.Add(hsizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        self.Update()

    def Update(self):
        if self.htdata == None:
            return

        for vtext, section, attribute in self.texts:
            value = getattr(self.htdata.get_section(section, True), attribute)
            vtext.SetLabel(value)

        self.GetSizer().RecalcSizes()
        self.GetSizer().Layout()

    def ChangeHRF(self, new_htdata):
        self.htdata = new_htdata
        self.Update()

class LineupPage(wxPanel):
    ID_POPUP_MENU    = 231
    ID_DATE_SELECTED = 232

    def __init__(self, parent, ID, filename):
        if filename == None:
            self.htdata = (None, None)
        else:
            self.htdata = (filename, HRFData.hrfstorage[filename])

        wxPanel.__init__(self, parent, ID)

        sizer = wxBoxSizer(wxVERTICAL)
        hsizer = wxBoxSizer(wxHORIZONTAL)

        self.htbutton = wxButton(self, LineupPage.ID_POPUP_MENU, \
                                 _("Change date"), style = wxBU_EXACTFIT)
        hsizer.Add(self.htbutton, 0, wxALIGN_CENTRE | wxALL, 5)

        self.current_hrf_info = wxStaticText(self, -1, self.__get_current_text())
        hsizer.Add(self.current_hrf_info, 1, wxEXPAND | wxALL, 5)

        sizer.Add(hsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        self.formation = Formation(self, -1, self.htdata[1])
        sizer.Add(self.formation, 0, wxEXPAND | wxALIGN_LEFT | wxALL, 0)

        self.spirit = Spirit(self, -1, self.htdata[1])
        sizer.Add(self.spirit, 0, wxEXPAND | wxALIGN_LEFT | wxALL, 0)

        self.lineup = LineupCtrl(self, -1, True, "htdata", self.htdata[1])
        sizer.Add(self.lineup, 1, wxEXPAND | wxALIGN_LEFT | wxALIGN_BOTTOM | wxALIGN_TOP, 5)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        EVT_BUTTON(self, LineupPage.ID_POPUP_MENU, self.OnPopupMenu)

    def ChangeHRF(self, hrffile):
        self.htdata = (hrffile, HRFData.hrfstorage[hrffile])
        self.current_hrf_info.SetLabel(self.__get_current_text())
        self.lineup.ChangeHRF("htdata", self.htdata[1])
        self.formation.ChangeHRF(self.htdata[1])
        self.spirit.ChangeHRF(self.htdata[1])

    def OnPopupMenu(self, event):
        sdmenu = SelectDateMenu.SelectDateMenu(self, LineupPage.ID_DATE_SELECTED, self.htdata[0])
        SelectDateMenu.EVT_DATE_SELECTED(sdmenu, LineupPage.ID_DATE_SELECTED, \
                                         self.OnDateSelected)
        rect = self.htbutton.GetRect()
        self.PopupMenu(sdmenu, (rect[0], rect[1] + rect[3]))

    def OnDateSelected(self, event):
        self.ChangeHRF(event.GetHRFFile())

    def __get_current_text(self):
        return htdata_text(self.htdata)
