# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import HRFData
import sys
import string

from Common import *
from DataInfo import *

filters = {
    "arena": [["arena.antalSitt", "arena.antalStaplats", "arena.antalTak",
              "arena.antalVIP", "arena.arenaname", "arena.expandingSitt",
              "arena.expandingStaplats", "arena.expandingTak",
              "arena.expandingVIP", "arena.isExpanding",
              "arena.seatTotal"],_("Arena")],

    "basics": [["basics.countryID", "basics.leagueID", "basics.matchround",
               "basics.owner", "basics.ownerEmail", "basics.ownerHomepage",
               "basics.ownerICQ", "basics.season", "basics.teamID",
               "basics.teamName"],_("Basics")],

    "club": [["club.ekonom", "club.fanclub", "club.hjTranare",
             "club.juniorverksamhet", "club.lakare", "club.massor",
             "club.mvTranare", "club.presstalesman", "club.psykolog",
             "club.undefeated", "club.victories"],_("Club")],

    "economy": [["economy.IncomeSponsorer", "economy.cash",
                "economy.costsArena", "economy.costsJuniorverksamhet",
                "economy.costsPersonal", "economy.costsRantor",
                "economy.costsSpelare", "economy.costsSumma",
                "economy.costsTillfalliga", "economy.incomeFinansiella",
                "economy.incomePublik", "economy.incomeSumma",
                "economy.incomeTillfalliga", "economy.lastCostsArena",
                "economy.lastCostsJuniorverksamhet",
                "economy.lastCostsPersonal", "economy.lastCostsRantor",
                "economy.lastCostsSpelare", "economy.lastCostsSumma",
                "economy.lastCostsTillfalliga",
                "economy.lastIncomeFinansiella",
                "economy.lastIncomePublik", "economy.lastIncomeSponsorer",
                "economy.lastIncomeSumma", "economy.lastIncomeTillfalliga",
                "economy.lastTotal", "economy.sponsors",
                "economy.supporters", "economy.total"],_("Economy")],

    "league": [["league.gjorda", "league.inslappta", "league.placering",
               "league.poang", "league.serie", "league.spelade"],_("League")],

    "lineup": [["lineup.behForward1", "lineup.behForward2",
               "lineup.behInsideBack1", "lineup.behInsideBack2",
               "lineup.behInsideMid1", "lineup.behInsideMid2",
               "lineup.behLeftBack", "lineup.behLeftWinger",
               "lineup.behRightBack", "lineup.behRightWinger",
               "lineup.captain", "lineup.forward1", "lineup.forward2",
               "lineup.insideBack1", "lineup.insideBack2",
               "lineup.insideMid1", "lineup.insideMid2",
               "lineup.installning", "lineup.keeper", "lineup.kicker1",
               "lineup.leftBack", "lineup.leftWinger", "lineup.rightBack",
               "lineup.rightWinger", "lineup.substBack",
               "lineup.substForward", "lineup.substInsideMid",
               "lineup.substKeeper", "lineup.substWinger",
               "lineup.trainer"],_("Lineup")],

    "team": [["team.exper343", "team.exper352", "team.exper433", "team.exper451",
             "team.exper532", "team.exper541", "team.sjalvfortroendeValue",
             "team.stamningValue", "team.trLevel", "team.trTypeValue"],_("Team")],

    "player": [["player.CountryID", "player.ald", "player.gev",
               "player.gtc", "player.gtl", "player.gttot", "player.gtt",
               "player.gtall", "player.hat", "player.name",
               "player.playerID", "player.surname","player.ska",
               "player.warnings"],_("Player")],

    "player_attr": [["player.Aggressiveness", "player.TrainerSkill",
                    "player.TrainerType", "player.bac", "player.fas",
                    "player.for", "player.fra", "player.gentleness",
                    "player.honesty", "player.led", "player.mal",
                    "player.mlv", "player.rut",
                    "player.spe", "player.speciality",
                    "player.uth", "player.ytt"],_("Player attributes")],

    "player_eval": [["player.ts_goalie", "player.ts_centraldefender",
                    "player.ts_centraldefender_offensive",
                    "player.ts_centraldefender_towards_the_wing",
                    "player.ts_wingback", "player.ts_wingback_offensive",
                    "player.ts_wingback_defensive", "player.ts_wingback_towards_the_middle",
                    "player.ts_innermid", "player.ts_innermid_offensive",
                    "player.ts_innermid_defensive", "player.ts_innermid_towards_the_wing",
                    "player.ts_winger", "player.ts_winger_offensive",
                    "player.ts_winger_defensive", "player.ts_winger_towards_the_middle",
                    "player.ts_forward", "player.ts_forward_defensive",
                    "player.ts_best_position", "team.teamval",],_("Player evaluation")],

    "player_value" : [["player.mkt", "player.sal"],_("Player value")],

    "player_rating" : [["player.rating"],_("Player rating")]
    }

class Listchanges:
    def __init__(self, h1, h2):
        self.h1 = h1
        self.h2 = h2

    def get_player_changes(self):
        res = []

        pids = self.h1.get_pids() + self.h2.get_pids()
        pids_done = []

        for pid in pids:
            if not pid in pids_done:
                pids_done.append(pid)

                has1 = self.h1.has_player(pid)
                has2 = self.h2.has_player(pid)
                if has1 and has2:
                    name = self.h1.get_player(pid).name
                    data = HRFData.compare(self.h1.get_player(pid), \
                                           self.h2.get_player(pid), True)
                elif has1 and not has2:
                    name = self.h1.get_player(pid).name
                    data = _("sold or kicked")
                elif has2 and not has1:
                    name = self.h2.get_player(pid).name
                    data = _("bought or junior")
                else:
                    name = "?"
                    data = _("How on Earth did we get here?")

                res.append((name, pid, data))

        return res

    def get_section_changes(self):
        res = []
        sections = self.h1.get_section_names() + self.h2.get_section_names()
        sections_done = []

        for section in sections:
            if not section in sections_done:
                sections_done.append(section)

                has1 = self.h1.has_section(section)
                has2 = self.h2.has_section(section)
                if has1 and has2:
                    data = HRFData.compare(self.h1.get_section(section), \
                                           self.h2.get_section(section), True)
                elif has1 and not has2:
                    data = _("section has disappeared")
                elif has2 and not has1:
                    data = _("new section")
                else:
                    data = _("How on Earth did we get here?")

                res.append((section, None, data))
        return res

    def get_changes_text(self, filter_list = []):
        res = self.get_player_changes()
        res.sort()
        res += self.get_section_changes()
        return self.__make_changes_text(res, filter_list)

    def __make_changes_text(self, res, filter_list = []):
        def max_length(x,y):
            if len(str(x)) < len(str(y)):
                return y
            else:
                return x

        attr_filter = []
        for f in filter_list:
            attr_filter += filters[f][0]
        attr_filter = unique(attr_filter)

        long_name, long_id =  reduce(lambda x,y: (max_length(x[0],y[0]), \
                                                  max_length(x[1],y[1])), res)
        long_name = len(str(long_name))
        long_id = len(str(long_id))

        res_text = ""

        for r in res:
            if len(r[2]) > 0:
                add = ""
                strextra = ""
                if r[1] != None:
                    sectname = "player"
                    add = " (" + str(r[1]) + ")"
                else:
                    sectname = r[0]
                if type(r[2]) == str:
                    strextra += " " + r[2]

                if len(strextra) == 0:
                    dv = r[2].__dict__.items()
                    if len(attr_filter) > 0:
                        dv = filter(lambda av: (sectname + "." + av[0]) in attr_filter, dv)
                    if len(dv) == 0:
                        continue

                res_text += r[0] + add + ":" + strextra + "\n"

                if len(strextra) == 0:
                    if r[1] == None:
                        dv = [(known_general_data_info[r[0]][attr][1], value, attr) \
                              for attr,value in dv]
                    else:
                        dv = [(known_player_data_info[attr][1], value, attr) \
                              for attr,value in dv]
                    dv.sort()
                    for description, value, attribute in dv:
                        if description == None:
                            description = attribute
                        if r[1] == None:
                            h1val = getattr(self.h1.get_section(r[0], True), attribute)
                            h2val = getattr(self.h2.get_section(r[0], True), attribute)
                        else:
                            h1val = getattr(self.h1.get_player(r[1], True), attribute)
                            h2val = getattr(self.h2.get_player(r[1], True), attribute)

                        strvalue = str(h1val) + " -> " + str(h2val)
                        if value == "differ":
                            res_text += "\t" + description + ": " + strvalue + "\n"
                        else:
                            res_text += "\t" + description + ": " + signedstr(value) + " (" + strvalue + ")\n"
        return res_text
