# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from Common import *
import XMLParse
from CommonData import get_xml_list_indices

import operator
import string
import sys
import time
from htmlentitydefs import entitydefs

class Live:
    def __init__(self):
        self.last_fetched = 0.0
        self.last_fetched_local = 0.0
        self.matches = {}

    def add_match(self, match_id):
        """Add a match to view."""
        data = XMLParse.get(["HT live", "addMatch", match_id], progress_dialog = False)
        return self.__parse_data(data)

    def delete_match(self, match_id):
        """Remove a match and reset it's match info."""
        data = XMLParse.get(["HT live", "deleteMatch", match_id], progress_dialog = False)
        if self.matches.has_key(match_id):
            del self.matches[match_id]
        return self.__parse_data(data)

    def get_new_events(self):
        """Get new events since last check."""
        data = XMLParse.get(["HT live", "viewNew"], progress_dialog = False)
        return self.__parse_data(data)

    def get_all_events(self):
        """Get all the events from viewed matches and reset the match info."""
        data = XMLParse.get(["HT live", "viewAll"], progress_dialog = False)
        self.matches = {}
        return self.__parse_data(data)

    def clear_all_matches(self):
        """Remove all matches and reset the match info."""
        data = XMLParse.get(["HT live", "clearAll"], progress_dialog = False)
        self.matches = {}
        return self.__parse_data(data)

    def __parse_data(self, data):
        self.last_fetched = data.get("FetchedDate", 0.0)
        self.last_fetched_local = time.time()

        match_indices = get_xml_list_indices("MatchList.Match", data)

        for mindex in match_indices:
            mid = data.get("MatchList.Match%s.MatchID" % mindex)
            date = data.get("MatchList.Match%s.MatchDate" % mindex)
            hometeam = (data.get("MatchList.Match%s.HomeTeam.HomeTeamID" % mindex),
                        data.get("MatchList.Match%s.HomeTeam.HomeTeamName" % mindex, "?"))
            awayteam = (data.get("MatchList.Match%s.AwayTeam.AwayTeamID" % mindex),
                        data.get("MatchList.Match%s.AwayTeam.AwayTeamName" % mindex, "?"))

            cs1 = data.get("MatchList.Match%s.HomeGoals" % mindex)
            cs2 = data.get("MatchList.Match%s.AwayGoals" % mindex)

            if cs1 is None or cs2 is None:
                played = False
            else:
                played = True

            current_score = (played, cs1, cs2)

            event_indices = get_xml_list_indices("MatchList.Match%s.EventList.Event" % mindex, data)

            if self.matches.has_key(mid):
                events = self.matches[mid][4]
            else:
                events = {}

            for eindex in event_indices:
                base_path = "MatchList.Match%s.EventList.Event%s" % (mindex, eindex)
                eid = data.get(base_path + ".xml_attrs.Index")
                minute = data.get(base_path + ".Minute")
                event_key = data.get(base_path + ".EventKey")
                event_text = data.get(base_path + ".EventText")
                subject_tid = data.get(base_path + ".SubjectTeamID")
                subject_pid = data.get(base_path + ".SubjectPlayerID")
                obj_pid = data.get(base_path + ".ObjectPlayerID")

                events[eid] = (minute, event_key, event_text,
                               subject_tid, subject_pid, obj_pid)

            finished = False
            for event in events.values():
                if event[1][0] == 599:
                    finished = True
                    break

            self.matches[mid] = (date, hometeam, awayteam, current_score, events, finished)

class TextLive:
    UPDATE_FREQUENCY = 120.0

    def __init__(self, *match_ids):
        self.match_ids = match_ids
        self.live = Live()

    def run(self):
        for mid in self.match_ids:
            self.live.add_match(mid)

        last_update = time.time()

        self.ended_matches = {}
        self.current_events = {}
        for mid in self.live.matches.keys():
            self.current_events[mid] = 0
            self.ended_matches[mid] = False

        (next_event_minute, next_event_eid, next_event_mid) = self.__get_next_event()
        while True:
            if next_event_mid is not None and next_event_eid is not None:
                self.__write_event(next_event_mid, next_event_eid)

                event_key = self.live.matches[next_event_mid][4][next_event_eid][1]
                if event_key[0] == 599:
                    self.ended_matches[next_event_mid] = True

                time.sleep(0.1)
            else:
                if reduce(operator.and_, self.ended_matches.values()):
                    break

                tslu = time.time() - last_update
                if tslu < self.UPDATE_FREQUENCY:
                    time.sleep(self.UPDATE_FREQUENCY - tslu)

                self.live.get_new_events()
                last_update = time.time()

            (next_event_minute, next_event_eid, next_event_mid) = self.__get_next_event()

    def __get_next_event(self):
        next_event_minute = 1000
        next_event_eid = None
        next_event_mid = None

        for mid in self.live.matches.keys():
            ce = self.current_events[mid]

            eids = self.live.matches[mid][4].keys()
            eids = filter((lambda x: x > ce), eids)

            if len(eids) > 0:
                eid = eids[0]

                minute = self.live.matches[mid][4][eid][0]
                if minute < next_event_minute:
                    next_event_minute = minute
                    next_event_eid = eid
                    next_event_mid = mid

        if next_event_mid is not None and next_event_eid is not None:
            self.current_events[next_event_mid] = next_event_eid
        return (next_event_minute, next_event_eid, next_event_mid)

    def __write_event(self, mid, eid):
        (minute, event_key, event_text, subject_tid, subject_pid,
         obj_pid) = self.live.matches[mid][4][eid]

        event_text = unhtmlify(event_text)
        event_text = break_up_row(event_text, 55)
        lines = string.split(event_text, "\n")
        lines[1:] = [" "*15 + l for l in lines[1:]]
        event_text = string.join(lines, "\n")

        sys.stdout.write("%s: %s %s\n" % (string.rjust(str(mid), 10),
                                          string.rjust(str(minute), 2),
                                          event_text))

def unhtmlify(str):
    pos = 0
    nstr = ""

    while True:
        lpos = str.find('<', pos)
        opos = str.find('&', pos)

        if lpos >= 0 and (lpos < opos or opos < 0):
            nstr += str[pos:lpos]
            lpose = str.find('>', lpos + 1)

            if lpose < 0:
                break
            pos = lpose + 1
        elif opos >= 0 and (opos <= lpos or lpos < 0):
            nstr += str[pos:opos]
            opose = str.find(';', opos + 1)

            if opose < 0:
                nstr += '&'
                pos = opos + 1
                continue

            e = str[opos + 1:opose]
            if entitydefs.has_key(e):
                nstr += entitydefs[e]
                pos = opose + 1
            else:
                nstr += '&'
                pos = opos + 1
        else:
            nstr += str[pos:]
            break

    return string.join(nstr.strip().split(), " ")

def break_up_row(rstr, row_length = 66):
    """Break up a row into suitable row lengths."""
    words = rstr.split()
    lines = []
    nstr = ""

    for word in words:
        if len(nstr) + len(word) + 1 > row_length:
            lines.append(nstr)
            nstr = word
        else:
            if len(nstr) == 0:
                nstr = word
            else:
                nstr += " " + word

    if len(nstr) > 0:
        lines.append(nstr)

    return string.join(lines, "\n")
