# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""The main frames and main loops of the graphical interface
of Mattricks."""

from Common import *
from Commonwx import *

import os
import os.path
import time
import sys

from Config import config
import Errorwx
import CommonData
import HRFData
import AboutPage
import Playerlist
import ComparisonPage
import Fetchdialog
import Fetch
import EconomyPage
import MatchListPage
import LeaguePage
import MatchPrediction
import MatchViewerPage
import SummaryPage
import JuniorSpy
import TeamSelectorPage
import Configdialog
import DialogSet
import Database
import XMLParse
import VersionChecker
import ArenaPage
import Players

from Images import Images

from wxPython.wx import *

mattricks_app = None
notebook_pages = {}

class StatusBar(wxPanel):
    def __init__(self, parent, ID):
        wxPanel.__init__(self, parent, ID)

        main_sizer = wxBoxSizer(wxHORIZONTAL)

        panel = wxPanel(self, -1, style = wxSUNKEN_BORDER)
        psizer = wxBoxSizer(wxVERTICAL)

        self.__idling = True
        self.__idle_message = ""

        self.__statustext = wxStaticText(panel, -1, self.__idle_message)
        psizer.Add(self.__statustext, 0, wxEXPAND | wxALIGN_LEFT | wxLEFT | wxRIGHT, 5)

        panel.SetSizer(psizer)
        panel.SetAutoLayout(True)
        psizer.Fit(panel)

        main_sizer.Add(panel, 3, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        self.__progressbar = wxGauge(self, -1, 10, style = wxGA_HORIZONTAL)
        self.__progressbar.SetValue(0)

        main_sizer.Add(self.__progressbar, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        self.SetSizer(main_sizer)
        self.SetAutoLayout(True)
        main_sizer.Fit(self)

        self.__extra_message_list = []
        self.__basevalue = 0

    def Reset(self):
        self.__statustext.SetLabel(self.__idle_message)
        self.__progressbar.SetValue(0)
        self.__progressbar.SetRange(10)
        self.__basevalue = 0
        self.__extra_message_list = []

    def SetRange(self, range):
        self.__progressbar.SetRange(range)

    def SetBaseValue(self, baseval):
        self.__basevalue = baseval

    def PushExtraMessage(self, message):
        self.__extra_message_list.append(message)

    def PopExtraMessage(self):
        return self.__extra_message_list.pop()

    def Update(self, value, message):
        text = string.join(self.__extra_message_list + [message], " ")
        self.__statustext.SetLabel(text)
        if value >= 0:
            self.__progressbar.SetValue(min(self.__basevalue + value,
                                            self.__progressbar.GetRange()))

        wxPanel.Update(self)
        wxGetApp().Yield(True)

    def SetIdleMessage(self, message):
        self.__idle_message = message
        if self.__idling:
            self.__statustext.SetLabel(self.__idle_message)

class ProgressBarWrapper:
    def __init__(self, statusbar):
        self.__start_level = 0
        self.__block = False
        self.__statusbar = statusbar
        self.__top_frame = wxGetTopLevelParent(self.__statusbar)

    def progress_start(self, title):
        if not self.__block and self.__start_level == 0:
            self.__windisabler = wxWindowDisabler(self.__top_frame)
            self.__top_frame.SpecialEnable(False)
        self.__start_level += 1
        self.__statusbar.PushExtraMessage(title)
        return self.__statusbar

    def progress_end(self):
        if self.__start_level > 0:
            self.__start_level -= 1
            self.__statusbar.PopExtraMessage()
            if not self.__block and self.__start_level == 0:
                self.__statusbar.Reset()
                self.__top_frame.SpecialEnable(True)
                del self.__windisabler
            else:
                self.__statusbar.Update(-1, "")

class ProgressMakerFuncWrapper:
    def __init__(self, pbw):
        self.__pbw = pbw

    def __call__(self, title):
        return self.__pbw.progress_start(title)

class ProgressDestroyFuncWrapper:
    def __init__(self, pbw):
        self.__pbw = pbw

    def __call__(self, dlg):
        return self.__pbw.progress_end()

class MainFrame(wxFrame):
    """The main windows of Mattricks."""

    ID_FETCH = 101
    ID_EXIT  = 103
    ID_CONFIG = 104
    ID_OPEN = 105

    def __init__(self, parent, ID, title, filename):
        """Initialize the main window.

         - parent -- the parent of the frame.
         - ID -- the id of the frame.
         - title -- the title of the window.
         - filename -- a string containing a filename."""

        global notebook_pages

        wxFrame.__init__(self, parent, ID, title)

        if filename != None:
            config.set("Fetch", "save directory", os.path.dirname(filename))

        sizer = wxBoxSizer(wxVERTICAL)

        hsizer = wxBoxSizer(wxHORIZONTAL)

        buttons = [(MainFrame.ID_FETCH, _("Download data"), self.OnFetch),
                   (MainFrame.ID_CONFIG, _("Preferences"), self.OnConfig),
                   #(MainFrame.ID_OPEN, _("Open data file"), self.OnOpen ),
                   (MainFrame.ID_EXIT, _("Exit"), self.OnExit)]
        self.__buttons = []
        for id, info, func in buttons:
            button = wxButton(self, id, info)
            hsizer.Add(button, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)
            self.__buttons.append(button)
            EVT_BUTTON(self, id, func)

        sizer.Add(hsizer, 0, wxEXPAND | wxBOTTOM, 0)

        notebook = wxNotebook(self, -1)
        nbsizer = wxNotebookSizer(notebook)

        the_pages = [
            ("about", "Mattricks", AboutPage.AboutPage(notebook, -1)),
            ("summary", _("Summary"), SummaryPage.SummaryPage(notebook, -1)),
            ("player", _("Player lists"), Playerlist.PlayerPage(notebook, -1)),
            ("comparison", _("Comparisons"), ComparisonPage.ComparisonPage(notebook, -1)),
            ("economy", _("Economy"), EconomyPage.EconomyPage(notebook, -1)),
            ("matchlist", _("Match list"), MatchListPage.MatchListPage(notebook, -1)),
            ("table", _("League table"), LeaguePage.LeaguePage(notebook, -1)),
            ("lineupselect", _("Lineup selection"),
             TeamSelectorPage.TeamSelectorPage(notebook, -1)),
            ("prediction", _("Game prediction"), MatchPrediction.MatchPrediction(notebook, -1)),
            ("matchview", _("Match viewer"), MatchViewerPage.MatchViewerPage(notebook, -1))
            ]
        self.pages = {}
        notebook_pages = self.pages
        for name, descr, page in the_pages:
            notebook.AddPage(page, descr)
            self.pages[name] = page

        sizer.Add(nbsizer, 1, wxEXPAND | wxALL, 0)
        self.__notebook = notebook

        self.__statusbar = StatusBar(self, -1)
        self.__pbw = ProgressBarWrapper(self.__statusbar)
        sizer.Add(self.__statusbar, 0, wxEXPAND | wxALL, 0)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        EVT_CLOSE(self, self.OnSystemExit)

        icons = wxIconBundle()
        for iconsize in [16, 32, 48, 64]:
            iconbmp = Images.get_bitmap_from_file("icon-%dx%d.png" % (iconsize, iconsize))
            icon = wxEmptyIcon()
            icon.CopyFromBitmap(iconbmp)
            icons.AddIcon(icon)
        self.SetIcons(icons)

    def AfterInit(self):
        """Initialization that needs to be done after __init__() but
        before everything else.

        This function also calls all pages in the notebook to after-
        initialize them."""
        if config.get("Fetch", "download at startup"):
            self.__do_fetch()

        # Update order (matchlist must be first at the moment)
        pagelist=["matchlist", "summary", "player",
                  "comparison", "economy", 
                  "lineupselect", "table", "matchview"]
        for name in pagelist:
            if getattr(self.pages[name], "AfterInit", None) is not None:
                self.pages[name].AfterInit()

        return True

    def SpecialEnable(self, enable = True):
        for b in self.__buttons:
            b.Enable(enable)
        self.__notebook.Enable(enable)

    #    def OnOpen(self, event):
    #        """Called when the user wants to open a hrf on harddisk."""
    #        wildcard = "HRF Files (*.hrf)|*.hrf|" \
    #                   "All files (*.*)|*.*"
    #
    #        default_dir = config.get("Fetch", "save directory")
    #
    #        dlg = wxFileDialog(self, _("Choose file with hattrick data."), wildcard = wildcard,
    #                           style = wxOPEN, defaultDir = default_dir)
    #        if dlg.ShowModal() == wxID_OK:
    #            self.__change_hrf(dlg.GetPath())
    #        dlg.Destroy()

    def OnFetch(self, event):
        """Called when the user wants to download a hrf."""
        self.__do_fetch()

    def OnExit(self, event):
        """Called when the user wants to exit the program."""
        if config.get("General", "warning on exit") == True:
            closeDialog = wxMessageDialog(self, _("Exit Mattricks?"), _("Exit Mattricks?"), wxYES_NO)
            ans = closeDialog.ShowModal()
            if ans == wxID_NO:
                return

        self.Close(True)

    def OnSystemExit(self, event):
        """Called when the user wants to exit the program."""
        if event.CanVeto():
            if config.get("General", "warning on exit") == True:
                closeDialog = wxMessageDialog(self, _("Exit Mattricks?"), _("Exit Mattricks?"), wxYES_NO)
                ans = closeDialog.ShowModal()
                if ans == wxID_NO:
                    event.Veto()
                    return
        self.Destroy()

    def GetTabHandle(self, tabname):
        """Get a page from the notebook by it's name."""
        return self.pages[tabname]

    def GetStatusBar(self):
        """Get the statusbar."""
        return self.__statusbar

    def GetPBW(self):
        """Get the progress bar wrapper."""
        return self.__pbw

    def OnConfig(self, event):
        """Called when the user wants to configure Mattricks."""
        if not Configdialog.ConfigDialog.conf_dialog_running:
            dlg = Configdialog.ConfigDialog(self, -1)
            dlg.Show()

    def __new_xmls(self):
        """Notify all pages that we have new xmls."""
        for name, page in self.pages.items():
            if getattr(page, "ChangeXML", None) is not None:
                page.ChangeXML()

    def __do_fetch(self):
        """Fetching common data."""

        if config.get("Fetch", "new version"):
            ret = VersionChecker.check_version(self)
            if ret == wxID_CANCEL:
                return

        loginname = Database.databases.get(None, True, "Mattricks.loginname")
        password = Database.databases.get(None, True, "Mattricks.password")


	if config.get("Fetch","junior spy"):
            p_old = Players.Players()	
            old_pids = p_old.all_players.keys()

        if None in [loginname, password]:
            res = Fetchdialog.graphical_password_dialog(self)

            if res is None:
                return

            loginname, password = res

        if CommonData.download_common_data():
            self.__new_xmls()

        if config.get("Fetch","junior spy"):
            p_new = Players.Players()
            new_pids = p_new.all_players.keys()
            new_players = []
            
            if len(old_pids) > 0:
                for pid in new_pids:
                    if pid not in old_pids:
                        new_players.append(pid)

                for pid in new_players:
                    tdata = XMLParse.get(["transfer history", "player", pid])
                    tcount = tdata.get("Transfers.xml_attrs.Count",1)
                    if not tcount:
                        player = p_new.all_players.get(pid)
                        if player is not None:
                            dlg = JuniorSpy.JuniorSpy(self,player)
                            dlg.ShowModal()
                            dlg.Destroy()

class Main(wxApp):
    """The main application of the program."""

    ID_FRAME = 201
    def __init__(self, filename = None):
        """First initialization."""

        self.initial_file = filename
        wxApp.__init__(self)

    def OnInit(self):
        """Create windows and change to graphical dialogs."""
        global progressbar_wrapper

        self.SetAppName("Mattricks")
        frame = MainFrame(NULL, Main.ID_FRAME, "Mattricks", self.initial_file)
        self.SetTopWindow(frame)
        frame.Centre()
        frame.Show(True)

        Errorwx.error.init_errorwx(frame)

        DialogSet.progress_dialog_maker_func = ProgressMakerFuncWrapper(frame.GetPBW())
        DialogSet.destroy_dialog_maker_func = ProgressDestroyFuncWrapper(frame.GetPBW())
        DialogSet.always_progress_dialog = True

        DialogSet.password_getting_dialog = (lambda p=frame: Fetchdialog.graphical_password_dialog(p))
        DialogSet.error_message_dialog = (lambda msg, p=frame: Fetchdialog.graphical_error_msg_dialog(p, msg))

        del self.initial_file

        frame.AfterInit()

        return true

def main(*hrfs):
    """The entry point for the graphical version of Mattricks.

     - *hrfs -- an amount of strings of filenames pointing to hrfs."""
    global mattricks_app

    filename = None
    if len(hrfs) == 0:
        hrfs = config.get("General", "hrf directories")
    for hrf in hrfs:
        hrf = os.path.realpath(os.path.expanduser(hrf))
        hrfdir = os.path.dirname(hrf)
        if os.path.isdir(hrf):
            HRFData.htdirs.add_dirs(hrf)
        else:
            HRFData.htdirs.add_from_filename(hrf)
            filename = hrf

    if filename == None:
        filename = HRFData.htdirs.get_from_time(time.time())

    app = Main(filename)
    app.MainLoop()
