# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import string
import Players

from Common import *
import Terms
import CommonData
import XMLParse
from Database import databases

def get_match_list(team_id = None):
    if team_id is None:
        team_id = CommonData.get_current_teamid()

    mlist = databases.get(None, False, "Mattricks.MatchList+%d" % team_id)
    if mlist is None or mlist == "":
        mlist = []
    else:
        mlist = [int(x) for x in mlist.split(",")]
    return mlist

def set_match_list(match_list = [], team_id = None):
    if team_id is None:
        team_id = CommonData.get_current_teamid()

    mlstr = string.join([str(x) for x in match_list], ",")
    databases.sets(None, False, {("Mattricks.MatchList+%d" % team_id): mlstr})

def get_goals(match_id, team_id = None):
    if team_id is None:
        team_id = CommonData.get_current_teamid()
    
    if match_id > 0:
        print "This should only be used for negative match-ID:s"
        
    home_goals = databases.get(None, False, "Mattricks.OldData+%d.Match%d.HomeTeam" % (team_id, match_id))
    away_goals = databases.get(None, False, "Mattricks.OldData+%d.Match%d.AwayTeam" % (team_id, match_id))
    match_type = databases.get(None, False, "Mattricks.OldData+%d.Match%d.MatchType" % (team_id, match_id))
    return ((int(home_goals), int(away_goals)), int(match_type))

def set_goals(match_id, score, match_type, team_id = None):
    if team_id is None:
        team_id = CommonData.get_current_teamid()

    home_goals = str(score[0])
    away_goals = str(score[1])
    
    if match_id > 0:
        print "This should only be used for negative match-ID:s"
        return
    
    databases.sets(None, False, {("Mattricks.OldData+%d.Match%d.HomeTeam" % (team_id, match_id)): home_goals})
    databases.sets(None, False, {("Mattricks.OldData+%d.Match%d.AwayTeam" % (team_id, match_id)): away_goals})
    databases.sets(None, False, {("Mattricks.OldData+%d.Match%d.MatchType" % (team_id, match_id)): str(match_type)})

class CurrentMatches:
    def __init__(self, team_id = None, force_update = False, progress_dialog = True):
        if team_id is None:
            team_id = CommonData.get_current_teamid()

        self.__extract_data(team_id, force_update, progress_dialog)
        self.__update_match_list()

    def __extract_data(self, team_id, f_upd, p_dlg):

        data = XMLParse.get(["match list", team_id], force_update=f_upd, progress_dialog = p_dlg)

        self.team_id = data.get("Team.TeamID", None)
        self.team_name = data.get("Team.TeamName", "")

        self.match = []
        self.match_d = {}

        if self.team_id is None:
            return

        keys = ["MatchID", "HomeTeam.HomeTeamID", "HomeTeam.HomeTeamName",
                "AwayTeam.AwayTeamID", "AwayTeam.AwayTeamName",
                "MatchDate", "MatchType", "HomeGoals", "AwayGoals",
                "Status", "OrdersGiven"]

        match_indices = CommonData.get_xml_list_indices("Team.MatchList.Match", data)
        for mindex in match_indices:
            mid = data.get("Team.MatchList.Match%s.MatchID" % mindex)
            mdata = {}
            for key in keys:
                mdata[key] = data.get("Team.MatchList.Match%s.%s" % (mindex, key), None)
            mdata = CommonData.DataStore(mdata, True)
            self.match.append(mdata)
            self.match_d[mid] = mdata

        CommonData.DataStore.set_comparison_key("MatchDate")
        self.match.sort()

    def __update_match_list(self):
        if self.team_id is None:
            return

        mlist = get_match_list(self.team_id)

        for m in self.match:
            if m.get("Status", "UPCOMING") == "FINISHED":
                mid = m.get("MatchID", None)
                if mid is not None and mid not in mlist:
                    mlist.append(mid)

        set_match_list(mlist, self.team_id)

    def GetNextMatch(self):
        for m in self.match:
            if m.get("Status", "FINISHED") in ["ONGOING", "UPCOMING"]:
                return m.get("MatchID")
        return None

class MatchDetails:

    rating_types = {"mf": "Midfield", "rd": "RightDef", "md": "MidDef",
                    "ld": "LeftDef", "ra": "RightAtt", "ma": "MidAtt",
                    "la": "LeftAtt"}

    def __init__(self, match_id, force_update = False, at_least_version = None):
        self.__extract_data(match_id, force_update, at_least_version)

    def __extract_data(self, match_id, force_update, at_least_version):

        data = XMLParse.get(["match details", match_id], force_update = force_update, at_least_version = at_least_version)
        self.version=data.get("Version")
        
        self.match_id = data.get("Match.MatchID")
        self.match_date = data.get("Match.MatchDate")
        
        self.arena = (data.get("Match.Arena.ArenaID"),
                      data.get("Match.Arena.ArenaName"),
                      data.get("Match.Arena.WeatherID"),
                      data.get("Match.Arena.SoldTotal"),
                      data.get("Match.Arena.SoldTerraces"),
                      data.get("Match.Arena.SoldBasic"),
                      data.get("Match.Arena.SoldRoof"),
                      data.get("Match.Arena.SoldVIP"))

        self.score = (data.get("Match.HomeTeam.HomeGoals"),
                      data.get("Match.AwayTeam.AwayGoals"))
    
        if self.score[0] is None or self.score[1] is None:
            self.played = False
        else:
            self.played = True

        self.home_team = (data.get("Match.HomeTeam.HomeTeamID"),
                          data.get("Match.HomeTeam.HomeTeamName", "?"),
                          data.get("Match.HomeTeam.Dress"),
                          data.get("Match.HomeTeam.TeamAttitude"),
                          data.get("Match.HomeTeam.TacticType"),
                          data.get("Match.HomeTeam.TacticSkill"))
        self.away_team = (data.get("Match.AwayTeam.AwayTeamID"),
                          data.get("Match.AwayTeam.AwayTeamName", "?"),
                          data.get("Match.AwayTeam.Dress"),
                          data.get("Match.AwayTeam.TeamAttitude"),
                          data.get("Match.AwayTeam.TacticType"),
                          data.get("Match.AwayTeam.TacticSkill"))

        if match_id < 0:
            (self.score, dummy) = get_goals(match_id)
            self.played = True
            return
        
        self.home_ratings = {}
        self.away_ratings = {}
        for rating, rating_ht in self.rating_types.items():
            self.home_ratings[rating] = data.get("Match.HomeTeam.Rating%s" % rating_ht,0)
            self.away_ratings[rating] = data.get("Match.AwayTeam.Rating%s" % rating_ht,0)

        self.hatstats_home_ratings = {}
        self.hatstats_away_ratings = {}
        
        self.hatstats_home_ratings["def"] = self.home_ratings["rd"] + self.home_ratings["md"] +  self.home_ratings["ld"]
        self.hatstats_home_ratings["mid"] = 3*self.home_ratings["mf"]
        self.hatstats_home_ratings["att"] = self.home_ratings["ra"] + self.home_ratings["ma"] +  self.home_ratings["la"]
        self.hatstats_home_ratings["tot"] = self.hatstats_home_ratings["def"] + self.hatstats_home_ratings["att"] + self.hatstats_home_ratings["mid"]

        self.hatstats_away_ratings["def"] = self.away_ratings["rd"] + self.away_ratings["md"] +  self.away_ratings["ld"]
        self.hatstats_away_ratings["mid"] = 3*self.away_ratings["mf"]
        self.hatstats_away_ratings["att"] = self.away_ratings["ra"] + self.away_ratings["ma"] +  self.away_ratings["la"]
        self.hatstats_away_ratings["tot"] = self.hatstats_away_ratings["def"] + self.hatstats_away_ratings["att"] + self.hatstats_away_ratings["mid"]

        if self.played:
            goals = data.get("Match.Scorers.xml_attrs.NumberOfGoals", 0)
            self.scorers = []
            if goals < 1:
                scorer_indices =  []
            else:
                scorer_indices = CommonData.get_xml_list_indices("Match.Scorers.Goal", data)
            for goal in scorer_indices:
                id = data.get("Match.Scorers.Goal%s.ScorerPlayerID" % goal)
                name = data.get("Match.Scorers.Goal%s.ScorerPlayerName" % goal, "?")
                team = data.get("Match.Scorers.Goal%s.ScorerTeamID" % goal)
                hgoals = data.get("Match.Scorers.Goal%s.ScorerHomeGoals" % goal)
                agoals = data.get("Match.Scorers.Goal%s.ScorerAwayGoals" % goal)
                minute = data.get("Match.Scorers.Goal%s.ScorerMinute" % goal)

                self.scorers.append((minute, hgoals, agoals, team, id, name))

            self.scorers.sort()

            books = data.get("Match.Bookings.xml_attrs.NumberOfBookings", 0)
            self.bookings = []
            if books < 1:
                booking_indices =  []
            else:
                booking_indices = CommonData.get_xml_list_indices("Match.Bookings.Booking", data)
            for booking in booking_indices:
                id = data.get("Match.Bookings.Booking%s.BookingPlayerID" % booking)
                name = data.get("Match.Bookings.Booking%s.BookingPlayerName" % booking, "?")
                team = data.get("Match.Bookings.Booking%s.BookingTeamID" % booking)
                type = data.get("Match.Bookings.Booking%s.BookingType" % booking)
                minute = data.get("Match.Bookings.Booking%s.BookingMinute" % booking)

                self.bookings.append((minute, type, team, id, name))
            self.bookings.sort()

class MatchLineup:
    def __init__(self, match_id, team_id):
        self.__extract_data(match_id, team_id)

    def __extract_data(self, match_id, team_id):

        if match_id < 1:
            return
        data = XMLParse.get(["match lineup", match_id, team_id])

        # basic data

        self.datatype = "Lineup"
        self.match_id = data.get("MatchID")
        self.team_id = data.get("Team.TeamID")
        self.team_name = data.get("Team.TeamName", "?")
        self.match_type = data.get("MatchType")
        self.match_date = data.get("MatchDate")
        self.home_team = (data.get("HomeTeam.HomeTeamID"), data.get("HomeTeam.HomeTeamName", "?"))
        self.away_team = (data.get("AwayTeam.AwayTeamID"), data.get("AwayTeam.AwayTeamName", "?"))
        self.match_type = data.get("MatchType")
        self.arena = (data.get("Arena.ArenaID"), data.get("Arena.ArenaName"))
        self. experience = data.get("Team.ExperienceLevel")
        
        lineup_indices = CommonData.get_xml_list_indices("Team.Lineup.Player", data)

        lineup_info = {}
        for i in lineup_indices:
            id = data.get("Team.Lineup.Player%s.PlayerID" % i)
            name = data.get("Team.Lineup.Player%s.PlayerName" % i, "?")
            role = data.get("Team.Lineup.Player%s.RoleID" % i)
            rating = data.get("Team.Lineup.Player%s.RatingStars" % i)
            position = data.get("Team.Lineup.Player%s.PositionCode" % i)
            behaviour = data.get("Team.Lineup.Player%s.Behaviour" % i)

            if role is not None:
                lineup_info[role] = (id, name, rating, position, behaviour)

        self.player_id_played=[]
        self.role_id_played=[]
        
        self.total_rating = 0.0
        for i in range(1,22):
            if not lineup_info.has_key(i):
                lineup_info[i] = (None, None, None, None, None)

            if i <= 11 and lineup_info[i][2] is not None:
                self.total_rating += lineup_info[i][2]
                self.player_id_played.append(lineup_info[i][0])
                self.role_id_played.append(i)

            if i >= 19 and lineup_info[i][2] is not None:
                self.player_id_played.append(lineup_info[i][0])
                self.role_id_played.append(i)

        self.lineup = lineup_info

    def write_lineup_text(self, outfile):
        if self.match_id is None:
            outfile.write(_("No match lineup to print") + "\n")
            return

        outfile.write("%s %d: %s - %s\n" % (_("Match"), self.match_id,
                                          self.home_team[1], self.away_team[1]))
        if self.match_type is not None:
            outfile.write(Terms.match_type[self.match_type] + "\n")
        if self.match_date is not None:
            outfile.write("%s: %s\n" % (_("Match date"), mktimestring(self.match_date)))

        outfile.write("\n%s:\n\n" % _("Lineup for %s") % self.team_name)

        parts = []
        for i in range(1,22):
            (id, name, rating, position, behaviour) = self.lineup[i]
            if id is None:
                continue

            first_part = Terms.team_role[i]
            if behaviour is not None:
                first_part += ", " + Terms.individual_order[behaviour]
            first_part += ":"

            second_part = name

            third_part = fourth_part = ""
            if rating is not None:
                third_part = "*"*(int(rating)) + ">"*(int(rating*2) % 2)
                fourth_part = "(" + str(rating) + "*)"

            parts.append((first_part, second_part, third_part, fourth_part))

        max_len = [max([len(x[i]) for x in parts]) for i in range(4)]
        for p1, p2, p3, p4 in parts:
            line = string.ljust(p1, max_len[0] + 2) + \
                   string.ljust(p2, max_len[1] + 2) + \
                   string.ljust(p3, max_len[2] + 1) + \
                   string.ljust(p4, max_len[3]) + "\n"
            outfile.write(line)

        outfile.flush()

class MatchOrders:
    def __init__(self, match_id):
        self.__extract_data(match_id)

    def __extract_data(self, match_id):

        data = XMLParse.get(["match orders", match_id])

        # basic data

        self.datatype = "Order"
        self.match_id = data.get("MatchID")
#        self.team_id = data.get("Team.TeamID")
#        self.team_name = data.get("Team.TeamName", "?")
        available = data.get("MatchData.xml_attrs.Available")
        if available == "true":
            self.available = True
            self.match_type = data.get("MatchData.MatchType")
            self.match_date = data.get("MatchData.MatchDate")
            self.home_team = (data.get("MatchData.HomeTeam.HomeTeamID"), data.get("MatchData.HomeTeam.HomeTeamName", "?"))
            self.away_team = (data.get("MatchData.AwayTeam.AwayTeamID"), data.get("MatchData.AwayTeam.AwayTeamName", "?"))
            self.match_type = data.get("MatchData.MatchType")
            self.arena = (data.get("MatchData.Arena.ArenaID"), data.get("MatchData.Arena.ArenaName"))
        
            lineup_indices = CommonData.get_xml_list_indices("MatchData.Lineup.Player", data)

            self.keeper_count=0
            self.wingback_count=0
            self.centraldefender_count=0
            self.winger_count=0
            self.innermidfielder_count=0
            self.forward_count=0

            playereval=0.0
            pdata = Players.Players()
            lineup_info = {}
            for i in lineup_indices:
                id = data.get("MatchData.Lineup.Player%s.PlayerID" % i)
                name = data.get("MatchData.Lineup.Player%s.PlayerName" % i, "?")
                role = data.get("MatchData.Lineup.Player%s.RoleID" % i)
                behaviour = data.get("MatchData.Lineup.Player%s.Behaviour" % i)

                if role is not None:
                    if role < 17:
                        if behaviour == 5:
                            self.forward_count += 1
                            pos = 16
                        elif behaviour == 6:
                            self.innermidfielder_count += 1
                            pos = 8
                        elif behaviour == 7:
                            self.centraldefender_count += 1
                            pos = 1
                        else:
                            if role == 1:
                                self.keeper_count += 1
                                pos = 0
                            elif role in [2, 5]:
                                self.wingback_count += 1
                                if behaviour == 1:
                                    pos = 5
                                elif behaviour == 2:
                                    pos = 6
                                elif behaviour == 3:
                                    pos = 7
                                else:
                                    pos = 4
                            elif role in [3, 4]:
                                self.centraldefender_count += 1
                                if behaviour == 1:
                                    pos = 2
                                elif behaviour == 4:
                                    pos = 3
                                else:
                                    pos = 1
                            elif role in [6, 9]:
                                self.winger_count += 1
                                if behaviour == 1:
                                    pos = 13
                                elif behaviour == 2:
                                    pos = 14
                                elif behaviour == 3:
                                    pos = 15
                                else:
                                    pos = 12
                            elif role in [7, 8]:
                                self.innermidfielder_count += 1
                                if behaviour == 1:
                                    pos = 9
                                elif behaviour == 2:
                                    pos = 10
                                elif behaviour == 4:
                                    pos = 11
                                else:
                                    pos = 8
                            elif role in [10, 11]:
                                self.forward_count += 1
                                if behaviour == 2:
                                    pos = 17
                                else:
                                    pos = 16
                            elif role in [12]:
                                pos = 0
                            elif role in [13]:
                                pos = 1
                            elif role in [14]:
                                pos = 8
                            elif role in [15]:
                                pos = 12
                            elif role in [16]:
                                pos = 16
                            else:
                                print "Strange role number"

                    playerval = ((0,0), "?")
                    if id != 0:
                        (position, descr) = Terms.team_positions[pos]
                        if pdata.all_players.has_key(id):
                            playerval = pdata.all_players[id].data[position]
                        else:
                            playerval = ((0, 0), _("Kicked/sold"))
                        if role < 12:
                            playereval += playerval[0][0]
                        
                    lineup_info[role] = (id, name, playerval, pos, behaviour)

            self.defender_count = self.wingback_count + self.centraldefender_count
            self.midfield_count = self.winger_count + self.innermidfielder_count

            self.formation_string = ("%d-%d-%d" % (self.defender_count, self.midfield_count, self.forward_count))
            self.formation_id = ("%d%d%d" % (self.defender_count, self.midfield_count, self.forward_count))

            self.teameval = (playereval/11.0, "%.2f" % (playereval/11.0))
            self.total_rating = 0.0
            for i in range(1,18):
                if not lineup_info.has_key(i):
                    lineup_info[i] = (None, None, None, None, None)                    

            self.lineup = lineup_info
        else:
            self.available = False
            
class LineupInfo:
    def __init__(self, match_id):
        self.__extract_data(match_id)

    def __extract_data(self, match_id):

        data = XMLParse.get(["match orders", match_id])

        # basic data

        self.match_id = data.get("MatchID")
        available = data.get("MatchData.xml_attrs.Available")
        if available == "true":
            self.available = True

            self.starting_players = []
            self.substitute_players = []
            self.player_roles = {}

            lineup_indices = CommonData.get_xml_list_indices("MatchData.Lineup.Player", data)

            lineup_info = {}
            for i in lineup_indices:
                id = data.get("MatchData.Lineup.Player%s.PlayerID" % i)
                name = data.get("MatchData.Lineup.Player%s.PlayerName" % i, "?")
                role = data.get("MatchData.Lineup.Player%s.RoleID" % i)
                behaviour = data.get("MatchData.Lineup.Player%s.Behaviour" % i)

                if role is not None:
                    if role < 17:
                        if behaviour == 5:
                            pos = 16
                        elif behaviour == 6:
                            pos = 8
                        elif behaviour == 7:
                            pos = 1
                        else:
                            if role == 1:
                                pos = 0
                            elif role in [2, 5]:
                                if behaviour == 1:
                                    pos = 5
                                elif behaviour == 2:
                                    pos = 6
                                elif behaviour == 3:
                                    pos = 7
                                else:
                                    pos = 4
                            elif role in [3, 4]:
                                if behaviour == 1:
                                    pos = 2
                                elif behaviour == 4:
                                    pos = 3
                                else:
                                    pos = 1
                            elif role in [6, 9]:
                                if behaviour == 1:
                                    pos = 13
                                elif behaviour == 2:
                                    pos = 14
                                elif behaviour == 3:
                                    pos = 15
                                else:
                                    pos = 12
                            elif role in [7, 8]:
                                if behaviour == 1:
                                    pos = 9
                                elif behaviour == 2:
                                    pos = 10
                                elif behaviour == 4:
                                    pos = 11
                                else:
                                    pos = 8
                            elif role in [10, 11]:
                                if behaviour == 2:
                                    pos = 17
                                else:
                                    pos = 16
                            elif role in [12]:
                                pos = 0
                            elif role in [13]:
                                pos = 1
                            elif role in [14]:
                                pos = 8
                            elif role in [15]:
                                pos = 12
                            elif role in [16]:
                                pos = 16
                            else:
                                print "Strange role number"

                    if role < 12:
                        if id is not None:
                            self.starting_players.append(id)
                            self.player_roles[id] = pos
                    elif role < 17:
                        if id is not None:
                            self.substitute_players.append(id)
                            self.player_roles[id] = pos

        else:
            self.available = False
            
