# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""A panel suited for placement in the main notebook."""

from Common import *
from Commonwx import *

import CommonData
import MatchInfo
import XMLParse
import Images
import Database
import Terms
import Matchdialogs
import MatchStatsDialog

from wxPython.wx import *

import time

class MatchListPage(wxPanel):
    ID_PREV = 743
    ID_NEXT = 744
    ID_FETCH_ALL = 745
    ID_UPDATE = 746
    ID_GET_ALL = 747
    ID_FIRST = 748
    ID_LAST = 749
    ID_MB_BASE = 755
    ID_TABLES = 750
    
    MATCHES_PER_PAGE = 15

    def __init__(self, parent, ID):
        wxPanel.__init__(self, parent, ID)

        main_sizer = wxBoxSizer(wxVERTICAL)

        hsizer = wxBoxSizer(wxHORIZONTAL)

        self.first_button = wxButton(self, self.ID_FIRST, "|< " + _("First"))
        hsizer.Add(self.first_button, 0, wxALIGN_LEFT | wxALL, 2)
        self.prev_button = wxButton(self, self.ID_PREV, "<< " + _("Previous"))
        hsizer.Add(self.prev_button, 0, wxALIGN_LEFT | wxALL, 2)

        hsizer.Add((0, 0), 1, wxEXPAND, 0)

        button = wxButton(self, self.ID_FETCH_ALL, _("Fetch all games"))
        hsizer.Add(button, 0, wxALIGN_CENTRE | wxALL, 2)

        hsizer.Add((0, 0), 1, wxEXPAND, 0)

        button = wxButton(self, self.ID_UPDATE, _("Update list"))
        hsizer.Add(button, 0, wxALIGN_CENTRE | wxALL, 2)

        hsizer.Add((0, 0), 1, wxEXPAND, 0)

        button = wxButton(self, self.ID_TABLES, _("Match statistics"))
        hsizer.Add(button, 0, wxALIGN_CENTRE | wxALL, 2)

        hsizer.Add((0, 0), 1, wxEXPAND, 0)

        self.next_button = wxButton(self, self.ID_NEXT, _("Next") + " >>")
        hsizer.Add(self.next_button, 0, wxALIGN_RIGHT | wxALL, 2)
        self.last_button = wxButton(self, self.ID_LAST, _("Last") + " >|")
        hsizer.Add(self.last_button, 0, wxALIGN_RIGHT | wxALL, 2)

        main_sizer.Add(hsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        self.match_panel = wxPanel(self, -1)

        main_sizer.Add(self.match_panel, 0, wxEXPAND | wxADJUST_MINSIZE | wxALIGN_CENTRE | wxALL, 2)

        self.SetSizer(main_sizer)
        self.SetAutoLayout(True)
        main_sizer.Fit(self)

        EVT_BUTTON(self, self.ID_PREV, self.OnPrevious)
        EVT_BUTTON(self, self.ID_NEXT, self.OnNext)
        EVT_BUTTON(self, self.ID_FIRST, self.OnFirst)
        EVT_BUTTON(self, self.ID_LAST, self.OnLast)
        EVT_BUTTON(self, self.ID_TABLES, self.OnTables)
        EVT_BUTTON(self, self.ID_FETCH_ALL, self.OnFetchAllGames)
        EVT_BUTTON(self, self.ID_UPDATE, self.OnUpdateList)

        self.star_img = Images.Images.get_bitmap_from_file("star.png")
        self.glasses_img = Images.Images.get_bitmap_from_file("glasses.png")
        mask = wxMaskColour(self.glasses_img, wxWHITE)
        self.glasses_img.SetMask(mask)
        self.field_img = Images.Images.get_bitmap_from_file("field.png")
        self.all_widgets = []
        self.__position = 0

    def AfterInit(self):
        self.__get_all_matches()
        self.__update()

    def OnFirst(self, event):
        old_pos = self.__position
        self.__position = len(self.all_matches) - self.MATCHES_PER_PAGE
        if old_pos != self.__position:
            self.next_button.Enable(True)
            self.__update()

    def OnLast(self, event):
        old_pos = self.__position
        self.__position = 0
        if old_pos != self.__position:
            self.prev_button.Enable(True)
            self.__update()

    def OnTables(self, event):
        ret = MatchStatsDialog.MatchStatsDialog(self, self.all_matches).Show()
        
    def OnPrevious(self, event):
        old_pos = self.__position
        self.__position = min(self.__position + self.MATCHES_PER_PAGE,
                              len(self.all_matches) - self.MATCHES_PER_PAGE)
        if old_pos != self.__position:
            self.next_button.Enable(True)
            self.__update()

    def OnNext(self, event):
        old_pos = self.__position
        self.__position = max(self.__position - self.MATCHES_PER_PAGE, 0)
        if old_pos != self.__position:
            self.prev_button.Enable(True)
            self.__update()
            
    def OnUpdateList(self, event):
        self.__get_all_matches(True, True)
        self.__fetch_all_orders()
        self.__fetch_all_games()
        self.__update()

        import Main
        Main.notebook_pages["player"].ChangeXML()

    def GetLastPlayedMatch(self):
        self.all_matches.reverse()

        i = 0
        stop=False
        while not stop:
            if self.all_matches[i][6] == "FINISHED":
                stop = True
            else:
                i+=1

        mid = self.all_matches[i][1]
        self.all_matches.sort()

        return mid

    def GetNextMatch(self):
        if not hasattr(self, "all_matches"):
            return None
        
        self.all_matches.reverse()

        i = 0
        stop=False
        while not stop:
            if self.all_matches[i][6] == "FINISHED":
                stop = True
            else:
                i+=1

        if i < 1:
            mid = None
        else:
            mid = self.all_matches[i-1][1]
        self.all_matches.sort()

        return mid

    def OnFetchAllGames(self, event):
        dlg = wxMessageDialog(self, _("Fetching all games may take long time and should be used with care!"),
                          _("Fetch all games"), wxOK | wxCANCEL | wxICON_INFORMATION)
        ret = dlg.ShowModal()
        dlg.Destroy()
        if ret == wxID_CANCEL:
            return
        
        tdata=XMLParse.get(["team details",CommonData.get_current_teamid()])
        first_date = tdata.get("User.ActivationDate")
        matches_left = True
        
        while matches_left:
            m=XMLParse.get(["match archive", first_date])
            c=m.get("Team.MatchList.xml_attrs.Count")
            if c is None:
                return
            if c < 50:
                matches_left = False
            for index in range(c):
                mid=m.get("Team.MatchList.Match+%s.MatchID" % index)
                hometeamname = m.get("Team.MatchList.Match+%s.HomeTeam.HomeTeamName" % index)
                hometeamid = m.get("Team.MatchList.Match+%s.HomeTeam.HomeTeamID" % index)
                home_team = (hometeamid, hometeamname)
                awayteamname = m.get("Team.MatchList.Match+%s.AwayTeam.AwayTeamName" % index)
                awayteamid = m.get("Team.MatchList.Match+%s.AwayTeam.AwayTeamID" % index)
                away_team = (awayteamid, awayteamname)
                score = (m.get("Team.MatchList.Match+%s.HomeGoals" % index),
                         m.get("Team.MatchList.Match+%s.AwayGoals" % index))
                status = "FINISHED"
                mtype = m.get("Team.MatchList.Match+%s.MatchType" % index)
                mdate = m.get("Team.MatchList.Match+%s.MatchDate" % index)
                has_md = False
                if mid < 0:
                    MatchInfo.set_goals(mid, score, mtype)
                if not mid in self.all_mids:
                    self.all_mids.append(mid)
                    self.all_matches.append((mdate, mid, mtype, home_team, away_team, score, status, has_md))

            first_date=mdate+86400.0 # Add one day

        self.all_matches.sort()
        MatchInfo.set_match_list(self.all_mids, CommonData.get_current_teamid())
        self.__fetch_all_games()
        self.__update()

    def OnMatchDetails(self, event):
        eid = event.GetId()
        match_id, status, has_md = self.match_button_ids[eid]
        if match_id < 1:
            return
            
        if status == "FINISHED" or status is None:
            dlg = Matchdialogs.MatchDetailsDialog(self, match_id)
            if not dlg.no_data:
                ret = dlg.Show()
        else:
            self.GetGrandParent().GetTabHandle("matchview").TryAddMatch(match_id)

    def OnMatchLineup(self, event):
        eid = event.GetId()
        match_id, team_id, status = self.match_button_ids[eid]
        if match_id < 1:
            return
        
        my_team = CommonData.get_current_teamid()
        if status == "FINISHED" or status is None:
            dlg = Matchdialogs.MatchLineupDialog(self, match_id, team_id)
            if not dlg.no_data:
                ret = dlg.Show()
        else:
            if team_id == my_team:
                dlg = Matchdialogs.MatchLineupDialog(self, match_id, None, False)
                if not dlg.no_data:
                    ret = dlg.Show()
            else:
                self.GetGrandParent().GetTabHandle("matchview").AddMatchFromTeamID(team_id)


    def __update(self):
        my_team = CommonData.get_current_teamid()
        for widget in self.all_widgets:
            widget.Destroy()

        self.all_widgets = []
        self.match_button_ids = {}
        self.__current_button_id = self.ID_MB_BASE

        fg_sizer = wxFlexGridSizer(cols=11, vgap=0, hgap=0)
        for i in [4,6]:
            fg_sizer.AddGrowableCol(i)

        fpos = max(len(self.all_matches) - (self.MATCHES_PER_PAGE + self.__position), 0)
        lpos = len(self.all_matches) - self.__position
        ms = self.all_matches[fpos:lpos]
        old_status = None

        if fpos <= 0:
            self.prev_button.Disable()
        if lpos >= len(self.all_matches):
            self.next_button.Disable()

        for mdate, mid, mtype, home_team, away_team, score, status, has_md in ms:
            if mdate is None:
                continue

            if (old_status is None or old_status == "FINISHED") and \
                   (status in ["UPCOMING", "ONGOING"]):
                for x in range(11):
                    fg_sizer.Add((0, 20), 0, wxEXPAND | wxADJUST_MINSIZE | wxALIGN_CENTRE | wxALL, 0)

            if mtype is None:
                mtypestr = ""
            else:
                if mtype == 1:
                    mtypestr = _("League")
                elif mtype == 2:
                    mtypestr = _("Qualification")
                elif mtype == 3:
                    mtypestr = _("Cup")
                elif mtype in [4,5,8,9]:
                    mtypestr = _("Friendly")
                elif mtype in [6,7]:
                    mtypestr = _("International")
                elif mtype in [10,11,12]:
                    mtypestr = _("National team")
                mtypestr = "(" + mtypestr + ")"

            texts = [0,
                     time.strftime("%d %b %Y", time.localtime(mdate)),
                     mtypestr,
                     1,
                     home_team[1],
                     "-",
                     away_team[1],
                     2,
                     str(score[0]),
                     "-",
                     str(score[1])]

            for t in range(11):
                text = texts[t]
                if text in [0,1,2]:
                    if text == 0:
                        b_id = self.__add_to_match_button_ids(True, mid, status, has_md)
                        if status == "FINISHED" or status is None:
                            widget = wxButton(self.match_panel, b_id,
                                              _("Match details"), style = wxBU_EXACTFIT)
                        else:
                            widget = wxButton(self.match_panel, b_id,
                                              _("View live"), style = wxBU_EXACTFIT)
                    else:
                        if status == "FINISHED":
                            tid = {1: home_team[0], 2: away_team[0]}[text]
                            if tid is None:
                                widget = None
                            else:
                                b_id = self.__add_to_match_button_ids(False, mid, tid, status)
                                widget = wxBitmapButton(self.match_panel, b_id,
                                                        self.star_img, style = wxBU_EXACTFIT)
                        else:
                            tid = {1: home_team[0], 2: away_team[0]}[text]
                            if tid is None:
                                widget = None
                            else:
                                b_id = self.__add_to_match_button_ids(False, mid, tid, status)
                                if tid == my_team:
                                    widget = wxBitmapButton(self.match_panel, b_id,
                                                            self.field_img, style = wxBU_EXACTFIT)
                                else:
                                    widget = wxBitmapButton(self.match_panel, b_id,
                                                            self.glasses_img, style = wxBU_EXACTFIT)
                else:
                    widget = wxStaticText(self.match_panel, -1, str(text))

                if t in [4,6]:
                    size = 1
                else:
                    size = 0

                if widget is None:
                    fg_sizer.Add((0, 0), size, wxEXPAND | wxADJUST_MINSIZE | wxALIGN_CENTRE | wxALL, 2)
                else:
                    fg_sizer.Add(widget, size, wxEXPAND | wxADJUST_MINSIZE | wxALIGN_CENTRE | wxALL, 2)
                    self.all_widgets.append(widget)
            old_status = status

        self.match_panel.SetSizer(fg_sizer)
        self.match_panel.SetAutoLayout(True)
        fg_sizer.Fit(self.match_panel)

        self.Layout()
        self.Refresh()

    def __add_to_match_button_ids(self, is_match, mid, *args):
        self.__current_button_id += 1
        if is_match:
            self.match_button_ids[self.__current_button_id] = (mid,) + args
            EVT_BUTTON(self, self.__current_button_id, self.OnMatchDetails)
        else:
            self.match_button_ids[self.__current_button_id] = (mid,) + args
            EVT_BUTTON(self, self.__current_button_id, self.OnMatchLineup)
        return self.__current_button_id

    def __fetch_all_orders(self):
        for m in self.all_matches:
            if m[6] == "UPCOMING":
                dummy = data = XMLParse.get(["match orders", m[1]], force_update = True)

    def __fetch_all_games(self):
        my_team = CommonData.get_current_teamid()

        dl_matches = []
        h_matches = []
        for m in self.all_matches:
            if m[0] is None or m[6] == "FINISHED":
                if m[7]:
                    if m[3][0] is None:
                        hdata = None
                    else:
                        hdata = XMLParse.get(["match lineup", m[1], m[3][0]],
                                             ["FileName"], nofetch = True)

                    if m[4][0] is None:
                        adata = None
                    else:
                        adata = XMLParse.get(["match lineup", m[1], m[4][0]],
                                             ["FileName"], nofetch = True)

                    if hdata is None or adata is None:
                        dl_matches.append(m)
                    else:
                        h_matches.append(m)
                else:
                    dl_matches.append(m)
            else:
                h_matches.append(m)

        n_matches = len(dl_matches)

        if n_matches == 0:
            return

        topframe = self.GetGrandParent()
        sbar = topframe.GetStatusBar()
        sbar.SetRange(n_matches*3*10)
        pbw = topframe.GetPBW()
        pbw.progress_start(_("Fetching all games."))

        new_matches = []
        match_number = 0
        for m in dl_matches:
            match_number += 1
            mdate, mid, mtype, home_team, away_team, score, status, has_md = m

            sbar.SetBaseValue((match_number - 1)*30)
            sbar.PushExtraMessage(_("Checking game %d of %d.") % (match_number, n_matches))
            sbar.Update(0, "")

            mdetails = MatchInfo.MatchDetails(mid)

            mdate = mdetails.match_date
            home_team = mdetails.home_team[0:2]
            away_team = mdetails.away_team[0:2]
            score = mdetails.score
            status = "FINISHED"

            if mdate is None or \
                   (home_team[0] != my_team and away_team[0] != my_team):
                sbar.PopExtraMessage()
                continue

            self.match_details_cache[mid] = mdetails

            if mid > 0:
                sbar.SetBaseValue((match_number - 1)*30 + 10)
                sbar.Update(0, "")
                mtype = XMLParse.get(["match lineup", mid, home_team[0]], ["MatchType"],
                                     progress_dialog = False)

                sbar.SetBaseValue((match_number - 1)*30 + 20)
                sbar.Update(0, "")
                mtype = XMLParse.get(["match lineup", mid, away_team[0]], ["MatchType"],
                                     progress_dialog = False)

                sbar.PopExtraMessage()
            else:
                (dummy, mtype) = MatchInfo.get_goals(mid)
                
            new_matches.append((mdate, mid, mtype, home_team, away_team, score, status, True))

        pbw.progress_end()

        self.all_matches = h_matches + new_matches
        self.all_matches.sort()

        amids = []
        for m in self.all_matches:
            amids.append(m[1])

        self.all_mids = amids

        MatchInfo.set_match_list(self.all_mids, my_team)

        self.__update()

    def __get_matches(self, force_update = False):
        self.matches = MatchInfo.CurrentMatches(None, force_update)
        self.all_mids = MatchInfo.get_match_list()

    def __get_all_matches(self, get_new_matches = True, force_update = False):
        if get_new_matches:
            self.__get_matches(force_update)

        self.match_details_cache = {}
        self.all_matches = []

        amids = self.all_mids + []
        for mid in self.matches.match_d.keys():
            if not mid in amids:
                amids.append(mid)

        for mid in amids:
            if self.match_details_cache.has_key(mid):
                data = True
                ddl = False
            else:
                data = XMLParse.get(["match details", mid], ["FileName"], nofetch = True)
                ddl = True

            if data is None:
                has_md = False
                if self.matches.match_d.has_key(mid):
                    m = self.matches.match_d[mid]
                    mdate = m.get("MatchDate")
                    home_team = (m.get("HomeTeam.HomeTeamID"),
                                 m.get("HomeTeam.HomeTeamName"))
                    away_team = (m.get("AwayTeam.AwayTeamID"),
                                 m.get("AwayTeam.AwayTeamName"))
                    if mid > 0:
                        mtype = m.get("MatchType")
                        status = m.get("Status", "UPCOMING")
                        if status == "UPCOMING":
                            score = ("x", "x")
                        else:
                            score = (m.get("HomeGoals"), m.get("AwayGoals"))
                    else:
                        (score, mtype) = MatchInfo.get_goals(mid)
                        status = "FINISHED"
                else:
                    mdate = None
                    mtype = None
                    home_team = (None, "")
                    away_team = (None, "")
                    status = None
                    score = ("x", "x")
            else:
                has_md = True
                if ddl:
                    self.match_details_cache[mid] = MatchInfo.MatchDetails(mid)
                mdate = self.match_details_cache[mid].match_date
                home_team = self.match_details_cache[mid].home_team[0:2]
                away_team = self.match_details_cache[mid].away_team[0:2]
                score = self.match_details_cache[mid].score
                status = "FINISHED"

                mtype = XMLParse.get(["match lineup", mid, home_team[0]],
                                     ["MatchType"], nofetch = True)

                if mtype is None:
                    mtype = XMLParse.get(["match lineup", mid, away_team[0]],
                                         ["MatchType"], nofetch = True)

                if mtype is None and self.matches.match_d.has_key(mid):
                    mtype = self.matches.match_d[mid].get("MatchType")

            self.all_matches.append((mdate, mid, mtype, home_team, away_team, score, status, has_md))
            #print (mdate, mid, mtype, home_team, away_team, score, status, has_md)
            #if mtype == 4:
            #    print (mid, home_team[0], away_team[0])

        self.all_matches.sort()
