# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import string

import HRFData
import SelectDateMenu
import DataInfo
import Terms

from Common import *
from Commonwx import *

from wxPython.wx import *

class MatchPrediction(wxPanel):
    HOME_TEXT = 271
    AWAY_TEXT = 272

    MAX_NO_ODDS = 21

    def __init__(self, parent, ID):
        wxPanel.__init__(self, parent, ID)

        main_sizer = wxBoxSizer(wxVERTICAL)
        hmain_sizer = wxBoxSizer(wxHORIZONTAL)
        vmain_sizer = wxBoxSizer(wxVERTICAL)

        # --- Inhmtning av data ---

        input_title = wxStaticBox(self, -1, _("Collecting of data"))
        sizer = wxStaticBoxSizer(input_title, wxVERTICAL)

        hsizer = wxBoxSizer(wxHORIZONTAL)
        for l in [_("Home team"), _("Away team")]:
            stext = wxStaticText(self, -1, l, style = wxALIGN_CENTRE)
            SetTextBold(stext)
            hsizer.Add(stext, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 4)
        sizer.Add(hsizer, 0, wxEXPAND | wxALL, 0)

        hsizer = wxBoxSizer(wxHORIZONTAL)

        self.hometext = wxButton(self, MatchPrediction.HOME_TEXT, _("Paste values"))
        hsizer.Add(self.hometext, 1, wxEXPAND | wxALL, 4)

        self.awaytext = wxButton(self, MatchPrediction.AWAY_TEXT, _("Paste values"))
        hsizer.Add(self.awaytext, 1, wxEXPAND | wxALL, 4)

        sizer.Add(hsizer, 0, wxEXPAND | wxALL, 0)

        hsizer = wxBoxSizer(wxHORIZONTAL)
        self.team_cbs = {"home": {}, "away": {}}
        self.cb_ids = {}
        id = 280
        for ha in ["home", "away"]:
            fsizer = wxFlexGridSizer(cols=2, vgap=4, hgap=10)
            for pos, p in Terms.team_parts:
                stext = wxStaticText(self, -1, p + ":")
                fsizer.Add(stext, 1, wxEXPAND | wxALIGN_LEFT | wxALL, 0)
                cb = wxComboBox(self, id, Terms.player_abilities[3],
                                choices = Terms.player_abilities.values()[1:],
                                style = wxCB_READONLY)
                cb.SetSelection(3)
                fsizer.Add(cb, 1, wxEXPAND | wxALIGN_RIGHT | wxALL, 0)
                self.team_cbs[ha][pos] = cb
                self.cb_ids[id] = (ha, p)
                id += 1
            hsizer.Add(fsizer, 1, wxEXPAND | wxALL, 4)
        sizer.Add(hsizer, 0, wxEXPAND | wxALL, 4)
        vmain_sizer.Add(sizer, 0, wxEXPAND | wxALL, 5)

        # --- Mlchanser ---

        output_title = wxStaticBox(self, -1, _("Scoring attempts"))
        sizer = wxStaticBoxSizer(output_title, wxVERTICAL)

        texts = [(_("Ball possession"), "%3.0f%%", (lambda vals: vals[0]*100.0)),
                 (_("Attempts"), "%d", (lambda vals: vals[1])),
                 (_("Chance of scoring"), None, None),
                 (_("Attack to the left"), "%3.1f%%", (lambda vals: vals[3]*100.0)),
                 (_("Attack in the middle"), "%3.1f%%", (lambda vals: vals[4]*100.0)),
                 (_("Attack to the right"), "%3.1f%%", (lambda vals: vals[2]*100.0)),
                 (_("Total"), "%3.1f%%", (lambda vals: vals[8]*100.0))]
        self.res_stexts = {}
        hsizer = wxBoxSizer(wxHORIZONTAL)
        for ha, hal in [("home", _("Home team")), ("away", _("Away team"))]:
            vsizer = wxBoxSizer(wxVERTICAL)
            stext = wxStaticText(self, -1, hal, style = wxALIGN_CENTRE)
            SetTextBold(stext)
            vsizer.Add(stext, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 4)

            fsizer = wxFlexGridSizer(cols=2, vgap=3, hgap=10)
            self.res_stexts[ha] = []
            for row in texts:
                descr, format, func = row
                if format == None or func == None:
                    vsizer.Add(fsizer, 0, wxEXPAND | wxALL, 3)
                    stext = wxStaticText(self, -1, descr, style = wxALIGN_CENTRE)
                    SetTextBold(stext)
                    vsizer.Add(stext, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 3)
                    fsizer = wxFlexGridSizer(cols=2, vgap=3, hgap=10)
                    continue

                stext = wxStaticText(self, -1, descr + ":")
                fsizer.Add(stext, 1, wxEXPAND | wxALIGN_LEFT | wxALL, 0)
                stext = wxStaticText(self, -1, "")
                fsizer.Add(stext, 1, wxEXPAND | wxADJUST_MINSIZE | wxALIGN_LEFT | wxALL, 0)
                self.res_stexts[ha].append((stext, format, func))

            vsizer.Add(fsizer, 0, wxEXPAND | wxALL, 3)
            hsizer.Add(vsizer, 1, wxEXPAND | wxALL, 5)
        sizer.Add(hsizer, 0, wxEXPAND | wxALL, 5)
        vmain_sizer.Add(sizer, 1, wxEXPAND | wxALL, 5)
        hmain_sizer.Add(vmain_sizer, 0, wxEXPAND | wxALL, 0)

        # --- Oddstabell ---

        odds_title = wxStaticBox(self, -1, _("Table of odds"))
        sizer = wxStaticBoxSizer(odds_title, wxHORIZONTAL)

        self.odds_table = {}
        for ha, t in [("home", _("Home victory")),
                      ("draw", _("Draw")),
                      ("away", _("Away victory"))]:
            self.odds_table[ha] = []

            vsizer = wxBoxSizer(wxVERTICAL)
            stext = wxStaticText(self, -1, t, style = wxALIGN_CENTRE)
            SetTextBold(stext)
            vsizer.Add(stext, 0, wxEXPAND | wxALL, 5)

            fsizer = wxFlexGridSizer(cols=2, vgap=3, hgap=10)
            for i in range(0, MatchPrediction.MAX_NO_ODDS):
                stext1 = wxStaticText(self, -1, "", style = wxALIGN_CENTRE)
                fsizer.Add(stext1, 1, wxEXPAND | wxADJUST_MINSIZE | wxALL, 0)

                stext2 = wxStaticText(self, -1, "", style = wxALIGN_CENTRE)
                fsizer.Add(stext2, 1, wxEXPAND | wxADJUST_MINSIZE | wxALL, 0)

                self.odds_table[ha].append((stext1, stext2))

            vsizer.Add(fsizer, 0, wxEXPAND | wxALL, 5)
            sizer.Add(vsizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        hmain_sizer.Add(sizer, 1, wxEXPAND | wxALL, 5)
        main_sizer.Add(hmain_sizer, 1, wxEXPAND | wxALL, 0)

        # --- Result ---

        result_title = wxStaticBox(self, -1, _("Result"))
        sizer = wxStaticBoxSizer(result_title, wxHORIZONTAL)

        for attr, descr in [("homewin_text", _("Home victory")),
                            ("draw_text", _("Draw")),
                            ("awaywin_text", _("Away victory"))]:
            hsizer = wxBoxSizer(wxHORIZONTAL)

            stext = wxStaticText(self, -1, descr + ":")
            SetTextBold(stext)
            hsizer.Add(stext, 0, wxEXPAND | wxALL, 5)

            stext = wxStaticText(self, -1, "")
            hsizer.Add(stext, 0, wxEXPAND | wxADJUST_MINSIZE | wxALL, 5)

            setattr(self, attr, stext)
            sizer.Add(hsizer, 1, wxEXPAND | wxALL, 0)

        main_sizer.Add(sizer, 0, wxEXPAND | wxALL, 5)

        self.SetSizer(main_sizer)
        self.SetAutoLayout(True)
        main_sizer.Fit(self)

        self.Update()

        EVT_BUTTON(self, MatchPrediction.HOME_TEXT,
                   (lambda event: self.TextChanged(event, True)))
        EVT_BUTTON(self, MatchPrediction.AWAY_TEXT,
                   (lambda event: self.TextChanged(event, False)))
        for id in self.cb_ids.keys():
            EVT_COMBOBOX(self, id, self.ValueChanged)

    def Update(self):
        team_values = {}
        for ha in ["home", "away"]:
            team_values[ha] = {}
            for name, cb in self.team_cbs[ha].items():
                team_values[ha][name] = self.__convert_part_value(cb.GetValue())

        home_compare = self.__compare_teams(team_values["home"], team_values["away"])
        away_compare = self.__compare_teams(team_values["away"], team_values["home"])

        for ha, vals in [("home", home_compare), ("away", away_compare)]:
            for stext, format, func in self.res_stexts[ha]:
                stext.SetLabel(format % func(vals))

        home_odds = []
        away_odds = []
        MAX_GOALS = 20
        for i in range(0,MAX_GOALS):
            home_odds.append(self.__odds(i, home_compare[1], home_compare[8]))
            away_odds.append(self.__odds(i, away_compare[1], away_compare[8]))

        homewin_list = []
        awaywin_list = []
        draw_list = []
        homewin_other = homewin_chance = 0.0
        awaywin_other = awaywin_chance = 0.0
        draw_other = draw_chance = 0.0
        for i in range(0,MAX_GOALS):
            for j in range(0,MAX_GOALS):
                odds = home_odds[i]*away_odds[j]
                if i > j:
                    homewin_chance += odds
                    if odds >= 0.001:
                        homewin_list.append((odds, i, j))
                    else:
                        homewin_other += odds
                elif i < j:
                    awaywin_chance += odds
                    if odds >= 0.001:
                        awaywin_list.append((odds, i, j))
                    else:
                        awaywin_other += odds
                else:
                    draw_chance += odds
                    if odds >= 0.001:
                        draw_list.append((odds, i, j))
                    else:
                        draw_other += odds
        for ha, the_list, other in [("home", homewin_list, homewin_other),
                                    ("draw", draw_list, draw_other),
                                    ("away", awaywin_list, awaywin_other)]:
            the_list.sort()
            the_list.reverse()
            count = 0
            for odds, i, j in the_list[:MatchPrediction.MAX_NO_ODDS-1]:
                self.odds_table[ha][count][0].SetLabel("%d-%d" % (i, j))
                self.odds_table[ha][count][1].SetLabel("%.2f" % (1.0/odds))
                count += 1
            rest_list = the_list[MatchPrediction.MAX_NO_ODDS-1:]
            if len(rest_list) > 0:
                other += reduce(lambda x, y: x + y, zip(*rest_list)[0])
            if other > 0.0:
                self.odds_table[ha][count][0].SetLabel(_("Oth."))
                self.odds_table[ha][count][1].SetLabel("%.2f" % min(1.0/other, 9999.99))
                count += 1
            while count < MatchPrediction.MAX_NO_ODDS:
                self.odds_table[ha][count][0].SetLabel("")
                self.odds_table[ha][count][1].SetLabel("")
                count += 1

        self.homewin_text.SetLabel("%3.1f%%" % (homewin_chance*100.0))
        self.draw_text.SetLabel("%3.1f%%" % (draw_chance*100.0))
        self.awaywin_text.SetLabel("%3.1f%%" % (awaywin_chance*100.0))

    def ValueChanged(self, event):
        self.Update()

    def TextChanged(self, event, home):
        if home:
            name = "home"
        else:
            name = "away"

        do = wxTextDataObject()
        wxTheClipboard.Open()
        wxTheClipboard.UsePrimarySelection(True)
        success = wxTheClipboard.GetData(do)
        wxTheClipboard.Close()

        if not success:
            return

        new_vals = self.__parse_text_input(do.GetText())

        for p, val in new_vals.items():
            self.team_cbs[name][p].SetValue(val)

        self.Update()

    def ChangeRatings(self, new_vals, home):
        if home:
            name = "home"
        else:
            name = "away"

        for p, val in new_vals.items():
            new_rating = int((val+3)/4)
            self.team_cbs[name][p].SetValue(Terms.player_abilities[new_rating])

        self.Update()

    def __parse_text_input(self, input):
        parts = string.split(input)

        res = {}
        tparts, tnames = zip(*Terms.team_parts)
        tdict = dict(zip(tnames,tparts))

        current_team_part = None
        last_p = None
        for p in parts:
            if p[-1] == ":":
                p = p[:-1]
            if current_team_part == None:
                if p in tnames:
                    current_team_part = p
                elif last_p != None and \
                         string.join([last_p, p], " ") in tnames:
                    current_team_part = string.join([last_p, p], " ")
            elif p in Terms.player_abilities.values():
                if current_team_part != None:
                    res[tdict[current_team_part]] = p
                    current_team_part = None
            last_p = p

        return res

    def __compare_teams(self, tv, ctv):
        pos = tv["mf"]/(tv["mf"]+ctv["mf"])
        chances = int(round(pos*10))
        rchance = self.__chance(tv["ra"], ctv["ld"])
        lchance = self.__chance(tv["la"], ctv["rd"])
        mchance = self.__chance(tv["ma"], ctv["md"])
        rrisk = 0.85 - self.__chance(tv["rd"], ctv["la"])
        lrisk = 0.85 - self.__chance(tv["ld"], ctv["ra"])
        mrisk = 0.85 - self.__chance(tv["md"], ctv["ma"])

        totalchance = (rchance + lchance + mchance)/3.0
        totalrisk = (rrisk + lrisk + mrisk)/3.0

        return (pos, chances,
                rchance, lchance, mchance,
                rrisk, lrisk, mrisk,
                totalchance, totalrisk)

    def __convert_part_value(self, s):
        pa = dict(zip(Terms.player_abilities.values(),
                      Terms.player_abilities.keys()))
        return float((pa[s] * 10) - 5)

    def __chance(self, l1, l2):
        ret = 0.425
        if l1 >= l2:
            ret += ((l1-l2) / (l1+l2)) * 0.6
        else:
            ret += ((l1-l2) / (l1+l2)) * 0.4
        return ret

    def __odds(self, goals, chances, p):
        modifiers = binomial_distribution(chances*2, 0.5)
        #modifiers = normal_distribution(float(chances), chances/3.29)
        odds = 0.0

        for i in range(0, chances*2+1):
            odds += modifiers(i) * self.__goalchance(goals, i, p)
        return odds

    def __goalchance(self, goals, chances, p):
        if goals > chances:
            return 0.0

        return pow(p, goals) * pow(1.0-p, chances-goals) * self.__permute(chances, goals)

    def __permute(self, n, p):
        ret = 1.0
        for i in range(n, n-p, -1):
            ret *= float(i)
        for j in range(p, 1, -1):
            ret /= float(j)
        return ret
