# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from Common import *

import CommonData
import Terms
import time
import Images
import Matchdialogs
import MatchInfo
import XMLParse
import StatHelper

from Commonwx import *
from wxPython.wx import *
from Config import config

class MatchStatsDialog(wxDialog):

    def __init__(self, parent, all_matches):

        wxDialog.__init__(self, parent, -1, _("Match statistics"),
                          style = wxRESIZE_BORDER | wxCAPTION | wxSYSTEM_MENU)

        main_sizer = wxBoxSizer(wxVERTICAL)

        # Some space

        main_sizer.Add((0, 5))

        # Notebook
        notebook = wxNotebook(self, -1)
        nbsizer = wxNotebookSizer(notebook)

        # Tabs
        notebook.AddPage(TablePanel(notebook, -1, all_matches), _("Tables"))

        notebook.AddPage(ResultPanel(notebook, -1, all_matches), _("Most/least goals"))
        notebook.AddPage(CommonPanel(notebook, -1, all_matches), _("Common results"))
        if CommonData.is_supporter():
            notebook.AddPage(PlayerPanel(notebook, -1, all_matches), _("Players"))
            notebook.AddPage(PlayerGoalPanel(notebook, -1, all_matches), _("Goal scorers"))
        notebook.AddPage(AttendancePanel(notebook, -1, all_matches), _("Attendance"))
            
        main_sizer.Add(nbsizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        # Line

        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        main_sizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        # Ok / Cancel buttons

        button = wxButton(self, wxID_OK, _("Ok"))
        button.SetDefault()

        main_sizer.Add(button, 0, wxALIGN_RIGHT | wxALL, 10)

        self.SetSizer(main_sizer)
        self.SetAutoLayout(True)
        main_sizer.Fit(self)

class TablePanel(wxPanel):
    ID_TYPE_TYPE = 1366
    ID_WHERE_TYPE = 1367

    def __init__(self, parent, ID, all_matches):
        wxPanel.__init__(self, parent, ID)

        ssizer = wxBoxSizer(wxVERTICAL)

        self.all_matches = all_matches

        self.no = config.get("Statistics","stat items")
        self.types = [[_("Total"), False],
                      [_("Average"), True]]

        self.average = self.types[0][1]
        self.wheres = [[_("All matches"), [_("Home"), _("Away")]],
                      [_("Home"), [_("Home")]],
                      [_("Away"), [_("Away")]]]

        self.where = self.wheres[0][1]


        row = 0
        row_col_alts = [0xF0F0F0, 0xF0F0F0, 0xE8E8E8, 0xE8E8E8]
        fg_sizer = wxFlexGridSizer(cols = 8, vgap = 0, hgap = 0)
        for i in range(8):
            fg_sizer.AddGrowableCol(i)

        aligns=[wxALIGN_LEFT] + [wxALIGN_RIGHT]*4 + [wxALIGN_CENTER] + [wxALIGN_RIGHT]*2  

        sizes = [1] + [0]*7
        i = 0
        for h in [_("Match type"), _("Pl"), _("W"), _("D"), _("L"),
                  _("For - Against"), _("Diff"), _("Po")]:
            stext = wxStaticText(self, -1, h, style = aligns[i])
            SetTextBold(stext)
            fg_sizer.Add(stext, sizes[i], wxEXPAND | wxADJUST_MINSIZE | wxALL, 5)
            i += 1

        self.text=[]
        for row in range(self.no):
            for col in range(8): 
                stext = wxStaticText(self, -1, "", style = aligns[col])
                self.text.append(stext)
                fg_sizer.Add(stext, sizes[col], wxEXPAND | wxADJUST_MINSIZE | wxALL, 2)

        ssizer.Add(fg_sizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        # Line

        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        ssizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        # Choice boxes

        choice_sizer = wxBoxSizer(wxHORIZONTAL)
        choice = []
        for f in self.types:
            choice.append(f[0])
        self.type_choice = wxChoice(self, self.ID_TYPE_TYPE, choices = choice)
        choice_sizer.Add(self.type_choice, 0, wxALL | wxALIGN_LEFT, 0)
        self.type_choice.SetSelection(0)

        choice_sizer.Add((0,0),1)
        
        choice = []
        for f in self.wheres:
            choice.append(f[0])
        self.where_choice = wxChoice(self, self.ID_WHERE_TYPE, choices = choice)
        choice_sizer.Add(self.where_choice, 0, wxALL | wxALIGN_RIGHT, 0)
        self.where_choice.SetSelection(0)

        ssizer.Add(choice_sizer, 0, wxEXPAND | wxALL, 5)

        # Events
        
        EVT_CHOICE(self, self.ID_WHERE_TYPE, self.OnChangeWhere)
        EVT_CHOICE(self, self.ID_TYPE_TYPE, self.OnChangeType)

        self.__update()

        self.SetSizer(ssizer)
        self.SetAutoLayout(True)
        ssizer.Fit(self)

    def OnChangeWhere(self, event):
        self.where = self.wheres[self.where_choice.GetSelection()][1]
        self.__update()

    def OnChangeType(self, event):
        self.average = self.types[self.type_choice.GetSelection()][1]
        self.__update()

    def __update(self):
        
        total = [0,0,0,0,0,0]
        self.__calc_tables()


        matches = []
        for matchtype in self.table.keys():
            matches.append((matchtype, self.table[matchtype]))

        matches.sort(lambda x, y: y[1][0]-x[1][0])

        row = 0
        for (matchtype, trow) in matches:
            if self.average:
                texts =  [Terms.match_type[matchtype],
                          str(trow[0]),
                          "%1.2f" % (float(trow[1])/float(trow[0])),
                          "%1.2f" % (float(trow[2])/float(trow[0])),
                          "%1.2f" % (float(trow[3])/float(trow[0])),
                          ("%1.2f - %1.2f" % (float(trow[4])/float(trow[0]),
                                              float(trow[5])/float(trow[0]))),
                          "%1.2f" % (float(trow[4]-trow[5])/float(trow[0])),
                          "%1.2f" % (float(3*trow[1]+trow[2])/float(trow[0]))]
            else:
                texts =  [Terms.match_type[matchtype],
                          str(trow[0]),
                          str(trow[1]),
                          str(trow[2]),
                          str(trow[3]),
                          ("%d - %d" % (trow[4], trow[5])),
                          signedstr(trow[4]-trow[5]),
                          str(3*trow[1]+trow[2])]
            col = 0          
            for t in texts:
                self.text[row*8+col].SetLabel(t)
                col += 1

            for i in range(6):
                total[i] += trow[i]

            row += 1

        
        if self.average:
            texts =  [_("All matches"),
                      str(total[0]),
                      "%1.2f" % (float(total[1])/float(total[0])),
                      "%1.2f" % (float(total[2])/float(total[0])),
                      "%1.2f" % (float(total[3])/float(total[0])),
                      ("%1.2f - %1.2f" % (float(total[4])/float(total[0]),
                                          float(total[5])/float(total[0]))),
                      "%1.2f" % (float(total[4]-total[5])/float(total[0])),
                      "%1.2f" % (float(3*total[1]+total[2])/float(total[0]))]
        else:
            texts =  [_("All matches"),
                      str(total[0]),
                      str(total[1]),
                      str(total[2]),
                      str(total[3]),
                      ("%d - %d" % (total[4], total[5])),
                      signedstr(total[4]-total[5]),
                      str(3*total[1]+total[2])]
        col = 0          
        for t in texts:
            self.text[row*8+col].SetLabel(t)
            col += 1
        row += 1

        for r in range(row, self.no):
            for col in range(8):
                self.text[r*8+col].SetLabel("")

        self.Layout()
 

    def __calc_tables(self):
        my_team = CommonData.get_current_teamid()
        self.table={}
        for (mdate, mid, mtype, home_team, away_team, score, status, has_md) in self.all_matches:
            if mtype == None:
                continue
            if status == "FINISHED":
                if home_team[0] == my_team:
                    (my_score, their_score) = score
                    where = _("Home")
                else:
                    (their_score, my_score) = score
                    where = _("Away")

                if not where in self.where:
                    continue

                if not self.table.has_key(mtype):
                    self.table[mtype]=[0, 0, 0, 0, 0, 0]
                    # (games, won, draw, lost, scored, against)
                    

                self.table[mtype][0] += 1  
                self.table[mtype][4] += my_score
                self.table[mtype][5] += their_score

                if my_score > their_score:
                    self.table[mtype][1] += 1
                elif my_score == their_score:
                    self.table[mtype][2] += 1
                else:
                    self.table[mtype][3] += 1

class ResultPanel(wxPanel):
    ID_GAME_TYPE = 1345
    ID_RESULT_TYPE = 1346
    ID_WHERE_TYPE = 1347

    def __init__(self, parent, ID, all_matches):
        wxPanel.__init__(self, parent, ID)

        ssizer = wxBoxSizer(wxVERTICAL)

        self.all_matches = all_matches

        self.match_img = Images.Images.get_bitmap_from_file("matchinfo.png")
        
        self.no = config.get("Statistics","stat items")
        self.filters = [[_("All matches"), range(13)],
                        [_("Competition match"), [1,2,3]],
                        [_("Friendly"), [4,5,8,9]],
                        [_("National friendly"), [4,5]],
                        [_("International friendly"), [8,9]],
                        [Terms.match_type[1], [1]],
                        [Terms.match_type[2], [2]],
                        [Terms.match_type[3], [3]],
                        [Terms.match_type[4], [4]],
                        [Terms.match_type[5], [5]],
                        [Terms.match_type[8], [8]],
                        [Terms.match_type[9], [9]]]

        self.filter = self.filters[0][1]

                        
        self.functions = [[_("Most total goals"), self.get_most_goals],
                          [_("Least total goals"), self.get_least_goals],
                          [_("Largest win"), self.get_largest_win],
                          [_("Largest loss"), self.get_largest_loss],
                          [_("Most goals scored"), self.get_most_scored],
                          [_("Most goals against"), self.get_most_against]]

        self.function = self.functions[0][1]

        self.wheres = [[_("All matches"), [_("Home"), _("Away")]],
                      [_("Home"), [_("Home")]],
                      [_("Away"), [_("Away")]]]

        self.where = self.wheres[0][1]

        fg_sizer = wxFlexGridSizer(cols = 5, vgap = 0, hgap = 0)
        for i in range(5):
            fg_sizer.AddGrowableCol(i)

        aligns=[wxALIGN_CENTER] + [wxALIGN_LEFT]*3 + [wxALIGN_CENTER]
        sizes = [0]*3 + [1, 0] 
        i = 0
        for h in [_("Result"), _("Opponent"), _("Date"), _("Match type"), _("Match details")]:
            stext = wxStaticText(self, -1, h, style = aligns[i])
            SetTextBold(stext)
            fg_sizer.Add(stext, sizes[i], wxEXPAND | wxADJUST_MINSIZE | wxALL, 5)
            i += 1

        self.buttonmapping = {}
        self.buttonlist = []
        self.text=[]
        for row in range(self.no):
            for col in range(4): 
                stext = wxStaticText(self, -1, "", style = aligns[col])
                self.text.append(stext)
                fg_sizer.Add(stext, sizes[col], wxEXPAND | wxADJUST_MINSIZE | wxALL, 2)
            b_id = wxNewId()
            widget = wxBitmapButton(self, b_id, self.match_img, style = wxBU_EXACTFIT)
            self.buttonlist.append(b_id)
            self.buttonmapping[b_id]=None
            fg_sizer.Add(widget, sizes[4], wxEXPAND | wxALL, 0)
            EVT_BUTTON(self, b_id, self.OnMatchDetails)

        ssizer.Add(fg_sizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        # Line

        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        ssizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        # Choice boxes
        
        choice_sizer = wxBoxSizer(wxHORIZONTAL)

        choice = []
        for f in self.functions:
            choice.append(f[0])
        self.result_choice = wxChoice(self, ResultPanel.ID_RESULT_TYPE, choices = choice)
        choice_sizer.Add(self.result_choice, 0, wxALL, 0)
        self.result_choice.SetSelection(0)

        choice_sizer.Add((0,0),1)
        
        choice = []
        for f in self.filters:
            choice.append(f[0])
        self.game_choice = wxChoice(self, ResultPanel.ID_GAME_TYPE, choices = choice)
        choice_sizer.Add(self.game_choice, 0, wxALL | wxALIGN_CENTER, 0)
        self.game_choice.SetSelection(0)

        choice_sizer.Add((0,0),1)
        
        choice = []
        for f in self.wheres:
            choice.append(f[0])
        self.where_choice = wxChoice(self, ResultPanel.ID_WHERE_TYPE, choices = choice)
        choice_sizer.Add(self.where_choice, 0, wxALL | wxALIGN_RIGHT, 0)
        self.where_choice.SetSelection(0)
        
        ssizer.Add(choice_sizer, 0, wxEXPAND | wxALL, 5)

        # Events
        
        EVT_CHOICE(self, ResultPanel.ID_RESULT_TYPE, self.OnChangeResult)
        EVT_CHOICE(self, ResultPanel.ID_GAME_TYPE, self.OnChangeGame)
        EVT_CHOICE(self, ResultPanel.ID_WHERE_TYPE, self.OnChangeWhere)

        self.__update()
        self.SetSizer(ssizer)
        self.SetAutoLayout(True)
        ssizer.Fit(self)

    def OnMatchDetails(self, event):
        eid = event.GetId()
        match_id = self.buttonmapping[eid]
        if match_id is not None:
            dlg = Matchdialogs.MatchDetailsDialog(self, match_id)
            if not dlg.no_data:
                ret = dlg.Show()
            
    
    def OnChangeResult(self, event):
        self.function = self.functions[self.result_choice.GetSelection()][1]
        self.__update()

    def OnChangeGame(self, event):
        self.filter = self.filters[self.game_choice.GetSelection()][1]
        self.__update()

    def OnChangeWhere(self, event):
        self.where = self.wheres[self.where_choice.GetSelection()][1]
        self.__update()

    def __update(self):
        self.__extract_results()
        matches=self.function(self.no)
        row = 0
        for (mdate, mid, mtype, my_score, their_score, opponent, where) in matches:
            col=0
            texts = [str(my_score) + " - " + str(their_score),
                     opponent + " (" + where + ")",
                     time.strftime("%d %b %Y", time.localtime(mdate)),
                     Terms.match_type[mtype]]

            for t in texts:
                self.text[row*4+col].SetLabel(t)
                col+=1
            self.buttonmapping[self.buttonlist[row]]=mid
            row += 1

        for r in range(row, self.no):
            for col in range(4):
                self.text[r*4+col].SetLabel("")
                self.buttonmapping[self.buttonlist[r]]=None

        self.Layout()

        
    def __extract_results(self):
        my_team = CommonData.get_current_teamid()
        self.games=[]
        for (mdate, mid, mtype, home_team, away_team, score, status, has_md) in self.all_matches:
            if mtype == None:
                continue
            if status == "FINISHED":
                if home_team[0] == my_team:
                    (my_score, their_score) = score
                    where = _("Home")
                    opponent = away_team[1]
                else:
                    (their_score, my_score) = score
                    where = _("Away")
                    opponent = home_team[1]

                if where in self.where:
                    if mtype in self.filter:
                        self.games.append((mdate, mid, mtype, my_score, their_score, opponent, where))
    
    def get_largest_loss(self, no):
        self.games.sort(lambda x, y: 100*((x[3]-x[4]) - (y[3] - y[4])) + (x[3]-y[3]))
        return self.games[0:no]
        
    def get_largest_win(self, no):
        self.games.sort(lambda x, y: 100*((x[4]-x[3]) - (y[4] - y[3])) + (y[3]-x[3]))
        return self.games[0:no]

    def get_least_goals(self, no):
        self.games.sort(lambda x, y: 100*((x[4]+x[3]) - (y[4] + y[3])) + (x[3]-y[3]))
        return self.games[0:no]

    def get_most_goals(self, no):
        self.games.sort(lambda x, y: 100*((y[4]+y[3]) - (x[4] + x[3])) + (y[3]-x[3]))
        return self.games[0:no]

    def get_most_scored(self, no):
        self.games.sort(lambda x, y: 100*((y[3]) - (x[3])) + (y[4]-x[4]))
        return self.games[0:no]

    def get_most_against(self, no):
        self.games.sort(lambda x, y: 100*((y[4]) - (x[4])) + (y[3]-x[3]))
        return self.games[0:no]

class CommonPanel(wxPanel):
    ID_GAME_TYPE = 1355
    ID_RESULT_TYPE = 1356
    ID_WHERE_TYPE = 1357

    def __init__(self, parent, ID, all_matches):
        wxPanel.__init__(self, parent, ID)

        ssizer = wxBoxSizer(wxVERTICAL)

        self.all_matches = all_matches

        self.no = config.get("Statistics","stat items")
        self.filters = [[_("All matches"), range(13)],
                        [_("Competition match"), [1,2,3]],
                        [_("Friendly"), [4,5,8,9]],
                        [_("National friendly"), [4,5]],
                        [_("International friendly"), [8,9]],
                        [Terms.match_type[1], [1]],
                        [Terms.match_type[2], [2]],
                        [Terms.match_type[3], [3]],
                        [Terms.match_type[4], [4]],
                        [Terms.match_type[5], [5]],
                        [Terms.match_type[8], [8]],
                        [Terms.match_type[9], [9]]]

        self.filter = self.filters[0][1]

                   
#        f = function(getattr(self, func))
        self.functions = [[_("Score"), self.get_score],
                          [_("Winning score"), self.get_wins],
                          [_("Draws"), self.get_draws],
                          [_("Losing score"), self.get_losses],
                          [_("Goals scored"), self.get_scored],
                          [_("Goals against"), self.get_against]]

        self.function = self.functions[0][1]

        self.wheres = [[_("All matches"), [_("Home"), _("Away")]],
                      [_("Home"), [_("Home")]],
                      [_("Away"), [_("Away")]]]

        self.where = self.wheres[0][1]

        fg_sizer = wxFlexGridSizer(cols = 2, vgap = 0, hgap = 0)
        for i in range(2):
            fg_sizer.AddGrowableCol(i)

        aligns=[wxALIGN_CENTER] + [wxALIGN_RIGHT]
        sizes = [0]*2
        i = 0
        for h in [_("Score"), _("Times")]:
            stext = wxStaticText(self, -1, h, style = aligns[i])
            SetTextBold(stext)
            fg_sizer.Add(stext, sizes[i], wxEXPAND | wxADJUST_MINSIZE | wxALL, 5)
            i += 1

        self.text=[]
        for row in range(self.no):
            for col in range(2): 
                stext = wxStaticText(self, -1, "", style = aligns[col])
                self.text.append(stext)
                fg_sizer.Add(stext, sizes[col], wxEXPAND |  wxADJUST_MINSIZE | wxALL, 2)

        ssizer.Add(fg_sizer, 0, wxALIGN_CENTRE | wxALL, 0)

        # Line

        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        ssizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        # Choice boxes
        
        choice_sizer = wxBoxSizer(wxHORIZONTAL)

        choice = []
        for f in self.functions:
            choice.append(f[0])
        self.result_choice = wxChoice(self, ResultPanel.ID_RESULT_TYPE, choices = choice)
        choice_sizer.Add(self.result_choice, 0, wxALL, 0)
        self.result_choice.SetSelection(0)

        choice_sizer.Add((0,0),1)
        
        choice = []
        for f in self.filters:
            choice.append(f[0])
        self.game_choice = wxChoice(self, ResultPanel.ID_GAME_TYPE, choices = choice)
        choice_sizer.Add(self.game_choice, 0, wxALL | wxALIGN_CENTER, 0)
        self.game_choice.SetSelection(0)

        choice_sizer.Add((0,0),1)
        
        choice = []
        for f in self.wheres:
            choice.append(f[0])
        self.where_choice = wxChoice(self, ResultPanel.ID_WHERE_TYPE, choices = choice)
        choice_sizer.Add(self.where_choice, 0, wxALL | wxALIGN_RIGHT, 0)
        self.where_choice.SetSelection(0)
        
        ssizer.Add(choice_sizer, 0, wxEXPAND | wxALL, 5)

        # Events
        
        EVT_CHOICE(self, ResultPanel.ID_RESULT_TYPE, self.OnChangeResult)
        EVT_CHOICE(self, ResultPanel.ID_GAME_TYPE, self.OnChangeGame)
        EVT_CHOICE(self, ResultPanel.ID_WHERE_TYPE, self.OnChangeWhere)

        self.__update()
        self.SetSizer(ssizer)
        self.SetAutoLayout(True)
        ssizer.Fit(self)

    def OnChangeResult(self, event):
        self.function = self.functions[self.result_choice.GetSelection()][1]
        self.__update()

    def OnChangeGame(self, event):
        self.filter = self.filters[self.game_choice.GetSelection()][1]
        self.__update()

    def OnChangeWhere(self, event):
        self.where = self.wheres[self.where_choice.GetSelection()][1]
        self.__update()

    def __update(self):
        self.__extract_results()
        matches=self.function(self.no)
        row = 0
        for (frequency, result) in matches:
            col=0
            texts = [result,
                     ngettext("%d time", "%d times", frequency) % frequency]

            for t in texts:
                self.text[row*2+col].SetLabel(t)
                col+=1
            row += 1

        for r in range(row, self.no):
            for col in range(2):
                self.text[r*2+col].SetLabel("")

        self.Layout()

        
    def __extract_results(self):
        my_team = CommonData.get_current_teamid()
        self.games=[]
        for (mdate, mid, mtype, home_team, away_team, score, status, has_md) in self.all_matches:
            if mtype == None:
                continue
            if status == "FINISHED":
                if home_team[0] == my_team:
                    (my_score, their_score) = score
                    where = _("Home")
                    opponent = away_team[1]
                else:
                    (their_score, my_score) = score
                    where = _("Away")
                    opponent = home_team[1]

                if where in self.where:
                    if mtype in self.filter:
                        self.games.append((my_score, their_score))
    
    def get_score(self, no):
        hist_data={}
        for (my_score, their_score) in self.games:
            score = "%d - %d" % (my_score, their_score)
            if not hist_data.has_key(score):
                hist_data[score]=0
            hist_data[score] += 1

        hist_result = []
        for score in hist_data.keys():
            hist_result.append((hist_data[score], score))

        hist_result.sort()
        hist_result.reverse()
        
        return hist_result[0:no]
    
    def get_wins(self, no):
        hist_data={}
        for (my_score, their_score) in self.games:
            if my_score <= their_score:
                continue
            score = "%d - %d" % (my_score, their_score)
            if not hist_data.has_key(score):
                hist_data[score]=0
            hist_data[score] += 1

        hist_result = []
        for score in hist_data.keys():
            hist_result.append((hist_data[score], score))

        hist_result.sort()
        hist_result.reverse()
        
        return hist_result[0:no]

    def get_draws(self, no):
        hist_data={}
        for (my_score, their_score) in self.games:
            if my_score != their_score:
                continue
            score = "%d - %d" % (my_score, their_score)
            if not hist_data.has_key(score):
                hist_data[score]=0
            hist_data[score] += 1

        hist_result = []
        for score in hist_data.keys():
            hist_result.append((hist_data[score], score))

        hist_result.sort()
        hist_result.reverse()
        
        return hist_result[0:no]
    
    def get_losses(self, no):
        hist_data={}
        for (my_score, their_score) in self.games:
            if my_score >= their_score:
                continue
            score = "%d - %d" % (my_score, their_score)
            if not hist_data.has_key(score):
                hist_data[score]=0
            hist_data[score] += 1

        hist_result = []
        for score in hist_data.keys():
            hist_result.append((hist_data[score], score))

        hist_result.sort()
        hist_result.reverse()
        
        return hist_result[0:no]
    
    def get_scored(self, no):
        hist_data={}
        for (my_score, their_score) in self.games:
            score = str(my_score)
            if not hist_data.has_key(score):
                hist_data[score]=0
            hist_data[score] += 1

        hist_result = []
        for score in hist_data.keys():
            hist_result.append((hist_data[score], score))

        hist_result.sort()
        hist_result.reverse()
        
        return hist_result[0:no]
    
    def get_against(self, no):
        hist_data={}
        for (my_score, their_score) in self.games:
            score = str(their_score)
            if not hist_data.has_key(score):
                hist_data[score]=0
            hist_data[score] += 1

        hist_result = []
        for score in hist_data.keys():
            hist_result.append((hist_data[score], score))

        hist_result.sort()
        hist_result.reverse()
        
        return hist_result[0:no]

class PlayerPanel(wxPanel):
    ID_GAME_TYPE = 1377

    def __init__(self, parent, ID, all_matches):
        wxPanel.__init__(self, parent, ID)

        self.no = config.get("Statistics","stat items")
        self.filters = [[_("All matches"), range(10)],
                        [_("Competition match"), [1,2,3]],
                        [_("Friendly"), [4,5,8,9]],
                        [_("National friendly"), [4,5]],
                        [_("International friendly"), [8,9]],
                        [Terms.match_type[1], [1]],
                        [Terms.match_type[2], [2]],
                        [Terms.match_type[3], [3]],
                        [Terms.match_type[4], [4]],
                        [Terms.match_type[5], [5]],
                        [Terms.match_type[8], [8]],
                        [Terms.match_type[9], [9]]]

        self.filter = self.filters[0][1]
        self.all_matches = all_matches

        ssizer = wxBoxSizer(wxVERTICAL)

        fg_sizer = wxFlexGridSizer(cols = 2, vgap = 0, hgap = 0)
        for i in range(2):
            fg_sizer.AddGrowableCol(i)

        aligns=[wxALIGN_LEFT] + [wxALIGN_RIGHT]
        sizes = [0]*2
        i = 0
        for h in [_("Player"), _("Games")]:
            stext = wxStaticText(self, -1, h, style = aligns[i])
            SetTextBold(stext)
            fg_sizer.Add(stext, sizes[i], wxEXPAND | wxADJUST_MINSIZE | wxALL, 5)
            i += 1

        self.text=[]
        for row in range(self.no):
            for col in range(2): 
                stext = wxStaticText(self, -1, "", style = aligns[col])
                self.text.append(stext)
                fg_sizer.Add(stext, sizes[col], wxEXPAND |  wxADJUST_MINSIZE | wxALL, 2)

        ssizer.Add(fg_sizer, 0, wxALIGN_CENTRE | wxALL, 0)

        # Line

        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        ssizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        # Choice boxes
        
        
        choice = []
        for f in self.filters:
            choice.append(f[0])
        self.game_choice = wxChoice(self, ResultPanel.ID_GAME_TYPE, choices = choice)
        ssizer.Add(self.game_choice, 0, wxALL | wxALIGN_RIGHT, 5)
        self.game_choice.SetSelection(0)

        # Events
        
        EVT_CHOICE(self, ResultPanel.ID_GAME_TYPE, self.OnChangeGame)
        
        self.__extract_results()
        self.__update()
        self.SetSizer(ssizer)
        self.SetAutoLayout(True)
        ssizer.Fit(self)
        
    def OnChangeGame(self, event):
        self.filter = self.filters[self.game_choice.GetSelection()][1]
        self.__update()

    def __update(self):

        gamelist = []
        for pid in self.playerlist.keys():
            mcount = 0
            for mtype in self.filter:
                mcount += self.playerlist[pid][1][mtype]

            if mcount > 0:
                gamelist.append((mcount, self.playerlist[pid][0]))
            
        gamelist.sort()
        gamelist.reverse()
        gamelist = gamelist[0:self.no]

        row = 0
        for (frequency, name) in gamelist:
            col=0
            texts = [name,
                     ngettext("%d game", "%d games", frequency) % frequency]

            for t in texts:
                self.text[row*2+col].SetLabel(t)
                col+=1
            row += 1

        for r in range(row, self.no):
            for col in range(2):
                self.text[r*2+col].SetLabel("")

        self.Layout()
        
    def __extract_results(self):
        my_team = CommonData.get_current_teamid()
        self.playerlist = {}
        for (mdate, mid, mtype, home_team, away_team, score, status, has_md) in self.all_matches:
            if mtype == None and mid < 0: 
                continue
            if status == "FINISHED":
                lineup = StatHelper.get_used_players(mid)
                for pid in lineup:
                    if not self.playerlist.has_key(pid):
                        self.playerlist[pid]=[StatHelper.get_player_name(pid, mid),
                                                  [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]]
                    self.playerlist[pid][1][mtype] += 1

class PlayerGoalPanel(wxPanel):
    ID_GAME_TYPE = 1387

    def __init__(self, parent, ID, all_matches):
        wxPanel.__init__(self, parent, ID)

        self.no = config.get("Statistics","stat items")
        self.filters = [[_("All matches"), range(10)],
                        [_("Competition match"), [1,2,3]],
                        [_("Friendly"), [4,5,8,9]],
                        [_("National friendly"), [4,5]],
                        [_("International friendly"), [8,9]],
                        [Terms.match_type[1], [1]],
                        [Terms.match_type[2], [2]],
                        [Terms.match_type[3], [3]],
                        [Terms.match_type[4], [4]],
                        [Terms.match_type[5], [5]],
                        [Terms.match_type[8], [8]],
                        [Terms.match_type[9], [9]]]

        self.filter = self.filters[0][1]
        self.all_matches = all_matches

        ssizer = wxBoxSizer(wxVERTICAL)

        fg_sizer = wxFlexGridSizer(cols = 2, vgap = 0, hgap = 0)
        for i in range(2):
            fg_sizer.AddGrowableCol(i)

        aligns=[wxALIGN_LEFT] + [wxALIGN_RIGHT]
        sizes = [0]*2
        i = 0
        for h in [_("Player"), _("Goals")]:
            stext = wxStaticText(self, -1, h, style = aligns[i])
            SetTextBold(stext)
            fg_sizer.Add(stext, sizes[i], wxEXPAND | wxADJUST_MINSIZE | wxALL, 5)
            i += 1

        self.text=[]
        for row in range(self.no):
            for col in range(2): 
                stext = wxStaticText(self, -1, "", style = aligns[col])
                self.text.append(stext)
                fg_sizer.Add(stext, sizes[col], wxEXPAND |  wxADJUST_MINSIZE | wxALL, 2)

        ssizer.Add(fg_sizer, 0, wxALIGN_CENTRE | wxALL, 0)

        # Line

        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        ssizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        # Choice boxes
        
        
        choice = []
        for f in self.filters:
            choice.append(f[0])
        self.game_choice = wxChoice(self, ResultPanel.ID_GAME_TYPE, choices = choice)
        ssizer.Add(self.game_choice, 0, wxALL | wxALIGN_RIGHT, 5)
        self.game_choice.SetSelection(0)

        # Events
        
        EVT_CHOICE(self, ResultPanel.ID_GAME_TYPE, self.OnChangeGame)
        
        self.__extract_results()
        self.__update()
        self.SetSizer(ssizer)
        self.SetAutoLayout(True)
        ssizer.Fit(self)
        
    def OnChangeGame(self, event):
        self.filter = self.filters[self.game_choice.GetSelection()][1]
        self.__update()

    def __update(self):

        gamelist = []
        for pid in self.playerlist.keys():
            mcount = 0
            for mtype in self.filter:
                mcount += self.playerlist[pid][1][mtype]

            if mcount > 0:
                gamelist.append((mcount, self.playerlist[pid][0]))
            
        gamelist.sort()
        gamelist.reverse()
        gamelist = gamelist[0:self.no]

        row = 0
        for (frequency, name) in gamelist:
            col=0
            texts = [name,
                     ngettext("%d goal", "%d goals", frequency) % frequency]

            for t in texts:
                self.text[row*2+col].SetLabel(t)
                col+=1
            row += 1

        for r in range(row, self.no):
            for col in range(2):
                self.text[r*2+col].SetLabel("")

        self.Layout()
        
    def __extract_results(self):
        my_team = CommonData.get_current_teamid()
        self.playerlist = {}
        for (mdate, mid, mtype, home_team, away_team, score, status, has_md) in self.all_matches:
            if mtype == None and mid < 0: 
                continue
            if status == "FINISHED":
                scorers = StatHelper.get_scorers(mid)
                for pid in scorers:
                    if not self.playerlist.has_key(pid):
                        self.playerlist[pid]=[StatHelper.get_player_name(pid, mid),
                                                  [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]]
                    self.playerlist[pid][1][mtype] += 1

class AttendancePanel(wxPanel):
    ID_GAME_TYPE = 1345
    ID_RESULT_TYPE = 1346
    ID_WHERE_TYPE = 1347

    def __init__(self, parent, ID, all_matches):
        wxPanel.__init__(self, parent, ID)

        ssizer = wxBoxSizer(wxVERTICAL)

        self.all_matches = all_matches


        self.__extract_results()

        self.match_img = Images.Images.get_bitmap_from_file("matchinfo.png")
        
        self.no = config.get("Statistics","stat items")
        self.filters = [[_("All matches"), range(13)],
                        [_("Competition match"), [1,2,3]],
                        [_("Friendly"), [4,5,8,9]],
                        [_("National friendly"), [4,5]],
                        [_("International friendly"), [8,9]],
                        [Terms.match_type[1], [1]],
                        [Terms.match_type[2], [2]],
                        [Terms.match_type[3], [3]],
                        [Terms.match_type[4], [4]],
                        [Terms.match_type[5], [5]],
                        [Terms.match_type[8], [8]],
                        [Terms.match_type[9], [9]]]

        self.filter = self.filters[0][1]

                        
        self.functions = [[_("Largest attendance"), self.get_largest],
                          [_("Smallest attendance"), self.get_smallest]]

        self.function = self.functions[0][1]

        self.wheres = [[_("All matches"), [_("Home"), _("Away")]],
                      [_("Home"), [_("Home")]],
                      [_("Away"), [_("Away")]]]

        self.where = self.wheres[1][1]

        fg_sizer = wxFlexGridSizer(cols = 5, vgap = 0, hgap = 0)
        for i in range(5):
            fg_sizer.AddGrowableCol(i)

        aligns=[wxALIGN_RIGHT] + [wxALIGN_LEFT]*3 + [wxALIGN_CENTER]
        sizes = [0]*3 + [1, 0] 
        i = 0
        for h in [_("Attendance"), _("Opponent"), _("Date"), _("Match type"), _("Match details")]:
            stext = wxStaticText(self, -1, h, style = aligns[i])
            SetTextBold(stext)
            fg_sizer.Add(stext, sizes[i], wxEXPAND | wxADJUST_MINSIZE | wxALL, 5)
            i += 1

        self.buttonmapping = {}
        self.buttonlist = []
        self.text=[]
        for row in range(self.no):
            for col in range(4): 
                stext = wxStaticText(self, -1, "", style = aligns[col])
                self.text.append(stext)
                fg_sizer.Add(stext, sizes[col], wxEXPAND | wxADJUST_MINSIZE | wxALL, 2)
            b_id = wxNewId()
            widget = wxBitmapButton(self, b_id, self.match_img, style = wxBU_EXACTFIT)
            self.buttonlist.append(b_id)
            self.buttonmapping[b_id]=None
            fg_sizer.Add(widget, sizes[4], wxEXPAND | wxALL, 0)
            EVT_BUTTON(self, b_id, self.OnMatchDetails)

        ssizer.Add(fg_sizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        # Line

        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        ssizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        # Choice boxes
        
        choice_sizer = wxBoxSizer(wxHORIZONTAL)

        choice = []
        for f in self.functions:
            choice.append(f[0])
        self.result_choice = wxChoice(self, ResultPanel.ID_RESULT_TYPE, choices = choice)
        choice_sizer.Add(self.result_choice, 0, wxALL, 0)
        self.result_choice.SetSelection(0)

        choice_sizer.Add((0,0),1)
        
        choice = []
        for f in self.filters:
            choice.append(f[0])
        self.game_choice = wxChoice(self, ResultPanel.ID_GAME_TYPE, choices = choice)
        choice_sizer.Add(self.game_choice, 0, wxALL | wxALIGN_CENTER, 0)
        self.game_choice.SetSelection(0)

        choice_sizer.Add((0,0),1)
        
        choice = []
        for f in self.wheres:
            choice.append(f[0])
        self.where_choice = wxChoice(self, ResultPanel.ID_WHERE_TYPE, choices = choice)
        choice_sizer.Add(self.where_choice, 0, wxALL | wxALIGN_RIGHT, 0)
        self.where_choice.SetSelection(1)
        
        ssizer.Add(choice_sizer, 0, wxEXPAND | wxALL, 5)

        # Events
        
        EVT_CHOICE(self, ResultPanel.ID_RESULT_TYPE, self.OnChangeResult)
        EVT_CHOICE(self, ResultPanel.ID_GAME_TYPE, self.OnChangeGame)
        EVT_CHOICE(self, ResultPanel.ID_WHERE_TYPE, self.OnChangeWhere)

        self.__update()
        self.SetSizer(ssizer)
        self.SetAutoLayout(True)
        ssizer.Fit(self)

    def OnMatchDetails(self, event):
        eid = event.GetId()
        match_id = self.buttonmapping[eid]
        if match_id is not None:
            dlg = Matchdialogs.MatchDetailsDialog(self, match_id)
            if not dlg.no_data:
                ret = dlg.Show()
            
    
    def OnChangeResult(self, event):
        self.function = self.functions[self.result_choice.GetSelection()][1]
        self.__update()

    def OnChangeGame(self, event):
        self.filter = self.filters[self.game_choice.GetSelection()][1]
        self.__update()

    def OnChangeWhere(self, event):
        self.where = self.wheres[self.where_choice.GetSelection()][1]
        self.__update()

    def __update(self):
        self.__extract_matches()
        matches=self.function(self.no)
        row = 0
        for (mdate, mid, mtype, attendance, opponent, where) in matches:
            col=0
            texts = [str(attendance)+"  ",
                     opponent + " (" + where + ")",
                     time.strftime("%d %b %Y", time.localtime(mdate)),
                     Terms.match_type[mtype]]

            for t in texts:
                self.text[row*4+col].SetLabel(t)
                col+=1
            self.buttonmapping[self.buttonlist[row]]=mid
            row += 1

        for r in range(row, self.no):
            for col in range(4):
                self.text[r*4+col].SetLabel("")
                self.buttonmapping[self.buttonlist[r]]=None

        self.Layout()

        
    def __extract_results(self):
        my_team = CommonData.get_current_teamid()
        self.all_games=[]
        for (mdate, mid, mtype, home_team, away_team, score, status, has_md) in self.all_matches:
            if mtype == None:
                continue
            if status == "FINISHED":
                attendance = XMLParse.get(["match details", mid],["Match.Arena.SoldTotal"])
                if home_team[0] == my_team:
                    (my_score, their_score) = score
                    where = _("Home")
                    opponent = away_team[1]
                else:
                    (their_score, my_score) = score
                    where = _("Away")
                    opponent = home_team[1]

                self.all_games.append((mdate, mid, mtype, attendance, opponent, where))

    def __extract_matches(self):
        self.games=[]
        for (mdate, mid, mtype, attendance, opponent, where) in self.all_games:
            if where in self.where:
                if mtype in self.filter:
                    self.games.append((mdate, mid, mtype, attendance, opponent, where))
                    
    
    def get_smallest(self, no):
        self.games.sort(lambda x, y: (x[3]) - (y[3]))
        return self.games[0:no]
        
    def get_largest(self, no):
        self.games.sort(lambda x, y: (y[3]) - (x[3]))
        return self.games[0:no]
