# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import Images
import Live
import Database
import MatchInfo
import TeamList
import Matchdialogs

import re
import string
import time
import math

from Common import *
from Commonwx import *
from wxPython.wx import *
from Config import config

from wxPython.lib.buttons import wxGenBitmapToggleButton
from wxPython.lib.scrolledpanel import wxScrolledPanel

class MyBitmapToggleButton(wxGenBitmapToggleButton):

    def _init_(self, parent, id, bitmap):
        wxGenBitmapButton.__init__(self, parent, id, bitmap)
        
    def SetBackgroundColour(self, colour):
        wxPyControl.SetBackgroundColour(self, colour)
        self.faceDnClr = colour

        
class MatchViewerPage(wxPanel):
    MAX_NO_MATCHES = 10
    UPDATE_INTERVAL = 120.0
    CHECK_INTERVAL = 600.0

    ID_ADD_MATCH = 386
    ID_REMOVE_ALL = 387
    ID_TIMER = 388
    ID_TEAMS = 389
    ID_ACK = 390
    
    def __init__(self, parent, ID):
        wxPanel.__init__(self, parent, ID)

        self.live = Live.Live()

        sizer = wxBoxSizer(wxHORIZONTAL)

        self.detailed_view = LivePanel(self, -1, self.live)
        sizer.Add(self.detailed_view, 1, wxEXPAND | wxALIGN_TOP | wxALL, 0)

        vsizer = wxBoxSizer(wxVERTICAL)

        ihsizer = wxBoxSizer(wxHORIZONTAL)

        button = wxButton(self, self.ID_REMOVE_ALL, _("Remove all"), style = wxBU_EXACTFIT)
        ihsizer.Add(button, 0, wxALIGN_LEFT | wxALL, 2)

        button = wxButton(self, self.ID_ADD_MATCH, _("Add match") + ":", style = wxBU_EXACTFIT)
        ihsizer.Add(button, 0, wxALIGN_LEFT | wxALL, 2)

        self.add_match_edit = wxTextCtrl(self, -1, size = (70, -1))
        self.add_match_edit.SetValue("")
        ihsizer.Add(self.add_match_edit, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        button = wxButton(self, self.ID_TEAMS, _("Teams"), style = wxBU_EXACTFIT)
        ihsizer.Add(button, 0, wxALIGN_LEFT | wxALL, 2)

        vsizer.Add(ihsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)
        
        jhsizer = wxBoxSizer(wxHORIZONTAL)
        button = wxButton(self, self.ID_ACK, _("Acknowledge all"), style = wxBU_EXACTFIT)
        button.Enable(False)
        self.btnAckAll = button
        jhsizer.Add(button, 0, wxALIGN_LEFT | wxALL, 2)
        vsizer.Add(jhsizer, 0, wxEXPAND | wxALIGN_CENTER | wxALL, 2)
        
        vsizer.Add((1, 10), 1, wxALL, 2)

        self.small_views = [None]*self.MAX_NO_MATCHES
        for i in range(self.MAX_NO_MATCHES):
            self.small_views[i] = LiveSmallPanel(self, -1, self.live)
            vsizer.Add(self.small_views[i], 1, wxEXPAND | wxALIGN_BOTTOM | wxALL, 2)

        sizer.Add(vsizer, 0, wxEXPAND | wxALIGN_TOP | wxALL, 5)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)
        
        self.viewed_matches = [None]*self.MAX_NO_MATCHES

        self.update_timer = wxTimer(self, self.ID_TIMER)

        EVT_TIMER(self, self.ID_TIMER, self.OnLiveUpdate)
        EVT_BUTTON(self, self.ID_ADD_MATCH, self.OnAddMatch)
        EVT_BUTTON(self, self.ID_REMOVE_ALL, self.OnRemoveAll)
        EVT_BUTTON(self, self.ID_TEAMS, self.OnTeams)
        EVT_BUTTON(self, self.ID_ACK, self.OnAck)
        
        self.update_timer.Start(5000)

    def AfterInit(self):
        path = "Mattricks.live_matches"
        ong_str = Database.databases.gets(None, False, [path])[path]
        if ong_str is not None:
            for mid in [int(mid) for mid in filter((lambda x : len(x) > 0), ong_str.split(","))]:
                self.TryAddMatch(mid, False)

    def OnTeams(self, event):
        tl=TeamList.TeamList(self)
        tl.Show()

    def OnAddMatch(self, event):
        mid = self.add_match_edit.GetValue()

        if config.get("Match viewer", "clear match-id"):
            self.add_match_edit.Clear()
            
        try:
            mid = int(mid)
            if mid < 1:
                raise ValueError
        except ValueError, err:
            self.__error_dlg(_("Match id needs to be a positive integer."))
            return

        self.TryAddMatch(mid)

    def OnAck(self, event):
        """Acknowledge all match events"""
        
        for match in self.small_views:
            match.event = false
            match.Update()
            
    def TryAddMatch(self, mid, add_live_match = True):
        if mid is None:
            return

        if mid in self.viewed_matches:
            self.__error_dlg(_("Match is already watched."))
            return

        res = None
        for i in range(self.MAX_NO_MATCHES):
            if self.viewed_matches[i] is None:
                res = i
                break

        if res is None:
            self.__error_dlg(_("All match slots are full."))
            return

        self.live.add_match(mid)
        m = self.live.matches.get(mid)
        if m is not None:
            self.viewed_matches[res] = mid
            self.small_views[res].ChangeMatch(mid)
            if self.detailed_view.match_id is None:
                self.detailed_view.ChangeMatch(mid)
            
            self.btnAckAll.Enable(True)

        self.__set_live_matches()

    def AddMatchFromTeamID(self, team_id):

        try:
            team_id = int(team_id)
            if team_id < 1:
                raise ValueError
        except ValueError, err:
            self.__error_dlg(_("Team id needs to be a positive integer."))
            return

        match_info=MatchInfo.CurrentMatches(team_id, force_update=True)
        if match_info.team_id is None:
            self.__error_dlg(_("Non-existing team."))
            return

        if match_info is None:
            self.__error_dlg(_("Problems with the Hattrick server."))
        else:
            self.TryAddMatch(match_info.GetNextMatch())


    def OnLiveUpdate(self, event):
        now = time.time()
        lf = self.live.last_fetched
        last_update = self.live.last_fetched_local

        if now - last_update > self.UPDATE_INTERVAL:
            update = False
            for mid, match in self.live.matches.items():
                if not match[5] and ((lf - match[0]) + (now - last_update)) >= 0.0:
                    update = True
                    break
            if update:
                if now - last_update > self.CHECK_INTERVAL:
                    self.live.get_all_events()

                    for cmid in filter((lambda x: x is not None), self.viewed_matches):
                        if not self.live.matches.has_key(cmid):
                            self.live.add_match(cmid)

                else:
                    self.live.get_new_events()

                for i in range(self.MAX_NO_MATCHES):
                    if self.viewed_matches[i] is not None:
                        self.small_views[i].Update()
                self.detailed_view.Update(False)
                self.__set_live_matches()
            else:
                self.detailed_view.Update(True)


    def SetDetailedMatch(self, match_id):
        if match_id is None or not match_id in self.viewed_matches:
            return

        self.detailed_view.ChangeMatch(match_id)

        if config.get("Match viewer", "copy match-id") == True:
            self.add_match_edit.SetValue(str(match_id))

    def OnRemoveAll(self, event):
        if config.get("Match viewer", "remove warning") == True:
            removeDialog = wxMessageDialog(self, _("Remove all matches?"),
                                           _("Remove match?"), wxYES_NO)
            ans = removeDialog.ShowModal()
            if ans == wxID_NO:
                return

        for i in range(self.MAX_NO_MATCHES):
            if self.viewed_matches[i] is not None:
                self.small_views[i].ChangeMatch(None)
                self.viewed_matches[i] = None
        self.detailed_view.ChangeMatch(None)

        self.btnAckAll.Enable(False)
        
        self.live.clear_all_matches()
        self.__set_live_matches()

    def RemoveMatch(self, match_id):
        if match_id is None or not match_id in self.viewed_matches:
            return

        if config.get("Match viewer", "remove warning") == True:
            removeDialog = wxMessageDialog(self, (_("Remove match %d?") % match_id),
                                           _("Remove match?"), wxYES_NO)
            ans = removeDialog.ShowModal()
            if ans == wxID_NO:
                return

        for i in range(self.MAX_NO_MATCHES):
            if self.viewed_matches[i] == match_id:
                self.viewed_matches[i] = None
                self.small_views[i].ChangeMatch(None)
                break

        if self.detailed_view.match_id == match_id:
            found = False
            for i in range(self.MAX_NO_MATCHES):
                if self.viewed_matches[i] is not None:
                    found = True
                    self.detailed_view.ChangeMatch(self.viewed_matches[i])

            if not found:
                self.detailed_view.ChangeMatch(None)
                self.btnAckAll.Enable(False)

        self.live.delete_match(match_id)
        self.__set_live_matches()

    def __error_dlg(self, msg):
        mdlg = wxMessageDialog(self, msg, _("Error"), wxOK | wxICON_ERROR)
        mdlg.ShowModal()
        mdlg.Destroy()

    def __set_live_matches(self):
        ongoing_matches = []
        for mid, match in self.live.matches.items():
            if not match[5]:
                ongoing_matches.append(mid)
        ong_str = string.join([str(mid) for mid in ongoing_matches], ",")
        Database.databases.sets(None, False, {"Mattricks.live_matches": ong_str})

class LivePanel(wxPanel):

    HOME_BUTTON = 800
    AWAY_BUTTON = 801
    MATCH_BUTTON = 802

    HOME_TEAM_COLOR = "#000066"
    AWAY_TEAM_COLOR = "#660000"

    def __init__(self, parent, ID, live):
        self.parent = parent
        self.match_id = None
        self.live = live
        self.displayed_events = 0
        self.report_texts = []
        self.goal_texts = {}
        self.book_texts = []
        self.ps_texts = []
        self.hometeam_id = None
        self.awayteam_id = None

        wxPanel.__init__(self, parent, ID)

        sizer = wxBoxSizer(wxVERTICAL)

        if wxPlatform == '__WXMSW__':
            pstyle = wxSTATIC_BORDER
        elif wxPlatform == '__WXGTK__':
            pstyle = wxRAISED_BORDER
        else:
            pstyle = wxSIMPLE_BORDER

        self.header_panel = wxPanel(self, -1, style=pstyle)
        self.header_panel.SetBackgroundColour("#E8E8E8")

        psizer = wxBoxSizer(wxHORIZONTAL)

        self.team_text = wxStaticText(self.header_panel, -1, "")
        SetTextFace(self.team_text, FACE_BOLD | FACE_BIG)
        psizer.Add(self.team_text, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        self.result_text = wxStaticText(self.header_panel, -1, "X - X", size = (75, -1),
                                        style = wxALIGN_RIGHT)
        SetTextFace(self.result_text, FACE_BOLD | FACE_BIG)
        psizer.Add(self.result_text, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        self.header_panel.SetSizer(psizer)
        self.header_panel.SetAutoLayout(True)
        psizer.Fit(self.header_panel)

        sizer.Add(self.header_panel, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        hsizer = wxBoxSizer(wxHORIZONTAL)

        self.ssizer = wxBoxSizer(wxVERTICAL)
        self.weather_img = None
        hsizer.Add(self.ssizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        self.minute_text = wxStaticText(self, -1, _("No match has been selected."))
        hsizer.Add(self.minute_text, 2, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        self.attendance_text = wxStaticText(self, -1, "", size = (100, -1), style = wxALIGN_LEFT)
        hsizer.Add(self.attendance_text, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        self.id_text = wxStaticText(self, -1, "", size = (100, -1), style = wxALIGN_RIGHT)
        hsizer.Add(self.id_text, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        sizer.Add(hsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        # Get panel ratios
        ratio=config.get("Match viewer", "panel ratios")
        ratiolist=string.split(ratio,":")

        # Important events

        sbox = wxStaticBox(self, -1, _("Important events"))
        ssizer = wxStaticBoxSizer(sbox, wxVERTICAL)

        self.imp_events_panel = wxScrolledPanel(self, -1)
        self.imp_events_panel.SetupScrolling(True, True)
        ssizer.Add(self.imp_events_panel, 1, wxEXPAND | wxALL, 3)

        self.star_img = Images.Images.get_bitmap_from_file("star.png")
        bsizer = wxBoxSizer(wxHORIZONTAL)
        self.homebutton= wxBitmapButton(self, self.HOME_BUTTON,
                                               self.star_img, style = wxBU_EXACTFIT)
        bsizer.Add(self.homebutton, 0, wxALL | wxALIGN_LEFT,1)
        bsizer.Add((1, 1), 1)

        self.matchbutton = wxButton(self, self.MATCH_BUTTON,
                                           _("Match details"), style = wxBU_EXACTFIT)

        bsizer.Add(self.matchbutton, 0, wxALL | wxALIGN_CENTER,1)
        bsizer.Add((1, 1), 1)

        self.awaybutton = wxBitmapButton(self, self.AWAY_BUTTON,
                                               self.star_img, style = wxBU_EXACTFIT)
        bsizer.Add(self.awaybutton, 0, wxALIGN_RIGHT | wxALL,1)
        ssizer.Add(bsizer, 0, wxEXPAND | wxALIGN_BOTTOM | wxALL, 0)

        sizer.Add(ssizer, int(ratiolist[0]), wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        # Match report

        sbox = wxStaticBox(self, -1, _("Match report"))
        ssizer = wxStaticBoxSizer(sbox, wxVERTICAL)

        self.match_report_panel = wxScrolledPanel(self, -1)
        self.match_report_panel.SetupScrolling(True, True)
        ssizer.Add(self.match_report_panel, 1, wxEXPAND | wxALL, 5)

        sizer.Add(ssizer, int(ratiolist[1]), wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        EVT_BUTTON(self, self.AWAY_BUTTON, self.OnAwayButton)
        EVT_BUTTON(self, self.HOME_BUTTON, self.OnHomeButton)
        EVT_BUTTON(self, self.MATCH_BUTTON, self.OnMatchButton)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        self.Enable(False)

    def ChangeMatch(self, match_id):
        self.match_id = match_id
        self.displayed_events = 0

        # Destroy all previous texts.
        for pid, (nametext, mintext) in self.goal_texts.items():
            nametext.Destroy()
            mintext.Destroy()
        for stext in self.report_texts + self.book_texts + self.ps_texts:
            stext.Destroy()
        self.report_texts = []
        self.book_texts = []
        self.goal_texts = {}
        self.ps_texts = []
        self.show_pen_shootout = False
        self.attendance_text.SetLabel("")
        self.__set_weather_image(None)

        # Set up new sizers for the important events.
        main_sizer = wxBoxSizer(wxVERTICAL)
        self.goal_sizers = []
        self.book_sizers = []
        self.pen_sizers = []
        i = 0
        hsizer = wxBoxSizer(wxHORIZONTAL)
        for i in range(2):
            side = [wxRIGHT, wxLEFT][i]
            vsizer = wxBoxSizer(wxVERTICAL)
            ss = wxFlexGridSizer(cols=2, vgap=0, hgap=5)
            ss.AddGrowableCol(0)
            ss.AddGrowableCol(1)
            vsizer.Add(ss, 0, wxEXPAND | wxALIGN_TOP | wxALL, 0)
            hsizer.Add(vsizer, 1, wxEXPAND | wxALIGN_TOP | side, 4)
            self.book_sizers.append(vsizer)
            self.goal_sizers.append(ss)
            i += 1
        main_sizer.Add(hsizer, 0, wxEXPAND | wxALIGN_TOP | wxBOTTOM, 3)
        ps_text = wxStaticText(self.imp_events_panel, -1, "", style = wxALIGN_LEFT)
        SetTextFace(ps_text, FACE_BOLD)
        main_sizer.Add(ps_text, 0, wxEXPAND | wxALIGN_CENTRE | wxTOP, 3)
        self.ps_texts.append(ps_text)
        i = 0
        hsizer = wxBoxSizer(wxHORIZONTAL)
        for i in range(2):
            side = [wxRIGHT, wxLEFT][i]
            vsizer = wxBoxSizer(wxVERTICAL)
            hsizer.Add(vsizer, 1, wxEXPAND | wxALIGN_TOP | side, 4)
            self.pen_sizers.append(vsizer)
        main_sizer.Add(hsizer, 0, wxEXPAND | wxALIGN_TOP | wxBOTTOM, 3)

        self.imp_events_panel.SetSizer(main_sizer, True)
        self.imp_events_panel.SetAutoLayout(True)
        main_sizer.Fit(self.imp_events_panel)

        # Set up new sizer for the match report.
        self.mrp_sizer = wxFlexGridSizer(cols=3, vgap=5, hgap=0)
        self.mrp_sizer.AddGrowableCol(2)
        self.match_report_panel.SetSizer(self.mrp_sizer, True)
        self.match_report_panel.SetAutoLayout(True)
        self.mrp_sizer.Fit(self.match_report_panel)

        if self.match_id is None:
            self.__reset()
            return

        self.id_text.SetLabel(str(match_id))
        self.Enable(True)

        self.Update()

    def Update(self, only_time = False):
        if self.match_id is None:
            return

        m = self.live.matches.get(self.match_id)
        if m is None:
            self.__reset()
            return

        (date, home_team, away_team, current_score, events, finished) = m

        self.hometeam_id=home_team[0]
        self.awayteam_id=away_team[0]

        if date is not None:
            diff = self.live.last_fetched - date
            diff2 = ((self.live.last_fetched - date) +
                     (time.time() - self.live.last_fetched_local))
        else:
            diff = diff2 = 1.0

        game_finished = False

        if diff < 0.0:
            if config.get("Match viewer", "detailed time"):
                self.minute_text.SetLabel(self.__time_until_match_string2(-diff2))
            else:
                self.minute_text.SetLabel(self.__time_until_match_string(-diff2))
            colour="#F8C8C8"
        elif finished:
            self.minute_text.SetLabel(_("The match is finished."))
            colour="#C8C8F8"
            game_finished = True
        else:
            self.minute_text.SetLabel(self.__time_during_match_string(diff2))
            colour="#C8F8C8"

        self.header_panel.SetBackgroundColour(colour)
        self.team_text.SetBackgroundColour(colour)
        self.result_text.SetBackgroundColour(colour)
        self.header_panel.Refresh()

        self.homebutton.Enable(game_finished)
        self.matchbutton.Enable(game_finished)
        self.awaybutton.Enable(game_finished)

        if only_time:
            self.GetSizer().Layout()
            self.Refresh()
            return

        tt = "%s - %s" % (home_team[1], away_team[1])
        self.team_text.SetLabel(tt)

        (played, cs1, cs2) = current_score
        if played:
            self.result_text.SetLabel("%d - %d" % (cs1, cs2))
        else:
            self.result_text.SetLabel("X - X")

        eids = filter((lambda x: x > self.displayed_events), events.keys())
    
        for eid in eids:
            (minute, event_key, event_text, subject_tid, subject_pid, obj_pid) = events[eid]

            if 30 <= event_key[0] <= 33:
                attendance = obj_pid
                self.attendance_text.SetLabel(_("Attendance") + ": " + str(attendance))
                self.__set_weather_image(event_key[0] - 30)

            if 55 <= event_key[0] <= 59 or \
               90 <= event_key[0] <= 99 or \
               510 <= event_key[0] <= 519 or \
               100 <= event_key[0] <= 199:

                r = re.compile("<A HREF.*?%d.*?>(.+?)</A>" % subject_pid, re.IGNORECASE | re.DOTALL)
                m = r.search(event_text)
                if m is not None:
                    pname = string.join(m.group(1).split(), " ")
                else:
                    pname = "Unkown"

                if subject_tid == home_team[0]:
                    team = 0
                else:
                    team = 1

            if 55 <= event_key[0] <= 59:
                if not self.show_pen_shootout and len(self.ps_texts) > 0:
                    self.ps_texts[0].SetLabel(_("Penalty shootout"))

                if event_key[0] in [55, 56, 57]:
                    mtext = _("%s scores")
                    col = "#0055AA"
                else:
                    mtext = _("%s misses")
                    col = "#006600"

                stext = wxStaticText(self.imp_events_panel, -1, mtext % pname)
                stext.SetForegroundColour(col)
                self.pen_sizers[team].Add(stext, 0, wxEXPAND | wxALIGN_TOP | wxALL, 0)
                self.ps_texts.append(stext)

            if 90 <= event_key[0] <= 99 or \
                   510 <= event_key[0] <= 519:
                hsizer = wxBoxSizer(wxHORIZONTAL)

                stext = wxStaticText(self.imp_events_panel, -1, str(minute))
                hsizer.Add(stext, 0, wxALIGN_LEFT | wxRIGHT, 3)
                self.book_texts.append(stext)

                if event_key[0] in [90, 94]:
                    mtext = _("%s bruised")
                    col = "#00AA00"
                elif event_key[0] in [91, 92, 93, 95, 96]:
                    mtext = _("%s injured")
                    col = "#008800"
                elif event_key[0] in [510, 511]:
                    mtext = _("%s booked")
                    col = "#AAAA00"
                elif event_key[0] in [512, 513, 514]:
                    mtext = _("%s sent off")
                    col = "#DD0000"
                else:
                    mtext = ""
                    col = "#000000"

                if len(mtext) > 0:
                    stext = wxStaticText(self.imp_events_panel, -1, mtext % pname)
                    stext.SetForegroundColour(col)
                    hsizer.Add(stext, 1, wxALIGN_LEFT | wxLEFT, 3)
                    self.book_sizers[team].Add(hsizer, 0, wxEXPAND | wxALIGN_TOP | wxALL, 0)
                    self.book_texts.append(stext)

            if 100 <= event_key[0] <= 199:

                tmint = str(minute)
                if event_key[0] in [104, 114, 124, 134, 144, 154, 164, 174, 184, 194]:
                    tmint += " (" + _("pen.") + ")"

                if self.goal_texts.has_key((subject_tid, subject_pid)):
                    mintext = self.goal_texts[(subject_tid, subject_pid)][1]
                    pre_label = mintext.GetLabel()
                    mintext.SetLabel(pre_label + ", " + tmint)
                else:
                    nametext = wxStaticText(self.imp_events_panel, -1, pname)
                    self.goal_sizers[team].Add(nametext, 0, wxEXPAND | wxALIGN_LEFT | wxALL, 0)

                    mintext = wxStaticText(self.imp_events_panel, -1, tmint)
                    self.goal_sizers[team].Add(mintext, 0, wxEXPAND | wxALIGN_LEFT | wxALL, 0)

                    self.goal_texts[(subject_tid, subject_pid)] = (nametext, mintext)

            stext = wxStaticText(self.match_report_panel, -1, str(minute))

            self.report_texts.append(stext)

            resum_text = ""
            
            if event_key[0] in [20]:
                resum_text += _("Team formation")
            elif event_key[0] in [21]:
            	resum_text += _("Team lineup")
            elif event_key[0] in [25]:
            	resum_text += _("Derby")
            elif event_key[0] in [26]:
            	resum_text += _("Neutral game")
            elif event_key[0] in [30, 31, 32, 33]:
            	resum_text += _("Spectators and weather")
            elif event_key[0] in [40, 47]:
            	resum_text += _("Ball possesion")
            elif event_key[0] in [41]:
            	resum_text += _("Best player")
            elif event_key[0] in [42]:
            	resum_text += _("Worst player")
            elif event_key[0] in [45]:
            	resum_text += _("Half-time")
            elif event_key[0] in [46]:
            	resum_text += _("Hattrick")
            elif event_key[0] in [56, 57,58, 59]:
            	resum_text += _("Penalty")
            elif event_key[0] in [60]:
            	resum_text += _("Underestimation")
            elif event_key[0] in [61]:
            	resum_text += _("Confusion")
            elif event_key[0] in [62]:
            	resum_text += _("Defensive")
            elif event_key[0] in [64]:
            	resum_text += _("Tactical briefing")
            elif event_key[0] in [65]:
            	resum_text += _("Nervous situation")
            elif event_key[0] in [68]:
            	resum_text += _("Pressing")
            elif event_key[0] in [70]:
            	resum_text += _("Overtime")
            elif event_key[0] in [71]:
            	resum_text += _("Penalties")
            elif event_key[0] in [72]:
            	resum_text += _("Victory in extratime")
            elif event_key[0] in [90, 94]:
            	resum_text += _("Bruised")
            elif event_key[0] in [91, 92, 93, 95, 96]:
            	resum_text += _("Injured")
            elif 100 <= event_key[0] <= 299:
                if 100 <= event_key[0] <= 199:
                    resum_text = _("Goal")
                else:
                    resum_text = _("Chance")
                if event_key[0] in [100, 110, 120, 130, 150, 160, 170, 180, 190] or \
                      event_key[0] in [200, 210, 220, 230, 250, 260, 270, 280, 290]:
                    resum_text += " (" + _("free kick") + ")"
                elif event_key[0] in [140] or \
                      event_key[0] in [240]:
                    resum_text += " (" + _("CA free kick") + ")"
                elif event_key[0] in [101, 111, 121, 131, 151, 161, 171, 181, 191] or \
                       event_key[0] in [201, 211, 221, 231, 251, 261, 271, 281, 291]:               	
                    resum_text += " (" + _("central attack") + ")"
                elif event_key[0] in [141] or \
                       event_key[0] in [241]:
                    resum_text += " (" + _("CA central attack") + ")"   
                elif event_key[0] in [102, 112, 122, 132, 152, 162, 172, 182, 192] or \
                      event_key[0] in [202, 212, 222, 232, 252, 262, 272, 282, 292]:
                    resum_text += " (" + _("left attack") + ")"
                elif event_key[0] in [142] or \
                      event_key[0] in [242]:
                     resum_text += " (" + _("CA left attack") + ")"   
                elif event_key[0] in [103, 113, 123, 133, 153, 163, 173, 183, 193] or \
                      event_key[0] in [203, 213, 223, 233, 253, 263, 273, 283, 293]:
                    resum_text += " (" + _("right attack") + ")"
                elif event_key[0] in [143] or \
                      event_key[0] in [243]:
                    resum_text += " (" + _("CA right attack") + ")"   
                elif event_key[0] in [104, 114, 124, 134, 154, 164, 174, 184, 194] or \
                      event_key[0] in [204, 214, 224, 234, 254, 264, 274, 284, 294]:
                     resum_text += " (" + _("penalty") + ")"
                elif event_key[0] in [105] or \
                      event_key[0] in [205]:
                    resum_text += " (" + _("unpred. long pass") + ")"
                elif event_key[0] in [106] or \
                      event_key[0] in [206]:
                    resum_text += " (" + _("unpred. interception") + ")"
                elif event_key[0] in [107] or \
                      event_key[0] in [207]:
                    resum_text += " (" + _("long shot") + ")"
                elif event_key[0] in [108] or \
                      event_key[0] in [208]:
                    resum_text += " (" + _("unpredictable pass") + ")"
                elif event_key[0] in [109] or \
                      event_key[0] in [209]:
                    resum_text += " (" + _("unpred. stupid pass") + ")"
                elif event_key[0] in [115] or \
                      event_key[0] in [215]:
                    resum_text += " (" + _("quick player") + ")"
                elif event_key[0] in [116] or \
                      event_key[0] in [216]:
                    resum_text += " (" + _("quick pass") + ")"
                elif event_key[0] in [117] or \
                      event_key[0] in [217]:
                    resum_text += " (" + _("tired defender") + ")"
                elif event_key[0] in [118] or \
                      event_key[0] in [218]:
                    resum_text += " (" + _("corner") + ")"
                elif event_key[0] in [119] or \
                      event_key[0] in [219]:
                    resum_text += " (" + _("corner to head") + ")"
                elif event_key[0] in [135] or \
                      event_key[0] in [235]:
                    resum_text += " (" + _("experienced") + ")"
                elif event_key[0] in [136] or \
                      event_key[0] in [236]:
                    resum_text += " (" + _("inexperienced") + ")"
                elif event_key[0] in [137] or \
                      event_key[0] in [237]:
                    resum_text += " (" + _("winger to head") + ")"
                elif event_key[0] in [138] or \
                      event_key[0] in [238]:
                    resum_text += " (" + _("winger pass to forw.") + ")"
                elif event_key[0] in [139] or \
                      event_key[0] in [239]:
                    resum_text += " (" + _("technical vs. head") + ")"
                else:
                	resum_text += " (" + str(event_key[0]) +"," + str(event_key[1]) + ")"
            elif event_key[0] in [301]:
            	resum_text += _("Technical weather loss")
            elif event_key[0] in [302]:
            	resum_text += _("Powerful weather win")
            elif event_key[0] in [303]:
            	resum_text += _("Technical weather win")
            elif event_key[0] in [304]:
            	resum_text += _("Powerful weather loss")
            elif event_key[0] in [305]:
            	resum_text += _("Quick weather loss")
            elif event_key[0] in [306]:
            	resum_text += _("Quick weather loss")
            elif event_key[0] in [331]:
            	resum_text += _("Pressing tactic")
            elif event_key[0] in [332]:
            	resum_text += _("Counter-attack tactic")
            elif event_key[0] in [333]:
            	resum_text += _("Attack in the middle")
            elif event_key[0] in [334]:
            	resum_text += _("Attack on wings")
            elif event_key[0] in [501, 502]:
            	resum_text += _("Walkover")
            elif event_key[0] in [510, 511]:
                resum_text += _("Booked")
            elif event_key[0] in [512, 513, 514]:
                resum_text += _("Sent off")
            elif event_key[0] in [599]:
            	resum_text += _("End")
            else:
                resum_text = event_key

            if subject_tid == home_team[0]:
               resum_col = self.HOME_TEAM_COLOR
            elif subject_tid == away_team[0]:
               resum_col = self.AWAY_TEAM_COLOR
            else:
               resum_col = "#000000"

            stext2 = wxStaticText(self.match_report_panel, -1, str(resum_text))
            stext2.SetForegroundColour(resum_col)
            self.report_texts.append(stext2)

            event_text = Live.break_up_row(Live.unhtmlify(event_text), 80)
            stext3 = wxStaticText(self.match_report_panel, -1, str(event_text))
            self.report_texts.append(stext3)

            if config.get("Match viewer", "report direction") == "normal":
                self.mrp_sizer.Add(stext, 0, wxALIGN_TOP | wxALL, 2)
                if config.get("Match viewer", "show eventinfo") == True:
                    self.mrp_sizer.Add(stext2, 1, wxEXPAND | wxALIGN_TOP | wxALL, 2)
                else:
                    self.mrp_sizer.Add((1,1))
                    stext2.Hide()
                self.mrp_sizer.Add(stext3, 2, wxEXPAND | wxALIGN_TOP | wxALL, 2)
            else:
                self.mrp_sizer.Prepend(stext3, 2, wxEXPAND | wxALIGN_TOP | wxALL, 2)
                if config.get("Match viewer", "show eventinfo") == True:
                    self.mrp_sizer.Prepend(stext2, 1, wxEXPAND | wxALIGN_TOP | wxALL, 2)
                else:
                    self.mrp_sizer.Prepend((1,1))
                    stext2.Hide()
                self.mrp_sizer.Prepend(stext, 0, wxALIGN_TOP | wxALL, 2)

        self.mrp_sizer.Layout()
        for team in range(2):
            self.goal_sizers[team].Layout()
            self.book_sizers[team].Layout()

        if len(eids) > 0:
            self.displayed_events = eids[-1]
            if config.get("Match viewer", "report direction") == "normal":
                self.match_report_panel.Scroll(-1, 100)
            else:
                self.match_report_panel.Scroll(-1,-100)

        self.GetSizer().Layout()
        self.Refresh()

    def OnHomeButton(self, event):
        dlg = Matchdialogs.MatchLineupDialog(self, self.match_id, self.hometeam_id)
        if not dlg.no_data:
            ret = dlg.Show()


    def OnAwayButton(self, event):
        dlg = Matchdialogs.MatchLineupDialog(self, self.match_id, self.awayteam_id)
        if not dlg.no_data:
            ret = dlg.Show()

    def OnMatchButton(self, event):
        dlg = Matchdialogs.MatchDetailsDialog(self, self.match_id)
        if not dlg.no_data:
             ret = dlg.Show()

    def __time_during_match_string(self, t):
        minutes = int(round(t / 60.0))
        if 45 < minutes < 60:
            return _("Half time")
        if minutes > 45:
            minutes -= min(minutes - 45, 15)
        return (ngettext("The match clock shows %d minute.",
                         "The match clock shows %d minutes.", minutes) % minutes)

    def __time_until_match_string(self, t):
        days = int(round(t / 86400.0))
        if days > 0:
            return (ngettext("The match will begin in %d day.",
                             "The match will begin in %d days.", days) % days)

        hours = int(round(t / 3600.0))
        if hours > 1:
            return (ngettext("The match will begin in %d hour.",
                             "The match will begin in %d hours.", hours) % hours)

        minutes = int(round(t / 60.0))
        if minutes > 0:
            return (ngettext("The match will begin in %d minute.",
                             "The match will begin in %d minutes.", minutes) % minutes)

        return _("The match is just about to begin.")

    def __time_until_match_string2(self, t):
        text_string = _("The match will begin in %s.")
        days = int(math.floor(t / 86400.0))
        hours = int(math.floor((t-(days*86400.0)) / 3600.0))
        minutes = int(math.floor((t-days*86400.0-hours*3600) / 60.0))

        day_string = (ngettext("%d day", "%d days", days) % days)
        hour_string = (ngettext("%d hour", "%d hours", hours) % hours)
        minute_string = (ngettext("%d minute", "%d minutes", minutes) % minutes)

        if days > 0:
            if hours > 0:
                if minutes > 0:
                    time_string = (_("%s, %s, and %s") % (day_string, hour_string, minute_string))
                else:
                    time_string = (_("%s and %s") % (day_string, hour_string))
            else:
                if minutes > 0:
                    time_string = (_("%s and %s") % (day_string, minute_string))
                else:
                    time_string = ("%s" % day_string)
        else:
            if hours > 0:
                if minutes > 0:
                    time_string = (_("%s and %s") % (hour_string, minute_string))
                else:
                    time_string = ("%s" % hour_string)
            else:
                if minutes > 0:
                    time_string = ("%s" % minute_string)
                else:
                    return _("The match is just about to begin.")
        return text_string % time_string

    def __set_weather_image(self, weather):
        if self.weather_img is not None:
            self.ssizer.Remove(self.weather_img)
            self.weather_img.Destroy()
            self.weather_img = None

        if weather in [0,1,2,3]:
            weather_images = ["weather_rain.png", "weather_overcast.png",
                              "weather_cloudy.png", "weather_sun.png"]
            img = Images.Images.get_bitmap_from_file(weather_images[weather])
            self.weather_img = wxStaticBitmap(self, -1, img)
            self.ssizer.Add(self.weather_img, 0, wxALIGN_CENTRE | wxALL, 0)

    def __reset(self):
        self.match_id = None
        self.team_text.SetLabel("")
        self.id_text.SetLabel("")
        self.result_text.SetLabel("X - X")
        self.attendance_text.SetLabel("")
        self.minute_text.SetLabel(_("No match has been selected."))
        self.__set_weather_image(None)
        self.header_panel.SetBackgroundColour("#E8E8E8")
        self.team_text.SetBackgroundColour("#E8E8E8")
        self.result_text.SetBackgroundColour("#E8E8E8")
        self.Enable(False)

        self.GetSizer().Layout()
        self.Refresh()

class LiveSmallPanel(wxPanel):
    ID_REMOVE = 344
    ID_SHOW_DETAILED = 345

    def __init__(self, parent, ID, live):
        self.parent = parent
        self.live = live
        self.match_id = None
        self.event = False
        self.displayed_events = 0
        
        if wxPlatform == '__WXMSW__':
            pstyle = wxSTATIC_BORDER
        elif wxPlatform == '__WXGTK__':
            pstyle = wxRAISED_BORDER
        else:
            pstyle = wxSIMPLE_BORDER

        wxPanel.__init__(self, parent, ID, style = pstyle)

        self.SetBackgroundColour("#E8E8E8")

        sizer = wxBoxSizer(wxVERTICAL)

        self.team_text = wxStaticText(self, -1, "", style=wxALIGN_CENTRE)
        sizer.Add(self.team_text, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        hsizer = wxBoxSizer(wxHORIZONTAL)
        for bt, bid, func in [(" + ", self.ID_SHOW_DETAILED, self.OnShowDetailed),
                              (" x ", self.ID_REMOVE, self.OnRemove)]:
            button = wxButton(self, bid, bt, style = wxBU_EXACTFIT)
            hsizer.Add(button, 0, wxEXPAND | wxALIGN_LEFT | wxRIGHT | wxLEFT, 2)
            EVT_BUTTON(self, bid, func)

        self.play_sound = MyBitmapToggleButton(self, -1, None)
        bmp = Images.Images.get_bitmap_from_file("soundoff.png")
        mask = wxMaskColour(bmp, wxWHITE)
        bmp.SetMask(mask)
        self.play_sound.SetBitmapLabel(bmp)
        bmp = Images.Images.get_bitmap_from_file("soundon.png")
        mask = wxMaskColour(bmp, wxWHITE)
        bmp.SetMask(mask)
        self.play_sound.SetBitmapSelected(bmp)
        self.play_sound.SetToggle(config.get("Match viewer", "play sound"))
        self.play_sound.labelDelta = 0
        self.play_sound.SetBezelWidth(0)
        self.play_sound.SetBestSize()
        hsizer.Add(self.play_sound, 0, wxALIGN_LEFT | wxALL, 2)
        self.play_sound.SetBackgroundColour("#E8E8E8")
        EVT_BUTTON(self, self.play_sound.GetId(), self.OnToggleButton)

        self.event_notify = MyBitmapToggleButton(self,-1, None)
        bmp = Images.Images.get_bitmap_from_file("eventoff.png")
        mask = wxMaskColour(bmp, wxWHITE)
        bmp.SetMask(mask)
        self.event_notify.SetBitmapLabel(bmp)
        bmp = Images.Images.get_bitmap_from_file("eventon.png")
        mask = wxMaskColour(bmp, wxWHITE)
        bmp.SetMask(mask)
        self.event_notify.SetBitmapSelected(bmp)
        self.event_notify.SetToggle(config.get("Match viewer", "indicate event"))
        self.event_notify.SetBezelWidth(0)
        self.event_notify.labelDelta = 0
        self.event_notify.SetBestSize()
        hsizer.Add(self.event_notify, 0, wxALIGN_LEFT | wxALL, 2)
        self.event_notify.SetBackgroundColour("#E8E8E8")
        EVT_BUTTON(self, self.event_notify.GetId(), self.OnToggleButton)
        self.current_colour="#E8E8E8"
        
        self.id_text = wxStaticText(self, -1, "")
        hsizer.Add(self.id_text, 1, wxALIGN_LEFT | wxALL, 2)

        self.result_text = wxStaticText(self, -1, "X - X", size = (50, -1),
                                        style = wxALIGN_RIGHT)
        hsizer.Add(self.result_text, 0, wxALIGN_RIGHT | wxALL, 2)

        sizer.Add(hsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxRIGHT | wxLEFT, 2)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        self.Enable(False)

    def ChangeMatch(self, match_id):
        self.match_id = match_id

        if self.match_id is None:
            self.__reset()
            return

        self.id_text.SetLabel(str(match_id))
        self.Enable(True)
        self.displayed_events = 0
        self.play_sound.SetToggle(config.get("Match viewer", "play sound"))
        self.event_notify.SetToggle(config.get("Match viewer", "indicate event"))
        self.Update()

    def SetEvent(self):
        colour="#F80000"
        self.__set_colour(colour)
        self.event = True
   
    def Update(self):
        if self.match_id is None:
            return

        m = self.live.matches.get(self.match_id)
        if m is None:
            self.__reset()
            return

        (date, home_team, away_team, current_score, events, finished) = m

        if date is not None:
            diff = self.live.last_fetched - date
        else:
            diff = 1.0

        eids = filter((lambda x: x > self.displayed_events), events.keys())

        if len(eids) > 0:
            self.displayed_events = eids[-1]

            if self.event_notify.GetToggle() == True:
                self.SetEvent()

            if self.play_sound.GetToggle() == True:
                for eid in eids:
                    (minute, event_key, event_text, subject_tid, subject_pid, obj_pid) = events[eid]
                    if 100 <= event_key[0] <= 199:
                        filename = config.get("Match viewer", "sound file")
                        if filename != "":
                            try:
                                wav = wxWave(filename)
                                if wav.IsOk():
                                    wav.Play()
                            except NotImplementedError, v:
                                pass

            
        if self.event == False:
            if diff < 0.0:
                colour="#F8C8C8"
            elif finished:
                colour="#C8C8F8"
            else:
                colour="#C8F8C8"

            self.__set_colour(colour)

        tt = "%s - %s" % (home_team[1], away_team[1])
        self.team_text.SetLabel(tt)

        (played, cs1, cs2) = current_score
        if played:
            self.result_text.SetLabel("%d - %d" % (cs1, cs2))
        else:
            self.result_text.SetLabel("X - X")

        self.GetSizer().Layout()
        self.Refresh()

    def __set_colour(self, colour):
        self.SetBackgroundColour(colour)
        self.team_text.SetBackgroundColour(colour)
        self.result_text.SetBackgroundColour(colour)
        self.id_text.SetBackgroundColour(colour)
        self.play_sound.SetBackgroundColour(colour)
        self.event_notify.SetBackgroundColour(colour)
        self.current_colour = colour

    def OnToggleButton(self, event):
        self.play_sound.SetBackgroundColour(self.current_colour)
        self.event_notify.SetBackgroundColour(self.current_colour)
        
    def OnRemove(self, event):
        self.parent.RemoveMatch(self.match_id)

    def OnShowDetailed(self, event):
        self.parent.SetDetailedMatch(self.match_id)
        self.event = False
        self.Update()
        
    def __reset(self):
        self.match_id = None
        self.team_text.SetLabel("")
        self.id_text.SetLabel("")
        self.result_text.SetLabel("X - X")
        self.__set_colour("#E8E8E8")

        self.Enable(False)

        self.GetSizer().Layout()
        self.Refresh()
