# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import Images
import MatchInfo
import Terms

import string

from Common import *
from Commonwx import *
from wxPython.wx import *

class MatchDetailsDialog(wxDialog):
    ID_MP_HOME = 428
    ID_MP_AWAY = 429
    ID_LINEUP_HOME = 432
    ID_LINEUP_AWAY = 433

    def __init__(self, parent, match_id):

        match = MatchInfo.MatchDetails(match_id, at_least_version = [1, 3])

        self.match = match

        if match.match_id is None:
            self.no_data = True
            return
        else:
            self.no_data = False

        hid = match.home_team[0]
        aid = match.away_team[0]

        title = "%s - %s" % (match.home_team[1], match.away_team[1])

        wxDialog.__init__(self, parent, -1, title, style = wxDEFAULT_DIALOG_STYLE)

        sizer=wxBoxSizer(wxVERTICAL)

        hsizer = wxBoxSizer(wxHORIZONTAL)

        dress_widget = self.__add_dress(match.home_team[2])
        hsizer.Add(dress_widget, 0, wxALIGN_TOP | wxALL, 10)

        vsizer = wxBoxSizer(wxVERTICAL)

        hsizer2 = wxBoxSizer(wxHORIZONTAL)

        self.star_img = Images.Images.get_bitmap_from_file("star.png")
        star_button = wxBitmapButton(self, self.ID_LINEUP_HOME, self.star_img, style = wxBU_EXACTFIT)
        hsizer2.Add(star_button, 0, wxALIGN_RIGHT | wxALL, 2)


        matchtext = "%s - %s" % (match.home_team[1], match.away_team[1])
        
        stext = wxStaticText(self, -1, matchtext)
        SetTextFace(stext, FACE_BOLD | FACE_BIG)
        hsizer2.Add(stext, 0, wxALIGN_CENTRE | wxALL, 2)

        self.star_img = Images.Images.get_bitmap_from_file("star.png")
        star_button = wxBitmapButton(self, self.ID_LINEUP_AWAY, self.star_img, style = wxBU_EXACTFIT)
        hsizer2.Add(star_button, 0, wxALIGN_LEFT | wxALL, 2)

        vsizer.Add(hsizer2, 0, wxALIGN_CENTRE | wxALL, 2)

        if match.played:
            resulttext = "%d - %d" % (match.score[0], match.score[1])
        else:
            resulttext = "x - x"

        stext = wxStaticText(self, -1, resulttext)
        SetTextFace(stext, FACE_BOLD | FACE_BIG)
        vsizer.Add(stext, 0, wxALIGN_CENTRE | wxALL, 2)

        arenasizer = wxBoxSizer(wxHORIZONTAL)
        (id, name, weather, total, terraces, basic, roof, vip) = match.arena
        if weather is not None:
            weather_images = ["weather_rain.png", "weather_overcast.png",
                              "weather_cloudy.png", "weather_sun.png"]
            img = Images.Images.get_bitmap_from_file(weather_images[weather])
            sbmp = wxStaticBitmap(self, -1, img)
            arenasizer.Add(sbmp, 0, wxALIGN_CENTRE | wxALL, 10)

        atexts = [_("Arena") + ": " +  name]
        if total is not None:
            atexts.append(_("Attendance") + ": %d" % total)

        btexts = []
        for desc, amount in [(_("Terraces"), terraces),
                             (_("Basic seating"), basic),
                             (_("Roofed seating"), roof),
                             (_("Seating in VIP boxes"), vip)]:
            if amount is not None:
                btexts.append(desc + ": %d" % amount)

        i = 0
        for texts in [atexts, btexts]:
            if len(texts) > 0:
                avsizer = wxBoxSizer(wxVERTICAL)
                for t in texts:
                    stext = wxStaticText(self, -1, t)
                    if i == 0:
                        SetTextFace(stext, FACE_BOLD | FACE_BIG)
                    avsizer.Add(stext, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)
                    i += 1
                arenasizer.Add(avsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        avsizer = wxBoxSizer(wxVERTICAL)
        arenasizer.Add(avsizer, 0, wxALIGN_LEFT | wxALL, 5)

        vsizer.Add(arenasizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        # Match events

        hteam_scores = {}
        ateam_scores = {}

        hid_orders = []
        aid_orders = []
        for minute, hgoals, agoals, team, id, name in match.scorers:
            if team == hid:
                scores = hteam_scores
                id_orders = hid_orders
            elif team == aid:
                scores = ateam_scores
                id_orders = aid_orders
            else:
                print match.scorers
                continue

            if not scores.has_key(id):
                scores[id] = (name, [])
                id_orders.append(id)
            scores[id][1].append(minute)

        hteam_bookings = []
        ateam_bookings = []

        for minute, type, team, id, name in match.bookings:
            if team == hid:
                hteam_bookings.append((minute, type, id, name))
            elif team == aid:
                ateam_bookings.append((minute, type, id, name))

        stext = wxStaticText(self, -1, _("Match events") + ":")
        SetTextFace(stext, FACE_BOLD | FACE_BIG)
        vsizer.Add(stext, 0, wxALIGN_LEFT | wxALL, 5)

        eventsizer = wxBoxSizer(wxHORIZONTAL)

        i = 0
        for scores, id_orders, bookings in [(hteam_scores, hid_orders, hteam_bookings),
                                            (ateam_scores, aid_orders, ateam_bookings)]:
            if i == 0:
                side = wxRIGHT
            else:
                side = wxLEFT
            evsizer = wxBoxSizer(wxVERTICAL)

            if len(scores.values()) > 0:
                fgsizer = wxFlexGridSizer(cols=2, vgap=0, hgap=0)
                for pid in id_orders:
                    (name, minutes) = scores[pid]
                    stext = wxStaticText(self, -1, name)
                    fgsizer.Add(stext, 0, wxALIGN_LEFT | wxRIGHT, 7)

                    min_text = string.join([str(m) for m in minutes], ", ")
                    stext = wxStaticText(self, -1, min_text)
                    fgsizer.Add(stext, 0, wxALIGN_LEFT | wxLEFT, 7)

                evsizer.Add(fgsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

            for minute, type, id, name in bookings:
                if type == 1:
                    stext = wxStaticText(self, -1, _("%s booked") % name)
                    stext.SetForegroundColour("#AAAA00")
                elif type == 2:
                    stext = wxStaticText(self, -1, _("%s sent off") % name)
                    stext.SetForegroundColour("#DD0000")
                evsizer.Add(stext, 0, wxEXPAND | wxALIGN_LEFT | wxALL, 0)

            eventsizer.Add(evsizer, 1, wxALIGN_TOP | wxTOP | wxBOTTOM | side, 5)
            i += 1

        vsizer.Add(eventsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        # Ratings

        hatstats_ratings = [("def", _("Defence")), ("mid", _("Midfield")),
                    ("att", _("Attack")), ("tot", _("Total"))]

        stext = wxStaticText(self, -1, _("Match ratings") + ":")
        SetTextFace(stext, FACE_BOLD | FACE_BIG)
        vsizer.Add(stext, 0, wxALIGN_LEFT | wxALL, 5)

        ratingsizer = wxBoxSizer(wxHORIZONTAL)

        i = 0
        for ratings in [match.home_ratings, match.away_ratings]:
            if i == 0:
                side = wxRIGHT
            else:
                side = wxLEFT

            rfsizer = wxFlexGridSizer(cols=2, vgap=0, hgap=0)

            for rname, rdesc in Terms.team_parts:
                r = ratings[rname]
                main_rating = int((r+3)/4)
                partial_rating = r-4*(main_rating-1)

                if Terms.player_abilities.has_key(main_rating):
                    main_rstr = Terms.player_abilities[main_rating]
                else:
                    main_rstr = _("Unknown")

                if Terms.partial_match_grade.has_key(partial_rating):
                    partial_rstr = Terms.partial_match_grade[partial_rating]
                else:
                    partial_rstr = _("Unknown")

                rstr = main_rstr + " (" + partial_rstr + ")"
                
                j = 0
                for t in [rdesc + ":", rstr]:
                    rside = (j == 0) and wxRIGHT or wxLEFT
                    stext = wxStaticText(self, -1, t)
                    rfsizer.Add(stext, 0, rside, 5)
                    j += 1


            stext = wxStaticText(self, -1, _("Hatstats ratings") + ":")
            SetTextFace(stext, FACE_BOLD)
            rfsizer.Add(stext, 0, wxALIGN_LEFT | wxALL, 5)
            rfsizer.Add((1,1))
            
            for rating_type, rating_name in hatstats_ratings:
                if i == 0:
                    hatstats = match.hatstats_home_ratings
                else:
                    hatstats = match.hatstats_away_ratings
                        
                j = 0
                for t in [rating_name + ":", str(hatstats[rating_type])]:
                    rside = (j == 0) and wxRIGHT or wxLEFT
                    stext = wxStaticText(self, -1, t)
                    rfsizer.Add(stext, 0, rside, 5)
                    j += 1
                
            
            button = wxButton(self, self.ID_MP_HOME + i*2, _("Matchprediction:\nHome team"))
            rfsizer.Add(button, 0, wxALIGN_TOP | wxTOP | wxBOTTOM | wxRIGHT, 5)
            EVT_BUTTON(self, self.ID_MP_HOME + i*2, self.MPCopy)

            button = wxButton(self, self.ID_MP_AWAY + i*2, _("Matchprediction:\nAway team"))
            rfsizer.Add(button, 0, wxALIGN_TOP | wxTOP | wxBOTTOM | wxLEFT, 5)
            EVT_BUTTON(self, self.ID_MP_AWAY + i*2, self.MPCopy)

            ratingsizer.Add(rfsizer, 1, wxALIGN_TOP | wxTOP | wxBOTTOM | side, 5)

            i += 1


        vsizer.Add(ratingsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        # Last dress

        hsizer.Add(vsizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        dress_widget = self.__add_dress(match.away_team[2])
        hsizer.Add(dress_widget, 0, wxALIGN_TOP | wxALL, 10)

        sizer.Add(hsizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

        # Line

        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        sizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        # Ok / Cancel buttons

        button = wxButton(self, wxID_OK, _("Ok"))
        button.SetDefault()

        sizer.Add(button, 0, wxALIGN_RIGHT | wxALL, 10)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        EVT_BUTTON(self, self.ID_LINEUP_HOME, self.OnLineupHome)
        EVT_BUTTON(self, self.ID_LINEUP_AWAY, self.OnLineupAway)

    def OnLineupHome(self, event):
        dlg = MatchLineupDialog(self, self.match.match_id, self.match.home_team[0])
        if not dlg.no_data:
            ret = dlg.Show()
                
    def OnLineupAway(self, event):
        dlg = MatchLineupDialog(self, self.match.match_id, self.match.away_team[0])
        if not dlg.no_data:
            ret = dlg.Show()

    def MPCopy(self, event):
        eid = event.GetId() - self.ID_MP_HOME

        if eid < 2:
            ratings = self.match.home_ratings
        else:
            ratings = self.match.away_ratings

        import Main
        Main.notebook_pages["prediction"].ChangeRatings(ratings, (eid in [0, 2]))

    def __add_dress(self, cols):
        sizer = wxBoxSizer(wxVERTICAL)

        def_cols = ('N', 2, 4, 4)

        if cols is not None:
            dc = cols
        else:
            dc = def_cols

        nc = []
        for i in range(1, 4):
            if Terms.dress_colours.has_key(dc[i]):
                nc.append(Terms.dress_colours[dc[i]])
            else:
                nc.append(Terms.dress_colours[def_cols[i]])
        dress_cols = (dc[0], nc[0], nc[1], nc[2])

        dress_imgs = Images.create_dress_images(*dress_cols)
        for img in dress_imgs:
            sbmp = wxStaticBitmap(self, -1, img)
            sizer.Add(sbmp, 1, wxALIGN_LEFT | wxALL, 0)

        return sizer

class MatchLineupDialog(wxDialog):
    def __init__(self, parent, match_id, team_id, lineup = True):

        if lineup == True:
            match_lineup = MatchInfo.MatchLineup(match_id, team_id)
        else:
            match_lineup = MatchInfo.MatchOrders(match_id)
            
        if match_lineup.match_id is None:
            self.no_data = True
            return
        else:
            self.no_data = False

        if lineup == True:
            title = _("Lineup for %s") % match_lineup.team_name
        else:
            title = _("Match orders")
            
        wxDialog.__init__(self, parent, -1, title, style = wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER)

        glineup = self.get_graphical_lineup(match_lineup.lineup)
        
        sizer=wxBoxSizer(wxVERTICAL)

        f_line = "%s %d: %s - %s\n" % (_("Match"),
                                       match_lineup.match_id,
                                       match_lineup.home_team[1],
                                       match_lineup.away_team[1])

        stext = wxStaticText(self, -1, f_line)
        SetTextFace(stext, FACE_BOLD | FACE_BIG)
        sizer.Add(stext, 0, wxEXPAND | wxALIGN_LEFT | wxALL, 5)

        lines = []
        if match_lineup.match_type is not None:
            lines.append(Terms.match_type[match_lineup.match_type])
        if match_lineup.match_date is not None:
            lines.append("%s: %s\n" % (_("Match date"),
                                       mktimestring(match_lineup.match_date)))

        if lineup == True:
            lines.append(_("Total rating") + ": %.1f*" % match_lineup.total_rating)
        else:
            lines.append(_("Formation") + ": %s" % match_lineup.formation_string)
            lines.append(_("Team value") + ": %s" % match_lineup.teameval[1])

        for line in lines:
            stext = wxStaticText(self, -1, line)
            sizer.Add(stext, 0, wxEXPAND | wxALIGN_LEFT | wxALL, 2)

        sizer.Add((5, 15))

        sizer.Add(SimpleLineupCtrl(self, -1, glineup, lineup), 0, wxALL | wxEXPAND | wxALIGN_CENTRE)

##        fgsizer = wxFlexGridSizer(cols=4, vgap=0, hgap=0)
##
##        if lineup == True:
##            maxpos = 22
##        else:
##            maxpos = 19
##            
##        for i in range(1, maxpos):
##            (id, name, rating, position, behaviour) = match_lineup.lineup[i]
##            if id is None:
##                continue
##
##            role = Terms.team_role[i]
##            if behaviour is not None:
##                role += ", " + Terms.individual_order[behaviour]
##            role += ":"
##
##            for line in [role, name]:
##                stext = wxStaticText(self, -1, line)
##                fgsizer.Add(stext, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 3)
##
##            if lineup == False:
##                if i in [17, 18]: # Set pieces and captain
##                    fgsizer.Add((5, 5))
##                else:
##                    stext = wxStaticText(self, -1, str(rating[1]))
##                    fgsizer.Add(stext, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 3)
##                fgsizer.Add((5, 5))
##            elif rating is None:
##                fgsizer.Add((5, 5))
##                fgsizer.Add((5, 5))
##            else:
##                img = Images.create_star_image(int(rating*2))
##                sbmp = wxStaticBitmap(self, -1, img)
##                fgsizer.Add(sbmp, 1, wxALIGN_LEFT | wxALL, 3)
##
##                stext = wxStaticText(self, -1, "(" + str(rating) + "*)")
##                fgsizer.Add(stext, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 3)
##
##        sizer.Add(fgsizer, 0, wxEXPAND | wxALIGN_LEFT | wxALL, 0)

        # Line

        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        sizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        # Ok / Cancel buttons

        button = wxButton(self, wxID_OK, _("Ok"))
        button.SetDefault()

        sizer.Add(button, 0, wxALIGN_RIGHT | wxALL, 10)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)
        self.Layout()

    def get_graphical_lineup(self, lineup):
        lines=[[(1,True)],
               [(2, True), (3, True), (None, "extra_cd"), (4, True), (5, True)],
               [(6, True), (7, False), (None, "extra_im"), (8, False), (9, True)],
               [(10, True), (None, "extra_fw"), (11, True)]]
        
        if len(lineup) > 11:
               lines.append([(17, False), (12, False), (18, False)])
               lines.append([(13, False), (14, False), (16, False), (15, False)])

        if len(lineup) > 18:
            lines.append([(19, False), (20, False), (21, False)])

        self.extra_cd = []
        self.extra_im = []
        self.extra_fw = []
        
        for i in range(1,12):
            pdata=lineup[i]
            if pdata[4] == 7:
                self.extra_cd.append((pdata, i))
            elif pdata[4] == 6:
                self.extra_im.append((pdata, i))
            elif pdata[4] == 5:
                self.extra_fw.append((pdata,i))

        graphical_lineup = []
        for line in lines:
            row = []
            for pos in line:
                if pos[0]:
                    if lineup[pos[0]][4] in [5,6,7]:
                        if pos[1]:
                            row.append(((None, None, None, None, None),pos[0]))
                    else:
                        if lineup[pos[0]][0] is not None and lineup[pos[0]][0] != 0:
                            row.append((lineup[pos[0]], pos[0]))
                        elif pos[1]:
                            row.append(((None, None, None, None, None),pos[0]))
                            
                else:
                    row = row + getattr(self, pos[1])
            graphical_lineup.append(row)
        return graphical_lineup

class SimpleLineupCtrl(wxPanel):
    def __init__(self, parent, ID, graphical_lineup, is_lineup = True):
        wxPanel.__init__(self, parent, ID, style = wxSUNKEN_BORDER)

        vsizer = wxBoxSizer(wxVERTICAL)
        line_count = 0
        for line in graphical_lineup:
            if len(line):
                hsizer=wxBoxSizer(wxHORIZONTAL)
                for item in line:
                    vsizer2 = wxBoxSizer(wxVERTICAL)
                    texts = []
                    positionname = Terms.team_role[item[1]]
                    if item[0][0] is None:
                        positionname = ""
                    texts.append(positionname)
                    name = item[0][1]
                    if name is None:
                        name = ""
                    texts.append(name)
                    if item[0][2] is None:
                        rating = ""                        
                    elif is_lineup:
                        rating = str(item[0][2]) + "*"
                    else:
                        if item[1] in [17, 18]:
                            rating = ""
                        else:
                            rating = item[0][2][1]
                    texts.append(rating)
                    if item[0][4] is None:
                        order = ""
                    else:
                        order = Terms.individual_order[item[0][4]]
                    if item[1] < 12:
                        texts.append(order)
                        
                    for text in texts:
                        vsizer2.Add(wxStaticText(self, -1, text),  1, wxALL | wxALIGN_CENTRE)
                    hsizer.Add(vsizer2, 1,wxALL | wxALIGN_CENTRE, 2)

                if line_count == 4:
                    # Line
                    line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
                    vsizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 2)
                line_count += 1   
                vsizer.Add(hsizer, 1, wxEXPAND | wxALL | wxALIGN_CENTRE,5)
        self.SetSizer(vsizer)
        self.SetAutoLayout(True)
        vsizer.Fit(self)
        vsizer.Layout()
        self.Layout()
