# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, os.path
import time

from Common import *

from Images import Images, create_star_image, create_small_dress
import Terms
import Columndialog
import Countries
import SelectDateMenu
import PlayerDialog
import Players
import SquadStatsDialog
import Database
import MatchInfo
import CommonData
import XMLParse

from wxPython.wx import *
from wxPython.lib.mixins.listctrl import wxColumnSorterMixin, wxListCtrlAutoWidthMixin

# "attribute": (colflags, sort_increasing_order)
# attribute   - The attribute in the player object.
# colflags    - Column flags (None if no flags).
# sort_increasing_order - If the column should be sorted in increasing order
playerlist_columns = {
    "PlayerName": (None, True),
    "PlayerNumber": (wxLIST_FORMAT_RIGHT, True),
    "PlayerID": (wxLIST_FORMAT_RIGHT, True),
    "Age": (wxLIST_FORMAT_RIGHT, True),
    "CountryID": (wxLIST_FORMAT_RIGHT, True),
    "Cards": (wxLIST_FORMAT_RIGHT, False),
    "InjuryLevel": (wxLIST_FORMAT_RIGHT, False),
    "PlayerForm": (None, False),
    "TrainerData.TrainerSkill": (None, False),
    "StaminaSkill": (None, False),
    "KeeperSkill": (None, False),
    "PlaymakerSkill": (None, False),
    "PassingSkill": (None, False),
    "WingerSkill": (None, False),
    "DefenderSkill": (None, False),
    "ScorerSkill": (None, False),
    "SetPiecesSkill": (None, False),
    "Experience": (None, False),
    "Leadership": (None, False),
    "Salary": (wxLIST_FORMAT_RIGHT, False),
    "MarketValue": (wxLIST_FORMAT_RIGHT, False),
    "TSI": (wxLIST_FORMAT_RIGHT, False),
    "Specialty": (None, True),
    "Aggressiveness": (None, True),
    "Honesty": (None, True),
    "Agreeability": (None, True),
    "LeagueGoals": (wxLIST_FORMAT_RIGHT,False),
    "CupGoals": (wxLIST_FORMAT_RIGHT,False),
    "CompetitionGoals": (wxLIST_FORMAT_RIGHT,False),
    "FriendliesGoals": (wxLIST_FORMAT_RIGHT,False),
    "SeasonGoals": (wxLIST_FORMAT_RIGHT,False),
    "CareerGoals": (wxLIST_FORMAT_RIGHT,False),
    "CareerHattricks": (wxLIST_FORMAT_RIGHT,False),
    "BestPosition": (None, False),
    "Caps": (wxLIST_FORMAT_RIGHT, False),
    "CapsU20": (wxLIST_FORMAT_RIGHT, False),
    "TransferListed": (None, False),
    "NationalTeamID": (wxLIST_FORMAT_RIGHT, False),
    "TrainerData.TrainerType": (None, False),
    "ArrivalDate": (wxLIST_FORMAT_RIGHT, False),
    "PricePaid": (wxLIST_FORMAT_RIGHT, False),
    "MotherClub": (None, False),
    "Position": (None, True),
    "BoughtFrom": (None, False),
    }

align_wx_convert = {
    Players.mALIGN_LEFT: None,
    Players.mALIGN_CENTRE: wxLIST_FORMAT_RIGHT,
    Players.mALIGN_RIGHT: wxLIST_FORMAT_RIGHT
    }

# Different column lists:
column_lists = {

    "custom": (N_("User defined"), ["PlayerName"]),

    "attributes": (N_("Player attributes"),
                   ["PlayerName", "PlayerID", "Age", "CountryID", "Cards",
                    "InjuryLevel", "PlayerForm", "TrainerData.TrainerSkill", "StaminaSkill",
                    "KeeperSkill", "PlaymakerSkill", "PassingSkill",
                    "WingerSkill", "DefenderSkill", "ScorerSkill", "SetPiecesSkill",
                    "Experience", "Leadership", "Salary", "MarketValue",
                    "Specialty", "Aggressiveness", "Honesty",
                    "Agreeability"]),

    "goals": (N_("Goals"),
              ["PlayerName", "LeagueGoals", "CupGoals", "CompetitionGoals",
               "FriendliesGoals", "SeasonGoals", "CareerGoals", "CareerHattricks"]),

    "evaluation": (N_("Player evaluation"),
                   ["PlayerName", "BestPosition"])
    }

def add_team_positions_to_column_lists():
    global pcolumn_lists
    for i, (position, descr) in Terms.team_positions.items():
        column_lists["evaluation"][1].append(position)
        playerlist_columns[position] = (None, False)

add_team_positions_to_column_lists()

def set_custom_columns():
    global column_lists
    column_str = Database.databases.gets(None, False, ["Mattricks.column_list.custom"])["Mattricks.column_list.custom"]
    cl= []
    if column_str is not None:
        for column in column_str.split(","):
            cl.append(column)
        column_lists["custom"]=(column_lists["custom"][0],cl)

set_custom_columns()

class PlayerListCtrl(wxListCtrl, wxListCtrlAutoWidthMixin):
    def __init__(self, parent, ID, pos=wxDefaultPosition,
                 size=wxDefaultSize, style=0):
        wxListCtrl.__init__(self, parent, ID, pos, size, style)
        wxListCtrlAutoWidthMixin.__init__(self)

class PlayerListCtrlPanel(wxPanel):
    def __init__(self, parent, column_list_pair):
        self.column_info = playerlist_columns
        self.curr_col_list = column_list_pair[0]
        self.curr_col_list_name = column_list_pair[1]
        self.sort_column = -1
        self.sort_colname = ""

        self.player_data = None
        self.player_sort_attribute = "PlayerName"
        wxPanel.__init__(self, parent, -1)

        tID = wxNewId()

        self.list = PlayerListCtrl(self, tID,
                                   style=wxLC_REPORT | wxSUNKEN_BORDER
                                   | wxLC_VRULES | wxLC_HRULES | wxLC_SINGLE_SEL
                                   )


        self.columns_increasing_sort = {}

        for attribute, (colflags, sort_increasing) in self.column_info.items():
            self.columns_increasing_sort[attribute] = sort_increasing

        EVT_SIZE(self, self.OnSize)
        EVT_LIST_ITEM_RIGHT_CLICK(self, tID, self.OnItemRightClick)
        EVT_LIST_ITEM_ACTIVATED(self, tID, self.OnItemRightClick)
        EVT_LIST_COL_CLICK(self, tID, self.OnColClick)


    def AfterInit(self):
        teamid = CommonData.get_current_teamid()
        dress = XMLParse.get(["team details", CommonData.get_current_teamid()], ["Team.Dress"])[1]
        if dress is None:
            dress = 2
        if Terms.dress_colours.has_key(dress):
            colour = Terms.dress_colours[dress]
            sub_colour = Terms.sub_dress_colours[dress]
        else:
            colour = Terms.dress_colours[2]
            sub_colour = Terms.sub_dress_colours[2]
        
        self.imagelist = wxImageList(20, 12)
        if wxPlatform != '__WXMSW__':
            self.images = {
                "up":   self.imagelist.Add(Images.get_bitmap_from_file("arrow_small_up.png")),
                "down": self.imagelist.Add(Images.get_bitmap_from_file("arrow_small_down.png")),
                "yellowcard": self.imagelist.Add(Images.get_bitmap_from_file("yellowcard.png")),
                "2yellowcards": self.imagelist.Add(Images.get_bitmap_from_file("2yellowcards.png")),
                "redcard": self.imagelist.Add(Images.get_bitmap_from_file("redcard.png")),
                "injury": self.imagelist.Add(Images.get_bitmap_from_file("injury.png")),
                "bruised": self.imagelist.Add(Images.get_bitmap_from_file("bruised.png")),
                "dress": self.imagelist.Add(create_small_dress(colour)),
                "subdress": self.imagelist.Add(create_small_dress(sub_colour)),
                }

            self.stars = {}
            for i in range(1,31):
                self.stars[i] = self.imagelist.Add(create_star_image(i))

            self.flags = {}
            for cid, cinfo in Countries.country_info.items():
                flagfile = cinfo[5]
                if flagfile != None:
                    bmp = Images.get_bitmap_from_file(flagfile)
                    self.flags[cid] = self.imagelist.Add(bmp)

        else:
            self.images = {
                "dress" : self.imagelist.Add(create_small_dress(colour)),
                "subdress" : self.imagelist.Add(create_small_dress(sub_colour))
                }

        self.list.SetImageList(self.imagelist, wxIMAGE_LIST_SMALL)
        
    def OnSize(self, event):
        w,h = self.GetClientSizeTuple()
        self.list.SetDimensions(0, 0, w, h)

    def OnItemRightClick(self, event):
        pid = self.pid_list[event.GetData()]
        player = self.player_data.all_players.get(pid)
        if player is not None:
            ret = PlayerDialog.PlayerDialog(self, player).Show()

    def OnColClick(self, event):
        self.SortList(event.GetColumn(), False)

    def PopulateList(self):
        if self.player_data == None:
            return

        self.starting_players = []
        self.substitute_players = []
        import Main
        mid = Main.notebook_pages["matchlist"].GetNextMatch()
        if mid is not None:
            mdata = MatchInfo.LineupInfo(mid)
            if mdata.available:
                self.starting_players = mdata.starting_players
                self.substitute_players = mdata.substitute_players

        self.pid_list=[]
        x = 0
        for pid in self.player_data.all_players.keys():
            self.pid_list.append(pid)
            self.list.InsertImageStringItem(x, self.player_data.all_players[pid].data["PlayerName"][1], -1)
            y = 0
            for attribute in self.curr_col_list:
                (colflags, sort_increasing) = self.column_info[attribute]
                colname = Players.player_attr_conf[attribute][0]
                if attribute in self.curr_col_list:
                    (cvalue, dvalue) = self.player_data.all_players[pid].data[attribute]
                    if dvalue == None:
                        value = ""
                    else:
                        value = dvalue
                    if wxPlatform != '__WXMSW__':
                        if attribute == "Cards":
                            if 0 < cvalue <= 3:
                                img = self.images[{1: "yellowcard",
                                                   2: "2yellowcards",
                                                   3: "redcard"}[cvalue]]
                                self.list.SetStringItem(x, y, "", img)
                            else:
                                self.list.SetStringItem(x, y, "")
                        elif attribute == "CountryID":
                            cid = Countries.country[cvalue]
                            if cid in self.flags.keys():
                                img = self.flags[cid]
                                self.list.SetStringItem(x, y, "", img)
                            else:
                                self.list.SetStringItem(x, y, value)
                        elif attribute == "rating":
                            if 0 < cvalue <= 30:
                                img = self.stars[cvalue]
                                self.list.SetStringItem(x, y, value, img)
                            elif cvalue == 0:
                                self.list.SetStringItem(x, y, "")
                            else:
                                self.list.SetStringItem(x, y, value)
                        elif attribute == "InjuryLevel":
                            if cvalue == 0:
                                img = self.images["bruised"]
                                self.list.SetStringItem(x, y, "", img)
                            elif cvalue > 0:
                                img = self.images["injury"]
                                self.list.SetStringItem(x, y, string.split(dvalue)[0], img)
                            else:
                                self.list.SetStringItem(x, y, value)

                        else:
                            if y == 0:
                                if pid in self.starting_players:
                                    img = self.images["dress"]
                                    self.list.SetStringItem(x, y, value, img)
                                elif pid in self.substitute_players:
                                    img = self.images["subdress"]
                                    self.list.SetStringItem(x, y, value, img)
                                else:
                                    self.list.SetStringItem(x, y, value)
                            else:
                                self.list.SetStringItem(x, y, value)
                    else:
                        if y == 0:
                            if pid in self.starting_players:
                                img = self.images["dress"]
                                self.list.SetStringItem(x, y, value, img)
                            elif pid in self.substitute_players:
                                img = self.images["subdress"]
                                self.list.SetStringItem(x, y, value, img)
                            else:
                                self.list.SetStringItem(x, y, value)
                        else:
                            self.list.SetStringItem(x, y, value)
                    y += 1
            self.list.SetItemData(x, x)
            x += 1

    def SortList(self, column, resort):
        if column == -1:
            column = 0
        old_col = self.sort_column
        real_col = self.__get_column_attribute(column)

        if real_col == None:
            return
        self.player_sort_attribute = real_col
        self.sort_column = column
        self.sort_colname = Players.player_attr_conf[real_col][0]

        if not resort and old_col == column:
            self.ascending = not self.ascending
            self.columns_increasing_sort[real_col] = self.ascending
        elif old_col != column:
            self.ascending = self.columns_increasing_sort[real_col]
        self.list.SortItems(self.__column_sorter)
        self.__update_column_images(old_col)

    def UpdateColumns(self):
        self.list.ClearAll()
        info = wxListItem()
        info.m_mask = wxLIST_MASK_TEXT | wxLIST_MASK_FORMAT
        if wxPlatform != '__WXMSW__':
            info.m_mask |= wxLIST_MASK_IMAGE
        info.m_image = -1
        new_sortcol = -1
        self.col_ids = {}
        i = 0
        for attribute in self.curr_col_list:
            (colflags, sort_increasing) = self.column_info[attribute]
            colname = Players.player_attr_conf[attribute][0]
            if attribute in self.curr_col_list:
                if colflags == None:
                    info.m_format = 0
                else:
                    info.m_format = colflags
                info.m_text = colname
                self.list.InsertColumnInfo(i, info)
                self.col_ids[i] = attribute
                if self.sort_colname == colname:
                    new_sortcol = i
                i += 1

        self.PopulateList()
        self.SortList(new_sortcol, True)

        for i in range(len(self.col_ids)):
            self.list.SetColumnWidth(i, wxLIST_AUTOSIZE)

        self.__set_column_widths()

    def SaveColumnWidths(self):
        widths = []
        cnt = self.list.GetColumnCount()
        for k in range(cnt):
            widths.append(self.list.GetColumnWidth(k))
        if len(widths) > 0:
            db_path = ("Mattricks.column_list.widths.%s" % self.curr_col_list_name)
            width_str = string.join([str(width) for width in widths], ",")
            Database.databases.sets(None, False,
                                    {db_path: width_str})
        
    def __set_column_widths(self):
        db_path = ("Mattricks.column_list.widths.%s" % self.curr_col_list_name)
        width_str = Database.databases.gets(None, False, [db_path])[db_path]
        widths = []

        if width_str is not None:
            for width in width_str.split(","):
                widths.append(int(width))

        cnt = self.list.GetColumnCount()
        i=0
        for w in widths:
            if i < cnt:
                self.list.SetColumnWidth(i,w)
            i=i+1

    def __get_column_attribute(self, index):
        if not self.col_ids.has_key(index):
            return None
        return self.col_ids[index]

    def __column_sorter(self, key1, key2):
        p1 = self.player_data.all_players[self.pid_list[key1]]
        p2 = self.player_data.all_players[self.pid_list[key2]]
        d1 = p1.data[self.player_sort_attribute][0]
        d2 = p2.data[self.player_sort_attribute][0]
        if d1 > d2:
            res = 1
        elif d1 < d2:
            res = -1
        else:
            res = 0

        if not self.ascending:
            res = -res

        return res

    def __update_column_images(self, old_col):
        if wxPlatform == '__WXMSW__':
            return

        if old_col != -1:
            self.list.ClearColumnImage(old_col)
        if self.sort_column != -1:
            img = self.images[{True: "down", False: "up"}[self.ascending]]
            self.list.SetColumnImage(self.sort_column, img)


class PlayerPage(wxPanel):
    ID_POPUP_MENU = 211
    ID_DATE_SELECTED = 212
    ID_EDIT_COLUMNS = 213
    ID_LISTCHOICE = 214
    ID_SQUAD_STATS = 215
    ID_SAVE_WIDTHS = 216

    def __init__(self, parent, ID):

        wxPanel.__init__(self, parent, ID)

        sizer = wxBoxSizer(wxVERTICAL)
        hsizer = wxBoxSizer(wxHORIZONTAL)
        vsizer = wxBoxSizer(wxVERTICAL)

        # Data change and info about data.

        button = wxButton(self, PlayerPage.ID_EDIT_COLUMNS, _("Edit columns"), \
                          style = wxBU_EXACTFIT)
        vsizer.Add(button, 0, wxALIGN_CENTRE | wxEXPAND | wxALL, 5)
        button = wxButton(self, PlayerPage.ID_SAVE_WIDTHS, _("Save column widths"), \
                          style = wxBU_EXACTFIT)
        vsizer.Add(button, 0, wxALIGN_CENTRE | wxALL, 5)

        hsizer.Add(vsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        # Squad statistics button

        vsizer = wxBoxSizer(wxVERTICAL)

        button = wxButton(self, self.ID_SQUAD_STATS, _("Squad statistics"))
        vsizer.Add(button, 0, wxALIGN_RIGHT | wxALL, 5)

        # Column views

        csizer = wxBoxSizer(wxHORIZONTAL)

        stext = wxStaticText(self, -1, _("Choose what columns to view") + ":")
        csizer.Add(stext, 0, wxALIGN_CENTRE | wxALL, 5)

        self.cl = dict([(_(descr), name) for name, (descr, the_list) in \
                        column_lists.items()])
        choices = self.cl.keys()
        choices.reverse()
        self.choice_of_list = wxChoice(self, self.ID_LISTCHOICE, choices = choices)
        self.choice_of_list.SetSelection(0) # Should do something smarter...
        csizer.Add(self.choice_of_list, 0, wxALIGN_CENTRE | wxALL, 5)

        vsizer.Add(csizer, 0, wxALIGN_RIGHT | wxALL, 0)
        hsizer.Add(vsizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        sizer.Add(hsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        # The player list

        self.playerlist = PlayerListCtrlPanel(self,
                                              self.__get_current_list())
        sizer.Add(self.playerlist, 1, wxEXPAND | wxALL, 1)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        EVT_BUTTON(self, PlayerPage.ID_EDIT_COLUMNS, self.OnEditCol)
        EVT_BUTTON(self, PlayerPage.ID_SAVE_WIDTHS, self.OnSaveWidths)
        EVT_BUTTON(self, PlayerPage.ID_SQUAD_STATS, self.OnSquadStatistics)
        EVT_CHOICE(self, PlayerPage.ID_LISTCHOICE, self.ChangeList)

    def AfterInit(self):
        self.playerlist.AfterInit()
        self.playerlist.player_data = Players.Players()
        self.playerlist.UpdateColumns()

    def ChangeXML(self):
        self.playerlist.player_data = Players.Players()
        self.playerlist.UpdateColumns()

    def OnSaveWidths(self, event):
        self.playerlist.SaveColumnWidths()

    def OnEditCol(self, event):
        cl = column_lists["custom"][1]
        hl = []
        tl = []
        for c in playerlist_columns.keys():
            if c not in cl:
                hl.append(c)
            tl.append(c)

        tl = dict([("%s (%s)" % (Players.player_attr_conf[c][0],
                Players.plattr_strings[Players.player_attr_conf[c][3]]), c) for c in tl])
        cl = ["%s (%s)" % (Players.player_attr_conf[c][0],
                Players.plattr_strings[Players.player_attr_conf[c][3]])  for c in cl]
        hl = ["%s (%s)" % (Players.player_attr_conf[c][0],
                Players.plattr_strings[Players.player_attr_conf[c][3]]) for c in hl]

        dlg = Columndialog.ColumnDialog(cl, hl)
        ret = dlg.ShowModal()
        if ret == wxID_OK:
            cl = dlg.GetEnabledStrings()
            cl = [tl[c] for c in cl]
            cdescr = column_lists["custom"][0]
            self.choice_of_list.SetStringSelection(_(cdescr))
            column_lists["custom"] = (cdescr, cl)
            if len(cl) > 0:
                column_str = string.join([str(column) for column in cl], ",")
                Database.databases.sets(None, False, {"Mattricks.column_list.custom": column_str})
            else:
                # Fetch from database to make sure it exists...
                dummy = Database.databases.gets(None, False, ["Mattricks.column_list.custom"])["Mattricks.column_list.custom"]
                Database.databases.sets(None, False, {"Mattricks.column_list.custom": None})
            self.playerlist.curr_col_list = cl
            self.playerlist.curr_col_list_name = "custom"
            self.playerlist.UpdateColumns()
        dlg.Destroy()

    def OnSquadStatistics(self, event):
        ret = SquadStatsDialog.SquadStatsDialog(self, self.playerlist.player_data).Show()

    def ChangeList(self, event):
        (self.playerlist.curr_col_list, self.playerlist.curr_col_list_name) = self.__get_current_list()
        self.playerlist.UpdateColumns()

    def __get_current_list(self):
        sl = self.choice_of_list.GetStringSelection()
        if not sl:
            return ([],None)
        return (column_lists[self.cl[sl]][1], self.cl[sl])
