# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Functions and classes to help pulling out relevant information of
the players xml."""

from Common import *
import XMLParse
import CommonData
from Database import databases
import TeamEvaluation
import Terms
import TeamInfo
import Countries
import string
from sets import Set
import time
import TransferHistory
import Main
import MatchInfo
import Config

known_player_attrs = Set([
    "PlayerID", "PlayerName", "PlayerNumber", "Age",
    "PlayerForm", "Statement", "Experience", "Leadership", "Salary",
    "Agreeability", "Aggressiveness", "Honesty", "LeagueGoals", "CupGoals",
    "FriendliesGoals", "CareerGoals", "CareerHattricks", "Specialty",
    "TransferListed", "Cards", "InjuryLevel", "StaminaSkill",
    "KeeperSkill", "PlaymakerSkill", "ScorerSkill", "PassingSkill",
    "WingerSkill", "DefenderSkill", "SetPiecesSkill",
    "TrainerData.TrainerType", "TrainerData.TrainerSkill", "CountryID",
    "Caps", "CapsU20", "NationalTeamID",
    ])

mALIGN_LEFT = 0
mALIGN_CENTRE = 1
mALIGN_RIGHT = 2

plattr_BASIC = 0
plattr_PERSONALITY = 1
plattr_ECONOMY = 2
plattr_GOALS = 3
plattr_STATUS = 4
plattr_SKILL = 5
plattr_COACH = 6
plattr_CAPS = 7
plattr_EVALUATION = 8

plattr_strings = {
    plattr_BASIC: _("Basic"),
    plattr_PERSONALITY: _("Personality"),
    plattr_ECONOMY: _("Economy"),
    plattr_GOALS: _("Goals"),
    plattr_STATUS: _("Status"),
    plattr_SKILL: _("Skill"),
    plattr_COACH: _("Coach"),
    plattr_CAPS: _("Caps"),
    plattr_EVALUATION: _("Evaluation"),
    }

player_attr_conf = {
    "PlayerID": (_("ID"), mALIGN_CENTRE, False, plattr_BASIC),
    "PlayerName": (_("Name"), mALIGN_LEFT, True, plattr_BASIC),
    "PlayerNumber": (_("Shirt number"), mALIGN_RIGHT, True, plattr_BASIC),
    "Age": (_("Age"), mALIGN_RIGHT, True, plattr_BASIC),
    "MarketValue": (_("Market value"), mALIGN_RIGHT, False, plattr_ECONOMY),
    "TSI": (_("Total Skill Index"), mALIGN_RIGHT, False, plattr_ECONOMY),
    "PlayerForm": (_("Form"), mALIGN_LEFT, False, plattr_STATUS),
    "Statement": (_("Statement"), mALIGN_LEFT, True, plattr_PERSONALITY),
    "Experience": (_("Experience"), mALIGN_LEFT, False, plattr_PERSONALITY),
    "Leadership": (_("Leadership"), mALIGN_LEFT, False, plattr_PERSONALITY),
    "Salary": (_("Salary"), mALIGN_RIGHT, False, plattr_ECONOMY),
    "Agreeability": (_("Agreeability"), mALIGN_LEFT, False, plattr_PERSONALITY),
    "Aggressiveness": (_("Aggressiveness"), mALIGN_LEFT, False, plattr_PERSONALITY),
    "Honesty": (_("Honesty"), mALIGN_LEFT, False, plattr_PERSONALITY),
    "LeagueGoals": (_("League goals"), mALIGN_RIGHT, False, plattr_GOALS),
    "CupGoals": (_("Cup goals"), mALIGN_RIGHT, False, plattr_GOALS),
    "FriendliesGoals": (_("Goals in friendlies"), mALIGN_RIGHT, False, plattr_GOALS),
    "CareerGoals": (_("Career goals"), mALIGN_RIGHT, False, plattr_GOALS),
    "CareerHattricks": (_("Career Hattricks"), mALIGN_RIGHT, False, plattr_GOALS),
    "CompetitionGoals": (_("Competition goals"), mALIGN_RIGHT, False, plattr_GOALS),
    "SeasonGoals": (_("Goals in this season"), mALIGN_RIGHT, False, plattr_GOALS),
    "Specialty": (_("Speciality"), mALIGN_LEFT, True, plattr_SKILL),
    "TransferListed": (_("Transfer listed?"), mALIGN_LEFT, True, plattr_ECONOMY),
    "Cards": (_("Bookings"), mALIGN_CENTRE, False, plattr_STATUS),
    "InjuryLevel": (_("Injury time"), mALIGN_RIGHT, False, plattr_STATUS),
    "StaminaSkill": (_("Stamina"), mALIGN_LEFT, False, plattr_SKILL),
    "KeeperSkill": (_("Goalkeeper"), mALIGN_LEFT, False, plattr_SKILL),
    "PlaymakerSkill": (_("Playmaking"), mALIGN_LEFT, False, plattr_SKILL),
    "ScorerSkill": (_("Scoring"), mALIGN_LEFT, False, plattr_SKILL),
    "PassingSkill": (_("Passing"), mALIGN_LEFT, False, plattr_SKILL),
    "WingerSkill": (_("Winger"), mALIGN_LEFT, False, plattr_SKILL),
    "DefenderSkill": (_("Defending"), mALIGN_LEFT, False, plattr_SKILL),
    "SetPiecesSkill": (_("Set pieces"), mALIGN_LEFT, False, plattr_SKILL),
    "TrainerData.TrainerType": (_("Coach type"), mALIGN_LEFT, False, plattr_COACH),
    "TrainerData.TrainerSkill": (_("Coaching ability"), mALIGN_LEFT, False, plattr_COACH),
    "CountryID": (_("Country"), mALIGN_RIGHT, True, plattr_BASIC),
    "Caps": (_("National caps"), mALIGN_RIGHT, False, plattr_CAPS),
    "CapsU20": (_("National U-20 caps"), mALIGN_RIGHT, False, plattr_CAPS),
    "BestPosition": (_("Best position"), mALIGN_LEFT, True, plattr_EVALUATION),
    "NationalTeamID": (_("National team"), mALIGN_LEFT, True, plattr_CAPS),
    "ArrivalDate": (_("Arrival date"), mALIGN_LEFT, True, plattr_ECONOMY),
    "PricePaid": (_("Price paid"), mALIGN_LEFT, True, plattr_ECONOMY),
    "MotherClub": (_("Mother club"), mALIGN_LEFT, True, plattr_ECONOMY),
    "Position": (_("Position in next game"), mALIGN_LEFT, True, plattr_STATUS),
    "BoughtFrom": (_("Bought from"), mALIGN_LEFT, True, plattr_ECONOMY),
}

def add_team_positions_to_player_attr_conf():
    global player_attr_conf
    for i, (position, descr) in Terms.team_positions.items():
        player_attr_conf[position] = (descr, mALIGN_RIGHT, False, plattr_EVALUATION)

add_team_positions_to_player_attr_conf()

class Player(CommonData.DataStore):
    """Class that contains information about a player."""

    def __init__(self, data):
        """Takes player data and initializes the class."""

        CommonData.DataStore.__init__(self, data)

        self.id = self.get("PlayerID", None, False)
        self.name = self.get("PlayerName", "", True)

class Players:
    """Class that extracts and stores information about all players
    from a particular date."""


    def __init__(self,  date = None):
        """Initialize the structure.

         - date -- A quadruple of (year, month, day, hour) telling which
                   day and hour to fetch players for."""

        team = CommonData.get_current_teamid()

        self.__extract_data(date, team)

    def __extract_data(self, date, team):
        """Extracts the data and put it into player structures.

         - date -- A quadruple of (year, month, day, hour) telling which
                   day and hour to fetch players for.
         - team -- Which teams players to fetch for."""

        if date is None:
            identifier = ["all players"]
            max_identifier = None
        else:
            identifier = ["all players", team, date]
            max_identifier = [team, date]

        data = XMLParse.get(identifier,
                            normal = True, display = True,
                            nofetch = True, get_latest_available = True,
                            max_id = max_identifier)

        if data is None or not(len(data)):
            data = XMLParse.get(identifier,
                                normal = True, display = True)


        ndata = {}
        ddata = {}
        for key, value in data.items():
            ndata[key] = value[0]
            ddata[key] = value[1]

        self.fetched_date = ndata.get("FetchedDate")
        self.fetched_date_text = ddata.get("FetchedDate")
        self.team_id = ndata.get("Team.TeamID")
        self.team_name = ddata.get("Team.TeamName", "")
        team=TeamInfo.TeamInfo(self.team_id)
        self.league_id=team.league_id

        player_indices = CommonData.get_xml_list_indices("Team.PlayerList.Player", ndata)
        self.all_players = {}

        self.player_roles = {}
        self.substitute_players = []
        mid = Main.notebook_pages["matchlist"].GetNextMatch()
        if mid is not None:
            mdata = MatchInfo.LineupInfo(mid)
            if mdata.available:
                self.player_roles = mdata.player_roles
                self.substitute_players = mdata.substitute_players
        
        for pindex in player_indices:
            if ndata.get("Team.PlayerList.Player%s.PlayerID" % pindex) is None:
                break
            pdata = {}
            for pattr in known_player_attrs:
                pdata[pattr] = (ndata.get("Team.PlayerList.Player%s.%s" % (pindex, pattr)),
                                ddata.get("Team.PlayerList.Player%s.%s" % (pindex, pattr)))

            player = Player(pdata)

            pnumber=player.data["PlayerNumber"]
            if pnumber[0] == 100:
                player.data["PlayerNumber"] = (100, "")

            mvalue = ndata.get("Team.PlayerList.Player%s.MarketValue" % (pindex))
            tsivalue = ndata.get("Team.PlayerList.Player%s.TSI" % (pindex))
            if mvalue == None:
                mvalue = 1000*tsivalue
            elif tsivalue == None:
                tsivalue = mvalue/1000
            player.data["MarketValue"] = (mvalue, CommonData.add_currency(mvalue))
            player.data["TSI"] = (tsivalue, string.join(split_equal(str(tsivalue), 3, True), " "))

            statement = player.get("Statement")
            if statement is None:
                player.data["Statement"] = ("", "")
                
            surname = name2surname(player.get("PlayerName", "?"))
            player.data["PlayerSurname"] = (surname, surname)

            compgoals = player.get("CupGoals", 0) + player.get("LeagueGoals", 0)
            player.data["CompetitionGoals"] = (compgoals,
                                               ngettext("%d goal", "%d goals", compgoals) % compgoals)

            seasgoals = player.get("CompetitionGoals", 0) + player.get("FriendliesGoals", 0)
            player.data["SeasonGoals"] = (seasgoals,
                                          ngettext("%d goal", "%d goals", seasgoals) % seasgoals)


            trandata = TransferHistory.TransferHistory(player.id, "player")
            if len(trandata.transfers):
                player.data["ArrivalDate"] = (trandata.transfers[0].get("Deadline", 0.0),
                                              trandata.transfers[0].get("Deadline", 0.0, display = True))
                player.data["PricePaid"] = (trandata.transfers[0].get("Price", 0.0),
                                            trandata.transfers[0].get("Price", 0.0, display = True))
                player.data["MotherClub"] = (trandata.transfers[len(trandata.transfers)-1].get("Seller.SellerTeamName",""),
                                             trandata.transfers[len(trandata.transfers)-1].get("Seller.SellerTeamName","",display = True))
                player.data["BoughtFrom"] = (trandata.transfers[0].get("Seller.SellerTeamName",""),
                                             trandata.transfers[0].get("Seller.SellerTeamName","",display = True))
            else:
                player.data["PricePaid"] = (0, "")
                player.data["ArrivalDate"] = (0.0, "")
                player.data["MotherClub"] = (self.team_name, self.team_name)
                player.data["BoughtFrom"] = ("", "")
                
            if self.player_roles.has_key(player.id):
                if player.id not in self.substitute_players:
                    player.data["Position"] = (self.player_roles[player.id],
                                               Terms.team_positions[self.player_roles[player.id]][1])
                else:
                    player.data["Position"] = (self.player_roles[player.id] + 20,
                                               Terms.team_positions[self.player_roles[player.id]][1] + " (" + _("Substitute") + ")")
            else:
                player.data["Position"] = (100, "")
                
            self.all_players[player.id] = player

        player_eval_table = TeamEvaluation.TeamEvaluation.calculate_player_table(self.all_players)

        bpos = {}
        for pid in self.all_players.keys():
            bpos[pid] = (None, None, None)

        for i, (position, descr) in Terms.team_positions.items():
            count = 0
            for pval, pid in player_eval_table[i]:
                count += 1

                self.all_players[pid].data[position] = ((pval, count),
                                                        "%.2f (%d)" % (pval, count))

                if pval > bpos[pid][2]:
                    bpos[pid] = (i, descr, pval)

        for pid, (i, descr, pval) in bpos.items():
            if i is not None:
                self.all_players[pid].data["BestPosition"] = ((i, pval),
                                                              "%s (%.2f)" % (descr, pval))

class Compare:
    """Class for making a comparison between two player objects."""

    def __init__(self):
        self.__cached_text = None

        self.__sort_on_attribute = False
        self.__attribute_filter = None
        self.__players1 = None
        self.__players2 = None

    def get_diff_list(self, p1, p2, attribute_list = None):
        if p1 is None or p2 is None:
            return []

        if attribute_list is None:
            attribute_list = player_attr_conf.keys()

        diff_list = {}
        p1_pids = p1.all_players.keys()
        p2_pids = p2.all_players.keys()
        pids = intersection(p1_pids, p2_pids)
        for attribute in attribute_list:
            for pid in pids:
                attr1 = p1.all_players[pid].data[attribute]
                attr2 = p2.all_players[pid].data[attribute]

                if attr1 != attr2:
                    if attr1 > attr2:
                        diff_list[(attribute, pid)] = 1
                    else:
                        diff_list[(attribute, pid)] = -1
        print diff_list
        return diff_list


    def get_text(self,
                 players1 = None,
                 players2 = None,
                 sort_on_attribute = False,
                 attribute_filter = None):

        if ((players1 != self.__players1) or
            (players2 != self.__players2) or
            (sort_on_attribute != self.__sort_on_attribute) or
            (attribute_filter != self.__attribute_filter)):

            self.__players1 = players1
            self.__players2 = players2
            self.__sort_on_attribute = sort_on_attribute
            self.__attribute_filter = attribute_filter

            self.__cached_text = self.__create_text()

        return self.__cached_text

    def __create_text(self):
        if self.__players1 is None or self.__players2 is None:
            return ""

        space = " "*5

        result = ""

        p1_pids = self.__players1.all_players.keys()
        p2_pids = self.__players2.all_players.keys()


        # Sold or sacked players
        first = True
        for pid in p1_pids:
            if pid not in p2_pids:
                if first:
                    result += _("Players that have been sold or sacked:") + "\n"
                    first = False
                result += space + self.__players1.all_players[pid].data["PlayerName"][1] + "\n"
        if not first:
            result += "\n"

        # Bought or juniors.
        first = True
        for pid in p2_pids:
            if pid not in p1_pids:
                if first:
                    result += _("Players that have been bought or taken up from the youth squad:") + "\n"
                    first = False
                result += space + self.__players2.all_players[pid].data["PlayerName"][1] + "\n"
        if not first:
            result += "\n"

        pids = intersection(p1_pids, p2_pids)
        
        # If we should sort on attribute we do like this
        if self.__sort_on_attribute:
            for attribute in player_attr_conf.keys():

                if Config.config.get("Format","MV or TSI") == "TSI":
                    if attribute == "MarketValue":
                        continue
                else:
                    if attribute == "TSI":
                        continue

                group = player_attr_conf[attribute][3]

                if self.__attribute_filter is None or \
                       group in self.__attribute_filter:

                    first = True
                    for pid in pids:
                        attr1 = self.__players1.all_players[pid].data[attribute]
                        attr2 = self.__players2.all_players[pid].data[attribute]

                        if attr1[0] != attr2[0]:
                            if first:
                                result += player_attr_conf[attribute][0] + \
                                          " (" + plattr_strings[group] + "): \n"
                                first = False
                            result += space + \
                                      self.__players2.all_players[pid].data["PlayerName"][1] + \
                                      ": " + attr1[1] + " -> " + attr2[1] + "\n"
                    if not first:
                        result += "\n"

        else:
            for pid in pids:

                first = True

                for attribute in player_attr_conf.keys():
                    if Config.config.get("Format","MV or TSI") == "TSI":
                        if attribute == "MarketValue":
                            continue
                    else:
                        if attribute == "TSI":
                            continue

                    group = player_attr_conf[attribute][3]
                    if self.__attribute_filter is None or \
                           group in self.__attribute_filter:

                        attr1 = self.__players1.all_players[pid].data[attribute]
                        attr2 = self.__players2.all_players[pid].data[attribute]

                        if attr1[0] != attr2[0]:
                            if first:
                                result += self.__players2.all_players[pid].data["PlayerName"][1] + ": \n"
                                first = False
                            result += space + \
                                      player_attr_conf[attribute][0] + \
                                      " (" + plattr_strings[group] + "): " + \
                                      attr1[1] + " -> " + attr2[1] + "\n"
                if not first:
                    result += "\n"


        return result

def get_players_from_one_week_ago(players):
    pass

class PlayerHistory:
    """Class that traces one or more attributes of a player through
    a period of time."""
