# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import time

import HRFData

from Common import *
from wxPython.wx import *

mEVT_DATE_SELECTED = wxNewEventType()

def EVT_DATE_SELECTED(win, id, func):
    win.Connect(id, -1, mEVT_DATE_SELECTED, func)

class DateSelectedEvent(wxPyCommandEvent):
    def __init__(self, evtType, id):
        wxPyCommandEvent.__init__(self, evtType, id)
        self.date = None
        self.hrfFile = None

    def GetDate(self):
        return self.date

    def GetHRFFile(self):
        return self.hrfFile

    def SetDate(self, date):
        self.date = date

    def SetHRFFile(self, hrfFile):
        self.hrfFile = hrfFile

class SelectDateMenu(wxMenu):
    ID_LATEST = 997
    ID_NEXT = 998
    ID_PREVIOUS = 999

    def __init__(self, parent, myid, current_hrf):
        self.parent = parent
        self.id = myid
        self.day_ids = {}
        self.current_id = 1000

        wxMenu.__init__(self)

        self.latest = None
        self.nearest_before = None
        self.nearest_after = None
        got_current = 0
        first = True
        years = {}
        for date, file in HRFData.htdirs.hrflist:
            if first:
                self.latest = (date, file)
                first = False
            if got_current == 1:
                self.nearest_before = (date, file)
                got_current = -1
            if got_current == 0 and current_hrf == file:
                got_current = 1
            if got_current == 0:
                self.nearest_after = (date, file)

            year, month, day = time.localtime(date)[:3]
            if not years.has_key(year):
                years[year] = {}
            if not years[year].has_key(month):
                years[year][month] = []
            if not day in years[year][month]:
                years[year][month].append(day)

        if self.latest != None:
            self.Append(SelectDateMenu.ID_LATEST, _("Latest"))
        if self.nearest_after != None:
            self.Append(SelectDateMenu.ID_NEXT, _("Next"))
        if self.nearest_before != None:
            self.Append(SelectDateMenu.ID_PREVIOUS, _("Previous"))

        month_names = ["", _("January"), _("February"), _("March"),
                       _("April"), _("May"), _("June"), _("July"),
                       _("August"), _("September"), _("October"),
                       _("November"), _("December")]

        this_year = time.localtime(time.time())[0]
        years = years.items()
        years.sort()
        years.reverse()
        firstyear = True
        for year, months in years:
            if year == this_year:
                year_menu = self
            else:
                year_menu = wxMenu()

            months = months.items()
            months.sort()
            months.reverse()
            for month, days in months:
                month_menu = wxMenu()

                days.sort()
                days.reverse()
                for day in days:
                    month_menu.Append(self.__get_id(year, month, day), str(day))

                month_name = month_names[month]
                year_menu.AppendMenu(self.__get_id(year, month), month_name, month_menu)
            if year == this_year:
                firstyear = False
            else:
                self.AppendMenu(self.__get_id(year), str(year), year_menu)

        EVT_MENU(parent, SelectDateMenu.ID_LATEST, self.OnSpecialClick)
        EVT_MENU(parent, SelectDateMenu.ID_NEXT, self.OnSpecialClick)
        EVT_MENU(parent, SelectDateMenu.ID_PREVIOUS, self.OnSpecialClick)
        [EVT_MENU(parent, day_id, self.OnDayClick) for day_id in self.day_ids.keys()]

    def OnSpecialClick(self, event):
        id = event.GetId()
        if id == SelectDateMenu.ID_LATEST:
            self.__generate__event(*self.latest)
        if id == SelectDateMenu.ID_NEXT:
            self.__generate__event(*self.nearest_after)
        if id == SelectDateMenu.ID_PREVIOUS:
            self.__generate__event(*self.nearest_before)

    def OnDayClick(self, event):
        id = event.GetId()
        year, month, day = self.day_ids[id]
        etime = time.mktime((year,month,day+1,0,0,0,0,0,0))-1
        hrffile = HRFData.htdirs.get_from_time(etime)
        self.__generate__event(etime, hrffile)

    def __generate__event(self, etime, hrffile):
        evt = DateSelectedEvent(mEVT_DATE_SELECTED, self.id)
        evt.SetDate(etime)
        evt.SetHRFFile(hrffile)
        self.GetEventHandler().ProcessEvent(evt)

    def __get_id(self, year = None, month = None, day = None):
        self.current_id += 1
        self.day_ids[self.current_id] = (year, month, day)
        return self.current_id
