# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Squad statistics dialog, showing statistics over the entrire squad."""

from Common import *

import Players
import PlotCtrl
import Terms

from wxPython.wx import *
from Commonwx import *

class SquadStatsDialog(wxDialog):

    disp_skills = [("StaminaSkill", (30, 255, 255)),
                   ("KeeperSkill", (255, 255, 30)),
                   ("PlaymakerSkill", (255, 30, 30)),
                   ("ScorerSkill", (30, 30, 255)),
                   ("PassingSkill", (255, 30, 255)),
                   ("WingerSkill", (255, 255, 255)),
                   ("DefenderSkill", (30, 255, 30)),
                   ("SetPiecesSkill", (255, 127, 30))]

    def __init__(self, parent, players):
        wxDialog.__init__(self, parent, -1, _("Squad statistics"),
                          style = wxRESIZE_BORDER | wxCAPTION | wxSYSTEM_MENU)

        self.__players = players

        main_sizer = wxBoxSizer(wxVERTICAL)
        hsizer = wxBoxSizer(wxHORIZONTAL)

        vsizer = wxBoxSizer(wxVERTICAL)

        for skill, colour in self.disp_skills:
            spanel = SimpleColouredPanel(self, -1, colour, wxVERTICAL)
            stext = wxStaticText(spanel, -1, Players.player_attr_conf[skill][0],
                                 style = wxALIGN_CENTRE)
            spanel.AddChild(stext, 0, wxALIGN_CENTRE | wxALL, 2)
            vsizer.Add(spanel, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        hsizer.Add(vsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        ptable = self.__make_plot_table()

        self.pc = PlotCtrl.PlotCtrl(self)
        self.pc.SetGridType(PlotCtrl.PLOT_GRID_CHECKERED)

        for skill, colour, stable in ptable:
            self.pc.AddData(skill, PlotCtrl.PlotDataHistogram(stable, colour = colour))

        self.pc.SetClientSize((600, 400))


        hsizer.Add(self.pc, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        main_sizer.Add(hsizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        # Line

        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        main_sizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        # Ok / Cancel buttons

        button = wxButton(self, wxID_OK, _("Ok"))
        button.SetDefault()
        main_sizer.Add(button, 0, wxALIGN_RIGHT | wxALL, 10)

        self.SetSizer(main_sizer)
        self.SetAutoLayout(True)
        main_sizer.Fit(self)

    def __make_plot_table(self):
        ptable = []
        for skill, colour in self.disp_skills:
            ptable += [(skill, colour, {})]

        for player in self.__players.all_players.values():
            for skill, colour, stable in ptable:
                val = player.get(skill, 0)
                if val > 0:
                    if stable.has_key(val):
                        stable[val] += 1
                    else:
                        stable[val] = 1

        restable = []
        for skill, colour, stable in ptable:
            ntable = []
            for val in range(1, max(stable.keys()) + 1):
                amount = stable.get(val, 0)
                ntable.append((Terms.player_abilities[val], amount))
            restable.append((skill, colour, ntable))

        return restable
