# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Some functions to speed-up stats generation"""

import XMLParse
import CommonData
import string
from Database import databases

def get_player_name(playerid, matchid):
    my_team = CommonData.get_current_teamid()
    if playerid > 0:        
        name = databases.get(None, False, "Mattricks.Player+%d.Name" % (playerid))
        if name is None:
            data = XMLParse.get(["match lineup", matchid, my_team])
            lineup_indices = CommonData.get_xml_list_indices("Team.Lineup.Player", data)
            for i in lineup_indices:
                pid = data.get("Team.Lineup.Player%s.PlayerID" % i)
                if pid == playerid:
                    name = data.get("Team.Lineup.Player%s.PlayerName" % i)
                    if name is not None:
                        databases.sets(None, False, {("Mattricks.Player+%d.Name" % (playerid)): name})
                    else:
                        name = "?"
        return name
    else:
        return _("Invalid player id")

def get_used_players(matchid):
    my_team = CommonData.get_current_teamid()
    if matchid > 0:
        lineupstring = databases.get(None, False, "Mattricks.Match+%d.Team+%d.Lineup" % (matchid, my_team))
        if lineupstring is None:
            data = XMLParse.get(["match lineup", matchid, my_team])
            lineup_indices = CommonData.get_xml_list_indices("Team.Lineup.Player", data)
            lineup=[]
            for i in lineup_indices:
                pid = data.get("Team.Lineup.Player%s.PlayerID" % i)
                role = data.get("Team.Lineup.Player%s.RoleID" % i)
                rating = data.get("Team.Lineup.Player%s.RatingStars" % i)

                if (role <= 11 or role >= 19) and rating is not None:
                    lineup.append(pid)

            if len(lineup):
                id_str = string.join([str(pid) for pid in lineup], ",")
                databases.sets(None, False, {("Mattricks.Match+%d.Team+%d.Lineup" % (matchid, my_team)): id_str})
        else:
            lineup = []
            for pidstring in lineupstring.split(","):
                lineup.append(int(pidstring))
        return lineup
    else:
        return []

def get_scorers(matchid):
    my_team = CommonData.get_current_teamid()
    if matchid > 0:
        scorerstring = databases.get(None, False, "Mattricks.Match+%d.Team+%d.Scorers" % (matchid, my_team))
        if scorerstring is None:
            data = XMLParse.get(["match details", matchid])
            goals = data.get("Match.Scorers.xml_attrs.NumberOfGoals", 0)
            scorers = []
            if goals < 1:
                scorer_indices =  []
            else:
                scorer_indices = CommonData.get_xml_list_indices("Match.Scorers.Goal", data)
            for goal in scorer_indices:
                id = data.get("Match.Scorers.Goal%s.ScorerPlayerID" % goal)
                team = data.get("Match.Scorers.Goal%s.ScorerTeamID" % goal)
                if team == my_team:
                    scorers.append(id)
                    
            if len(scorers):
                id_str = string.join([str(pid) for pid in scorers], ",")
                databases.sets(None, False, {("Mattricks.Match+%d.Team+%d.Scorers" % (matchid, my_team)): id_str})
        else:
            scorers = []
            for pidstring in scorerstring.split(","):
                scorers.append(int(pidstring))
        return scorers
    else:
        return []
