# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import Terms
import Players
import XMLParse
import XMLDataInfo
import CommonData

from Common import *
from Commonwx import *

from wxPython.wx import *
from Config import config
from Images import Images

class SummaryPage(wxPanel):
    def __init__(self, parent, ID):

        wxPanel.__init__(self, parent, ID)

        sizer = wxBoxSizer(wxVERTICAL)

        hsizer2 = wxBoxSizer(wxHORIZONTAL)
        fsizer = wxFlexGridSizer(cols=2, vgap=5, hgap=10)
        fsizer.AddGrowableCol(1)
        info = [
            (_('Team'), self.__get_display_value, ("Team.TeamName",)),
            (_('Country'), self.__get_display_value, ("Team.League.LeagueName",)),
            (_('Arena'), self.__get_display_value, ("Team.Arena.ArenaName",)),
            (_('Series'), self.__get_display_value, ("Team.LeagueLevelUnit.LeagueLevelUnitName",)),
            (None, None, None),
            (_('Troop value'), self.__calc_player_value, ("MarketValue", False)),
            (_('Average player value'), self.__calc_player_value, ("MarketValue", True)),
            (_('Troop salaries'), self.__calc_player_value, ("Salary", False)),
            (_('Average player salaries'), self.__calc_player_value, ("Salary", True)),
            (_('Average age'), self.__calc_player_value, ("Age", True)),
            (_('Current form (average)'), self.__calc_player_display_value,
             ("PlayerForm", True, Terms.player_abilities)),
            (_('Experience (average)'), self.__calc_player_display_value,
             ("Experience", True, Terms.player_abilities)),
            (_('Number of players'), self.__calc_player_value, (None, False)),
            (_('Number of nationalities'), self.__calc_nationalities, ()),
            ]
        self.info = []
        for name, func, args in info:
            if name == None:
                fsizer.Add((15,15))
                fsizer.Add((15,15))
            else:
                nametext = wxStaticText(self, -1, name + ":")
                SetTextBold(nametext)
                fsizer.Add(nametext, 0, wxALIGN_LEFT | wxALIGN_BOTTOM | wxALIGN_TOP)
                stext = wxStaticText(self, -1, "                     ")
                fsizer.Add(stext, 1, wxEXPAND | wxALIGN_LEFT | wxALIGN_BOTTOM | wxALIGN_TOP)
                self.info.append((stext, func, args))

        hsizer2.Add(fsizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 10)
        self.logobmp=wxStaticBitmap(self,-1,Images.get_bitmap_from_file(config.get("Team info", "logo file")))
        hsizer2.Add(self.logobmp,0, wxEXPAND | wxALIGN_CENTRE | wxALL, 10)
        sizer.Add(hsizer2, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

    def AfterInit(self):
        self.Update()

    def Update(self):
        self.cdata = XMLParse.get(["team details", CommonData.get_current_teamid()])
        self.pdata = Players.Players()

        for stext, func, args in self.info:
            stext.SetLabel(str(func(*args)))

        self.GetSizer().Layout()
        self.Refresh()

    def OnChangeXML(self):
        self.Update()
        
    def __get_display_value(self, attribute):
        return self.cdata.get(attribute)

    def __calc_player_display_value(self, attribute, average, convertor):
        val = self.__calc_player_value(attribute, average, False)
        return "%s (%.1f)" % (convertor[round(val)], val)

    def __calc_player_value(self, attribute, average, make_str = True):
        if config.get("Statistics", "include coach"):
            coach = None
        else:
            coach = self.cdata.get("Team.Trainer.PlayerID")
        sum = 0
        num = 0
        for pid in self.pdata.all_players.keys():
            if pid != coach:
                num += 1
                if attribute != None:
                    tmp = self.pdata.all_players[pid].data[attribute]
                    if tmp[0] != None:
                        sum += tmp[0]
        if attribute != None:
            f2=XMLDataInfo.xml_known_data["all players"]["Team.PlayerList.Player.%s" % attribute][2]
            if average:
                if make_str:
                    return f2(int(round(float(sum)/float(num))))
                else:
                    return float(sum)/float(num)
            else:
                if make_str:
                    return f2(sum)
                else:
                    return sum
        else:
            if make_str:
                return str(num)
            else:
                return num

    def __calc_nationalities(self):
        nationalities = []
        for pid in self.pdata.all_players.keys():
            n = self.pdata.all_players[pid].data["CountryID"]
            if not n in nationalities:
                nationalities.append(n)
        return str(len(nationalities))
