# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""League table plot dialog, plotting information extracted from the league table."""

from Common import *

import Players
import PlotCtrl
import Terms

from wxPython.wx import *
from Commonwx import *

class TablePlotsDialog(wxDialog):

    colours = [(30, 255, 255),
                   (255, 255, 30),
                   (255, 30, 30),
                   (30, 30, 255),
                   (255, 30, 255),
                   (255, 255, 255),
                   (30, 255, 30),
                   (255, 127, 30)]

    ID_CHOICE = 700

    plot_choices=[_("Team position"), _("Team points"),
                  _("Goals for"), _("Goals against"),
                  _("Goal difference"), _("Goals for histogram"), _("Goals against histogram")]

    def __init__(self, parent, fixture):
        wxDialog.__init__(self, parent, -1, _("League table statistics"),
                          style = wxRESIZE_BORDER | wxCAPTION | wxSYSTEM_MENU)

        self.__fixture = fixture

        main_sizer = wxBoxSizer(wxVERTICAL)
        hsizer = wxBoxSizer(wxHORIZONTAL)

        vsizer = wxBoxSizer(wxVERTICAL)

        (self.fdata, self.adata)=self.__fixture.get_goal_hist()
        
        self.plotinfo={}
        for i in range(1,self.__fixture.GetLatestRound()+1,1):
            (allteams, hteams, ateams)=self.__fixture.calc_table(i)
            for j in range(8):
                (pl, w, d, l, f, a, po, n, tid) = allteams[j]
                if not self.plotinfo.has_key(tid):
                    self.plotinfo[tid]={}
                    for k in range(len(self.plot_choices)):
                        if not self.plotinfo[tid].has_key(k):
                            self.plotinfo[tid][k]=[]
                            if k > 0:
                                self.plotinfo[tid][k].append((0.0, 0.0))
                self.plotinfo[tid][0].append((float(i), float(j + 1)))
                self.plotinfo[tid][1].append((float(i), float(po)))
                self.plotinfo[tid][2].append((float(i), float(f)))
                self.plotinfo[tid][3].append((float(i), float(a)))
                self.plotinfo[tid][4].append((float(i), float(f-a)))

        self.pc = PlotCtrl.PlotCtrl(self)
        self.pc.SetGridType(PlotCtrl.PLOT_GRID_CHECKERED)

        self.team_ids=[]
        for j in range(8):
            (pl, w, d, l, f, a, po, n, tid) = allteams[j]
            spanel = SimpleColouredPanel(self, -1, self.colours[j], wxVERTICAL)
            stext = wxStaticText(spanel, -1, n,
                                 style = wxALIGN_CENTRE)
            spanel.AddChild(stext, 0, wxALIGN_CENTRE | wxALL, 2)
            vsizer.Add(spanel, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)
            self.team_ids.append(tid)

        # Select plot
        self.choice=wxChoice(self, self.ID_CHOICE, choices = self.plot_choices)
        self.choice.SetSelection(0)
        vsizer.Add(self.choice, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)
        
        hsizer.Add(vsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        self.pc.SetClientSize((600, 400))
        hsizer.Add(self.pc, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        main_sizer.Add(hsizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        # Line

        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        main_sizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        # Ok / Cancel buttons

        button = wxButton(self, wxID_OK, _("Ok"))
        button.SetDefault()
        main_sizer.Add(button, 0, wxALIGN_RIGHT | wxALL, 10)

        EVT_CHOICE(self, self.ID_CHOICE, self.OnPlotChange) 

        self.__change_plot()
        self.SetSizer(main_sizer)
        self.SetAutoLayout(True)
        main_sizer.Fit(self)

    def OnPlotChange(self, event):
        self.__change_plot()
        
    def __change_plot(self):
        plot=self.choice.GetSelection()

        # Clean up previous plot
        self.pc.ClearData()
        self.pc.RemoveTicks()
        self.pc.ReverseAxis(False, False)

        # Add data
        if (plot <= 4):
            for i in range(8):
                self.pc.AddData(str(self.team_ids[i]), PlotCtrl.PlotDataLine(self.plotinfo[self.team_ids[i]][plot], colour=self.colours[i]))
            # Set axes
            if plot == 0: # Positions
                yticks=[(0,"")] + [(float(j), str(j)) for j in range(1, 9)] + [(9.0, "")]
                self.pc.SetTicks(yticks, True, True)
                self.pc.ReverseAxis(False)
                xticks=[(float(j), str(j)) for j in range(1, 15)]
                self.pc.SetTicks(xticks, False, True)
            else: # The rest
                xticks=[(float(j), str(j)) for j in range(15)]
                self.pc.SetTicks(xticks, False, True)
        elif (plot == 5):
            for i in range(8):
                self.pc.AddData(str(self.team_ids[i]), PlotCtrl.PlotDataHistogram(self.fdata[self.team_ids[i]], colour=self.colours[i]))
        elif (plot == 6):
            for i in range(8):
                self.pc.AddData(str(self.team_ids[i]), PlotCtrl.PlotDataHistogram(self.adata[self.team_ids[i]], colour=self.colours[i]))
                
        # Redraw
        self.pc.Redraw()
            
