# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import string
import MatchInfo
import Database
import TeamInfo
import Main
import CommonData
import XMLParse
import Images

from wxPython.wx import *
from Common import *
from Commonwx import *


class TeamList(wxDialog):
    ID_ADD_MATCH=400
    ID_ADD_TEAM=401
    ID_ADD_MATCH_FROM_LIST=402
    ID_REMOVE_TEAM_FROM_LIST=403
    ID_VIEW_LEAGUE_TABLE=404
    ID_IMPORT_BOOKMARKS=405
    ID_UP_ARROW=406
    ID_DOWN_ARROW=407
    ID_LISTBOX=408


    def __init__(self, parent):

        pos = wxDefaultPosition
        size = wxDefaultSize,
        style = wxDEFAULT_DIALOG_STYLE

        wxDialog.__init__(self, parent, -1, _("Team List"), pos, wxSize(400, 600), style, "Team List")

        self.parent=parent

        sizer = wxBoxSizer(wxVERTICAL)
        hsizer = wxBoxSizer(wxHORIZONTAL)
        hsizer.Add(wxStaticText(self, -1, _("Team ID")+":"),0,wxALL | wxALIGN_CENTRE,0)
        self.add_team_edit = wxTextCtrl(self, -1, size = (50, -1))
        self.add_team_edit.SetValue("")
        hsizer.Add(self.add_team_edit, 1, wxALIGN_CENTRE | wxALL, 2)
        button=wxButton(self, self.ID_ADD_MATCH, _("Add next match"))
        hsizer.Add(button,0,wxALL,5)
        button=wxButton(self, self.ID_ADD_TEAM, _("Add to team list"))
        hsizer.Add(button,0,wxALL,5)
        sizer.Add(hsizer,0,wxALL,5)

        # Line
        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        sizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 5)

        # List box and related

        hsizer = wxBoxSizer(wxHORIZONTAL)
        teams_text=wxStaticText(self, -1, _("Teams"))
        SetTextFace(teams_text, FACE_BOLD | FACE_BIG)
        sizer.Add(teams_text,0,wxALL, 5)
        self.listbox = wxListBox(self, self.ID_LISTBOX, style=wxLB_MULTIPLE, size = (-1, 200))
        hsizer.Add(self.listbox,1,wxALL|wxEXPAND,0)
        hsizer.Add((5, 5), 0)
        up_img = Images.Images.get_bitmap_from_file("arrow_small_up.png")
        down_img = Images.Images.get_bitmap_from_file("arrow_small_down.png")
        vsizer = wxBoxSizer(wxVERTICAL)
        up_button = wxBitmapButton(self, self.ID_UP_ARROW, up_img, style=wxBU_AUTODRAW | wxBU_EXACTFIT)
        vsizer.Add(up_button, 0, wxALL)
        down_button = wxBitmapButton(self, self.ID_DOWN_ARROW, down_img, style=wxBU_AUTODRAW | wxBU_EXACTFIT)
        vsizer.Add(down_button, 0, wxALL)
        hsizer.Add(vsizer, 0, wxALL | wxALIGN_CENTER)
        sizer.Add(hsizer,1,wxALL|wxEXPAND,5)
        hsizer = wxBoxSizer(wxHORIZONTAL)
        button=wxButton(self, self.ID_ADD_MATCH_FROM_LIST, _("Add next match"))
        hsizer.Add(button,0,wxALL,0)
        hsizer.Add((1, 1), 1)
        button=wxButton(self, self.ID_VIEW_LEAGUE_TABLE, _("View league table"))
        hsizer.Add(button,0,wxALL,0)
        hsizer.Add((1, 1), 1)
        button=wxButton(self, self.ID_REMOVE_TEAM_FROM_LIST, _("Remove team"))
        hsizer.Add(button,0,wxALL,0)
        sizer.Add(hsizer,0,wxEXPAND|wxALL,5)

        # Populate list box
        path = "Mattricks.team_list"
        id_str = Database.databases.gets(None, False, [path])[path]
        self.teams = {}

        if id_str is not None:
            for team_id in id_str.split(","):
                if team_id.isdigit():
                    team_id=int(team_id)
                    self.__add_team(team_id)
                else:
                    self.__add_separator(team_id)

        # Line
        line = wxStaticLine(self, -1, style = wxLI_HORIZONTAL)
        sizer.Add(line, 0, wxEXPAND | wxALIGN_CENTRE, 0)


        hsizer = wxBoxSizer(wxHORIZONTAL)

        # Check for supporter and add import bookmarks
        if CommonData.is_supporter():

            button=wxButton(self, self.ID_IMPORT_BOOKMARKS, _("Import bookmarks"))
            hsizer.Add(button,0,wxALL,0)
            hsizer.Add((1, 1), 1)

        # Ok / Cancel buttons
        button = wxButton(self, wxID_OK, _("Ok"))
        button.SetDefault()
        hsizer.Add(button, 0, wxALIGN_RIGHT | wxALL, 0)

        sizer.Add(hsizer, 0, wxEXPAND| wxALL, 5)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        EVT_BUTTON(self, self.ID_ADD_MATCH, self.OnAddMatch)
        EVT_BUTTON(self, self.ID_ADD_MATCH_FROM_LIST, self.OnAddMatchList)
        EVT_BUTTON(self, self.ID_ADD_TEAM, self.OnAddTeam)
        EVT_BUTTON(self, self.ID_REMOVE_TEAM_FROM_LIST, self.OnRemoveTeam)
        EVT_BUTTON(self, self.ID_VIEW_LEAGUE_TABLE, self.OnViewLeagueTable)
        EVT_BUTTON(self, self.ID_IMPORT_BOOKMARKS, self.OnImportBookmarks)
        EVT_BUTTON(self, self.ID_UP_ARROW, self.OnMoveUp)
        EVT_BUTTON(self, self.ID_DOWN_ARROW, self.OnMoveDown)
        EVT_LISTBOX_DCLICK(self, self.ID_LISTBOX, self.OnDoubleClick)

    def OnMoveUp(self, event):
        id=self.listbox.GetSelections()
        for i in id:
            if (i == 0):
                self.listbox.Deselect(0)
            else:
                temp_team_info=self.teams[i]
                self.teams[i]=self.teams[i-1]
                self.teams[i-1]=temp_team_info
                temp = self.listbox.GetString(i)
                self.listbox.SetString(i, self.listbox.GetString(i-1))
                self.listbox.SetString(i-1, temp)
                self.listbox.Deselect(i)
                self.listbox.SetSelection(i-1)
        self.__update_database()

    def OnMoveDown(self, event):
        id=self.listbox.GetSelections()
        count =self.listbox.GetCount()
        list = range(count)
        list.reverse()

        for i in list:
            if (self.listbox.Selected(i)):
                if (i!=(count-1)):
                    temp_team_info=self.teams[i]
                    self.teams[i]=self.teams[i+1]
                    self.teams[i+1]=temp_team_info
                    temp = self.listbox.GetString(i)
                    self.listbox.SetString(i, self.listbox.GetString(i+1))
                    self.listbox.SetString(i+1, temp)
                    self.listbox.Deselect(i)
                    self.listbox.SetSelection(i+1)
        self.__update_database()

    def OnDoubleClick(self, event):
        id=self.listbox.GetSelections()
        for i in id:
            team_id=self.teams[i][0]
            if isinstance(team_id,int):
                if team_id not in range(3000,4999):
                    team_info=TeamInfo.TeamInfo(team_id, progress_dialog=True, force_update=True)
                    self.teams[i][1]=team_info.team_name
                    self.teams[i][2]=team_info.league_level_id
                    self.teams[i][3]=team_info.league_level_name
                    self.teams[i][4]=team_info.league_name
                    self.teams[i][5]=team_info.league_id
                    self.listbox.SetString(i, "%s (%s), %s, %s" % (self.teams[i][1], self.teams[i][0], self.teams[i][3], self.teams[i][4]))

    def OnAddMatch(self, event):
        team_id = self.add_team_edit.GetValue()
        import Main
        Main.notebook_pages["matchview"].AddMatchFromTeamID(team_id)

    def OnAddMatchList(self, event):
        id=self.listbox.GetSelections()
        import Main
        for i in id:
            team_id=self.teams[i][0]
            if isinstance(team_id,int):
                Main.notebook_pages["matchview"].AddMatchFromTeamID(team_id)
            self.listbox.Deselect(i)

    def OnViewLeagueTable(self, event):
        id=self.listbox.GetSelections()
        if len(id) > 0:
            team_id=self.teams[id[0]][0]
            if isinstance(team_id,int):
                if team_id not in range(3000,4999):
                    import Main
                    Main.notebook_pages["table"].SetLeague(self.teams[id[0]][2], self.teams[id[0]][5])
            self.listbox.Deselect(id[0])

    def OnAddTeam(self, event):
        team_id = self.add_team_edit.GetValue()
        self.__add_team(team_id, quiet=False)

    def OnImportBookmarks(self, event):
        data = XMLParse.get(["bookmarks", ""], force_update = True)

        bookmarks = data.get("UserBookmarks.xml_attrs.Count")
        for ind in range(bookmarks):
            team_id = data.get("UserBookmarks.UserBookmark+%s.TeamID" % ind)
            self.__add_team(team_id, quiet = True)

    def OnRemoveTeam(self, event):
        id = self.listbox.GetSelections()
        i = []
        for x in id:
            i.append(x)
        i.reverse()
        for x in i:
            self.listbox.Delete(x)
            for k in range(len(self.teams)-1):
                if k >= x:
                    self.teams[k]=self.teams[k+1]
        for x in range(len(id)):
            del self.teams[len(self.teams)-1]
        self.__update_database()

    def __add_separator(self, name):
        id = len(self.teams)

        if not self.teams.has_key(id):
            self.teams[id] = {}
        self.teams[id][0]=name
        l=len(name)
        st=""
        m=int((50-l)/2)
        for k in range(m):
            st+="-"

        st = st + " " + name + " "

        for k in range(m):
            st+="-"

        self.listbox.Append(st)

        self.__update_database()

    def __add_team(self, team_id, quiet = False):

        if isinstance(team_id,str):
            if not team_id.isdigit():
                self.__add_separator(team_id)
                return

        try:
            team_id = int(team_id)
            if team_id < 1:
                raise ValueError
        except ValueError, err:
            self.__error_dlg(_("Team id needs to be a positive integer."))
            return


        id = len(self.teams)


        if id > 0:
            for k in range(id):
                if self.teams[k][0]==team_id:
                    if not quiet:
                        self.__error_dlg(_("Team id already in the list."))
                    return


        if team_id in range(3000,4999):
            match_info=MatchInfo.CurrentMatches(team_id, progress_dialog=False)
            if match_info is not None:
                if not self.teams.has_key(id):
                    self.teams[id] = {}
                self.teams[id][0]=team_id
                self.teams[id][1]=match_info.team_name
                self.listbox.Append("%s (%s), " % (self.teams[id][1], self.teams[id][0]) + _("National team"))
            else:
                self.__error_dlg(_("Non-existing team."))
                return
        else:
            try:
                team_info=TeamInfo.TeamInfo(team_id, progress_dialog=False)
            except:
                self.__error_dlg(_("Non-existing team."))
                return

            if not self.teams.has_key(id):
                self.teams[id] = {}
            self.teams[id][0]=team_id
            self.teams[id][1]=team_info.team_name
            self.teams[id][2]=team_info.league_level_id
            self.teams[id][3]=team_info.league_level_name
            self.teams[id][4]=team_info.league_name
            self.teams[id][5]=team_info.league_id
            self.listbox.Append("%s (%s), %s, %s" % (self.teams[id][1], self.teams[id][0], self.teams[id][3], self.teams[id][4]))
        self.__update_database()

    def __update_database(self):
        team_ids = []
        if len(self.teams) > 0:
            for id in range(len(self.teams)):
                team_ids.append(self.teams[id][0])
            id_str = string.join([str(team_id) for team_id in team_ids], ",")
            Database.databases.sets(None, False, {"Mattricks.team_list": id_str})
        else:
            # Fetch from database to check if it exists...
            dummy = Database.databases.gets(None, False, ["Mattricks.team_list"])["Mattricks.team_list"]
            if dummy is not None:
                Database.databases.sets(None, False, {"Mattricks.team_list": None})

    def __error_dlg(self, msg):
        mdlg = wxMessageDialog(self, msg, _("Error"), wxOK | wxICON_ERROR)
        mdlg.ShowModal()
        mdlg.Destroy()
