# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from Common import *
from Commonwx import *

from LineupCtrl import LineupCtrl
import SelectDateMenu
import TeamSelectorXML
import Terms
import Players
import MatchInfo
import CommonData

from wxPython.wx import *

class TeamSelectorPage(wxPanel):
    ID_POPUP_MENU    = 301
    ID_DATE_SELECTED = 302
    ID_CALC_TEAM     = 303
    ID_COPY_LAST     = 304
    ID_CLEAR_PLAYERS = 305

    def __init__(self, parent, ID):
        wxPanel.__init__(self, parent, ID)

        sizer = wxBoxSizer(wxVERTICAL)

        hsizer = wxBoxSizer(wxHORIZONTAL)

        vsizer = wxBoxSizer(wxVERTICAL)

        self.manual_formation = wxCheckBox(self, -1, _("Use manual formation?"))
        self.manual_formation.SetValue(False)
        vsizer.Add(self.manual_formation, 0, wxEXPAND | wxALIGN_LEFT | wxALL, 5)


        self.calc_button = wxButton(self, TeamSelectorPage.ID_CALC_TEAM, \
                                    _("Let the computer pick a team"), style = wxBU_EXACTFIT)
        vsizer.Add(self.calc_button, 0, wxALIGN_CENTRE | wxALL, 5)

        mhsizer = wxBoxSizer(wxHORIZONTAL)

        ttext = wxStaticText(self, -1, _("Team value") + ":")
        mhsizer.Add(ttext, 0, wxEXPAND | wxADJUST_MINSIZE | wxALL, 5)

        self.teamval_text = wxStaticText(self, -1, "")
        mhsizer.Add(self.teamval_text, 1, wxEXPAND | wxADJUST_MINSIZE | wxALL, 5)

        vsizer.Add(mhsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        hsizer.Add(vsizer, 0, wxALIGN_CENTER_VERTICAL | wxEXPAND | wxALIGN_LEFT | wxALL, 0)

        # Eligible players
        if wxPlatform in ['__WXMSW__', '__WXGTK__']:
            vsizer = wxBoxSizer(wxVERTICAL)
            ptext = wxStaticText(self, -1, _("Players not to use") + ":")
            vsizer.Add(ptext, 0, wxEXPAND | wxADJUST_MINSIZE | wxALL, 2)

            self.listbox = wxCheckListBox(self, -1, style=wxLB_SINGLE) # Not working on __WXMAC__
            vsizer.Add(self.listbox,1,wxEXPAND|wxALL,2)

            self.last_button = wxButton(self, TeamSelectorPage.ID_COPY_LAST, \
                                        _("Previous game players"), style = wxBU_EXACTFIT)
            vsizer.Add(self.last_button, 0, wxALIGN_CENTRE | wxALL, 2)
            self.clear_button = wxButton(self, TeamSelectorPage.ID_CLEAR_PLAYERS, \
                                        _("Clear selections"), style = wxBU_EXACTFIT)
            vsizer.Add(self.clear_button, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 2)

            hsizer.Add(vsizer, 0, wxEXPAND | wxALIGN_LEFT | wxALL, 3)

            EVT_BUTTON(self, TeamSelectorPage.ID_COPY_LAST, self.OnCopyLast)
            EVT_BUTTON(self, TeamSelectorPage.ID_CLEAR_PLAYERS, self.OnClearPlayers)

        else: # When wxCheckListBox is not available (__WXOSX__)
            self.previous_players = wxCheckBox(self, -1, _("Skip previous game players?"))
            self.previous_players.SetValue(False)
            hsizer.Add(self.previous_players, 0, wxEXPAND | wxALIGN_LEFT | wxALL, 5)

        # Manual select of formation.

        vsizer = wxBoxSizer(wxVERTICAL)

        self.roles = []
        self.role_choices = {}
        pos_info = TeamSelectorXML.TeamSelector.pos_info
        for line in [[(2, "wingback"), (3,"centraldefender"),
                      (4, "centraldefender"), (5, "wingback")],
                     [(6, "winger"), (7, "innermid"), (8, "innermid"), (9, "winger")],
                     [(10, "forward"), (11, "forward")]]:
            mhsizer = wxBoxSizer(wxHORIZONTAL)
            for roleid, c in line:
                self.roles.append((roleid, c))
                mvsizer = wxBoxSizer(wxVERTICAL)

                stext = wxStaticText(self, -1, Terms.team_role[roleid] + ":")
                mvsizer.Add(stext, 0, wxALIGN_CENTRE | wxBOTTOM, 2)

                choice_list = [Terms.individual_order[x] for x in pos_info[c]]
                choice = wxChoice(self, -1, choices = choice_list)
                choice.SetSelection(0)
                mvsizer.Add(choice, 0, wxALIGN_CENTRE | wxALL, 0)
                self.role_choices[roleid] = choice
                EVT_CHOICE(self, choice.GetId(), self.OnChoice) 

                mhsizer.Add(mvsizer, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

            vsizer.Add(mhsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        hsizer.Add(vsizer, 1, wxEXPAND | wxALIGN_LEFT | wxALL, 0)

        sizer.Add(hsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        self.lineup = LineupCtrl(self, -1, False, "teamselect", None)
        sizer.Add(self.lineup, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        EVT_BUTTON(self, TeamSelectorPage.ID_CALC_TEAM, self.OnCalcTeam)

    def AfterInit(self):
        self.__update_player_list()

    def ChangeXML(self):
        self.__update_player_list()

    def OnChoice(self, event):
        self.manual_formation.SetValue(True)

    def OnCopyLast(self, event):
        import Main
        mid = Main.notebook_pages["matchlist"].GetLastPlayedMatch()
        mdata = MatchInfo.MatchLineup(mid, CommonData.get_current_teamid())

        i=0
        player_ids = mdata.player_id_played
        for (name, pid) in self.name_pid:
            if pid in player_ids:
               self.listbox.Check(i)
            i+=1

    def OnClearPlayers(self, event):
        for i in range(self.listbox.GetCount()):
            self.listbox.Check(i, False)

    def OnCalcTeam(self, event):
        dontuse_ids = []
        if wxPlatform in ['__WXMSW__', '__WXGTK__']:
            i=0
            for (name, pid) in self.name_pid:
                if self.listbox.IsChecked(i):
                    dontuse_ids.append(pid)
                i+=1
        else:
            if self.previous_players.GetValue():
                import Main
                mid = Main.notebook_pages["matchlist"].GetLastPlayedMatch()
                mdata = MatchInfo.MatchLineup(mid, CommonData.get_current_teamid())
                dontuse_ids = mdata.player_id_played


        formation_data = None
        if self.manual_formation.GetValue():
            formation_data = {"keeper": 0}

            rolenames = {2: "rightBack", 3: "insideBack1",
                         4: "insideBack2", 5: "leftBack",
                         6: "rightWinger", 7: "insideMid1",
                         8: "insideMid2", 9: "leftWinger",
                         10: "forward1", 11: "forward2"}

            pos_info = TeamSelectorXML.TeamSelector.pos_info
            for roleid, c in self.roles:
                s = self.role_choices[roleid].GetSelection()
                s = pos_info[c][s]
                formation_data[rolenames[roleid]] = s

        ts = TeamSelectorXML.TeamSelector(dontuse_ids)
        lineup = ts.select_team(formation_data)
        self.lineup.ChangeData("teamselect", lineup)
        self.teamval_text.SetLabel("%.2f" % ts.teamval(lineup))

        self.Refresh()

    def __update_player_list(self):
        if wxPlatform not in ['__WXMSW__', '__WXGTK__']:
            return


        self.player_list=Players.Players()
        self.pids=self.player_list.all_players.keys()

        self.name_pid = []
        for pid in self.pids:
            self.name_pid.append((self.player_list.all_players.get(pid).name, pid))

        self.name_pid.sort()
        self.listbox.Clear()
        for (name, pid) in self.name_pid:
            self.listbox.Append(name)
