# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Functions and classes to help a user evaluate a good lineup."""

from Common import *

import copy
import string
import math
import operator

import Terms
import Config
import TeamEvaluation
import Players

class TeamSelector:
    """Class handling the calculation of the best team possible."""

    lineup_info = [
        (_("Goalkeeper"), "keeper", "goalie"),
        (_("Right wingback"), "rightBack", "wingback"),
        (_("Central defender 1"), "insideBack1", "centraldefender"),
        (_("Central defender 2"), "insideBack2", "centraldefender"),
        (_("Left wingback"), "leftBack", "wingback"),
        (_("Right winger"), "rightWinger", "winger"),
        (_("Midfielder 1"), "insideMid1", "innermid"),
        (_("Midfielder 2"), "insideMid2", "innermid"),
        (_("Left winger"), "leftWinger", "winger"),
        (_("Forward 1"), "forward1", "forward"),
        (_("Forward 2"), "forward2", "forward")
        ]

    pos_info = {
        "goalie": [0],
        "wingback": [0, 1, 2, 3, 5, 6],
        "centraldefender": [0, 1, 4, 5, 6],
        "winger": [0, 1, 2, 3, 5, 7],
        "innermid": [0, 1, 2, 4, 5, 7],
        "forward": [0, 2, 6, 7]
        }

    funcs = ["goalie",
             "centraldefender", "centraldefender_offensive", "centraldefender_towards_the_wing",
             "wingback", "wingback_offensive",
             "wingback_defensive", "wingback_towards_the_middle",
             "innermid", "innermid_offensive", "innermid_defensive", "innermid_towards_the_wing",
             "winger", "winger_offensive", "winger_defensive", "winger_towards_the_middle",
             "forward", "forward_defensive"]

    order_funcs_info = {
        0: "P",
        1: "P_offensive",
        2: "P_defensive",
        3: "P_towards_the_middle",
        4: "P_towards_the_wing",
        5: "forward",
        6: "innermid",
        7: "centraldefender"
        }

    def __init__(self, dontuse_pids = []):
        self.dontuse_pids = dontuse_pids
        self.player_table = None

    def get_player_table(self):
        if self.player_table == None:
            self.__calc_player_table()

        return self.player_table

    def select_team(self, formation_data = None):
        if self.player_table == None:
            self.__calc_player_table()

        return self.__internal_select_team(formation_data)

    def set_real_pos(pos, extra):
        f = TeamSelector.order_funcs_info[extra]
        if f[0] == "P":
            return pos + f[1:]
        return f

    set_real_pos = staticmethod(set_real_pos)

    def teamval(self, lineup):
        """Function for evaluating the value of the team, given a lineup."""
        wing_balance = 0.0
        back_value = 0.0
        midfield_value = 0.0
        forward_value = 0.0
        pvals = []
        for cpos, (pid, val, n, pos, extra) in lineup.items():
            if pid is not None:
                if cpos[0] == "r" and extra in [0, 1, 2]:
                    wing_balance += val/15
                elif cpos[0] == "l" and extra in [0, 1, 2]:
                    wing_balance -= val/15
                elif cpos[-1] == "1" and extra == 4:
                    wing_balance += val/30
                elif cpos[-1] == "2" and extra == 4:
                    wing_balance -= val/30

                if (pos in ["centraldefender", "wingback"] and extra < 5) or \
                       extra == 7:
                    back_value += val
                elif (pos in ["innermid", "winger"] and extra < 5) or \
                         extra == 6:
                    midfield_value += val
                elif (pos == "forward" and extra < 5) or \
                         extra == 5:
                    forward_value += val

                if extra in [5, 6, 7]:
                    val = 0.95*val
                pvals.append(val)

        back_value *= 1.0
        midfield_value *= 0.8
        forward_value *= 2.0

        minval = min(back_value, midfield_value, forward_value)
        maxval = max(back_value, midfield_value, forward_value)
        diffval = maxval - minval
        #print wing_balance
        #pvals = zip(*lineup.values())[1]
        basic = reduce(operator.add, pvals)/float(len(pvals))
        ret = basic - abs(wing_balance) - diffval/35
        #print ret, back_value, midfield_value, forward_value, diffval
        return ret

    def __calc_team(self, the_lineup):
        start_over = True
        while start_over:
            old_team_val = self.teamval(the_lineup)
            start_over = False
            for cpos, (pid, val, n, pos, extra) in the_lineup.items():
                if pid is not None:
                    rpos = self.set_real_pos(pos, extra)
                    for i in range(1,n):
                        spid = self.player_table[pos][i-1][1]
                        for ocpos, (opid, dummy, dummy, opos, oextra) in the_lineup.items():
                            if spid == opid:
                                break

                        ropos = self.set_real_pos(opos, oextra)

                        #if ropos == rpos:
                        #    continue

                        n = 0
                        for pval, ppid in self.player_table[rpos]:
                            n += 1
                            if ppid == opid:
                                oval = pval
                                on = n
                                break
                        n = 0
                        for pval, ppid in self.player_table[ropos]:
                            n += 1
                            if ppid == pid or \
                                   (not ppid in zip(*the_lineup.values())[0]):
                                npid = ppid
                                nval = pval
                                nn = n
                                break

                        new_lineup = copy.copy(the_lineup)
                        new_lineup[ocpos] = (npid, nval, nn, opos, oextra)
                        new_lineup[cpos] = (opid, oval, on, pos, extra)
                        team_val = self.teamval(new_lineup)
                        #print team_val, old_team_val, (pos, extra), (opos, oextra)
                        if team_val > old_team_val:
                            the_lineup = new_lineup
                            start_over = True
                            break
                    if start_over:
                        break
        return the_lineup

    def __calc_formation(self, the_lineup):
        start_over = True
        while start_over:
            old_team_val = self.teamval(the_lineup)
            start_over = False
            for cpos, (pid, val, n, pos, extra) in the_lineup.items():
                if pid is not None:
                    for nextra in TeamSelector.pos_info[pos]:
                        if nextra == extra:
                            continue

                        rpos = self.set_real_pos(pos, nextra)

                        n = 0
                        for pval, ppid in self.player_table[rpos]:
                            n += 1
                            if ppid == pid or \
                                   (not ppid in zip(*the_lineup.values())[0]):
                                npid = ppid
                                nval = pval
                                nn = n
                                break

                        new_lineup = copy.copy(the_lineup)
                        new_lineup[cpos] = (npid, nval, nn, pos, nextra)
                        new_lineup = self.__calc_team(new_lineup)
                        team_val = self.teamval(new_lineup)

                        if team_val > old_team_val:
                            the_lineup = new_lineup
                            start_over = True
                            break
                    if start_over:
                        break
        return the_lineup

    def __internal_select_team(self, formation_data = None):
        the_lineup = dict([(c,(None, None, None, None, None))
                           for d,c,p in TeamSelector.lineup_info])

        for descr, cpos, pos in TeamSelector.lineup_info:
            n = 0
            if formation_data is not None and formation_data.has_key(cpos):
                extra = formation_data[cpos]
                npos = self.set_real_pos(pos, extra)
            else:
                extra = 0
                npos = pos
            for val, pid in self.player_table[npos]:
                n += 1
                if not pid in zip(*the_lineup.values())[0]:
                    the_lineup[cpos] = (pid, val, n, pos, extra)
                    break

        the_lineup = self.__calc_team(the_lineup)
        if formation_data is None:
            the_lineup = self.__calc_formation(the_lineup)
        return the_lineup

    def __calc_player_table(self):
        player_table = {}
        players = Players.Players()
        for f in TeamEvaluation.PlayerEvaluation.eval_positions.keys():
            player_table[f] = []
            for pid in players.all_players.keys():
                if pid not in self.dontuse_pids:
                    player_table[f].append((players.all_players[pid].data[f][0][0], pid))
            player_table[f].sort()
            player_table[f].reverse()
        self.player_table = player_table
