# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Functions and classes to help pulling out relevant information of
the transfer history xml."""

from Common import *

import XMLParse
import CommonData

class TransferHistory:
    """Class for extracting and storing information about
    transfer history in a convenient way."""

    def __init__(self, aid, action = "player"):
        """Initialization

         - aid -- the id of the action, if action is player, a player id,
                  and if team a team id.
         - action -- the type of transfer history, player or team."""

        if action == "team" and aid is None:
            aid = CommonData.get_current_teamid()

        self.__extract_data(aid, action)

    def __extract_data(self, aid, action):
        """Extraction of tranfer history data.

         - aid -- the id of the action, if action is player, a player id,
                  and if team a team id.
         - action -- the type of transfer history, player or team."""

        if action == "player":
            data = XMLParse.get(["transfer history", action, aid],
                                normal = True, display = True)
        elif action == "team":
            raise NotImplemented("The 'team' action hasn't been implemented yet.")
            data = XMLParse.get(["transfer history", action, aid, None],
                                normal = True, display = True)

        if data.get("FileName", None) is None:
            self.has_data = False
            self.transfers = []
            return

        ver=data.get("Version", '0.5')
        if ver[0] < '1.2':
            if action == "player":
                data = XMLParse.get(["transfer history", action, aid],
                                    normal = True, display = True, force_update = True)
            elif action == "team":
                raise NotImplemented("The 'team' action hasn't been implemented yet.")
                data = XMLParse.get(["transfer history", action, aid, None],
                                    normal = True, display = True, force_update = True)

        self.has_data = True

        ndata = {}
        ddata = {}
        for key, (normal, display) in data.items():
            ndata[key] = normal
            ddata[key] = display

        self.startdate = ndata.get("Transfers.StartDate", 0.0)
        self.enddate = ndata.get("Transfers.EndDate", 0.0)

        if action == "player":
            self.player_id = ndata.get("Transfers.Player.PlayerID")
            self.player_name = ndata.get("Transfers.Player.PlayerName", "")
            transfer_keys = ["TransferID", "Deadline",
                             "Buyer.BuyerTeamID", "Buyer.BuyerTeamName",
                             "Seller.SellerTeamID", "Seller.SellerTeamName",
                             "Price", "TSI"]

        self.transfers = []
        transfer_indices = CommonData.get_xml_list_indices("Transfers.Transfer", ndata)
        for tindex in transfer_indices:
            tdata = {}
            for key in transfer_keys:
                tdata[key] = data.get("Transfers.Transfer%s.%s" % (tindex, key))
        
            TSI=tdata["TSI"]
            MV=1000*TSI[0]
            tdata["MarketValue"]=(MV, CommonData.add_currency(MV))
            self.transfers.append(CommonData.DataStore(tdata))

        CommonData.DataStore.set_comparison_key("Deadline")
        self.transfers.sort()
        self.transfers.reverse()
