# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""File for parsing xml:s and related functions."""

from Common import *

import cStringIO
import operator
import os
import os.path
import string
import sys

from xml.sax import saxlib, saxexts, SAXException

import XMLDataInfo
from Database import databases
import Fetch
from Config import config
import DialogSet
import I18n
import CommonData

def convert_value(value, conv_func, display_conv_func, normal = True, display = False):
    """Takes a string value and converts it using conv_func and display_conv_func.

     - normal -- if True, then only the normal conversion is done and returned.
     - display -- if True, then the value is converted for suitability of display.

    If both normal and display is True a tuple with both conversions is returned."""

    if not (normal or display):
        return None

    if value is None:
        if normal and display:
            return (None, None)
        return None

    if conv_func is not None:
        try:
            value = conv_func(value)
        except Exception, e:
            print "Conversion error: %s" % str(e)

    if display:
        if display_conv_func is not None:
            try:
                disp_value = display_conv_func(value)
            except Exception, err:
                print "Conversion error: %s" % str(err)
                disp_value = value
        else:
            disp_value = value
        disp_value = str(disp_value)
        if normal:
            value = (value, disp_value)
        else:
            value = disp_value

    return value

def get_xml_info(data):
    fn = data.get("FileName")
    if fn is None:
        fn = "unknown"
    else:
        if not XMLDataInfo.xml_known_xmls.has_key(fn):
            fn = "unknown"
    return XMLDataInfo.xml_known_xmls[fn]

def make_data_identifier(data):
    xml_info = get_xml_info(data)

    identifier = [xml_info[0]]
    for element_path, conversion in xml_info[3]:
        known_data = XMLDataInfo.xml_known_data[xml_info[0]].get(element_path)
        value = data.get(element_path)

        if known_data is None:
            return None

        val = convert_value(value, known_data[1], known_data[2], True, False)

        if val is not None and conversion is not None:
            try:
                cval = conversion(val)
            except:
                cval = val
        else:
            cval = val
        if cval is not None:
            identifier.append(cval)
    return identifier

def save_data(data):
    identifier = make_data_identifier(data)
    xml_info = get_xml_info(data)
    idstr = xml_info[5](identifier)
    if xml_info[1]:
        userid = None
    else:
        userid = int(data.get("UserID"))
    data = dict([(idstr + "." + key, value) for key, value in data.items()])
    if not data.has_key(idstr):
        data[idstr] = ""
    databases.sets(userid, xml_info[2], data, idstr)
    if xml_info[7]:
        xl_key = xml_info[0] + ".xml_list"
        xlist = databases.get(userid, xml_info[2], xl_key)
        if xlist is None or len(xlist) == 0:
            databases.sets(userid, xml_info[2], {xl_key: idstr})
        else:
            xll = xlist.split(",")
            if not idstr in xll:
                databases.sets(userid, xml_info[2], {xl_key: xlist + "," + idstr})
    return identifier

class HTXMLHandler(saxlib.HandlerBase):
    """Handles the actual xml parsing. The resulting data will be stored in
    HTXMLHandler.parsed_data.

    The result is put in a dictionary of element paths as keys.

    The attributes of the element is put in element_path.xml_attrs.attribute.

    If there are attributes ID (primarily or Index, the value of those
    attributes will be appended to the element path.

    If the element path is occuring more than once, the old value will be
    overwritten."""

    def __init__(self):
        self.__reset()

    def startDocument(self):
        self.__reset()

    def endDocument(self):
        self.__chain_recurse(self.current_chain[0], [], None, True)

    def __chain_recurse(self, chain, namechain, extra, first):
        (name, attrs, content, children) = chain

        if content.strip() == "NOT AVAILABLE":
            raise Fetch.FetchError(_("Team is currently playing match."))

        if extra is not None:
            ename = name + "+" + extra
        else:
            ename = name
        if not first:
            namechain.append(ename)
            element_path = string.join(namechain, ".")
            if len(content) > 0:
                self.parsed_data[element_path] = content
            if len(attrs) > 0:
                for key, value in attrs.items():
                    self.parsed_data[element_path + ".xml_attrs." + key] = value

        childrens_names = [x[0] for x in children]
        i = -1
        nextras = {}
        for child in children:
            nname = child[0]
            nattrs = child[1]
            nextra = None
            if childrens_names.count(nname) > 1:
                if nattrs.has_key("ID"):
                    nextra = nattrs["ID"]
                elif nattrs.has_key("Index"):
                    nextra = nattrs["Index"]
                else:
                    i += 1
                    nextra = str(i)

                if not nextras.has_key(nname):
                    nextras[nname] = []

                while nextra in nextras[nname]:
                    i += 1
                    nextra = str(i)

                nextras[nname].append(nextra)

                npath = string.join(namechain + [nname, "xml_list"], ".")
                if self.parsed_data.has_key(npath):
                    self.parsed_data[npath] += "," + nextra
                else:
                    self.parsed_data[npath] = nextra
            self.__chain_recurse(child, namechain + [], nextra, False)

    def startElement(self, name, attrs):
        """Function that handles an opening element tag."""
        try:
            name = name.encode(I18n.def_charset)
        except:
            name = name.encode("utf8")

        self.current_chain.append([name, {}, "", []])

        for key, value in attrs.items():
            try:
                key = key.encode(I18n.def_charset)
            except:
                key = key.encode("utf8")

            try:
                value = value.encode(I18n.def_charset)
            except:
                value = value.encode("utf8")

            self.current_chain[-1][1][key] = value

    def endElement(self, name):
        """Function that handles an ending element tag."""
        try:
            name = name.encode(I18n.def_charset)
        except:
            name = name.encode("utf8")

        if len(self.current_chain) == 0 or name != self.current_chain[-1][0]:
            sys.stdout.write("*** XML Parse Error: Strange end element '%s'" % name)
            sys.stdout.write("\n")
            sys.stdout.flush()
            return

        content = self.current_chain[-1][2].strip()

        try:
            content = content.encode(I18n.def_charset)
        except:
            content = content.encode("utf8")

        self.current_chain[-1][2] = content

        if len(self.current_chain) > 1:
            self.current_chain[-2][3].append(self.current_chain[-1])
            self.current_chain = self.current_chain[:-1]

    def characters(self, content, offset, length):
        """Function that handles characters that will be the content of an
        element."""
        if len(self.current_chain) > 0:
            self.current_chain[-1][2] += content[offset:offset+length]

    def __reset(self):
        """Reset everything."""
        self.current_chain = []
        self.parsed_data = {}

def parse_xml_file(xmlfile):
    """Takes a string to an xml file and parses it and returns the
    resulting structure."""
    handler = HTXMLHandler()
    parser = saxexts.make_parser()

    parser.setDocumentHandler(handler)
    inFile = file(xmlfile, 'r')

    parser.parseFile(inFile)
    inFile.close()

    return handler.parsed_data

def parse_xml_string(xmldata):
    """Takes a string consisting of xml data and parses it and returns the
    resulting structure."""

    handler = HTXMLHandler()
    parser = saxexts.make_parser()

    parser.setDocumentHandler(handler)
    inFile = cStringIO.StringIO(xmldata)

    parser.parseFile(inFile)
    inFile.close()

    return handler.parsed_data

def get(identifier,
        paths = [],
        normal = True, display = False,
        userid = None,
        nofetch = False,
        force_update = False,
        progress_dialog = True,
        at_least_version = None,
        get_latest_available = False,
        max_id = None):
    """Get data from the database.

     - userid -- The user to get the data from.
     - identifier -- a structure to identify the xml to get data
                     from (see XMLDataInfo.py).
     - paths -- A list of paths within the xml to the desired data.
     - normal -- Get the normal value.
     - display -- Get the value in display form.
     - nofetch -- If fetching new data is allowed or not.
     - progress_dialog -- Show a progress dialog during download.
     - at_least_version -- If the xml in the database isn't at least of
                           this version, fetch a new one. No effect if
                           nofetch is True.
     - get_latest_available -- Get the latest available version in the
                               database. No effect if force_update is
                               True.
     - max_id -- Identifier to get latest available, before this id.
                 NOTE: Do not include the mattricks name in the identifier.

    If both normal and display is True, then both is returned as a tuple.

    Returns a dictionary to the data if more than one path supplied,
    otherwise returns just the data."""

    xml_info = XMLDataInfo.xml_known_xmls[XMLDataInfo.xml_name_table[identifier[0]]]
    if len(identifier) <= 1:
        identifier = [identifier[0]] + xml_info[4]()
    idstr = xml_info[5](identifier)

    if not xml_info[1]:
        if userid is None:
            nuserid = CommonData.get_current_teamid(True)
        else:
            nuserid = userid
    else:
        nuserid = None

    filename = databases.get(nuserid, xml_info[2], idstr + "." + "FileName")
    if at_least_version is not None and filename is not None:
        version_str = databases.get(nuserid, xml_info[2], idstr + "." + "Version")
        if version_str is None:
            force_update = True
        else:
            version = []
            for vn in version_str.split('.'):
                try:
                    vn_val = int(vn)
                except ValueError:
                    vn_val = vn
                version.append(vn_val)
            if at_least_version > version:
                force_update = True

    if (filename is None or force_update) and not nofetch:
        args = xml_info[6](identifier)
        if args is not None:

            do_fetch = True
            while do_fetch:
                do_fetch = False

                loginname = databases.get(None, True, "Mattricks.loginname")
                password = databases.get(None, True, "Mattricks.password")
                if loginname is None or password is None:
                    res = DialogSet.password_getting_dialog()
                    if res is not None:
                        loginname, password = res
                if loginname is not None and password is not None:

                    if DialogSet.progress_dialog_maker_func is not None:
                        if DialogSet.always_progress_dialog or progress_dialog:
                            title = _("Downloading %s.") % xml_info[8]
                            dlg = DialogSet.progress_dialog_maker_func(title)
                        else:
                            dlg = None
                    else:
                        dlg = None

                    try:
                        identifier = Fetch.conn.fetch_xml(loginname, password,
                                                          XMLDataInfo.xml_name_table[identifier[0]],
                                                          args,
                                                          None,
                                                          dlg)
                    except Fetch.FetchError, err:
                        if dlg is not None:
                            DialogSet.destroy_dialog_maker_func(dlg)
                            dlg = None

                        DialogSet.error_message_dialog(str(err))

                        if err.type == Fetch.FETCH_ERROR_LOGIN_ERROR:
                            databases.sets(None, True, {"Mattricks.loginname": None,
                                                        "Mattricks.password": None})
                            do_fetch = True

                        continue

                    if dlg is not None:
                        DialogSet.destroy_dialog_maker_func(dlg)
                        dlg = None

                    return get(identifier, paths, normal,
                               display, userid, True, False,
                               at_least_version, get_latest_available,
                               max_id)

    elif get_latest_available and filename is None:
        # Get latest available xml.
        xl_key = xml_info[0] + ".xml_list"
        xlist = databases.get(nuserid, xml_info[2], xl_key)

        if xlist is None or len(xlist) == 0:
            xlist = []
        else:
            xlist = xlist.split(",")

        less_fn = xml_info[9]
        if less_fn is None:
            less_fn = operator.lt

        if max_id is not None:
            max_id = [identifier[0]] + max_id
            max_idstr = xml_info[5](max_id)

        curr_idstr = None
        for xl_idstr in xlist:
            if max_id is None:
                if curr_idstr is None or less_fn(curr_idstr, xl_idstr):
                    curr_idstr = xl_idstr
            else:
                if curr_idstr is None or \
                   (less_fn(xl_idstr, max_idstr) and \
                    (less_fn(curr_idstr, xl_idstr) or \
                     not less_fn(curr_idstr, max_idstr))):

                    curr_idstr = xl_idstr

        if curr_idstr is not None:
            filename = databases.get(nuserid, xml_info[2], xl_idstr + "." + "FileName")
            idstr = curr_idstr
        else:
            get(identifier, paths, normal,
                display, userid, False, True,
                at_least_version, False, None)

    just_one = (len(paths) == 1)

    if len(paths) > 0:
        keys = []
        for path in paths:
            keys.append(idstr + "." + path)

        values = databases.gets(nuserid, xml_info[2], keys)
    else:
        values = databases.get_all(nuserid, xml_info[2], idstr)
        keys = values.keys()
        idslen = len(idstr + ".")
        paths = [x[idslen:] for x in keys]

    resvalues = {}
    known_data = XMLDataInfo.xml_known_data[xml_info[0]]
    for i in range(len(paths)):
        key = string.join([x.split("+")[0] for x in paths[i].split(".")], ".")
        kd = known_data.get(key)
        value = values[keys[i]]

        if kd is None:
            if normal and display:
                resvalues[paths[i]] = (value, value)
            else:
                resvalues[paths[i]] = value
        else:
            resvalues[paths[i]] = convert_value(value, kd[1], kd[2], normal, display)

    if just_one:
        return resvalues[paths[0]]
    return resvalues
