#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


# Internationalization
from Mattricks import I18n

I18n.install_translation()

from distutils.core import setup
try:
    import py2exe
    py2exe = True
except ImportError:
    py2exe = False

from Mattricks import Version
from Mattricks import Countries

import os
import re
import string

if py2exe:
    try:
        import sys
        import modulefinder
        import win32com
        for p in win32com.__path__[1:]:
            modulefinder.AddPackagePath("win32com", p)
        for extra in ["win32com.shell","win32com.mapi"]:
            __import__(extra)
            m = sys.modules[extra]
            for p in m.__path__[1:]:
                modulefinder.AddPackagePath(extra, p)
    except ImportError:
        # no build path setup, no worries.
        pass

def convert_files(files, vartable, varlisttable):
    re_fdd = re.compile("###(.*?)###", re.DOTALL | re.MULTILINE)

    for infile, outfile in files:
        f = open(infile, 'r')
        fdata = f.read()
        f.close()

        m = re_fdd.search(fdata)
        while m:
            s = m.group(1)
            res = ""
            for var, valuelist in varlisttable:
                re_var = re.compile("#" + var + "#", re.DOTALL | re.MULTILINE)
                m2 = re_var.search(s)
                if m2:
                    for value in valuelist:
                        res += s.replace("#" + var + "#", str(value))
            fdata = fdata[:m.start(0)] + res + fdata[m.end(0):]
            m = re_fdd.search(fdata)

        for var, value in vartable:
            fdata = fdata.replace("#" + var + "#", str(value))

        f = open(outfile, 'w')
        f.write(fdata)
        f.close()

def crlf_convert_file(filename):
    content = open(filename, 'r').read()
    content = string.join(content.splitlines() + [''], "\n")
    open(filename, 'w').write(content)

def get_image_files(ignores):
    files = ["Data/" + f for f in os.listdir('Data')]
    files = filter((lambda x: x.endswith('.png')), files)
    files = filter((lambda x: x not in ignores), files)
    return files

if Version.version_cvs:
    VERSION = Version.version + Version.cvsdate_str
else:
    VERSION = Version.version

real_version = VERSION
if py2exe:
    VERSION = str(Version.version_major) + "." + str(Version.version_minor)


DESCRIPTION = """\
Mattricks is an attempt to create a free helper program for hattricks players
with a multitude of tools to help fetching data, comparing and other things.
The program is currently under heavy development.
"""

languages = ["sv", "pl", "nl", "de", "it", "es", "pt"]

lang_files = []
for l in languages:
    lang_files.append(("share/locale/%s/LC_MESSAGES" % l,
                       ["locale/%s/LC_MESSAGES/mattricks.mo" % l]))

convert_files([("scripts/mattricks.iss.in", "scripts/mattricks.iss"),
               ("debian/changelog.in", "debian/changelog")],
              [("VERSION", real_version)],
              [("LOCALE", languages)])

all_flags = ["Data/" + f for f in
             filter(lambda f: f != None, list(zip(*Countries.country_info.values())[5]))]

image_files = get_image_files(all_flags)

if py2exe:
    for filename in ["AUTHORS", "BUGS", "COPYING", "ChangeLog", "INSTALL", "NEWS", "README", "README.MacOSX", "TODO"]:
        crlf_convert_file(filename)

setup(name="Mattricks",
      version=VERSION,
      license="GPL",
      description="A free helper program for hattrick players.",
      long_description=DESCRIPTION,
      author="Mattricks Team",
      author_email="mattricks@lists.lysator.liu.se",
      url="http://www.lysator.liu.se/mattricks/",
      packages=["Mattricks"],
      scripts=["mattricks.py"],
      windows=[{"script": "mattricks.py",
                "icon_resources": [(1, "scripts/mattricks-icons.ico")]
                }],
      data_files=[("share/Mattricks",
                   image_files + all_flags),
#                  ("share/doc/Mattricks-" + VERSION,
#                   ["COPYING", "README", "AUTHORS", "NEWS", "scripts/testfile.hrf"])
                  ] + lang_files)
